(function(){const R=document.createElement("link").relList;if(R&&R.supports&&R.supports("modulepreload"))return;for(const p of document.querySelectorAll('link[rel="modulepreload"]'))d(p);new MutationObserver(p=>{for(const o of p)if(o.type==="childList")for(const y of o.addedNodes)y.tagName==="LINK"&&y.rel==="modulepreload"&&d(y)}).observe(document,{childList:!0,subtree:!0});function c(p){const o={};return p.integrity&&(o.integrity=p.integrity),p.referrerPolicy&&(o.referrerPolicy=p.referrerPolicy),p.crossOrigin==="use-credentials"?o.credentials="include":p.crossOrigin==="anonymous"?o.credentials="omit":o.credentials="same-origin",o}function d(p){if(p.ep)return;p.ep=!0;const o=c(p);fetch(p.href,o)}})();var Qa=typeof globalThis<"u"?globalThis:typeof window<"u"?window:typeof global<"u"?global:typeof self<"u"?self:{};function M0(s){return s&&s.__esModule&&Object.prototype.hasOwnProperty.call(s,"default")?s.default:s}var Us={exports:{}},Wl={};/**
 * @license React
 * react-jsx-runtime.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var Zd;function j0(){if(Zd)return Wl;Zd=1;var s=Symbol.for("react.transitional.element"),R=Symbol.for("react.fragment");function c(d,p,o){var y=null;if(o!==void 0&&(y=""+o),p.key!==void 0&&(y=""+p.key),"key"in p){o={};for(var m in p)m!=="key"&&(o[m]=p[m])}else o=p;return p=o.ref,{$$typeof:s,type:d,key:y,ref:p!==void 0?p:null,props:o}}return Wl.Fragment=R,Wl.jsx=c,Wl.jsxs=c,Wl}var Gd;function B0(){return Gd||(Gd=1,Us.exports=j0()),Us.exports}var Z=B0(),Ms={exports:{}},Ot={};/**
 * @license React
 * react.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var Xd;function H0(){if(Xd)return Ot;Xd=1;var s=Symbol.for("react.transitional.element"),R=Symbol.for("react.portal"),c=Symbol.for("react.fragment"),d=Symbol.for("react.strict_mode"),p=Symbol.for("react.profiler"),o=Symbol.for("react.consumer"),y=Symbol.for("react.context"),m=Symbol.for("react.forward_ref"),h=Symbol.for("react.suspense"),r=Symbol.for("react.memo"),E=Symbol.for("react.lazy"),A=Symbol.iterator;function w(C){return C===null||typeof C!="object"?null:(C=A&&C[A]||C["@@iterator"],typeof C=="function"?C:null)}var _={isMounted:function(){return!1},enqueueForceUpdate:function(){},enqueueReplaceState:function(){},enqueueSetState:function(){}},z=Object.assign,g={};function S(C,N,B){this.props=C,this.context=N,this.refs=g,this.updater=B||_}S.prototype.isReactComponent={},S.prototype.setState=function(C,N){if(typeof C!="object"&&typeof C!="function"&&C!=null)throw Error("takes an object of state variables to update or a function which returns an object of state variables.");this.updater.enqueueSetState(this,C,N,"setState")},S.prototype.forceUpdate=function(C){this.updater.enqueueForceUpdate(this,C,"forceUpdate")};function v(){}v.prototype=S.prototype;function T(C,N,B){this.props=C,this.context=N,this.refs=g,this.updater=B||_}var D=T.prototype=new v;D.constructor=T,z(D,S.prototype),D.isPureReactComponent=!0;var L=Array.isArray,j={H:null,A:null,T:null,S:null,V:null},tt=Object.prototype.hasOwnProperty;function F(C,N,B,nt,it,dt){return B=dt.ref,{$$typeof:s,type:C,key:N,ref:B!==void 0?B:null,props:dt}}function ft(C,N){return F(C.type,N,void 0,void 0,void 0,C.props)}function $(C){return typeof C=="object"&&C!==null&&C.$$typeof===s}function rt(C){var N={"=":"=0",":":"=2"};return"$"+C.replace(/[=:]/g,function(B){return N[B]})}var vt=/\/+/g;function M(C,N){return typeof C=="object"&&C!==null&&C.key!=null?rt(""+C.key):N.toString(36)}function et(){}function b(C){switch(C.status){case"fulfilled":return C.value;case"rejected":throw C.reason;default:switch(typeof C.status=="string"?C.then(et,et):(C.status="pending",C.then(function(N){C.status==="pending"&&(C.status="fulfilled",C.value=N)},function(N){C.status==="pending"&&(C.status="rejected",C.reason=N)})),C.status){case"fulfilled":return C.value;case"rejected":throw C.reason}}throw C}function at(C,N,B,nt,it){var dt=typeof C;(dt==="undefined"||dt==="boolean")&&(C=null);var _t=!1;if(C===null)_t=!0;else switch(dt){case"bigint":case"string":case"number":_t=!0;break;case"object":switch(C.$$typeof){case s:case R:_t=!0;break;case E:return _t=C._init,at(_t(C._payload),N,B,nt,it)}}if(_t)return it=it(C),_t=nt===""?"."+M(C,0):nt,L(it)?(B="",_t!=null&&(B=_t.replace(vt,"$&/")+"/"),at(it,N,B,"",function(Ct){return Ct})):it!=null&&($(it)&&(it=ft(it,B+(it.key==null||C&&C.key===it.key?"":(""+it.key).replace(vt,"$&/")+"/")+_t)),N.push(it)),1;_t=0;var Lt=nt===""?".":nt+":";if(L(C))for(var xt=0;xt<C.length;xt++)nt=C[xt],dt=Lt+M(nt,xt),_t+=at(nt,N,B,dt,it);else if(xt=w(C),typeof xt=="function")for(C=xt.call(C),xt=0;!(nt=C.next()).done;)nt=nt.value,dt=Lt+M(nt,xt++),_t+=at(nt,N,B,dt,it);else if(dt==="object"){if(typeof C.then=="function")return at(b(C),N,B,nt,it);throw N=String(C),Error("Objects are not valid as a React child (found: "+(N==="[object Object]"?"object with keys {"+Object.keys(C).join(", ")+"}":N)+"). If you meant to render a collection of children, use an array instead.")}return _t}function G(C,N,B){if(C==null)return C;var nt=[],it=0;return at(C,nt,"","",function(dt){return N.call(B,dt,it++)}),nt}function X(C){if(C._status===-1){var N=C._result;N=N(),N.then(function(B){(C._status===0||C._status===-1)&&(C._status=1,C._result=B)},function(B){(C._status===0||C._status===-1)&&(C._status=2,C._result=B)}),C._status===-1&&(C._status=0,C._result=N)}if(C._status===1)return C._result.default;throw C._result}var gt=typeof reportError=="function"?reportError:function(C){if(typeof window=="object"&&typeof window.ErrorEvent=="function"){var N=new window.ErrorEvent("error",{bubbles:!0,cancelable:!0,message:typeof C=="object"&&C!==null&&typeof C.message=="string"?String(C.message):String(C),error:C});if(!window.dispatchEvent(N))return}else if(typeof process=="object"&&typeof process.emit=="function"){process.emit("uncaughtException",C);return}console.error(C)};function st(){}return Ot.Children={map:G,forEach:function(C,N,B){G(C,function(){N.apply(this,arguments)},B)},count:function(C){var N=0;return G(C,function(){N++}),N},toArray:function(C){return G(C,function(N){return N})||[]},only:function(C){if(!$(C))throw Error("React.Children.only expected to receive a single React element child.");return C}},Ot.Component=S,Ot.Fragment=c,Ot.Profiler=p,Ot.PureComponent=T,Ot.StrictMode=d,Ot.Suspense=h,Ot.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE=j,Ot.__COMPILER_RUNTIME={__proto__:null,c:function(C){return j.H.useMemoCache(C)}},Ot.cache=function(C){return function(){return C.apply(null,arguments)}},Ot.cloneElement=function(C,N,B){if(C==null)throw Error("The argument must be a React element, but you passed "+C+".");var nt=z({},C.props),it=C.key,dt=void 0;if(N!=null)for(_t in N.ref!==void 0&&(dt=void 0),N.key!==void 0&&(it=""+N.key),N)!tt.call(N,_t)||_t==="key"||_t==="__self"||_t==="__source"||_t==="ref"&&N.ref===void 0||(nt[_t]=N[_t]);var _t=arguments.length-2;if(_t===1)nt.children=B;else if(1<_t){for(var Lt=Array(_t),xt=0;xt<_t;xt++)Lt[xt]=arguments[xt+2];nt.children=Lt}return F(C.type,it,void 0,void 0,dt,nt)},Ot.createContext=function(C){return C={$$typeof:y,_currentValue:C,_currentValue2:C,_threadCount:0,Provider:null,Consumer:null},C.Provider=C,C.Consumer={$$typeof:o,_context:C},C},Ot.createElement=function(C,N,B){var nt,it={},dt=null;if(N!=null)for(nt in N.key!==void 0&&(dt=""+N.key),N)tt.call(N,nt)&&nt!=="key"&&nt!=="__self"&&nt!=="__source"&&(it[nt]=N[nt]);var _t=arguments.length-2;if(_t===1)it.children=B;else if(1<_t){for(var Lt=Array(_t),xt=0;xt<_t;xt++)Lt[xt]=arguments[xt+2];it.children=Lt}if(C&&C.defaultProps)for(nt in _t=C.defaultProps,_t)it[nt]===void 0&&(it[nt]=_t[nt]);return F(C,dt,void 0,void 0,null,it)},Ot.createRef=function(){return{current:null}},Ot.forwardRef=function(C){return{$$typeof:m,render:C}},Ot.isValidElement=$,Ot.lazy=function(C){return{$$typeof:E,_payload:{_status:-1,_result:C},_init:X}},Ot.memo=function(C,N){return{$$typeof:r,type:C,compare:N===void 0?null:N}},Ot.startTransition=function(C){var N=j.T,B={};j.T=B;try{var nt=C(),it=j.S;it!==null&&it(B,nt),typeof nt=="object"&&nt!==null&&typeof nt.then=="function"&&nt.then(st,gt)}catch(dt){gt(dt)}finally{j.T=N}},Ot.unstable_useCacheRefresh=function(){return j.H.useCacheRefresh()},Ot.use=function(C){return j.H.use(C)},Ot.useActionState=function(C,N,B){return j.H.useActionState(C,N,B)},Ot.useCallback=function(C,N){return j.H.useCallback(C,N)},Ot.useContext=function(C){return j.H.useContext(C)},Ot.useDebugValue=function(){},Ot.useDeferredValue=function(C,N){return j.H.useDeferredValue(C,N)},Ot.useEffect=function(C,N,B){var nt=j.H;if(typeof B=="function")throw Error("useEffect CRUD overload is not enabled in this build of React.");return nt.useEffect(C,N)},Ot.useId=function(){return j.H.useId()},Ot.useImperativeHandle=function(C,N,B){return j.H.useImperativeHandle(C,N,B)},Ot.useInsertionEffect=function(C,N){return j.H.useInsertionEffect(C,N)},Ot.useLayoutEffect=function(C,N){return j.H.useLayoutEffect(C,N)},Ot.useMemo=function(C,N){return j.H.useMemo(C,N)},Ot.useOptimistic=function(C,N){return j.H.useOptimistic(C,N)},Ot.useReducer=function(C,N,B){return j.H.useReducer(C,N,B)},Ot.useRef=function(C){return j.H.useRef(C)},Ot.useState=function(C){return j.H.useState(C)},Ot.useSyncExternalStore=function(C,N,B){return j.H.useSyncExternalStore(C,N,B)},Ot.useTransition=function(){return j.H.useTransition()},Ot.version="19.1.1",Ot}var Qd;function Vs(){return Qd||(Qd=1,Ms.exports=H0()),Ms.exports}var I=Vs(),js={exports:{}},$l={},Bs={exports:{}},Hs={};/**
 * @license React
 * scheduler.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var Vd;function L0(){return Vd||(Vd=1,function(s){function R(G,X){var gt=G.length;G.push(X);t:for(;0<gt;){var st=gt-1>>>1,C=G[st];if(0<p(C,X))G[st]=X,G[gt]=C,gt=st;else break t}}function c(G){return G.length===0?null:G[0]}function d(G){if(G.length===0)return null;var X=G[0],gt=G.pop();if(gt!==X){G[0]=gt;t:for(var st=0,C=G.length,N=C>>>1;st<N;){var B=2*(st+1)-1,nt=G[B],it=B+1,dt=G[it];if(0>p(nt,gt))it<C&&0>p(dt,nt)?(G[st]=dt,G[it]=gt,st=it):(G[st]=nt,G[B]=gt,st=B);else if(it<C&&0>p(dt,gt))G[st]=dt,G[it]=gt,st=it;else break t}}return X}function p(G,X){var gt=G.sortIndex-X.sortIndex;return gt!==0?gt:G.id-X.id}if(s.unstable_now=void 0,typeof performance=="object"&&typeof performance.now=="function"){var o=performance;s.unstable_now=function(){return o.now()}}else{var y=Date,m=y.now();s.unstable_now=function(){return y.now()-m}}var h=[],r=[],E=1,A=null,w=3,_=!1,z=!1,g=!1,S=!1,v=typeof setTimeout=="function"?setTimeout:null,T=typeof clearTimeout=="function"?clearTimeout:null,D=typeof setImmediate<"u"?setImmediate:null;function L(G){for(var X=c(r);X!==null;){if(X.callback===null)d(r);else if(X.startTime<=G)d(r),X.sortIndex=X.expirationTime,R(h,X);else break;X=c(r)}}function j(G){if(g=!1,L(G),!z)if(c(h)!==null)z=!0,tt||(tt=!0,M());else{var X=c(r);X!==null&&at(j,X.startTime-G)}}var tt=!1,F=-1,ft=5,$=-1;function rt(){return S?!0:!(s.unstable_now()-$<ft)}function vt(){if(S=!1,tt){var G=s.unstable_now();$=G;var X=!0;try{t:{z=!1,g&&(g=!1,T(F),F=-1),_=!0;var gt=w;try{e:{for(L(G),A=c(h);A!==null&&!(A.expirationTime>G&&rt());){var st=A.callback;if(typeof st=="function"){A.callback=null,w=A.priorityLevel;var C=st(A.expirationTime<=G);if(G=s.unstable_now(),typeof C=="function"){A.callback=C,L(G),X=!0;break e}A===c(h)&&d(h),L(G)}else d(h);A=c(h)}if(A!==null)X=!0;else{var N=c(r);N!==null&&at(j,N.startTime-G),X=!1}}break t}finally{A=null,w=gt,_=!1}X=void 0}}finally{X?M():tt=!1}}}var M;if(typeof D=="function")M=function(){D(vt)};else if(typeof MessageChannel<"u"){var et=new MessageChannel,b=et.port2;et.port1.onmessage=vt,M=function(){b.postMessage(null)}}else M=function(){v(vt,0)};function at(G,X){F=v(function(){G(s.unstable_now())},X)}s.unstable_IdlePriority=5,s.unstable_ImmediatePriority=1,s.unstable_LowPriority=4,s.unstable_NormalPriority=3,s.unstable_Profiling=null,s.unstable_UserBlockingPriority=2,s.unstable_cancelCallback=function(G){G.callback=null},s.unstable_forceFrameRate=function(G){0>G||125<G?console.error("forceFrameRate takes a positive int between 0 and 125, forcing frame rates higher than 125 fps is not supported"):ft=0<G?Math.floor(1e3/G):5},s.unstable_getCurrentPriorityLevel=function(){return w},s.unstable_next=function(G){switch(w){case 1:case 2:case 3:var X=3;break;default:X=w}var gt=w;w=X;try{return G()}finally{w=gt}},s.unstable_requestPaint=function(){S=!0},s.unstable_runWithPriority=function(G,X){switch(G){case 1:case 2:case 3:case 4:case 5:break;default:G=3}var gt=w;w=G;try{return X()}finally{w=gt}},s.unstable_scheduleCallback=function(G,X,gt){var st=s.unstable_now();switch(typeof gt=="object"&&gt!==null?(gt=gt.delay,gt=typeof gt=="number"&&0<gt?st+gt:st):gt=st,G){case 1:var C=-1;break;case 2:C=250;break;case 5:C=1073741823;break;case 4:C=1e4;break;default:C=5e3}return C=gt+C,G={id:E++,callback:X,priorityLevel:G,startTime:gt,expirationTime:C,sortIndex:-1},gt>st?(G.sortIndex=gt,R(r,G),c(h)===null&&G===c(r)&&(g?(T(F),F=-1):g=!0,at(j,gt-st))):(G.sortIndex=C,R(h,G),z||_||(z=!0,tt||(tt=!0,M()))),G},s.unstable_shouldYield=rt,s.unstable_wrapCallback=function(G){var X=w;return function(){var gt=w;w=X;try{return G.apply(this,arguments)}finally{w=gt}}}}(Hs)),Hs}var Kd;function q0(){return Kd||(Kd=1,Bs.exports=L0()),Bs.exports}var Ls={exports:{}},ye={};/**
 * @license React
 * react-dom.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var Jd;function Y0(){if(Jd)return ye;Jd=1;var s=Vs();function R(h){var r="https://react.dev/errors/"+h;if(1<arguments.length){r+="?args[]="+encodeURIComponent(arguments[1]);for(var E=2;E<arguments.length;E++)r+="&args[]="+encodeURIComponent(arguments[E])}return"Minified React error #"+h+"; visit "+r+" for the full message or use the non-minified dev environment for full errors and additional helpful warnings."}function c(){}var d={d:{f:c,r:function(){throw Error(R(522))},D:c,C:c,L:c,m:c,X:c,S:c,M:c},p:0,findDOMNode:null},p=Symbol.for("react.portal");function o(h,r,E){var A=3<arguments.length&&arguments[3]!==void 0?arguments[3]:null;return{$$typeof:p,key:A==null?null:""+A,children:h,containerInfo:r,implementation:E}}var y=s.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE;function m(h,r){if(h==="font")return"";if(typeof r=="string")return r==="use-credentials"?r:""}return ye.__DOM_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE=d,ye.createPortal=function(h,r){var E=2<arguments.length&&arguments[2]!==void 0?arguments[2]:null;if(!r||r.nodeType!==1&&r.nodeType!==9&&r.nodeType!==11)throw Error(R(299));return o(h,r,null,E)},ye.flushSync=function(h){var r=y.T,E=d.p;try{if(y.T=null,d.p=2,h)return h()}finally{y.T=r,d.p=E,d.d.f()}},ye.preconnect=function(h,r){typeof h=="string"&&(r?(r=r.crossOrigin,r=typeof r=="string"?r==="use-credentials"?r:"":void 0):r=null,d.d.C(h,r))},ye.prefetchDNS=function(h){typeof h=="string"&&d.d.D(h)},ye.preinit=function(h,r){if(typeof h=="string"&&r&&typeof r.as=="string"){var E=r.as,A=m(E,r.crossOrigin),w=typeof r.integrity=="string"?r.integrity:void 0,_=typeof r.fetchPriority=="string"?r.fetchPriority:void 0;E==="style"?d.d.S(h,typeof r.precedence=="string"?r.precedence:void 0,{crossOrigin:A,integrity:w,fetchPriority:_}):E==="script"&&d.d.X(h,{crossOrigin:A,integrity:w,fetchPriority:_,nonce:typeof r.nonce=="string"?r.nonce:void 0})}},ye.preinitModule=function(h,r){if(typeof h=="string")if(typeof r=="object"&&r!==null){if(r.as==null||r.as==="script"){var E=m(r.as,r.crossOrigin);d.d.M(h,{crossOrigin:E,integrity:typeof r.integrity=="string"?r.integrity:void 0,nonce:typeof r.nonce=="string"?r.nonce:void 0})}}else r==null&&d.d.M(h)},ye.preload=function(h,r){if(typeof h=="string"&&typeof r=="object"&&r!==null&&typeof r.as=="string"){var E=r.as,A=m(E,r.crossOrigin);d.d.L(h,E,{crossOrigin:A,integrity:typeof r.integrity=="string"?r.integrity:void 0,nonce:typeof r.nonce=="string"?r.nonce:void 0,type:typeof r.type=="string"?r.type:void 0,fetchPriority:typeof r.fetchPriority=="string"?r.fetchPriority:void 0,referrerPolicy:typeof r.referrerPolicy=="string"?r.referrerPolicy:void 0,imageSrcSet:typeof r.imageSrcSet=="string"?r.imageSrcSet:void 0,imageSizes:typeof r.imageSizes=="string"?r.imageSizes:void 0,media:typeof r.media=="string"?r.media:void 0})}},ye.preloadModule=function(h,r){if(typeof h=="string")if(r){var E=m(r.as,r.crossOrigin);d.d.m(h,{as:typeof r.as=="string"&&r.as!=="script"?r.as:void 0,crossOrigin:E,integrity:typeof r.integrity=="string"?r.integrity:void 0})}else d.d.m(h)},ye.requestFormReset=function(h){d.d.r(h)},ye.unstable_batchedUpdates=function(h,r){return h(r)},ye.useFormState=function(h,r,E){return y.H.useFormState(h,r,E)},ye.useFormStatus=function(){return y.H.useHostTransitionStatus()},ye.version="19.1.1",ye}var Fd;function Z0(){if(Fd)return Ls.exports;Fd=1;function s(){if(!(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__>"u"||typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE!="function"))try{__REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE(s)}catch(R){console.error(R)}}return s(),Ls.exports=Y0(),Ls.exports}/**
 * @license React
 * react-dom-client.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var Wd;function G0(){if(Wd)return $l;Wd=1;var s=q0(),R=Vs(),c=Z0();function d(t){var e="https://react.dev/errors/"+t;if(1<arguments.length){e+="?args[]="+encodeURIComponent(arguments[1]);for(var a=2;a<arguments.length;a++)e+="&args[]="+encodeURIComponent(arguments[a])}return"Minified React error #"+t+"; visit "+e+" for the full message or use the non-minified dev environment for full errors and additional helpful warnings."}function p(t){return!(!t||t.nodeType!==1&&t.nodeType!==9&&t.nodeType!==11)}function o(t){var e=t,a=t;if(t.alternate)for(;e.return;)e=e.return;else{t=e;do e=t,(e.flags&4098)!==0&&(a=e.return),t=e.return;while(t)}return e.tag===3?a:null}function y(t){if(t.tag===13){var e=t.memoizedState;if(e===null&&(t=t.alternate,t!==null&&(e=t.memoizedState)),e!==null)return e.dehydrated}return null}function m(t){if(o(t)!==t)throw Error(d(188))}function h(t){var e=t.alternate;if(!e){if(e=o(t),e===null)throw Error(d(188));return e!==t?null:t}for(var a=t,n=e;;){var l=a.return;if(l===null)break;var i=l.alternate;if(i===null){if(n=l.return,n!==null){a=n;continue}break}if(l.child===i.child){for(i=l.child;i;){if(i===a)return m(l),t;if(i===n)return m(l),e;i=i.sibling}throw Error(d(188))}if(a.return!==n.return)a=l,n=i;else{for(var u=!1,x=l.child;x;){if(x===a){u=!0,a=l,n=i;break}if(x===n){u=!0,n=l,a=i;break}x=x.sibling}if(!u){for(x=i.child;x;){if(x===a){u=!0,a=i,n=l;break}if(x===n){u=!0,n=i,a=l;break}x=x.sibling}if(!u)throw Error(d(189))}}if(a.alternate!==n)throw Error(d(190))}if(a.tag!==3)throw Error(d(188));return a.stateNode.current===a?t:e}function r(t){var e=t.tag;if(e===5||e===26||e===27||e===6)return t;for(t=t.child;t!==null;){if(e=r(t),e!==null)return e;t=t.sibling}return null}var E=Object.assign,A=Symbol.for("react.element"),w=Symbol.for("react.transitional.element"),_=Symbol.for("react.portal"),z=Symbol.for("react.fragment"),g=Symbol.for("react.strict_mode"),S=Symbol.for("react.profiler"),v=Symbol.for("react.provider"),T=Symbol.for("react.consumer"),D=Symbol.for("react.context"),L=Symbol.for("react.forward_ref"),j=Symbol.for("react.suspense"),tt=Symbol.for("react.suspense_list"),F=Symbol.for("react.memo"),ft=Symbol.for("react.lazy"),$=Symbol.for("react.activity"),rt=Symbol.for("react.memo_cache_sentinel"),vt=Symbol.iterator;function M(t){return t===null||typeof t!="object"?null:(t=vt&&t[vt]||t["@@iterator"],typeof t=="function"?t:null)}var et=Symbol.for("react.client.reference");function b(t){if(t==null)return null;if(typeof t=="function")return t.$$typeof===et?null:t.displayName||t.name||null;if(typeof t=="string")return t;switch(t){case z:return"Fragment";case S:return"Profiler";case g:return"StrictMode";case j:return"Suspense";case tt:return"SuspenseList";case $:return"Activity"}if(typeof t=="object")switch(t.$$typeof){case _:return"Portal";case D:return(t.displayName||"Context")+".Provider";case T:return(t._context.displayName||"Context")+".Consumer";case L:var e=t.render;return t=t.displayName,t||(t=e.displayName||e.name||"",t=t!==""?"ForwardRef("+t+")":"ForwardRef"),t;case F:return e=t.displayName||null,e!==null?e:b(t.type)||"Memo";case ft:e=t._payload,t=t._init;try{return b(t(e))}catch{}}return null}var at=Array.isArray,G=R.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE,X=c.__DOM_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE,gt={pending:!1,data:null,method:null,action:null},st=[],C=-1;function N(t){return{current:t}}function B(t){0>C||(t.current=st[C],st[C]=null,C--)}function nt(t,e){C++,st[C]=t.current,t.current=e}var it=N(null),dt=N(null),_t=N(null),Lt=N(null);function xt(t,e){switch(nt(_t,e),nt(dt,t),nt(it,null),e.nodeType){case 9:case 11:t=(t=e.documentElement)&&(t=t.namespaceURI)?gd(t):0;break;default:if(t=e.tagName,e=e.namespaceURI)e=gd(e),t=yd(e,t);else switch(t){case"svg":t=1;break;case"math":t=2;break;default:t=0}}B(it),nt(it,t)}function Ct(){B(it),B(dt),B(_t)}function Pt(t){t.memoizedState!==null&&nt(Lt,t);var e=it.current,a=yd(e,t.type);e!==a&&(nt(dt,t),nt(it,a))}function Vt(t){dt.current===t&&(B(it),B(dt)),Lt.current===t&&(B(Lt),Ql._currentValue=gt)}var _e=Object.prototype.hasOwnProperty,ve=s.unstable_scheduleCallback,f=s.unstable_cancelCallback,ot=s.unstable_shouldYield,P=s.unstable_requestPaint,k=s.unstable_now,O=s.unstable_getCurrentPriorityLevel,q=s.unstable_ImmediatePriority,ht=s.unstable_UserBlockingPriority,mt=s.unstable_NormalPriority,K=s.unstable_LowPriority,pt=s.unstable_IdlePriority,bt=s.log,yt=s.unstable_setDisableYieldValue,Et=null,kt=null;function Rt(t){if(typeof bt=="function"&&yt(t),kt&&typeof kt.setStrictMode=="function")try{kt.setStrictMode(Et,t)}catch{}}var Kt=Math.clz32?Math.clz32:Ka,Va=Math.log,Ue=Math.LN2;function Ka(t){return t>>>=0,t===0?32:31-(Va(t)/Ue|0)|0}var Jt=256,aa=4194304;function Xe(t){var e=t&42;if(e!==0)return e;switch(t&-t){case 1:return 1;case 2:return 2;case 4:return 4;case 8:return 8;case 16:return 16;case 32:return 32;case 64:return 64;case 128:return 128;case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:case 262144:case 524288:case 1048576:case 2097152:return t&4194048;case 4194304:case 8388608:case 16777216:case 33554432:return t&62914560;case 67108864:return 67108864;case 134217728:return 134217728;case 268435456:return 268435456;case 536870912:return 536870912;case 1073741824:return 0;default:return t}}function Se(t,e,a){var n=t.pendingLanes;if(n===0)return 0;var l=0,i=t.suspendedLanes,u=t.pingedLanes;t=t.warmLanes;var x=n&134217727;return x!==0?(n=x&~i,n!==0?l=Xe(n):(u&=x,u!==0?l=Xe(u):a||(a=x&~t,a!==0&&(l=Xe(a))))):(x=n&~i,x!==0?l=Xe(x):u!==0?l=Xe(u):a||(a=n&~t,a!==0&&(l=Xe(a)))),l===0?0:e!==0&&e!==l&&(e&i)===0&&(i=l&-l,a=e&-e,i>=a||i===32&&(a&4194048)!==0)?e:l}function Sa(t,e){return(t.pendingLanes&~(t.suspendedLanes&~t.pingedLanes)&e)===0}function Er(t,e){switch(t){case 1:case 2:case 4:case 8:case 64:return e+250;case 16:case 32:case 128:case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:case 262144:case 524288:case 1048576:case 2097152:return e+5e3;case 4194304:case 8388608:case 16777216:case 33554432:return-1;case 67108864:case 134217728:case 268435456:case 536870912:case 1073741824:return-1;default:return-1}}function ni(){var t=Jt;return Jt<<=1,(Jt&4194048)===0&&(Jt=256),t}function li(){var t=aa;return aa<<=1,(aa&62914560)===0&&(aa=4194304),t}function hn(t){for(var e=[],a=0;31>a;a++)e.push(t);return e}function Qe(t,e){t.pendingLanes|=e,e!==268435456&&(t.suspendedLanes=0,t.pingedLanes=0,t.warmLanes=0)}function mn(t,e,a,n,l,i){var u=t.pendingLanes;t.pendingLanes=a,t.suspendedLanes=0,t.pingedLanes=0,t.warmLanes=0,t.expiredLanes&=a,t.entangledLanes&=a,t.errorRecoveryDisabledLanes&=a,t.shellSuspendCounter=0;var x=t.entanglements,U=t.expirationTimes,V=t.hiddenUpdates;for(a=u&~a;0<a;){var lt=31-Kt(a),ct=1<<lt;x[lt]=0,U[lt]=-1;var J=V[lt];if(J!==null)for(V[lt]=null,lt=0;lt<J.length;lt++){var W=J[lt];W!==null&&(W.lane&=-536870913)}a&=~ct}n!==0&&Ps(t,n,0),i!==0&&l===0&&t.tag!==0&&(t.suspendedLanes|=i&~(u&~e))}function Ps(t,e,a){t.pendingLanes|=e,t.suspendedLanes&=~e;var n=31-Kt(e);t.entangledLanes|=e,t.entanglements[n]=t.entanglements[n]|1073741824|a&4194090}function to(t,e){var a=t.entangledLanes|=e;for(t=t.entanglements;a;){var n=31-Kt(a),l=1<<n;l&e|t[n]&e&&(t[n]|=e),a&=~l}}function wr(t){switch(t){case 2:t=1;break;case 8:t=4;break;case 32:t=16;break;case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:case 262144:case 524288:case 1048576:case 2097152:case 4194304:case 8388608:case 16777216:case 33554432:t=128;break;case 268435456:t=134217728;break;default:t=0}return t}function Ar(t){return t&=-t,2<t?8<t?(t&134217727)!==0?32:268435456:8:2}function eo(){var t=X.p;return t!==0?t:(t=window.event,t===void 0?32:jd(t.type))}function wh(t,e){var a=X.p;try{return X.p=t,e()}finally{X.p=a}}var xa=Math.random().toString(36).slice(2),pe="__reactFiber$"+xa,xe="__reactProps$"+xa,pn="__reactContainer$"+xa,zr="__reactEvents$"+xa,Ah="__reactListeners$"+xa,zh="__reactHandles$"+xa,ao="__reactResources$"+xa,al="__reactMarker$"+xa;function Tr(t){delete t[pe],delete t[xe],delete t[zr],delete t[Ah],delete t[zh]}function gn(t){var e=t[pe];if(e)return e;for(var a=t.parentNode;a;){if(e=a[pn]||a[pe]){if(a=e.alternate,e.child!==null||a!==null&&a.child!==null)for(t=Sd(t);t!==null;){if(a=t[pe])return a;t=Sd(t)}return e}t=a,a=t.parentNode}return null}function yn(t){if(t=t[pe]||t[pn]){var e=t.tag;if(e===5||e===6||e===13||e===26||e===27||e===3)return t}return null}function nl(t){var e=t.tag;if(e===5||e===26||e===27||e===6)return t.stateNode;throw Error(d(33))}function vn(t){var e=t[ao];return e||(e=t[ao]={hoistableStyles:new Map,hoistableScripts:new Map}),e}function se(t){t[al]=!0}var no=new Set,lo={};function Ja(t,e){bn(t,e),bn(t+"Capture",e)}function bn(t,e){for(lo[t]=e,t=0;t<e.length;t++)no.add(e[t])}var Th=RegExp("^[:A-Z_a-z\\u00C0-\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02FF\\u0370-\\u037D\\u037F-\\u1FFF\\u200C-\\u200D\\u2070-\\u218F\\u2C00-\\u2FEF\\u3001-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFFD][:A-Z_a-z\\u00C0-\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02FF\\u0370-\\u037D\\u037F-\\u1FFF\\u200C-\\u200D\\u2070-\\u218F\\u2C00-\\u2FEF\\u3001-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFFD\\-.0-9\\u00B7\\u0300-\\u036F\\u203F-\\u2040]*$"),io={},ro={};function Rh(t){return _e.call(ro,t)?!0:_e.call(io,t)?!1:Th.test(t)?ro[t]=!0:(io[t]=!0,!1)}function ii(t,e,a){if(Rh(e))if(a===null)t.removeAttribute(e);else{switch(typeof a){case"undefined":case"function":case"symbol":t.removeAttribute(e);return;case"boolean":var n=e.toLowerCase().slice(0,5);if(n!=="data-"&&n!=="aria-"){t.removeAttribute(e);return}}t.setAttribute(e,""+a)}}function ri(t,e,a){if(a===null)t.removeAttribute(e);else{switch(typeof a){case"undefined":case"function":case"symbol":case"boolean":t.removeAttribute(e);return}t.setAttribute(e,""+a)}}function na(t,e,a,n){if(n===null)t.removeAttribute(a);else{switch(typeof n){case"undefined":case"function":case"symbol":case"boolean":t.removeAttribute(a);return}t.setAttributeNS(e,a,""+n)}}var Rr,uo;function _n(t){if(Rr===void 0)try{throw Error()}catch(a){var e=a.stack.trim().match(/\n( *(at )?)/);Rr=e&&e[1]||"",uo=-1<a.stack.indexOf(`
    at`)?" (<anonymous>)":-1<a.stack.indexOf("@")?"@unknown:0:0":""}return`
`+Rr+t+uo}var Or=!1;function kr(t,e){if(!t||Or)return"";Or=!0;var a=Error.prepareStackTrace;Error.prepareStackTrace=void 0;try{var n={DetermineComponentFrameRoot:function(){try{if(e){var ct=function(){throw Error()};if(Object.defineProperty(ct.prototype,"props",{set:function(){throw Error()}}),typeof Reflect=="object"&&Reflect.construct){try{Reflect.construct(ct,[])}catch(W){var J=W}Reflect.construct(t,[],ct)}else{try{ct.call()}catch(W){J=W}t.call(ct.prototype)}}else{try{throw Error()}catch(W){J=W}(ct=t())&&typeof ct.catch=="function"&&ct.catch(function(){})}}catch(W){if(W&&J&&typeof W.stack=="string")return[W.stack,J.stack]}return[null,null]}};n.DetermineComponentFrameRoot.displayName="DetermineComponentFrameRoot";var l=Object.getOwnPropertyDescriptor(n.DetermineComponentFrameRoot,"name");l&&l.configurable&&Object.defineProperty(n.DetermineComponentFrameRoot,"name",{value:"DetermineComponentFrameRoot"});var i=n.DetermineComponentFrameRoot(),u=i[0],x=i[1];if(u&&x){var U=u.split(`
`),V=x.split(`
`);for(l=n=0;n<U.length&&!U[n].includes("DetermineComponentFrameRoot");)n++;for(;l<V.length&&!V[l].includes("DetermineComponentFrameRoot");)l++;if(n===U.length||l===V.length)for(n=U.length-1,l=V.length-1;1<=n&&0<=l&&U[n]!==V[l];)l--;for(;1<=n&&0<=l;n--,l--)if(U[n]!==V[l]){if(n!==1||l!==1)do if(n--,l--,0>l||U[n]!==V[l]){var lt=`
`+U[n].replace(" at new "," at ");return t.displayName&&lt.includes("<anonymous>")&&(lt=lt.replace("<anonymous>",t.displayName)),lt}while(1<=n&&0<=l);break}}}finally{Or=!1,Error.prepareStackTrace=a}return(a=t?t.displayName||t.name:"")?_n(a):""}function Oh(t){switch(t.tag){case 26:case 27:case 5:return _n(t.type);case 16:return _n("Lazy");case 13:return _n("Suspense");case 19:return _n("SuspenseList");case 0:case 15:return kr(t.type,!1);case 11:return kr(t.type.render,!1);case 1:return kr(t.type,!0);case 31:return _n("Activity");default:return""}}function so(t){try{var e="";do e+=Oh(t),t=t.return;while(t);return e}catch(a){return`
Error generating stack: `+a.message+`
`+a.stack}}function Me(t){switch(typeof t){case"bigint":case"boolean":case"number":case"string":case"undefined":return t;case"object":return t;default:return""}}function oo(t){var e=t.type;return(t=t.nodeName)&&t.toLowerCase()==="input"&&(e==="checkbox"||e==="radio")}function kh(t){var e=oo(t)?"checked":"value",a=Object.getOwnPropertyDescriptor(t.constructor.prototype,e),n=""+t[e];if(!t.hasOwnProperty(e)&&typeof a<"u"&&typeof a.get=="function"&&typeof a.set=="function"){var l=a.get,i=a.set;return Object.defineProperty(t,e,{configurable:!0,get:function(){return l.call(this)},set:function(u){n=""+u,i.call(this,u)}}),Object.defineProperty(t,e,{enumerable:a.enumerable}),{getValue:function(){return n},setValue:function(u){n=""+u},stopTracking:function(){t._valueTracker=null,delete t[e]}}}}function ui(t){t._valueTracker||(t._valueTracker=kh(t))}function co(t){if(!t)return!1;var e=t._valueTracker;if(!e)return!0;var a=e.getValue(),n="";return t&&(n=oo(t)?t.checked?"true":"false":t.value),t=n,t!==a?(e.setValue(t),!0):!1}function si(t){if(t=t||(typeof document<"u"?document:void 0),typeof t>"u")return null;try{return t.activeElement||t.body}catch{return t.body}}var Ch=/[\n"\\]/g;function je(t){return t.replace(Ch,function(e){return"\\"+e.charCodeAt(0).toString(16)+" "})}function Cr(t,e,a,n,l,i,u,x){t.name="",u!=null&&typeof u!="function"&&typeof u!="symbol"&&typeof u!="boolean"?t.type=u:t.removeAttribute("type"),e!=null?u==="number"?(e===0&&t.value===""||t.value!=e)&&(t.value=""+Me(e)):t.value!==""+Me(e)&&(t.value=""+Me(e)):u!=="submit"&&u!=="reset"||t.removeAttribute("value"),e!=null?Dr(t,u,Me(e)):a!=null?Dr(t,u,Me(a)):n!=null&&t.removeAttribute("value"),l==null&&i!=null&&(t.defaultChecked=!!i),l!=null&&(t.checked=l&&typeof l!="function"&&typeof l!="symbol"),x!=null&&typeof x!="function"&&typeof x!="symbol"&&typeof x!="boolean"?t.name=""+Me(x):t.removeAttribute("name")}function fo(t,e,a,n,l,i,u,x){if(i!=null&&typeof i!="function"&&typeof i!="symbol"&&typeof i!="boolean"&&(t.type=i),e!=null||a!=null){if(!(i!=="submit"&&i!=="reset"||e!=null))return;a=a!=null?""+Me(a):"",e=e!=null?""+Me(e):a,x||e===t.value||(t.value=e),t.defaultValue=e}n=n??l,n=typeof n!="function"&&typeof n!="symbol"&&!!n,t.checked=x?t.checked:!!n,t.defaultChecked=!!n,u!=null&&typeof u!="function"&&typeof u!="symbol"&&typeof u!="boolean"&&(t.name=u)}function Dr(t,e,a){e==="number"&&si(t.ownerDocument)===t||t.defaultValue===""+a||(t.defaultValue=""+a)}function Sn(t,e,a,n){if(t=t.options,e){e={};for(var l=0;l<a.length;l++)e["$"+a[l]]=!0;for(a=0;a<t.length;a++)l=e.hasOwnProperty("$"+t[a].value),t[a].selected!==l&&(t[a].selected=l),l&&n&&(t[a].defaultSelected=!0)}else{for(a=""+Me(a),e=null,l=0;l<t.length;l++){if(t[l].value===a){t[l].selected=!0,n&&(t[l].defaultSelected=!0);return}e!==null||t[l].disabled||(e=t[l])}e!==null&&(e.selected=!0)}}function ho(t,e,a){if(e!=null&&(e=""+Me(e),e!==t.value&&(t.value=e),a==null)){t.defaultValue!==e&&(t.defaultValue=e);return}t.defaultValue=a!=null?""+Me(a):""}function mo(t,e,a,n){if(e==null){if(n!=null){if(a!=null)throw Error(d(92));if(at(n)){if(1<n.length)throw Error(d(93));n=n[0]}a=n}a==null&&(a=""),e=a}a=Me(e),t.defaultValue=a,n=t.textContent,n===a&&n!==""&&n!==null&&(t.value=n)}function xn(t,e){if(e){var a=t.firstChild;if(a&&a===t.lastChild&&a.nodeType===3){a.nodeValue=e;return}}t.textContent=e}var Dh=new Set("animationIterationCount aspectRatio borderImageOutset borderImageSlice borderImageWidth boxFlex boxFlexGroup boxOrdinalGroup columnCount columns flex flexGrow flexPositive flexShrink flexNegative flexOrder gridArea gridRow gridRowEnd gridRowSpan gridRowStart gridColumn gridColumnEnd gridColumnSpan gridColumnStart fontWeight lineClamp lineHeight opacity order orphans scale tabSize widows zIndex zoom fillOpacity floodOpacity stopOpacity strokeDasharray strokeDashoffset strokeMiterlimit strokeOpacity strokeWidth MozAnimationIterationCount MozBoxFlex MozBoxFlexGroup MozLineClamp msAnimationIterationCount msFlex msZoom msFlexGrow msFlexNegative msFlexOrder msFlexPositive msFlexShrink msGridColumn msGridColumnSpan msGridRow msGridRowSpan WebkitAnimationIterationCount WebkitBoxFlex WebKitBoxFlexGroup WebkitBoxOrdinalGroup WebkitColumnCount WebkitColumns WebkitFlex WebkitFlexGrow WebkitFlexPositive WebkitFlexShrink WebkitLineClamp".split(" "));function po(t,e,a){var n=e.indexOf("--")===0;a==null||typeof a=="boolean"||a===""?n?t.setProperty(e,""):e==="float"?t.cssFloat="":t[e]="":n?t.setProperty(e,a):typeof a!="number"||a===0||Dh.has(e)?e==="float"?t.cssFloat=a:t[e]=(""+a).trim():t[e]=a+"px"}function go(t,e,a){if(e!=null&&typeof e!="object")throw Error(d(62));if(t=t.style,a!=null){for(var n in a)!a.hasOwnProperty(n)||e!=null&&e.hasOwnProperty(n)||(n.indexOf("--")===0?t.setProperty(n,""):n==="float"?t.cssFloat="":t[n]="");for(var l in e)n=e[l],e.hasOwnProperty(l)&&a[l]!==n&&po(t,l,n)}else for(var i in e)e.hasOwnProperty(i)&&po(t,i,e[i])}function Nr(t){if(t.indexOf("-")===-1)return!1;switch(t){case"annotation-xml":case"color-profile":case"font-face":case"font-face-src":case"font-face-uri":case"font-face-format":case"font-face-name":case"missing-glyph":return!1;default:return!0}}var Nh=new Map([["acceptCharset","accept-charset"],["htmlFor","for"],["httpEquiv","http-equiv"],["crossOrigin","crossorigin"],["accentHeight","accent-height"],["alignmentBaseline","alignment-baseline"],["arabicForm","arabic-form"],["baselineShift","baseline-shift"],["capHeight","cap-height"],["clipPath","clip-path"],["clipRule","clip-rule"],["colorInterpolation","color-interpolation"],["colorInterpolationFilters","color-interpolation-filters"],["colorProfile","color-profile"],["colorRendering","color-rendering"],["dominantBaseline","dominant-baseline"],["enableBackground","enable-background"],["fillOpacity","fill-opacity"],["fillRule","fill-rule"],["floodColor","flood-color"],["floodOpacity","flood-opacity"],["fontFamily","font-family"],["fontSize","font-size"],["fontSizeAdjust","font-size-adjust"],["fontStretch","font-stretch"],["fontStyle","font-style"],["fontVariant","font-variant"],["fontWeight","font-weight"],["glyphName","glyph-name"],["glyphOrientationHorizontal","glyph-orientation-horizontal"],["glyphOrientationVertical","glyph-orientation-vertical"],["horizAdvX","horiz-adv-x"],["horizOriginX","horiz-origin-x"],["imageRendering","image-rendering"],["letterSpacing","letter-spacing"],["lightingColor","lighting-color"],["markerEnd","marker-end"],["markerMid","marker-mid"],["markerStart","marker-start"],["overlinePosition","overline-position"],["overlineThickness","overline-thickness"],["paintOrder","paint-order"],["panose-1","panose-1"],["pointerEvents","pointer-events"],["renderingIntent","rendering-intent"],["shapeRendering","shape-rendering"],["stopColor","stop-color"],["stopOpacity","stop-opacity"],["strikethroughPosition","strikethrough-position"],["strikethroughThickness","strikethrough-thickness"],["strokeDasharray","stroke-dasharray"],["strokeDashoffset","stroke-dashoffset"],["strokeLinecap","stroke-linecap"],["strokeLinejoin","stroke-linejoin"],["strokeMiterlimit","stroke-miterlimit"],["strokeOpacity","stroke-opacity"],["strokeWidth","stroke-width"],["textAnchor","text-anchor"],["textDecoration","text-decoration"],["textRendering","text-rendering"],["transformOrigin","transform-origin"],["underlinePosition","underline-position"],["underlineThickness","underline-thickness"],["unicodeBidi","unicode-bidi"],["unicodeRange","unicode-range"],["unitsPerEm","units-per-em"],["vAlphabetic","v-alphabetic"],["vHanging","v-hanging"],["vIdeographic","v-ideographic"],["vMathematical","v-mathematical"],["vectorEffect","vector-effect"],["vertAdvY","vert-adv-y"],["vertOriginX","vert-origin-x"],["vertOriginY","vert-origin-y"],["wordSpacing","word-spacing"],["writingMode","writing-mode"],["xmlnsXlink","xmlns:xlink"],["xHeight","x-height"]]),Uh=/^[\u0000-\u001F ]*j[\r\n\t]*a[\r\n\t]*v[\r\n\t]*a[\r\n\t]*s[\r\n\t]*c[\r\n\t]*r[\r\n\t]*i[\r\n\t]*p[\r\n\t]*t[\r\n\t]*:/i;function oi(t){return Uh.test(""+t)?"javascript:throw new Error('React has blocked a javascript: URL as a security precaution.')":t}var Ur=null;function Mr(t){return t=t.target||t.srcElement||window,t.correspondingUseElement&&(t=t.correspondingUseElement),t.nodeType===3?t.parentNode:t}var En=null,wn=null;function yo(t){var e=yn(t);if(e&&(t=e.stateNode)){var a=t[xe]||null;t:switch(t=e.stateNode,e.type){case"input":if(Cr(t,a.value,a.defaultValue,a.defaultValue,a.checked,a.defaultChecked,a.type,a.name),e=a.name,a.type==="radio"&&e!=null){for(a=t;a.parentNode;)a=a.parentNode;for(a=a.querySelectorAll('input[name="'+je(""+e)+'"][type="radio"]'),e=0;e<a.length;e++){var n=a[e];if(n!==t&&n.form===t.form){var l=n[xe]||null;if(!l)throw Error(d(90));Cr(n,l.value,l.defaultValue,l.defaultValue,l.checked,l.defaultChecked,l.type,l.name)}}for(e=0;e<a.length;e++)n=a[e],n.form===t.form&&co(n)}break t;case"textarea":ho(t,a.value,a.defaultValue);break t;case"select":e=a.value,e!=null&&Sn(t,!!a.multiple,e,!1)}}}var jr=!1;function vo(t,e,a){if(jr)return t(e,a);jr=!0;try{var n=t(e);return n}finally{if(jr=!1,(En!==null||wn!==null)&&(Ji(),En&&(e=En,t=wn,wn=En=null,yo(e),t)))for(e=0;e<t.length;e++)yo(t[e])}}function ll(t,e){var a=t.stateNode;if(a===null)return null;var n=a[xe]||null;if(n===null)return null;a=n[e];t:switch(e){case"onClick":case"onClickCapture":case"onDoubleClick":case"onDoubleClickCapture":case"onMouseDown":case"onMouseDownCapture":case"onMouseMove":case"onMouseMoveCapture":case"onMouseUp":case"onMouseUpCapture":case"onMouseEnter":(n=!n.disabled)||(t=t.type,n=!(t==="button"||t==="input"||t==="select"||t==="textarea")),t=!n;break t;default:t=!1}if(t)return null;if(a&&typeof a!="function")throw Error(d(231,e,typeof a));return a}var la=!(typeof window>"u"||typeof window.document>"u"||typeof window.document.createElement>"u"),Br=!1;if(la)try{var il={};Object.defineProperty(il,"passive",{get:function(){Br=!0}}),window.addEventListener("test",il,il),window.removeEventListener("test",il,il)}catch{Br=!1}var Ea=null,Hr=null,ci=null;function bo(){if(ci)return ci;var t,e=Hr,a=e.length,n,l="value"in Ea?Ea.value:Ea.textContent,i=l.length;for(t=0;t<a&&e[t]===l[t];t++);var u=a-t;for(n=1;n<=u&&e[a-n]===l[i-n];n++);return ci=l.slice(t,1<n?1-n:void 0)}function fi(t){var e=t.keyCode;return"charCode"in t?(t=t.charCode,t===0&&e===13&&(t=13)):t=e,t===10&&(t=13),32<=t||t===13?t:0}function di(){return!0}function _o(){return!1}function Ee(t){function e(a,n,l,i,u){this._reactName=a,this._targetInst=l,this.type=n,this.nativeEvent=i,this.target=u,this.currentTarget=null;for(var x in t)t.hasOwnProperty(x)&&(a=t[x],this[x]=a?a(i):i[x]);return this.isDefaultPrevented=(i.defaultPrevented!=null?i.defaultPrevented:i.returnValue===!1)?di:_o,this.isPropagationStopped=_o,this}return E(e.prototype,{preventDefault:function(){this.defaultPrevented=!0;var a=this.nativeEvent;a&&(a.preventDefault?a.preventDefault():typeof a.returnValue!="unknown"&&(a.returnValue=!1),this.isDefaultPrevented=di)},stopPropagation:function(){var a=this.nativeEvent;a&&(a.stopPropagation?a.stopPropagation():typeof a.cancelBubble!="unknown"&&(a.cancelBubble=!0),this.isPropagationStopped=di)},persist:function(){},isPersistent:di}),e}var Fa={eventPhase:0,bubbles:0,cancelable:0,timeStamp:function(t){return t.timeStamp||Date.now()},defaultPrevented:0,isTrusted:0},hi=Ee(Fa),rl=E({},Fa,{view:0,detail:0}),Mh=Ee(rl),Lr,qr,ul,mi=E({},rl,{screenX:0,screenY:0,clientX:0,clientY:0,pageX:0,pageY:0,ctrlKey:0,shiftKey:0,altKey:0,metaKey:0,getModifierState:Zr,button:0,buttons:0,relatedTarget:function(t){return t.relatedTarget===void 0?t.fromElement===t.srcElement?t.toElement:t.fromElement:t.relatedTarget},movementX:function(t){return"movementX"in t?t.movementX:(t!==ul&&(ul&&t.type==="mousemove"?(Lr=t.screenX-ul.screenX,qr=t.screenY-ul.screenY):qr=Lr=0,ul=t),Lr)},movementY:function(t){return"movementY"in t?t.movementY:qr}}),So=Ee(mi),jh=E({},mi,{dataTransfer:0}),Bh=Ee(jh),Hh=E({},rl,{relatedTarget:0}),Yr=Ee(Hh),Lh=E({},Fa,{animationName:0,elapsedTime:0,pseudoElement:0}),qh=Ee(Lh),Yh=E({},Fa,{clipboardData:function(t){return"clipboardData"in t?t.clipboardData:window.clipboardData}}),Zh=Ee(Yh),Gh=E({},Fa,{data:0}),xo=Ee(Gh),Xh={Esc:"Escape",Spacebar:" ",Left:"ArrowLeft",Up:"ArrowUp",Right:"ArrowRight",Down:"ArrowDown",Del:"Delete",Win:"OS",Menu:"ContextMenu",Apps:"ContextMenu",Scroll:"ScrollLock",MozPrintableKey:"Unidentified"},Qh={8:"Backspace",9:"Tab",12:"Clear",13:"Enter",16:"Shift",17:"Control",18:"Alt",19:"Pause",20:"CapsLock",27:"Escape",32:" ",33:"PageUp",34:"PageDown",35:"End",36:"Home",37:"ArrowLeft",38:"ArrowUp",39:"ArrowRight",40:"ArrowDown",45:"Insert",46:"Delete",112:"F1",113:"F2",114:"F3",115:"F4",116:"F5",117:"F6",118:"F7",119:"F8",120:"F9",121:"F10",122:"F11",123:"F12",144:"NumLock",145:"ScrollLock",224:"Meta"},Vh={Alt:"altKey",Control:"ctrlKey",Meta:"metaKey",Shift:"shiftKey"};function Kh(t){var e=this.nativeEvent;return e.getModifierState?e.getModifierState(t):(t=Vh[t])?!!e[t]:!1}function Zr(){return Kh}var Jh=E({},rl,{key:function(t){if(t.key){var e=Xh[t.key]||t.key;if(e!=="Unidentified")return e}return t.type==="keypress"?(t=fi(t),t===13?"Enter":String.fromCharCode(t)):t.type==="keydown"||t.type==="keyup"?Qh[t.keyCode]||"Unidentified":""},code:0,location:0,ctrlKey:0,shiftKey:0,altKey:0,metaKey:0,repeat:0,locale:0,getModifierState:Zr,charCode:function(t){return t.type==="keypress"?fi(t):0},keyCode:function(t){return t.type==="keydown"||t.type==="keyup"?t.keyCode:0},which:function(t){return t.type==="keypress"?fi(t):t.type==="keydown"||t.type==="keyup"?t.keyCode:0}}),Fh=Ee(Jh),Wh=E({},mi,{pointerId:0,width:0,height:0,pressure:0,tangentialPressure:0,tiltX:0,tiltY:0,twist:0,pointerType:0,isPrimary:0}),Eo=Ee(Wh),$h=E({},rl,{touches:0,targetTouches:0,changedTouches:0,altKey:0,metaKey:0,ctrlKey:0,shiftKey:0,getModifierState:Zr}),Ih=Ee($h),Ph=E({},Fa,{propertyName:0,elapsedTime:0,pseudoElement:0}),tm=Ee(Ph),em=E({},mi,{deltaX:function(t){return"deltaX"in t?t.deltaX:"wheelDeltaX"in t?-t.wheelDeltaX:0},deltaY:function(t){return"deltaY"in t?t.deltaY:"wheelDeltaY"in t?-t.wheelDeltaY:"wheelDelta"in t?-t.wheelDelta:0},deltaZ:0,deltaMode:0}),am=Ee(em),nm=E({},Fa,{newState:0,oldState:0}),lm=Ee(nm),im=[9,13,27,32],Gr=la&&"CompositionEvent"in window,sl=null;la&&"documentMode"in document&&(sl=document.documentMode);var rm=la&&"TextEvent"in window&&!sl,wo=la&&(!Gr||sl&&8<sl&&11>=sl),Ao=" ",zo=!1;function To(t,e){switch(t){case"keyup":return im.indexOf(e.keyCode)!==-1;case"keydown":return e.keyCode!==229;case"keypress":case"mousedown":case"focusout":return!0;default:return!1}}function Ro(t){return t=t.detail,typeof t=="object"&&"data"in t?t.data:null}var An=!1;function um(t,e){switch(t){case"compositionend":return Ro(e);case"keypress":return e.which!==32?null:(zo=!0,Ao);case"textInput":return t=e.data,t===Ao&&zo?null:t;default:return null}}function sm(t,e){if(An)return t==="compositionend"||!Gr&&To(t,e)?(t=bo(),ci=Hr=Ea=null,An=!1,t):null;switch(t){case"paste":return null;case"keypress":if(!(e.ctrlKey||e.altKey||e.metaKey)||e.ctrlKey&&e.altKey){if(e.char&&1<e.char.length)return e.char;if(e.which)return String.fromCharCode(e.which)}return null;case"compositionend":return wo&&e.locale!=="ko"?null:e.data;default:return null}}var om={color:!0,date:!0,datetime:!0,"datetime-local":!0,email:!0,month:!0,number:!0,password:!0,range:!0,search:!0,tel:!0,text:!0,time:!0,url:!0,week:!0};function Oo(t){var e=t&&t.nodeName&&t.nodeName.toLowerCase();return e==="input"?!!om[t.type]:e==="textarea"}function ko(t,e,a,n){En?wn?wn.push(n):wn=[n]:En=n,e=tr(e,"onChange"),0<e.length&&(a=new hi("onChange","change",null,a,n),t.push({event:a,listeners:e}))}var ol=null,cl=null;function cm(t){fd(t,0)}function pi(t){var e=nl(t);if(co(e))return t}function Co(t,e){if(t==="change")return e}var Do=!1;if(la){var Xr;if(la){var Qr="oninput"in document;if(!Qr){var No=document.createElement("div");No.setAttribute("oninput","return;"),Qr=typeof No.oninput=="function"}Xr=Qr}else Xr=!1;Do=Xr&&(!document.documentMode||9<document.documentMode)}function Uo(){ol&&(ol.detachEvent("onpropertychange",Mo),cl=ol=null)}function Mo(t){if(t.propertyName==="value"&&pi(cl)){var e=[];ko(e,cl,t,Mr(t)),vo(cm,e)}}function fm(t,e,a){t==="focusin"?(Uo(),ol=e,cl=a,ol.attachEvent("onpropertychange",Mo)):t==="focusout"&&Uo()}function dm(t){if(t==="selectionchange"||t==="keyup"||t==="keydown")return pi(cl)}function hm(t,e){if(t==="click")return pi(e)}function mm(t,e){if(t==="input"||t==="change")return pi(e)}function pm(t,e){return t===e&&(t!==0||1/t===1/e)||t!==t&&e!==e}var Te=typeof Object.is=="function"?Object.is:pm;function fl(t,e){if(Te(t,e))return!0;if(typeof t!="object"||t===null||typeof e!="object"||e===null)return!1;var a=Object.keys(t),n=Object.keys(e);if(a.length!==n.length)return!1;for(n=0;n<a.length;n++){var l=a[n];if(!_e.call(e,l)||!Te(t[l],e[l]))return!1}return!0}function jo(t){for(;t&&t.firstChild;)t=t.firstChild;return t}function Bo(t,e){var a=jo(t);t=0;for(var n;a;){if(a.nodeType===3){if(n=t+a.textContent.length,t<=e&&n>=e)return{node:a,offset:e-t};t=n}t:{for(;a;){if(a.nextSibling){a=a.nextSibling;break t}a=a.parentNode}a=void 0}a=jo(a)}}function Ho(t,e){return t&&e?t===e?!0:t&&t.nodeType===3?!1:e&&e.nodeType===3?Ho(t,e.parentNode):"contains"in t?t.contains(e):t.compareDocumentPosition?!!(t.compareDocumentPosition(e)&16):!1:!1}function Lo(t){t=t!=null&&t.ownerDocument!=null&&t.ownerDocument.defaultView!=null?t.ownerDocument.defaultView:window;for(var e=si(t.document);e instanceof t.HTMLIFrameElement;){try{var a=typeof e.contentWindow.location.href=="string"}catch{a=!1}if(a)t=e.contentWindow;else break;e=si(t.document)}return e}function Vr(t){var e=t&&t.nodeName&&t.nodeName.toLowerCase();return e&&(e==="input"&&(t.type==="text"||t.type==="search"||t.type==="tel"||t.type==="url"||t.type==="password")||e==="textarea"||t.contentEditable==="true")}var gm=la&&"documentMode"in document&&11>=document.documentMode,zn=null,Kr=null,dl=null,Jr=!1;function qo(t,e,a){var n=a.window===a?a.document:a.nodeType===9?a:a.ownerDocument;Jr||zn==null||zn!==si(n)||(n=zn,"selectionStart"in n&&Vr(n)?n={start:n.selectionStart,end:n.selectionEnd}:(n=(n.ownerDocument&&n.ownerDocument.defaultView||window).getSelection(),n={anchorNode:n.anchorNode,anchorOffset:n.anchorOffset,focusNode:n.focusNode,focusOffset:n.focusOffset}),dl&&fl(dl,n)||(dl=n,n=tr(Kr,"onSelect"),0<n.length&&(e=new hi("onSelect","select",null,e,a),t.push({event:e,listeners:n}),e.target=zn)))}function Wa(t,e){var a={};return a[t.toLowerCase()]=e.toLowerCase(),a["Webkit"+t]="webkit"+e,a["Moz"+t]="moz"+e,a}var Tn={animationend:Wa("Animation","AnimationEnd"),animationiteration:Wa("Animation","AnimationIteration"),animationstart:Wa("Animation","AnimationStart"),transitionrun:Wa("Transition","TransitionRun"),transitionstart:Wa("Transition","TransitionStart"),transitioncancel:Wa("Transition","TransitionCancel"),transitionend:Wa("Transition","TransitionEnd")},Fr={},Yo={};la&&(Yo=document.createElement("div").style,"AnimationEvent"in window||(delete Tn.animationend.animation,delete Tn.animationiteration.animation,delete Tn.animationstart.animation),"TransitionEvent"in window||delete Tn.transitionend.transition);function $a(t){if(Fr[t])return Fr[t];if(!Tn[t])return t;var e=Tn[t],a;for(a in e)if(e.hasOwnProperty(a)&&a in Yo)return Fr[t]=e[a];return t}var Zo=$a("animationend"),Go=$a("animationiteration"),Xo=$a("animationstart"),ym=$a("transitionrun"),vm=$a("transitionstart"),bm=$a("transitioncancel"),Qo=$a("transitionend"),Vo=new Map,Wr="abort auxClick beforeToggle cancel canPlay canPlayThrough click close contextMenu copy cut drag dragEnd dragEnter dragExit dragLeave dragOver dragStart drop durationChange emptied encrypted ended error gotPointerCapture input invalid keyDown keyPress keyUp load loadedData loadedMetadata loadStart lostPointerCapture mouseDown mouseMove mouseOut mouseOver mouseUp paste pause play playing pointerCancel pointerDown pointerMove pointerOut pointerOver pointerUp progress rateChange reset resize seeked seeking stalled submit suspend timeUpdate touchCancel touchEnd touchStart volumeChange scroll toggle touchMove waiting wheel".split(" ");Wr.push("scrollEnd");function Ve(t,e){Vo.set(t,e),Ja(e,[t])}var Ko=new WeakMap;function Be(t,e){if(typeof t=="object"&&t!==null){var a=Ko.get(t);return a!==void 0?a:(e={value:t,source:e,stack:so(e)},Ko.set(t,e),e)}return{value:t,source:e,stack:so(e)}}var He=[],Rn=0,$r=0;function gi(){for(var t=Rn,e=$r=Rn=0;e<t;){var a=He[e];He[e++]=null;var n=He[e];He[e++]=null;var l=He[e];He[e++]=null;var i=He[e];if(He[e++]=null,n!==null&&l!==null){var u=n.pending;u===null?l.next=l:(l.next=u.next,u.next=l),n.pending=l}i!==0&&Jo(a,l,i)}}function yi(t,e,a,n){He[Rn++]=t,He[Rn++]=e,He[Rn++]=a,He[Rn++]=n,$r|=n,t.lanes|=n,t=t.alternate,t!==null&&(t.lanes|=n)}function Ir(t,e,a,n){return yi(t,e,a,n),vi(t)}function On(t,e){return yi(t,null,null,e),vi(t)}function Jo(t,e,a){t.lanes|=a;var n=t.alternate;n!==null&&(n.lanes|=a);for(var l=!1,i=t.return;i!==null;)i.childLanes|=a,n=i.alternate,n!==null&&(n.childLanes|=a),i.tag===22&&(t=i.stateNode,t===null||t._visibility&1||(l=!0)),t=i,i=i.return;return t.tag===3?(i=t.stateNode,l&&e!==null&&(l=31-Kt(a),t=i.hiddenUpdates,n=t[l],n===null?t[l]=[e]:n.push(e),e.lane=a|536870912),i):null}function vi(t){if(50<Bl)throw Bl=0,ls=null,Error(d(185));for(var e=t.return;e!==null;)t=e,e=t.return;return t.tag===3?t.stateNode:null}var kn={};function _m(t,e,a,n){this.tag=t,this.key=a,this.sibling=this.child=this.return=this.stateNode=this.type=this.elementType=null,this.index=0,this.refCleanup=this.ref=null,this.pendingProps=e,this.dependencies=this.memoizedState=this.updateQueue=this.memoizedProps=null,this.mode=n,this.subtreeFlags=this.flags=0,this.deletions=null,this.childLanes=this.lanes=0,this.alternate=null}function Re(t,e,a,n){return new _m(t,e,a,n)}function Pr(t){return t=t.prototype,!(!t||!t.isReactComponent)}function ia(t,e){var a=t.alternate;return a===null?(a=Re(t.tag,e,t.key,t.mode),a.elementType=t.elementType,a.type=t.type,a.stateNode=t.stateNode,a.alternate=t,t.alternate=a):(a.pendingProps=e,a.type=t.type,a.flags=0,a.subtreeFlags=0,a.deletions=null),a.flags=t.flags&65011712,a.childLanes=t.childLanes,a.lanes=t.lanes,a.child=t.child,a.memoizedProps=t.memoizedProps,a.memoizedState=t.memoizedState,a.updateQueue=t.updateQueue,e=t.dependencies,a.dependencies=e===null?null:{lanes:e.lanes,firstContext:e.firstContext},a.sibling=t.sibling,a.index=t.index,a.ref=t.ref,a.refCleanup=t.refCleanup,a}function Fo(t,e){t.flags&=65011714;var a=t.alternate;return a===null?(t.childLanes=0,t.lanes=e,t.child=null,t.subtreeFlags=0,t.memoizedProps=null,t.memoizedState=null,t.updateQueue=null,t.dependencies=null,t.stateNode=null):(t.childLanes=a.childLanes,t.lanes=a.lanes,t.child=a.child,t.subtreeFlags=0,t.deletions=null,t.memoizedProps=a.memoizedProps,t.memoizedState=a.memoizedState,t.updateQueue=a.updateQueue,t.type=a.type,e=a.dependencies,t.dependencies=e===null?null:{lanes:e.lanes,firstContext:e.firstContext}),t}function bi(t,e,a,n,l,i){var u=0;if(n=t,typeof t=="function")Pr(t)&&(u=1);else if(typeof t=="string")u=x0(t,a,it.current)?26:t==="html"||t==="head"||t==="body"?27:5;else t:switch(t){case $:return t=Re(31,a,e,l),t.elementType=$,t.lanes=i,t;case z:return Ia(a.children,l,i,e);case g:u=8,l|=24;break;case S:return t=Re(12,a,e,l|2),t.elementType=S,t.lanes=i,t;case j:return t=Re(13,a,e,l),t.elementType=j,t.lanes=i,t;case tt:return t=Re(19,a,e,l),t.elementType=tt,t.lanes=i,t;default:if(typeof t=="object"&&t!==null)switch(t.$$typeof){case v:case D:u=10;break t;case T:u=9;break t;case L:u=11;break t;case F:u=14;break t;case ft:u=16,n=null;break t}u=29,a=Error(d(130,t===null?"null":typeof t,"")),n=null}return e=Re(u,a,e,l),e.elementType=t,e.type=n,e.lanes=i,e}function Ia(t,e,a,n){return t=Re(7,t,n,e),t.lanes=a,t}function tu(t,e,a){return t=Re(6,t,null,e),t.lanes=a,t}function eu(t,e,a){return e=Re(4,t.children!==null?t.children:[],t.key,e),e.lanes=a,e.stateNode={containerInfo:t.containerInfo,pendingChildren:null,implementation:t.implementation},e}var Cn=[],Dn=0,_i=null,Si=0,Le=[],qe=0,Pa=null,ra=1,ua="";function tn(t,e){Cn[Dn++]=Si,Cn[Dn++]=_i,_i=t,Si=e}function Wo(t,e,a){Le[qe++]=ra,Le[qe++]=ua,Le[qe++]=Pa,Pa=t;var n=ra;t=ua;var l=32-Kt(n)-1;n&=~(1<<l),a+=1;var i=32-Kt(e)+l;if(30<i){var u=l-l%5;i=(n&(1<<u)-1).toString(32),n>>=u,l-=u,ra=1<<32-Kt(e)+l|a<<l|n,ua=i+t}else ra=1<<i|a<<l|n,ua=t}function au(t){t.return!==null&&(tn(t,1),Wo(t,1,0))}function nu(t){for(;t===_i;)_i=Cn[--Dn],Cn[Dn]=null,Si=Cn[--Dn],Cn[Dn]=null;for(;t===Pa;)Pa=Le[--qe],Le[qe]=null,ua=Le[--qe],Le[qe]=null,ra=Le[--qe],Le[qe]=null}var be=null,te=null,Ht=!1,en=null,Fe=!1,lu=Error(d(519));function an(t){var e=Error(d(418,""));throw pl(Be(e,t)),lu}function $o(t){var e=t.stateNode,a=t.type,n=t.memoizedProps;switch(e[pe]=t,e[xe]=n,a){case"dialog":Mt("cancel",e),Mt("close",e);break;case"iframe":case"object":case"embed":Mt("load",e);break;case"video":case"audio":for(a=0;a<Ll.length;a++)Mt(Ll[a],e);break;case"source":Mt("error",e);break;case"img":case"image":case"link":Mt("error",e),Mt("load",e);break;case"details":Mt("toggle",e);break;case"input":Mt("invalid",e),fo(e,n.value,n.defaultValue,n.checked,n.defaultChecked,n.type,n.name,!0),ui(e);break;case"select":Mt("invalid",e);break;case"textarea":Mt("invalid",e),mo(e,n.value,n.defaultValue,n.children),ui(e)}a=n.children,typeof a!="string"&&typeof a!="number"&&typeof a!="bigint"||e.textContent===""+a||n.suppressHydrationWarning===!0||pd(e.textContent,a)?(n.popover!=null&&(Mt("beforetoggle",e),Mt("toggle",e)),n.onScroll!=null&&Mt("scroll",e),n.onScrollEnd!=null&&Mt("scrollend",e),n.onClick!=null&&(e.onclick=er),e=!0):e=!1,e||an(t)}function Io(t){for(be=t.return;be;)switch(be.tag){case 5:case 13:Fe=!1;return;case 27:case 3:Fe=!0;return;default:be=be.return}}function hl(t){if(t!==be)return!1;if(!Ht)return Io(t),Ht=!0,!1;var e=t.tag,a;if((a=e!==3&&e!==27)&&((a=e===5)&&(a=t.type,a=!(a!=="form"&&a!=="button")||_s(t.type,t.memoizedProps)),a=!a),a&&te&&an(t),Io(t),e===13){if(t=t.memoizedState,t=t!==null?t.dehydrated:null,!t)throw Error(d(317));t:{for(t=t.nextSibling,e=0;t;){if(t.nodeType===8)if(a=t.data,a==="/$"){if(e===0){te=Je(t.nextSibling);break t}e--}else a!=="$"&&a!=="$!"&&a!=="$?"||e++;t=t.nextSibling}te=null}}else e===27?(e=te,La(t.type)?(t=ws,ws=null,te=t):te=e):te=be?Je(t.stateNode.nextSibling):null;return!0}function ml(){te=be=null,Ht=!1}function Po(){var t=en;return t!==null&&(ze===null?ze=t:ze.push.apply(ze,t),en=null),t}function pl(t){en===null?en=[t]:en.push(t)}var iu=N(null),nn=null,sa=null;function wa(t,e,a){nt(iu,e._currentValue),e._currentValue=a}function oa(t){t._currentValue=iu.current,B(iu)}function ru(t,e,a){for(;t!==null;){var n=t.alternate;if((t.childLanes&e)!==e?(t.childLanes|=e,n!==null&&(n.childLanes|=e)):n!==null&&(n.childLanes&e)!==e&&(n.childLanes|=e),t===a)break;t=t.return}}function uu(t,e,a,n){var l=t.child;for(l!==null&&(l.return=t);l!==null;){var i=l.dependencies;if(i!==null){var u=l.child;i=i.firstContext;t:for(;i!==null;){var x=i;i=l;for(var U=0;U<e.length;U++)if(x.context===e[U]){i.lanes|=a,x=i.alternate,x!==null&&(x.lanes|=a),ru(i.return,a,t),n||(u=null);break t}i=x.next}}else if(l.tag===18){if(u=l.return,u===null)throw Error(d(341));u.lanes|=a,i=u.alternate,i!==null&&(i.lanes|=a),ru(u,a,t),u=null}else u=l.child;if(u!==null)u.return=l;else for(u=l;u!==null;){if(u===t){u=null;break}if(l=u.sibling,l!==null){l.return=u.return,u=l;break}u=u.return}l=u}}function gl(t,e,a,n){t=null;for(var l=e,i=!1;l!==null;){if(!i){if((l.flags&524288)!==0)i=!0;else if((l.flags&262144)!==0)break}if(l.tag===10){var u=l.alternate;if(u===null)throw Error(d(387));if(u=u.memoizedProps,u!==null){var x=l.type;Te(l.pendingProps.value,u.value)||(t!==null?t.push(x):t=[x])}}else if(l===Lt.current){if(u=l.alternate,u===null)throw Error(d(387));u.memoizedState.memoizedState!==l.memoizedState.memoizedState&&(t!==null?t.push(Ql):t=[Ql])}l=l.return}t!==null&&uu(e,t,a,n),e.flags|=262144}function xi(t){for(t=t.firstContext;t!==null;){if(!Te(t.context._currentValue,t.memoizedValue))return!0;t=t.next}return!1}function ln(t){nn=t,sa=null,t=t.dependencies,t!==null&&(t.firstContext=null)}function ge(t){return tc(nn,t)}function Ei(t,e){return nn===null&&ln(t),tc(t,e)}function tc(t,e){var a=e._currentValue;if(e={context:e,memoizedValue:a,next:null},sa===null){if(t===null)throw Error(d(308));sa=e,t.dependencies={lanes:0,firstContext:e},t.flags|=524288}else sa=sa.next=e;return a}var Sm=typeof AbortController<"u"?AbortController:function(){var t=[],e=this.signal={aborted:!1,addEventListener:function(a,n){t.push(n)}};this.abort=function(){e.aborted=!0,t.forEach(function(a){return a()})}},xm=s.unstable_scheduleCallback,Em=s.unstable_NormalPriority,re={$$typeof:D,Consumer:null,Provider:null,_currentValue:null,_currentValue2:null,_threadCount:0};function su(){return{controller:new Sm,data:new Map,refCount:0}}function yl(t){t.refCount--,t.refCount===0&&xm(Em,function(){t.controller.abort()})}var vl=null,ou=0,Nn=0,Un=null;function wm(t,e){if(vl===null){var a=vl=[];ou=0,Nn=fs(),Un={status:"pending",value:void 0,then:function(n){a.push(n)}}}return ou++,e.then(ec,ec),e}function ec(){if(--ou===0&&vl!==null){Un!==null&&(Un.status="fulfilled");var t=vl;vl=null,Nn=0,Un=null;for(var e=0;e<t.length;e++)(0,t[e])()}}function Am(t,e){var a=[],n={status:"pending",value:null,reason:null,then:function(l){a.push(l)}};return t.then(function(){n.status="fulfilled",n.value=e;for(var l=0;l<a.length;l++)(0,a[l])(e)},function(l){for(n.status="rejected",n.reason=l,l=0;l<a.length;l++)(0,a[l])(void 0)}),n}var ac=G.S;G.S=function(t,e){typeof e=="object"&&e!==null&&typeof e.then=="function"&&wm(t,e),ac!==null&&ac(t,e)};var rn=N(null);function cu(){var t=rn.current;return t!==null?t:Ft.pooledCache}function wi(t,e){e===null?nt(rn,rn.current):nt(rn,e.pool)}function nc(){var t=cu();return t===null?null:{parent:re._currentValue,pool:t}}var bl=Error(d(460)),lc=Error(d(474)),Ai=Error(d(542)),fu={then:function(){}};function ic(t){return t=t.status,t==="fulfilled"||t==="rejected"}function zi(){}function rc(t,e,a){switch(a=t[a],a===void 0?t.push(e):a!==e&&(e.then(zi,zi),e=a),e.status){case"fulfilled":return e.value;case"rejected":throw t=e.reason,sc(t),t;default:if(typeof e.status=="string")e.then(zi,zi);else{if(t=Ft,t!==null&&100<t.shellSuspendCounter)throw Error(d(482));t=e,t.status="pending",t.then(function(n){if(e.status==="pending"){var l=e;l.status="fulfilled",l.value=n}},function(n){if(e.status==="pending"){var l=e;l.status="rejected",l.reason=n}})}switch(e.status){case"fulfilled":return e.value;case"rejected":throw t=e.reason,sc(t),t}throw _l=e,bl}}var _l=null;function uc(){if(_l===null)throw Error(d(459));var t=_l;return _l=null,t}function sc(t){if(t===bl||t===Ai)throw Error(d(483))}var Aa=!1;function du(t){t.updateQueue={baseState:t.memoizedState,firstBaseUpdate:null,lastBaseUpdate:null,shared:{pending:null,lanes:0,hiddenCallbacks:null},callbacks:null}}function hu(t,e){t=t.updateQueue,e.updateQueue===t&&(e.updateQueue={baseState:t.baseState,firstBaseUpdate:t.firstBaseUpdate,lastBaseUpdate:t.lastBaseUpdate,shared:t.shared,callbacks:null})}function za(t){return{lane:t,tag:0,payload:null,callback:null,next:null}}function Ta(t,e,a){var n=t.updateQueue;if(n===null)return null;if(n=n.shared,(qt&2)!==0){var l=n.pending;return l===null?e.next=e:(e.next=l.next,l.next=e),n.pending=e,e=vi(t),Jo(t,null,a),e}return yi(t,n,e,a),vi(t)}function Sl(t,e,a){if(e=e.updateQueue,e!==null&&(e=e.shared,(a&4194048)!==0)){var n=e.lanes;n&=t.pendingLanes,a|=n,e.lanes=a,to(t,a)}}function mu(t,e){var a=t.updateQueue,n=t.alternate;if(n!==null&&(n=n.updateQueue,a===n)){var l=null,i=null;if(a=a.firstBaseUpdate,a!==null){do{var u={lane:a.lane,tag:a.tag,payload:a.payload,callback:null,next:null};i===null?l=i=u:i=i.next=u,a=a.next}while(a!==null);i===null?l=i=e:i=i.next=e}else l=i=e;a={baseState:n.baseState,firstBaseUpdate:l,lastBaseUpdate:i,shared:n.shared,callbacks:n.callbacks},t.updateQueue=a;return}t=a.lastBaseUpdate,t===null?a.firstBaseUpdate=e:t.next=e,a.lastBaseUpdate=e}var pu=!1;function xl(){if(pu){var t=Un;if(t!==null)throw t}}function El(t,e,a,n){pu=!1;var l=t.updateQueue;Aa=!1;var i=l.firstBaseUpdate,u=l.lastBaseUpdate,x=l.shared.pending;if(x!==null){l.shared.pending=null;var U=x,V=U.next;U.next=null,u===null?i=V:u.next=V,u=U;var lt=t.alternate;lt!==null&&(lt=lt.updateQueue,x=lt.lastBaseUpdate,x!==u&&(x===null?lt.firstBaseUpdate=V:x.next=V,lt.lastBaseUpdate=U))}if(i!==null){var ct=l.baseState;u=0,lt=V=U=null,x=i;do{var J=x.lane&-536870913,W=J!==x.lane;if(W?(jt&J)===J:(n&J)===J){J!==0&&J===Nn&&(pu=!0),lt!==null&&(lt=lt.next={lane:0,tag:x.tag,payload:x.payload,callback:null,next:null});t:{var Tt=t,At=x;J=e;var Xt=a;switch(At.tag){case 1:if(Tt=At.payload,typeof Tt=="function"){ct=Tt.call(Xt,ct,J);break t}ct=Tt;break t;case 3:Tt.flags=Tt.flags&-65537|128;case 0:if(Tt=At.payload,J=typeof Tt=="function"?Tt.call(Xt,ct,J):Tt,J==null)break t;ct=E({},ct,J);break t;case 2:Aa=!0}}J=x.callback,J!==null&&(t.flags|=64,W&&(t.flags|=8192),W=l.callbacks,W===null?l.callbacks=[J]:W.push(J))}else W={lane:J,tag:x.tag,payload:x.payload,callback:x.callback,next:null},lt===null?(V=lt=W,U=ct):lt=lt.next=W,u|=J;if(x=x.next,x===null){if(x=l.shared.pending,x===null)break;W=x,x=W.next,W.next=null,l.lastBaseUpdate=W,l.shared.pending=null}}while(!0);lt===null&&(U=ct),l.baseState=U,l.firstBaseUpdate=V,l.lastBaseUpdate=lt,i===null&&(l.shared.lanes=0),Ma|=u,t.lanes=u,t.memoizedState=ct}}function oc(t,e){if(typeof t!="function")throw Error(d(191,t));t.call(e)}function cc(t,e){var a=t.callbacks;if(a!==null)for(t.callbacks=null,t=0;t<a.length;t++)oc(a[t],e)}var Mn=N(null),Ti=N(0);function fc(t,e){t=ga,nt(Ti,t),nt(Mn,e),ga=t|e.baseLanes}function gu(){nt(Ti,ga),nt(Mn,Mn.current)}function yu(){ga=Ti.current,B(Mn),B(Ti)}var Ra=0,Dt=null,Zt=null,le=null,Ri=!1,jn=!1,un=!1,Oi=0,wl=0,Bn=null,zm=0;function ae(){throw Error(d(321))}function vu(t,e){if(e===null)return!1;for(var a=0;a<e.length&&a<t.length;a++)if(!Te(t[a],e[a]))return!1;return!0}function bu(t,e,a,n,l,i){return Ra=i,Dt=e,e.memoizedState=null,e.updateQueue=null,e.lanes=0,G.H=t===null||t.memoizedState===null?Jc:Fc,un=!1,i=a(n,l),un=!1,jn&&(i=hc(e,a,n,l)),dc(t),i}function dc(t){G.H=Mi;var e=Zt!==null&&Zt.next!==null;if(Ra=0,le=Zt=Dt=null,Ri=!1,wl=0,Bn=null,e)throw Error(d(300));t===null||oe||(t=t.dependencies,t!==null&&xi(t)&&(oe=!0))}function hc(t,e,a,n){Dt=t;var l=0;do{if(jn&&(Bn=null),wl=0,jn=!1,25<=l)throw Error(d(301));if(l+=1,le=Zt=null,t.updateQueue!=null){var i=t.updateQueue;i.lastEffect=null,i.events=null,i.stores=null,i.memoCache!=null&&(i.memoCache.index=0)}G.H=Nm,i=e(a,n)}while(jn);return i}function Tm(){var t=G.H,e=t.useState()[0];return e=typeof e.then=="function"?Al(e):e,t=t.useState()[0],(Zt!==null?Zt.memoizedState:null)!==t&&(Dt.flags|=1024),e}function _u(){var t=Oi!==0;return Oi=0,t}function Su(t,e,a){e.updateQueue=t.updateQueue,e.flags&=-2053,t.lanes&=~a}function xu(t){if(Ri){for(t=t.memoizedState;t!==null;){var e=t.queue;e!==null&&(e.pending=null),t=t.next}Ri=!1}Ra=0,le=Zt=Dt=null,jn=!1,wl=Oi=0,Bn=null}function we(){var t={memoizedState:null,baseState:null,baseQueue:null,queue:null,next:null};return le===null?Dt.memoizedState=le=t:le=le.next=t,le}function ie(){if(Zt===null){var t=Dt.alternate;t=t!==null?t.memoizedState:null}else t=Zt.next;var e=le===null?Dt.memoizedState:le.next;if(e!==null)le=e,Zt=t;else{if(t===null)throw Dt.alternate===null?Error(d(467)):Error(d(310));Zt=t,t={memoizedState:Zt.memoizedState,baseState:Zt.baseState,baseQueue:Zt.baseQueue,queue:Zt.queue,next:null},le===null?Dt.memoizedState=le=t:le=le.next=t}return le}function Eu(){return{lastEffect:null,events:null,stores:null,memoCache:null}}function Al(t){var e=wl;return wl+=1,Bn===null&&(Bn=[]),t=rc(Bn,t,e),e=Dt,(le===null?e.memoizedState:le.next)===null&&(e=e.alternate,G.H=e===null||e.memoizedState===null?Jc:Fc),t}function ki(t){if(t!==null&&typeof t=="object"){if(typeof t.then=="function")return Al(t);if(t.$$typeof===D)return ge(t)}throw Error(d(438,String(t)))}function wu(t){var e=null,a=Dt.updateQueue;if(a!==null&&(e=a.memoCache),e==null){var n=Dt.alternate;n!==null&&(n=n.updateQueue,n!==null&&(n=n.memoCache,n!=null&&(e={data:n.data.map(function(l){return l.slice()}),index:0})))}if(e==null&&(e={data:[],index:0}),a===null&&(a=Eu(),Dt.updateQueue=a),a.memoCache=e,a=e.data[e.index],a===void 0)for(a=e.data[e.index]=Array(t),n=0;n<t;n++)a[n]=rt;return e.index++,a}function ca(t,e){return typeof e=="function"?e(t):e}function Ci(t){var e=ie();return Au(e,Zt,t)}function Au(t,e,a){var n=t.queue;if(n===null)throw Error(d(311));n.lastRenderedReducer=a;var l=t.baseQueue,i=n.pending;if(i!==null){if(l!==null){var u=l.next;l.next=i.next,i.next=u}e.baseQueue=l=i,n.pending=null}if(i=t.baseState,l===null)t.memoizedState=i;else{e=l.next;var x=u=null,U=null,V=e,lt=!1;do{var ct=V.lane&-536870913;if(ct!==V.lane?(jt&ct)===ct:(Ra&ct)===ct){var J=V.revertLane;if(J===0)U!==null&&(U=U.next={lane:0,revertLane:0,action:V.action,hasEagerState:V.hasEagerState,eagerState:V.eagerState,next:null}),ct===Nn&&(lt=!0);else if((Ra&J)===J){V=V.next,J===Nn&&(lt=!0);continue}else ct={lane:0,revertLane:V.revertLane,action:V.action,hasEagerState:V.hasEagerState,eagerState:V.eagerState,next:null},U===null?(x=U=ct,u=i):U=U.next=ct,Dt.lanes|=J,Ma|=J;ct=V.action,un&&a(i,ct),i=V.hasEagerState?V.eagerState:a(i,ct)}else J={lane:ct,revertLane:V.revertLane,action:V.action,hasEagerState:V.hasEagerState,eagerState:V.eagerState,next:null},U===null?(x=U=J,u=i):U=U.next=J,Dt.lanes|=ct,Ma|=ct;V=V.next}while(V!==null&&V!==e);if(U===null?u=i:U.next=x,!Te(i,t.memoizedState)&&(oe=!0,lt&&(a=Un,a!==null)))throw a;t.memoizedState=i,t.baseState=u,t.baseQueue=U,n.lastRenderedState=i}return l===null&&(n.lanes=0),[t.memoizedState,n.dispatch]}function zu(t){var e=ie(),a=e.queue;if(a===null)throw Error(d(311));a.lastRenderedReducer=t;var n=a.dispatch,l=a.pending,i=e.memoizedState;if(l!==null){a.pending=null;var u=l=l.next;do i=t(i,u.action),u=u.next;while(u!==l);Te(i,e.memoizedState)||(oe=!0),e.memoizedState=i,e.baseQueue===null&&(e.baseState=i),a.lastRenderedState=i}return[i,n]}function mc(t,e,a){var n=Dt,l=ie(),i=Ht;if(i){if(a===void 0)throw Error(d(407));a=a()}else a=e();var u=!Te((Zt||l).memoizedState,a);u&&(l.memoizedState=a,oe=!0),l=l.queue;var x=yc.bind(null,n,l,t);if(zl(2048,8,x,[t]),l.getSnapshot!==e||u||le!==null&&le.memoizedState.tag&1){if(n.flags|=2048,Hn(9,Di(),gc.bind(null,n,l,a,e),null),Ft===null)throw Error(d(349));i||(Ra&124)!==0||pc(n,e,a)}return a}function pc(t,e,a){t.flags|=16384,t={getSnapshot:e,value:a},e=Dt.updateQueue,e===null?(e=Eu(),Dt.updateQueue=e,e.stores=[t]):(a=e.stores,a===null?e.stores=[t]:a.push(t))}function gc(t,e,a,n){e.value=a,e.getSnapshot=n,vc(e)&&bc(t)}function yc(t,e,a){return a(function(){vc(e)&&bc(t)})}function vc(t){var e=t.getSnapshot;t=t.value;try{var a=e();return!Te(t,a)}catch{return!0}}function bc(t){var e=On(t,2);e!==null&&Ne(e,t,2)}function Tu(t){var e=we();if(typeof t=="function"){var a=t;if(t=a(),un){Rt(!0);try{a()}finally{Rt(!1)}}}return e.memoizedState=e.baseState=t,e.queue={pending:null,lanes:0,dispatch:null,lastRenderedReducer:ca,lastRenderedState:t},e}function _c(t,e,a,n){return t.baseState=a,Au(t,Zt,typeof n=="function"?n:ca)}function Rm(t,e,a,n,l){if(Ui(t))throw Error(d(485));if(t=e.action,t!==null){var i={payload:l,action:t,next:null,isTransition:!0,status:"pending",value:null,reason:null,listeners:[],then:function(u){i.listeners.push(u)}};G.T!==null?a(!0):i.isTransition=!1,n(i),a=e.pending,a===null?(i.next=e.pending=i,Sc(e,i)):(i.next=a.next,e.pending=a.next=i)}}function Sc(t,e){var a=e.action,n=e.payload,l=t.state;if(e.isTransition){var i=G.T,u={};G.T=u;try{var x=a(l,n),U=G.S;U!==null&&U(u,x),xc(t,e,x)}catch(V){Ru(t,e,V)}finally{G.T=i}}else try{i=a(l,n),xc(t,e,i)}catch(V){Ru(t,e,V)}}function xc(t,e,a){a!==null&&typeof a=="object"&&typeof a.then=="function"?a.then(function(n){Ec(t,e,n)},function(n){return Ru(t,e,n)}):Ec(t,e,a)}function Ec(t,e,a){e.status="fulfilled",e.value=a,wc(e),t.state=a,e=t.pending,e!==null&&(a=e.next,a===e?t.pending=null:(a=a.next,e.next=a,Sc(t,a)))}function Ru(t,e,a){var n=t.pending;if(t.pending=null,n!==null){n=n.next;do e.status="rejected",e.reason=a,wc(e),e=e.next;while(e!==n)}t.action=null}function wc(t){t=t.listeners;for(var e=0;e<t.length;e++)(0,t[e])()}function Ac(t,e){return e}function zc(t,e){if(Ht){var a=Ft.formState;if(a!==null){t:{var n=Dt;if(Ht){if(te){e:{for(var l=te,i=Fe;l.nodeType!==8;){if(!i){l=null;break e}if(l=Je(l.nextSibling),l===null){l=null;break e}}i=l.data,l=i==="F!"||i==="F"?l:null}if(l){te=Je(l.nextSibling),n=l.data==="F!";break t}}an(n)}n=!1}n&&(e=a[0])}}return a=we(),a.memoizedState=a.baseState=e,n={pending:null,lanes:0,dispatch:null,lastRenderedReducer:Ac,lastRenderedState:e},a.queue=n,a=Qc.bind(null,Dt,n),n.dispatch=a,n=Tu(!1),i=Nu.bind(null,Dt,!1,n.queue),n=we(),l={state:e,dispatch:null,action:t,pending:null},n.queue=l,a=Rm.bind(null,Dt,l,i,a),l.dispatch=a,n.memoizedState=t,[e,a,!1]}function Tc(t){var e=ie();return Rc(e,Zt,t)}function Rc(t,e,a){if(e=Au(t,e,Ac)[0],t=Ci(ca)[0],typeof e=="object"&&e!==null&&typeof e.then=="function")try{var n=Al(e)}catch(u){throw u===bl?Ai:u}else n=e;e=ie();var l=e.queue,i=l.dispatch;return a!==e.memoizedState&&(Dt.flags|=2048,Hn(9,Di(),Om.bind(null,l,a),null)),[n,i,t]}function Om(t,e){t.action=e}function Oc(t){var e=ie(),a=Zt;if(a!==null)return Rc(e,a,t);ie(),e=e.memoizedState,a=ie();var n=a.queue.dispatch;return a.memoizedState=t,[e,n,!1]}function Hn(t,e,a,n){return t={tag:t,create:a,deps:n,inst:e,next:null},e=Dt.updateQueue,e===null&&(e=Eu(),Dt.updateQueue=e),a=e.lastEffect,a===null?e.lastEffect=t.next=t:(n=a.next,a.next=t,t.next=n,e.lastEffect=t),t}function Di(){return{destroy:void 0,resource:void 0}}function kc(){return ie().memoizedState}function Ni(t,e,a,n){var l=we();n=n===void 0?null:n,Dt.flags|=t,l.memoizedState=Hn(1|e,Di(),a,n)}function zl(t,e,a,n){var l=ie();n=n===void 0?null:n;var i=l.memoizedState.inst;Zt!==null&&n!==null&&vu(n,Zt.memoizedState.deps)?l.memoizedState=Hn(e,i,a,n):(Dt.flags|=t,l.memoizedState=Hn(1|e,i,a,n))}function Cc(t,e){Ni(8390656,8,t,e)}function Dc(t,e){zl(2048,8,t,e)}function Nc(t,e){return zl(4,2,t,e)}function Uc(t,e){return zl(4,4,t,e)}function Mc(t,e){if(typeof e=="function"){t=t();var a=e(t);return function(){typeof a=="function"?a():e(null)}}if(e!=null)return t=t(),e.current=t,function(){e.current=null}}function jc(t,e,a){a=a!=null?a.concat([t]):null,zl(4,4,Mc.bind(null,e,t),a)}function Ou(){}function Bc(t,e){var a=ie();e=e===void 0?null:e;var n=a.memoizedState;return e!==null&&vu(e,n[1])?n[0]:(a.memoizedState=[t,e],t)}function Hc(t,e){var a=ie();e=e===void 0?null:e;var n=a.memoizedState;if(e!==null&&vu(e,n[1]))return n[0];if(n=t(),un){Rt(!0);try{t()}finally{Rt(!1)}}return a.memoizedState=[n,e],n}function ku(t,e,a){return a===void 0||(Ra&1073741824)!==0?t.memoizedState=e:(t.memoizedState=a,t=Zf(),Dt.lanes|=t,Ma|=t,a)}function Lc(t,e,a,n){return Te(a,e)?a:Mn.current!==null?(t=ku(t,a,n),Te(t,e)||(oe=!0),t):(Ra&42)===0?(oe=!0,t.memoizedState=a):(t=Zf(),Dt.lanes|=t,Ma|=t,e)}function qc(t,e,a,n,l){var i=X.p;X.p=i!==0&&8>i?i:8;var u=G.T,x={};G.T=x,Nu(t,!1,e,a);try{var U=l(),V=G.S;if(V!==null&&V(x,U),U!==null&&typeof U=="object"&&typeof U.then=="function"){var lt=Am(U,n);Tl(t,e,lt,De(t))}else Tl(t,e,n,De(t))}catch(ct){Tl(t,e,{then:function(){},status:"rejected",reason:ct},De())}finally{X.p=i,G.T=u}}function km(){}function Cu(t,e,a,n){if(t.tag!==5)throw Error(d(476));var l=Yc(t).queue;qc(t,l,e,gt,a===null?km:function(){return Zc(t),a(n)})}function Yc(t){var e=t.memoizedState;if(e!==null)return e;e={memoizedState:gt,baseState:gt,baseQueue:null,queue:{pending:null,lanes:0,dispatch:null,lastRenderedReducer:ca,lastRenderedState:gt},next:null};var a={};return e.next={memoizedState:a,baseState:a,baseQueue:null,queue:{pending:null,lanes:0,dispatch:null,lastRenderedReducer:ca,lastRenderedState:a},next:null},t.memoizedState=e,t=t.alternate,t!==null&&(t.memoizedState=e),e}function Zc(t){var e=Yc(t).next.queue;Tl(t,e,{},De())}function Du(){return ge(Ql)}function Gc(){return ie().memoizedState}function Xc(){return ie().memoizedState}function Cm(t){for(var e=t.return;e!==null;){switch(e.tag){case 24:case 3:var a=De();t=za(a);var n=Ta(e,t,a);n!==null&&(Ne(n,e,a),Sl(n,e,a)),e={cache:su()},t.payload=e;return}e=e.return}}function Dm(t,e,a){var n=De();a={lane:n,revertLane:0,action:a,hasEagerState:!1,eagerState:null,next:null},Ui(t)?Vc(e,a):(a=Ir(t,e,a,n),a!==null&&(Ne(a,t,n),Kc(a,e,n)))}function Qc(t,e,a){var n=De();Tl(t,e,a,n)}function Tl(t,e,a,n){var l={lane:n,revertLane:0,action:a,hasEagerState:!1,eagerState:null,next:null};if(Ui(t))Vc(e,l);else{var i=t.alternate;if(t.lanes===0&&(i===null||i.lanes===0)&&(i=e.lastRenderedReducer,i!==null))try{var u=e.lastRenderedState,x=i(u,a);if(l.hasEagerState=!0,l.eagerState=x,Te(x,u))return yi(t,e,l,0),Ft===null&&gi(),!1}catch{}finally{}if(a=Ir(t,e,l,n),a!==null)return Ne(a,t,n),Kc(a,e,n),!0}return!1}function Nu(t,e,a,n){if(n={lane:2,revertLane:fs(),action:n,hasEagerState:!1,eagerState:null,next:null},Ui(t)){if(e)throw Error(d(479))}else e=Ir(t,a,n,2),e!==null&&Ne(e,t,2)}function Ui(t){var e=t.alternate;return t===Dt||e!==null&&e===Dt}function Vc(t,e){jn=Ri=!0;var a=t.pending;a===null?e.next=e:(e.next=a.next,a.next=e),t.pending=e}function Kc(t,e,a){if((a&4194048)!==0){var n=e.lanes;n&=t.pendingLanes,a|=n,e.lanes=a,to(t,a)}}var Mi={readContext:ge,use:ki,useCallback:ae,useContext:ae,useEffect:ae,useImperativeHandle:ae,useLayoutEffect:ae,useInsertionEffect:ae,useMemo:ae,useReducer:ae,useRef:ae,useState:ae,useDebugValue:ae,useDeferredValue:ae,useTransition:ae,useSyncExternalStore:ae,useId:ae,useHostTransitionStatus:ae,useFormState:ae,useActionState:ae,useOptimistic:ae,useMemoCache:ae,useCacheRefresh:ae},Jc={readContext:ge,use:ki,useCallback:function(t,e){return we().memoizedState=[t,e===void 0?null:e],t},useContext:ge,useEffect:Cc,useImperativeHandle:function(t,e,a){a=a!=null?a.concat([t]):null,Ni(4194308,4,Mc.bind(null,e,t),a)},useLayoutEffect:function(t,e){return Ni(4194308,4,t,e)},useInsertionEffect:function(t,e){Ni(4,2,t,e)},useMemo:function(t,e){var a=we();e=e===void 0?null:e;var n=t();if(un){Rt(!0);try{t()}finally{Rt(!1)}}return a.memoizedState=[n,e],n},useReducer:function(t,e,a){var n=we();if(a!==void 0){var l=a(e);if(un){Rt(!0);try{a(e)}finally{Rt(!1)}}}else l=e;return n.memoizedState=n.baseState=l,t={pending:null,lanes:0,dispatch:null,lastRenderedReducer:t,lastRenderedState:l},n.queue=t,t=t.dispatch=Dm.bind(null,Dt,t),[n.memoizedState,t]},useRef:function(t){var e=we();return t={current:t},e.memoizedState=t},useState:function(t){t=Tu(t);var e=t.queue,a=Qc.bind(null,Dt,e);return e.dispatch=a,[t.memoizedState,a]},useDebugValue:Ou,useDeferredValue:function(t,e){var a=we();return ku(a,t,e)},useTransition:function(){var t=Tu(!1);return t=qc.bind(null,Dt,t.queue,!0,!1),we().memoizedState=t,[!1,t]},useSyncExternalStore:function(t,e,a){var n=Dt,l=we();if(Ht){if(a===void 0)throw Error(d(407));a=a()}else{if(a=e(),Ft===null)throw Error(d(349));(jt&124)!==0||pc(n,e,a)}l.memoizedState=a;var i={value:a,getSnapshot:e};return l.queue=i,Cc(yc.bind(null,n,i,t),[t]),n.flags|=2048,Hn(9,Di(),gc.bind(null,n,i,a,e),null),a},useId:function(){var t=we(),e=Ft.identifierPrefix;if(Ht){var a=ua,n=ra;a=(n&~(1<<32-Kt(n)-1)).toString(32)+a,e="«"+e+"R"+a,a=Oi++,0<a&&(e+="H"+a.toString(32)),e+="»"}else a=zm++,e="«"+e+"r"+a.toString(32)+"»";return t.memoizedState=e},useHostTransitionStatus:Du,useFormState:zc,useActionState:zc,useOptimistic:function(t){var e=we();e.memoizedState=e.baseState=t;var a={pending:null,lanes:0,dispatch:null,lastRenderedReducer:null,lastRenderedState:null};return e.queue=a,e=Nu.bind(null,Dt,!0,a),a.dispatch=e,[t,e]},useMemoCache:wu,useCacheRefresh:function(){return we().memoizedState=Cm.bind(null,Dt)}},Fc={readContext:ge,use:ki,useCallback:Bc,useContext:ge,useEffect:Dc,useImperativeHandle:jc,useInsertionEffect:Nc,useLayoutEffect:Uc,useMemo:Hc,useReducer:Ci,useRef:kc,useState:function(){return Ci(ca)},useDebugValue:Ou,useDeferredValue:function(t,e){var a=ie();return Lc(a,Zt.memoizedState,t,e)},useTransition:function(){var t=Ci(ca)[0],e=ie().memoizedState;return[typeof t=="boolean"?t:Al(t),e]},useSyncExternalStore:mc,useId:Gc,useHostTransitionStatus:Du,useFormState:Tc,useActionState:Tc,useOptimistic:function(t,e){var a=ie();return _c(a,Zt,t,e)},useMemoCache:wu,useCacheRefresh:Xc},Nm={readContext:ge,use:ki,useCallback:Bc,useContext:ge,useEffect:Dc,useImperativeHandle:jc,useInsertionEffect:Nc,useLayoutEffect:Uc,useMemo:Hc,useReducer:zu,useRef:kc,useState:function(){return zu(ca)},useDebugValue:Ou,useDeferredValue:function(t,e){var a=ie();return Zt===null?ku(a,t,e):Lc(a,Zt.memoizedState,t,e)},useTransition:function(){var t=zu(ca)[0],e=ie().memoizedState;return[typeof t=="boolean"?t:Al(t),e]},useSyncExternalStore:mc,useId:Gc,useHostTransitionStatus:Du,useFormState:Oc,useActionState:Oc,useOptimistic:function(t,e){var a=ie();return Zt!==null?_c(a,Zt,t,e):(a.baseState=t,[t,a.queue.dispatch])},useMemoCache:wu,useCacheRefresh:Xc},Ln=null,Rl=0;function ji(t){var e=Rl;return Rl+=1,Ln===null&&(Ln=[]),rc(Ln,t,e)}function Ol(t,e){e=e.props.ref,t.ref=e!==void 0?e:null}function Bi(t,e){throw e.$$typeof===A?Error(d(525)):(t=Object.prototype.toString.call(e),Error(d(31,t==="[object Object]"?"object with keys {"+Object.keys(e).join(", ")+"}":t)))}function Wc(t){var e=t._init;return e(t._payload)}function $c(t){function e(Y,H){if(t){var Q=Y.deletions;Q===null?(Y.deletions=[H],Y.flags|=16):Q.push(H)}}function a(Y,H){if(!t)return null;for(;H!==null;)e(Y,H),H=H.sibling;return null}function n(Y){for(var H=new Map;Y!==null;)Y.key!==null?H.set(Y.key,Y):H.set(Y.index,Y),Y=Y.sibling;return H}function l(Y,H){return Y=ia(Y,H),Y.index=0,Y.sibling=null,Y}function i(Y,H,Q){return Y.index=Q,t?(Q=Y.alternate,Q!==null?(Q=Q.index,Q<H?(Y.flags|=67108866,H):Q):(Y.flags|=67108866,H)):(Y.flags|=1048576,H)}function u(Y){return t&&Y.alternate===null&&(Y.flags|=67108866),Y}function x(Y,H,Q,ut){return H===null||H.tag!==6?(H=tu(Q,Y.mode,ut),H.return=Y,H):(H=l(H,Q),H.return=Y,H)}function U(Y,H,Q,ut){var St=Q.type;return St===z?lt(Y,H,Q.props.children,ut,Q.key):H!==null&&(H.elementType===St||typeof St=="object"&&St!==null&&St.$$typeof===ft&&Wc(St)===H.type)?(H=l(H,Q.props),Ol(H,Q),H.return=Y,H):(H=bi(Q.type,Q.key,Q.props,null,Y.mode,ut),Ol(H,Q),H.return=Y,H)}function V(Y,H,Q,ut){return H===null||H.tag!==4||H.stateNode.containerInfo!==Q.containerInfo||H.stateNode.implementation!==Q.implementation?(H=eu(Q,Y.mode,ut),H.return=Y,H):(H=l(H,Q.children||[]),H.return=Y,H)}function lt(Y,H,Q,ut,St){return H===null||H.tag!==7?(H=Ia(Q,Y.mode,ut,St),H.return=Y,H):(H=l(H,Q),H.return=Y,H)}function ct(Y,H,Q){if(typeof H=="string"&&H!==""||typeof H=="number"||typeof H=="bigint")return H=tu(""+H,Y.mode,Q),H.return=Y,H;if(typeof H=="object"&&H!==null){switch(H.$$typeof){case w:return Q=bi(H.type,H.key,H.props,null,Y.mode,Q),Ol(Q,H),Q.return=Y,Q;case _:return H=eu(H,Y.mode,Q),H.return=Y,H;case ft:var ut=H._init;return H=ut(H._payload),ct(Y,H,Q)}if(at(H)||M(H))return H=Ia(H,Y.mode,Q,null),H.return=Y,H;if(typeof H.then=="function")return ct(Y,ji(H),Q);if(H.$$typeof===D)return ct(Y,Ei(Y,H),Q);Bi(Y,H)}return null}function J(Y,H,Q,ut){var St=H!==null?H.key:null;if(typeof Q=="string"&&Q!==""||typeof Q=="number"||typeof Q=="bigint")return St!==null?null:x(Y,H,""+Q,ut);if(typeof Q=="object"&&Q!==null){switch(Q.$$typeof){case w:return Q.key===St?U(Y,H,Q,ut):null;case _:return Q.key===St?V(Y,H,Q,ut):null;case ft:return St=Q._init,Q=St(Q._payload),J(Y,H,Q,ut)}if(at(Q)||M(Q))return St!==null?null:lt(Y,H,Q,ut,null);if(typeof Q.then=="function")return J(Y,H,ji(Q),ut);if(Q.$$typeof===D)return J(Y,H,Ei(Y,Q),ut);Bi(Y,Q)}return null}function W(Y,H,Q,ut,St){if(typeof ut=="string"&&ut!==""||typeof ut=="number"||typeof ut=="bigint")return Y=Y.get(Q)||null,x(H,Y,""+ut,St);if(typeof ut=="object"&&ut!==null){switch(ut.$$typeof){case w:return Y=Y.get(ut.key===null?Q:ut.key)||null,U(H,Y,ut,St);case _:return Y=Y.get(ut.key===null?Q:ut.key)||null,V(H,Y,ut,St);case ft:var Nt=ut._init;return ut=Nt(ut._payload),W(Y,H,Q,ut,St)}if(at(ut)||M(ut))return Y=Y.get(Q)||null,lt(H,Y,ut,St,null);if(typeof ut.then=="function")return W(Y,H,Q,ji(ut),St);if(ut.$$typeof===D)return W(Y,H,Q,Ei(H,ut),St);Bi(H,ut)}return null}function Tt(Y,H,Q,ut){for(var St=null,Nt=null,wt=H,zt=H=0,fe=null;wt!==null&&zt<Q.length;zt++){wt.index>zt?(fe=wt,wt=null):fe=wt.sibling;var Bt=J(Y,wt,Q[zt],ut);if(Bt===null){wt===null&&(wt=fe);break}t&&wt&&Bt.alternate===null&&e(Y,wt),H=i(Bt,H,zt),Nt===null?St=Bt:Nt.sibling=Bt,Nt=Bt,wt=fe}if(zt===Q.length)return a(Y,wt),Ht&&tn(Y,zt),St;if(wt===null){for(;zt<Q.length;zt++)wt=ct(Y,Q[zt],ut),wt!==null&&(H=i(wt,H,zt),Nt===null?St=wt:Nt.sibling=wt,Nt=wt);return Ht&&tn(Y,zt),St}for(wt=n(wt);zt<Q.length;zt++)fe=W(wt,Y,zt,Q[zt],ut),fe!==null&&(t&&fe.alternate!==null&&wt.delete(fe.key===null?zt:fe.key),H=i(fe,H,zt),Nt===null?St=fe:Nt.sibling=fe,Nt=fe);return t&&wt.forEach(function(Xa){return e(Y,Xa)}),Ht&&tn(Y,zt),St}function At(Y,H,Q,ut){if(Q==null)throw Error(d(151));for(var St=null,Nt=null,wt=H,zt=H=0,fe=null,Bt=Q.next();wt!==null&&!Bt.done;zt++,Bt=Q.next()){wt.index>zt?(fe=wt,wt=null):fe=wt.sibling;var Xa=J(Y,wt,Bt.value,ut);if(Xa===null){wt===null&&(wt=fe);break}t&&wt&&Xa.alternate===null&&e(Y,wt),H=i(Xa,H,zt),Nt===null?St=Xa:Nt.sibling=Xa,Nt=Xa,wt=fe}if(Bt.done)return a(Y,wt),Ht&&tn(Y,zt),St;if(wt===null){for(;!Bt.done;zt++,Bt=Q.next())Bt=ct(Y,Bt.value,ut),Bt!==null&&(H=i(Bt,H,zt),Nt===null?St=Bt:Nt.sibling=Bt,Nt=Bt);return Ht&&tn(Y,zt),St}for(wt=n(wt);!Bt.done;zt++,Bt=Q.next())Bt=W(wt,Y,zt,Bt.value,ut),Bt!==null&&(t&&Bt.alternate!==null&&wt.delete(Bt.key===null?zt:Bt.key),H=i(Bt,H,zt),Nt===null?St=Bt:Nt.sibling=Bt,Nt=Bt);return t&&wt.forEach(function(U0){return e(Y,U0)}),Ht&&tn(Y,zt),St}function Xt(Y,H,Q,ut){if(typeof Q=="object"&&Q!==null&&Q.type===z&&Q.key===null&&(Q=Q.props.children),typeof Q=="object"&&Q!==null){switch(Q.$$typeof){case w:t:{for(var St=Q.key;H!==null;){if(H.key===St){if(St=Q.type,St===z){if(H.tag===7){a(Y,H.sibling),ut=l(H,Q.props.children),ut.return=Y,Y=ut;break t}}else if(H.elementType===St||typeof St=="object"&&St!==null&&St.$$typeof===ft&&Wc(St)===H.type){a(Y,H.sibling),ut=l(H,Q.props),Ol(ut,Q),ut.return=Y,Y=ut;break t}a(Y,H);break}else e(Y,H);H=H.sibling}Q.type===z?(ut=Ia(Q.props.children,Y.mode,ut,Q.key),ut.return=Y,Y=ut):(ut=bi(Q.type,Q.key,Q.props,null,Y.mode,ut),Ol(ut,Q),ut.return=Y,Y=ut)}return u(Y);case _:t:{for(St=Q.key;H!==null;){if(H.key===St)if(H.tag===4&&H.stateNode.containerInfo===Q.containerInfo&&H.stateNode.implementation===Q.implementation){a(Y,H.sibling),ut=l(H,Q.children||[]),ut.return=Y,Y=ut;break t}else{a(Y,H);break}else e(Y,H);H=H.sibling}ut=eu(Q,Y.mode,ut),ut.return=Y,Y=ut}return u(Y);case ft:return St=Q._init,Q=St(Q._payload),Xt(Y,H,Q,ut)}if(at(Q))return Tt(Y,H,Q,ut);if(M(Q)){if(St=M(Q),typeof St!="function")throw Error(d(150));return Q=St.call(Q),At(Y,H,Q,ut)}if(typeof Q.then=="function")return Xt(Y,H,ji(Q),ut);if(Q.$$typeof===D)return Xt(Y,H,Ei(Y,Q),ut);Bi(Y,Q)}return typeof Q=="string"&&Q!==""||typeof Q=="number"||typeof Q=="bigint"?(Q=""+Q,H!==null&&H.tag===6?(a(Y,H.sibling),ut=l(H,Q),ut.return=Y,Y=ut):(a(Y,H),ut=tu(Q,Y.mode,ut),ut.return=Y,Y=ut),u(Y)):a(Y,H)}return function(Y,H,Q,ut){try{Rl=0;var St=Xt(Y,H,Q,ut);return Ln=null,St}catch(wt){if(wt===bl||wt===Ai)throw wt;var Nt=Re(29,wt,null,Y.mode);return Nt.lanes=ut,Nt.return=Y,Nt}finally{}}}var qn=$c(!0),Ic=$c(!1),Ye=N(null),We=null;function Oa(t){var e=t.alternate;nt(ue,ue.current&1),nt(Ye,t),We===null&&(e===null||Mn.current!==null||e.memoizedState!==null)&&(We=t)}function Pc(t){if(t.tag===22){if(nt(ue,ue.current),nt(Ye,t),We===null){var e=t.alternate;e!==null&&e.memoizedState!==null&&(We=t)}}else ka()}function ka(){nt(ue,ue.current),nt(Ye,Ye.current)}function fa(t){B(Ye),We===t&&(We=null),B(ue)}var ue=N(0);function Hi(t){for(var e=t;e!==null;){if(e.tag===13){var a=e.memoizedState;if(a!==null&&(a=a.dehydrated,a===null||a.data==="$?"||Es(a)))return e}else if(e.tag===19&&e.memoizedProps.revealOrder!==void 0){if((e.flags&128)!==0)return e}else if(e.child!==null){e.child.return=e,e=e.child;continue}if(e===t)break;for(;e.sibling===null;){if(e.return===null||e.return===t)return null;e=e.return}e.sibling.return=e.return,e=e.sibling}return null}function Uu(t,e,a,n){e=t.memoizedState,a=a(n,e),a=a==null?e:E({},e,a),t.memoizedState=a,t.lanes===0&&(t.updateQueue.baseState=a)}var Mu={enqueueSetState:function(t,e,a){t=t._reactInternals;var n=De(),l=za(n);l.payload=e,a!=null&&(l.callback=a),e=Ta(t,l,n),e!==null&&(Ne(e,t,n),Sl(e,t,n))},enqueueReplaceState:function(t,e,a){t=t._reactInternals;var n=De(),l=za(n);l.tag=1,l.payload=e,a!=null&&(l.callback=a),e=Ta(t,l,n),e!==null&&(Ne(e,t,n),Sl(e,t,n))},enqueueForceUpdate:function(t,e){t=t._reactInternals;var a=De(),n=za(a);n.tag=2,e!=null&&(n.callback=e),e=Ta(t,n,a),e!==null&&(Ne(e,t,a),Sl(e,t,a))}};function tf(t,e,a,n,l,i,u){return t=t.stateNode,typeof t.shouldComponentUpdate=="function"?t.shouldComponentUpdate(n,i,u):e.prototype&&e.prototype.isPureReactComponent?!fl(a,n)||!fl(l,i):!0}function ef(t,e,a,n){t=e.state,typeof e.componentWillReceiveProps=="function"&&e.componentWillReceiveProps(a,n),typeof e.UNSAFE_componentWillReceiveProps=="function"&&e.UNSAFE_componentWillReceiveProps(a,n),e.state!==t&&Mu.enqueueReplaceState(e,e.state,null)}function sn(t,e){var a=e;if("ref"in e){a={};for(var n in e)n!=="ref"&&(a[n]=e[n])}if(t=t.defaultProps){a===e&&(a=E({},a));for(var l in t)a[l]===void 0&&(a[l]=t[l])}return a}var Li=typeof reportError=="function"?reportError:function(t){if(typeof window=="object"&&typeof window.ErrorEvent=="function"){var e=new window.ErrorEvent("error",{bubbles:!0,cancelable:!0,message:typeof t=="object"&&t!==null&&typeof t.message=="string"?String(t.message):String(t),error:t});if(!window.dispatchEvent(e))return}else if(typeof process=="object"&&typeof process.emit=="function"){process.emit("uncaughtException",t);return}console.error(t)};function af(t){Li(t)}function nf(t){console.error(t)}function lf(t){Li(t)}function qi(t,e){try{var a=t.onUncaughtError;a(e.value,{componentStack:e.stack})}catch(n){setTimeout(function(){throw n})}}function rf(t,e,a){try{var n=t.onCaughtError;n(a.value,{componentStack:a.stack,errorBoundary:e.tag===1?e.stateNode:null})}catch(l){setTimeout(function(){throw l})}}function ju(t,e,a){return a=za(a),a.tag=3,a.payload={element:null},a.callback=function(){qi(t,e)},a}function uf(t){return t=za(t),t.tag=3,t}function sf(t,e,a,n){var l=a.type.getDerivedStateFromError;if(typeof l=="function"){var i=n.value;t.payload=function(){return l(i)},t.callback=function(){rf(e,a,n)}}var u=a.stateNode;u!==null&&typeof u.componentDidCatch=="function"&&(t.callback=function(){rf(e,a,n),typeof l!="function"&&(ja===null?ja=new Set([this]):ja.add(this));var x=n.stack;this.componentDidCatch(n.value,{componentStack:x!==null?x:""})})}function Um(t,e,a,n,l){if(a.flags|=32768,n!==null&&typeof n=="object"&&typeof n.then=="function"){if(e=a.alternate,e!==null&&gl(e,a,l,!0),a=Ye.current,a!==null){switch(a.tag){case 13:return We===null?rs():a.alternate===null&&ee===0&&(ee=3),a.flags&=-257,a.flags|=65536,a.lanes=l,n===fu?a.flags|=16384:(e=a.updateQueue,e===null?a.updateQueue=new Set([n]):e.add(n),ss(t,n,l)),!1;case 22:return a.flags|=65536,n===fu?a.flags|=16384:(e=a.updateQueue,e===null?(e={transitions:null,markerInstances:null,retryQueue:new Set([n])},a.updateQueue=e):(a=e.retryQueue,a===null?e.retryQueue=new Set([n]):a.add(n)),ss(t,n,l)),!1}throw Error(d(435,a.tag))}return ss(t,n,l),rs(),!1}if(Ht)return e=Ye.current,e!==null?((e.flags&65536)===0&&(e.flags|=256),e.flags|=65536,e.lanes=l,n!==lu&&(t=Error(d(422),{cause:n}),pl(Be(t,a)))):(n!==lu&&(e=Error(d(423),{cause:n}),pl(Be(e,a))),t=t.current.alternate,t.flags|=65536,l&=-l,t.lanes|=l,n=Be(n,a),l=ju(t.stateNode,n,l),mu(t,l),ee!==4&&(ee=2)),!1;var i=Error(d(520),{cause:n});if(i=Be(i,a),jl===null?jl=[i]:jl.push(i),ee!==4&&(ee=2),e===null)return!0;n=Be(n,a),a=e;do{switch(a.tag){case 3:return a.flags|=65536,t=l&-l,a.lanes|=t,t=ju(a.stateNode,n,t),mu(a,t),!1;case 1:if(e=a.type,i=a.stateNode,(a.flags&128)===0&&(typeof e.getDerivedStateFromError=="function"||i!==null&&typeof i.componentDidCatch=="function"&&(ja===null||!ja.has(i))))return a.flags|=65536,l&=-l,a.lanes|=l,l=uf(l),sf(l,t,a,n),mu(a,l),!1}a=a.return}while(a!==null);return!1}var of=Error(d(461)),oe=!1;function de(t,e,a,n){e.child=t===null?Ic(e,null,a,n):qn(e,t.child,a,n)}function cf(t,e,a,n,l){a=a.render;var i=e.ref;if("ref"in n){var u={};for(var x in n)x!=="ref"&&(u[x]=n[x])}else u=n;return ln(e),n=bu(t,e,a,u,i,l),x=_u(),t!==null&&!oe?(Su(t,e,l),da(t,e,l)):(Ht&&x&&au(e),e.flags|=1,de(t,e,n,l),e.child)}function ff(t,e,a,n,l){if(t===null){var i=a.type;return typeof i=="function"&&!Pr(i)&&i.defaultProps===void 0&&a.compare===null?(e.tag=15,e.type=i,df(t,e,i,n,l)):(t=bi(a.type,null,n,e,e.mode,l),t.ref=e.ref,t.return=e,e.child=t)}if(i=t.child,!Xu(t,l)){var u=i.memoizedProps;if(a=a.compare,a=a!==null?a:fl,a(u,n)&&t.ref===e.ref)return da(t,e,l)}return e.flags|=1,t=ia(i,n),t.ref=e.ref,t.return=e,e.child=t}function df(t,e,a,n,l){if(t!==null){var i=t.memoizedProps;if(fl(i,n)&&t.ref===e.ref)if(oe=!1,e.pendingProps=n=i,Xu(t,l))(t.flags&131072)!==0&&(oe=!0);else return e.lanes=t.lanes,da(t,e,l)}return Bu(t,e,a,n,l)}function hf(t,e,a){var n=e.pendingProps,l=n.children,i=t!==null?t.memoizedState:null;if(n.mode==="hidden"){if((e.flags&128)!==0){if(n=i!==null?i.baseLanes|a:a,t!==null){for(l=e.child=t.child,i=0;l!==null;)i=i|l.lanes|l.childLanes,l=l.sibling;e.childLanes=i&~n}else e.childLanes=0,e.child=null;return mf(t,e,n,a)}if((a&536870912)!==0)e.memoizedState={baseLanes:0,cachePool:null},t!==null&&wi(e,i!==null?i.cachePool:null),i!==null?fc(e,i):gu(),Pc(e);else return e.lanes=e.childLanes=536870912,mf(t,e,i!==null?i.baseLanes|a:a,a)}else i!==null?(wi(e,i.cachePool),fc(e,i),ka(),e.memoizedState=null):(t!==null&&wi(e,null),gu(),ka());return de(t,e,l,a),e.child}function mf(t,e,a,n){var l=cu();return l=l===null?null:{parent:re._currentValue,pool:l},e.memoizedState={baseLanes:a,cachePool:l},t!==null&&wi(e,null),gu(),Pc(e),t!==null&&gl(t,e,n,!0),null}function Yi(t,e){var a=e.ref;if(a===null)t!==null&&t.ref!==null&&(e.flags|=4194816);else{if(typeof a!="function"&&typeof a!="object")throw Error(d(284));(t===null||t.ref!==a)&&(e.flags|=4194816)}}function Bu(t,e,a,n,l){return ln(e),a=bu(t,e,a,n,void 0,l),n=_u(),t!==null&&!oe?(Su(t,e,l),da(t,e,l)):(Ht&&n&&au(e),e.flags|=1,de(t,e,a,l),e.child)}function pf(t,e,a,n,l,i){return ln(e),e.updateQueue=null,a=hc(e,n,a,l),dc(t),n=_u(),t!==null&&!oe?(Su(t,e,i),da(t,e,i)):(Ht&&n&&au(e),e.flags|=1,de(t,e,a,i),e.child)}function gf(t,e,a,n,l){if(ln(e),e.stateNode===null){var i=kn,u=a.contextType;typeof u=="object"&&u!==null&&(i=ge(u)),i=new a(n,i),e.memoizedState=i.state!==null&&i.state!==void 0?i.state:null,i.updater=Mu,e.stateNode=i,i._reactInternals=e,i=e.stateNode,i.props=n,i.state=e.memoizedState,i.refs={},du(e),u=a.contextType,i.context=typeof u=="object"&&u!==null?ge(u):kn,i.state=e.memoizedState,u=a.getDerivedStateFromProps,typeof u=="function"&&(Uu(e,a,u,n),i.state=e.memoizedState),typeof a.getDerivedStateFromProps=="function"||typeof i.getSnapshotBeforeUpdate=="function"||typeof i.UNSAFE_componentWillMount!="function"&&typeof i.componentWillMount!="function"||(u=i.state,typeof i.componentWillMount=="function"&&i.componentWillMount(),typeof i.UNSAFE_componentWillMount=="function"&&i.UNSAFE_componentWillMount(),u!==i.state&&Mu.enqueueReplaceState(i,i.state,null),El(e,n,i,l),xl(),i.state=e.memoizedState),typeof i.componentDidMount=="function"&&(e.flags|=4194308),n=!0}else if(t===null){i=e.stateNode;var x=e.memoizedProps,U=sn(a,x);i.props=U;var V=i.context,lt=a.contextType;u=kn,typeof lt=="object"&&lt!==null&&(u=ge(lt));var ct=a.getDerivedStateFromProps;lt=typeof ct=="function"||typeof i.getSnapshotBeforeUpdate=="function",x=e.pendingProps!==x,lt||typeof i.UNSAFE_componentWillReceiveProps!="function"&&typeof i.componentWillReceiveProps!="function"||(x||V!==u)&&ef(e,i,n,u),Aa=!1;var J=e.memoizedState;i.state=J,El(e,n,i,l),xl(),V=e.memoizedState,x||J!==V||Aa?(typeof ct=="function"&&(Uu(e,a,ct,n),V=e.memoizedState),(U=Aa||tf(e,a,U,n,J,V,u))?(lt||typeof i.UNSAFE_componentWillMount!="function"&&typeof i.componentWillMount!="function"||(typeof i.componentWillMount=="function"&&i.componentWillMount(),typeof i.UNSAFE_componentWillMount=="function"&&i.UNSAFE_componentWillMount()),typeof i.componentDidMount=="function"&&(e.flags|=4194308)):(typeof i.componentDidMount=="function"&&(e.flags|=4194308),e.memoizedProps=n,e.memoizedState=V),i.props=n,i.state=V,i.context=u,n=U):(typeof i.componentDidMount=="function"&&(e.flags|=4194308),n=!1)}else{i=e.stateNode,hu(t,e),u=e.memoizedProps,lt=sn(a,u),i.props=lt,ct=e.pendingProps,J=i.context,V=a.contextType,U=kn,typeof V=="object"&&V!==null&&(U=ge(V)),x=a.getDerivedStateFromProps,(V=typeof x=="function"||typeof i.getSnapshotBeforeUpdate=="function")||typeof i.UNSAFE_componentWillReceiveProps!="function"&&typeof i.componentWillReceiveProps!="function"||(u!==ct||J!==U)&&ef(e,i,n,U),Aa=!1,J=e.memoizedState,i.state=J,El(e,n,i,l),xl();var W=e.memoizedState;u!==ct||J!==W||Aa||t!==null&&t.dependencies!==null&&xi(t.dependencies)?(typeof x=="function"&&(Uu(e,a,x,n),W=e.memoizedState),(lt=Aa||tf(e,a,lt,n,J,W,U)||t!==null&&t.dependencies!==null&&xi(t.dependencies))?(V||typeof i.UNSAFE_componentWillUpdate!="function"&&typeof i.componentWillUpdate!="function"||(typeof i.componentWillUpdate=="function"&&i.componentWillUpdate(n,W,U),typeof i.UNSAFE_componentWillUpdate=="function"&&i.UNSAFE_componentWillUpdate(n,W,U)),typeof i.componentDidUpdate=="function"&&(e.flags|=4),typeof i.getSnapshotBeforeUpdate=="function"&&(e.flags|=1024)):(typeof i.componentDidUpdate!="function"||u===t.memoizedProps&&J===t.memoizedState||(e.flags|=4),typeof i.getSnapshotBeforeUpdate!="function"||u===t.memoizedProps&&J===t.memoizedState||(e.flags|=1024),e.memoizedProps=n,e.memoizedState=W),i.props=n,i.state=W,i.context=U,n=lt):(typeof i.componentDidUpdate!="function"||u===t.memoizedProps&&J===t.memoizedState||(e.flags|=4),typeof i.getSnapshotBeforeUpdate!="function"||u===t.memoizedProps&&J===t.memoizedState||(e.flags|=1024),n=!1)}return i=n,Yi(t,e),n=(e.flags&128)!==0,i||n?(i=e.stateNode,a=n&&typeof a.getDerivedStateFromError!="function"?null:i.render(),e.flags|=1,t!==null&&n?(e.child=qn(e,t.child,null,l),e.child=qn(e,null,a,l)):de(t,e,a,l),e.memoizedState=i.state,t=e.child):t=da(t,e,l),t}function yf(t,e,a,n){return ml(),e.flags|=256,de(t,e,a,n),e.child}var Hu={dehydrated:null,treeContext:null,retryLane:0,hydrationErrors:null};function Lu(t){return{baseLanes:t,cachePool:nc()}}function qu(t,e,a){return t=t!==null?t.childLanes&~a:0,e&&(t|=Ze),t}function vf(t,e,a){var n=e.pendingProps,l=!1,i=(e.flags&128)!==0,u;if((u=i)||(u=t!==null&&t.memoizedState===null?!1:(ue.current&2)!==0),u&&(l=!0,e.flags&=-129),u=(e.flags&32)!==0,e.flags&=-33,t===null){if(Ht){if(l?Oa(e):ka(),Ht){var x=te,U;if(U=x){t:{for(U=x,x=Fe;U.nodeType!==8;){if(!x){x=null;break t}if(U=Je(U.nextSibling),U===null){x=null;break t}}x=U}x!==null?(e.memoizedState={dehydrated:x,treeContext:Pa!==null?{id:ra,overflow:ua}:null,retryLane:536870912,hydrationErrors:null},U=Re(18,null,null,0),U.stateNode=x,U.return=e,e.child=U,be=e,te=null,U=!0):U=!1}U||an(e)}if(x=e.memoizedState,x!==null&&(x=x.dehydrated,x!==null))return Es(x)?e.lanes=32:e.lanes=536870912,null;fa(e)}return x=n.children,n=n.fallback,l?(ka(),l=e.mode,x=Zi({mode:"hidden",children:x},l),n=Ia(n,l,a,null),x.return=e,n.return=e,x.sibling=n,e.child=x,l=e.child,l.memoizedState=Lu(a),l.childLanes=qu(t,u,a),e.memoizedState=Hu,n):(Oa(e),Yu(e,x))}if(U=t.memoizedState,U!==null&&(x=U.dehydrated,x!==null)){if(i)e.flags&256?(Oa(e),e.flags&=-257,e=Zu(t,e,a)):e.memoizedState!==null?(ka(),e.child=t.child,e.flags|=128,e=null):(ka(),l=n.fallback,x=e.mode,n=Zi({mode:"visible",children:n.children},x),l=Ia(l,x,a,null),l.flags|=2,n.return=e,l.return=e,n.sibling=l,e.child=n,qn(e,t.child,null,a),n=e.child,n.memoizedState=Lu(a),n.childLanes=qu(t,u,a),e.memoizedState=Hu,e=l);else if(Oa(e),Es(x)){if(u=x.nextSibling&&x.nextSibling.dataset,u)var V=u.dgst;u=V,n=Error(d(419)),n.stack="",n.digest=u,pl({value:n,source:null,stack:null}),e=Zu(t,e,a)}else if(oe||gl(t,e,a,!1),u=(a&t.childLanes)!==0,oe||u){if(u=Ft,u!==null&&(n=a&-a,n=(n&42)!==0?1:wr(n),n=(n&(u.suspendedLanes|a))!==0?0:n,n!==0&&n!==U.retryLane))throw U.retryLane=n,On(t,n),Ne(u,t,n),of;x.data==="$?"||rs(),e=Zu(t,e,a)}else x.data==="$?"?(e.flags|=192,e.child=t.child,e=null):(t=U.treeContext,te=Je(x.nextSibling),be=e,Ht=!0,en=null,Fe=!1,t!==null&&(Le[qe++]=ra,Le[qe++]=ua,Le[qe++]=Pa,ra=t.id,ua=t.overflow,Pa=e),e=Yu(e,n.children),e.flags|=4096);return e}return l?(ka(),l=n.fallback,x=e.mode,U=t.child,V=U.sibling,n=ia(U,{mode:"hidden",children:n.children}),n.subtreeFlags=U.subtreeFlags&65011712,V!==null?l=ia(V,l):(l=Ia(l,x,a,null),l.flags|=2),l.return=e,n.return=e,n.sibling=l,e.child=n,n=l,l=e.child,x=t.child.memoizedState,x===null?x=Lu(a):(U=x.cachePool,U!==null?(V=re._currentValue,U=U.parent!==V?{parent:V,pool:V}:U):U=nc(),x={baseLanes:x.baseLanes|a,cachePool:U}),l.memoizedState=x,l.childLanes=qu(t,u,a),e.memoizedState=Hu,n):(Oa(e),a=t.child,t=a.sibling,a=ia(a,{mode:"visible",children:n.children}),a.return=e,a.sibling=null,t!==null&&(u=e.deletions,u===null?(e.deletions=[t],e.flags|=16):u.push(t)),e.child=a,e.memoizedState=null,a)}function Yu(t,e){return e=Zi({mode:"visible",children:e},t.mode),e.return=t,t.child=e}function Zi(t,e){return t=Re(22,t,null,e),t.lanes=0,t.stateNode={_visibility:1,_pendingMarkers:null,_retryCache:null,_transitions:null},t}function Zu(t,e,a){return qn(e,t.child,null,a),t=Yu(e,e.pendingProps.children),t.flags|=2,e.memoizedState=null,t}function bf(t,e,a){t.lanes|=e;var n=t.alternate;n!==null&&(n.lanes|=e),ru(t.return,e,a)}function Gu(t,e,a,n,l){var i=t.memoizedState;i===null?t.memoizedState={isBackwards:e,rendering:null,renderingStartTime:0,last:n,tail:a,tailMode:l}:(i.isBackwards=e,i.rendering=null,i.renderingStartTime=0,i.last=n,i.tail=a,i.tailMode=l)}function _f(t,e,a){var n=e.pendingProps,l=n.revealOrder,i=n.tail;if(de(t,e,n.children,a),n=ue.current,(n&2)!==0)n=n&1|2,e.flags|=128;else{if(t!==null&&(t.flags&128)!==0)t:for(t=e.child;t!==null;){if(t.tag===13)t.memoizedState!==null&&bf(t,a,e);else if(t.tag===19)bf(t,a,e);else if(t.child!==null){t.child.return=t,t=t.child;continue}if(t===e)break t;for(;t.sibling===null;){if(t.return===null||t.return===e)break t;t=t.return}t.sibling.return=t.return,t=t.sibling}n&=1}switch(nt(ue,n),l){case"forwards":for(a=e.child,l=null;a!==null;)t=a.alternate,t!==null&&Hi(t)===null&&(l=a),a=a.sibling;a=l,a===null?(l=e.child,e.child=null):(l=a.sibling,a.sibling=null),Gu(e,!1,l,a,i);break;case"backwards":for(a=null,l=e.child,e.child=null;l!==null;){if(t=l.alternate,t!==null&&Hi(t)===null){e.child=l;break}t=l.sibling,l.sibling=a,a=l,l=t}Gu(e,!0,a,null,i);break;case"together":Gu(e,!1,null,null,void 0);break;default:e.memoizedState=null}return e.child}function da(t,e,a){if(t!==null&&(e.dependencies=t.dependencies),Ma|=e.lanes,(a&e.childLanes)===0)if(t!==null){if(gl(t,e,a,!1),(a&e.childLanes)===0)return null}else return null;if(t!==null&&e.child!==t.child)throw Error(d(153));if(e.child!==null){for(t=e.child,a=ia(t,t.pendingProps),e.child=a,a.return=e;t.sibling!==null;)t=t.sibling,a=a.sibling=ia(t,t.pendingProps),a.return=e;a.sibling=null}return e.child}function Xu(t,e){return(t.lanes&e)!==0?!0:(t=t.dependencies,!!(t!==null&&xi(t)))}function Mm(t,e,a){switch(e.tag){case 3:xt(e,e.stateNode.containerInfo),wa(e,re,t.memoizedState.cache),ml();break;case 27:case 5:Pt(e);break;case 4:xt(e,e.stateNode.containerInfo);break;case 10:wa(e,e.type,e.memoizedProps.value);break;case 13:var n=e.memoizedState;if(n!==null)return n.dehydrated!==null?(Oa(e),e.flags|=128,null):(a&e.child.childLanes)!==0?vf(t,e,a):(Oa(e),t=da(t,e,a),t!==null?t.sibling:null);Oa(e);break;case 19:var l=(t.flags&128)!==0;if(n=(a&e.childLanes)!==0,n||(gl(t,e,a,!1),n=(a&e.childLanes)!==0),l){if(n)return _f(t,e,a);e.flags|=128}if(l=e.memoizedState,l!==null&&(l.rendering=null,l.tail=null,l.lastEffect=null),nt(ue,ue.current),n)break;return null;case 22:case 23:return e.lanes=0,hf(t,e,a);case 24:wa(e,re,t.memoizedState.cache)}return da(t,e,a)}function Sf(t,e,a){if(t!==null)if(t.memoizedProps!==e.pendingProps)oe=!0;else{if(!Xu(t,a)&&(e.flags&128)===0)return oe=!1,Mm(t,e,a);oe=(t.flags&131072)!==0}else oe=!1,Ht&&(e.flags&1048576)!==0&&Wo(e,Si,e.index);switch(e.lanes=0,e.tag){case 16:t:{t=e.pendingProps;var n=e.elementType,l=n._init;if(n=l(n._payload),e.type=n,typeof n=="function")Pr(n)?(t=sn(n,t),e.tag=1,e=gf(null,e,n,t,a)):(e.tag=0,e=Bu(null,e,n,t,a));else{if(n!=null){if(l=n.$$typeof,l===L){e.tag=11,e=cf(null,e,n,t,a);break t}else if(l===F){e.tag=14,e=ff(null,e,n,t,a);break t}}throw e=b(n)||n,Error(d(306,e,""))}}return e;case 0:return Bu(t,e,e.type,e.pendingProps,a);case 1:return n=e.type,l=sn(n,e.pendingProps),gf(t,e,n,l,a);case 3:t:{if(xt(e,e.stateNode.containerInfo),t===null)throw Error(d(387));n=e.pendingProps;var i=e.memoizedState;l=i.element,hu(t,e),El(e,n,null,a);var u=e.memoizedState;if(n=u.cache,wa(e,re,n),n!==i.cache&&uu(e,[re],a,!0),xl(),n=u.element,i.isDehydrated)if(i={element:n,isDehydrated:!1,cache:u.cache},e.updateQueue.baseState=i,e.memoizedState=i,e.flags&256){e=yf(t,e,n,a);break t}else if(n!==l){l=Be(Error(d(424)),e),pl(l),e=yf(t,e,n,a);break t}else{switch(t=e.stateNode.containerInfo,t.nodeType){case 9:t=t.body;break;default:t=t.nodeName==="HTML"?t.ownerDocument.body:t}for(te=Je(t.firstChild),be=e,Ht=!0,en=null,Fe=!0,a=Ic(e,null,n,a),e.child=a;a;)a.flags=a.flags&-3|4096,a=a.sibling}else{if(ml(),n===l){e=da(t,e,a);break t}de(t,e,n,a)}e=e.child}return e;case 26:return Yi(t,e),t===null?(a=Ad(e.type,null,e.pendingProps,null))?e.memoizedState=a:Ht||(a=e.type,t=e.pendingProps,n=ar(_t.current).createElement(a),n[pe]=e,n[xe]=t,me(n,a,t),se(n),e.stateNode=n):e.memoizedState=Ad(e.type,t.memoizedProps,e.pendingProps,t.memoizedState),null;case 27:return Pt(e),t===null&&Ht&&(n=e.stateNode=xd(e.type,e.pendingProps,_t.current),be=e,Fe=!0,l=te,La(e.type)?(ws=l,te=Je(n.firstChild)):te=l),de(t,e,e.pendingProps.children,a),Yi(t,e),t===null&&(e.flags|=4194304),e.child;case 5:return t===null&&Ht&&((l=n=te)&&(n=o0(n,e.type,e.pendingProps,Fe),n!==null?(e.stateNode=n,be=e,te=Je(n.firstChild),Fe=!1,l=!0):l=!1),l||an(e)),Pt(e),l=e.type,i=e.pendingProps,u=t!==null?t.memoizedProps:null,n=i.children,_s(l,i)?n=null:u!==null&&_s(l,u)&&(e.flags|=32),e.memoizedState!==null&&(l=bu(t,e,Tm,null,null,a),Ql._currentValue=l),Yi(t,e),de(t,e,n,a),e.child;case 6:return t===null&&Ht&&((t=a=te)&&(a=c0(a,e.pendingProps,Fe),a!==null?(e.stateNode=a,be=e,te=null,t=!0):t=!1),t||an(e)),null;case 13:return vf(t,e,a);case 4:return xt(e,e.stateNode.containerInfo),n=e.pendingProps,t===null?e.child=qn(e,null,n,a):de(t,e,n,a),e.child;case 11:return cf(t,e,e.type,e.pendingProps,a);case 7:return de(t,e,e.pendingProps,a),e.child;case 8:return de(t,e,e.pendingProps.children,a),e.child;case 12:return de(t,e,e.pendingProps.children,a),e.child;case 10:return n=e.pendingProps,wa(e,e.type,n.value),de(t,e,n.children,a),e.child;case 9:return l=e.type._context,n=e.pendingProps.children,ln(e),l=ge(l),n=n(l),e.flags|=1,de(t,e,n,a),e.child;case 14:return ff(t,e,e.type,e.pendingProps,a);case 15:return df(t,e,e.type,e.pendingProps,a);case 19:return _f(t,e,a);case 31:return n=e.pendingProps,a=e.mode,n={mode:n.mode,children:n.children},t===null?(a=Zi(n,a),a.ref=e.ref,e.child=a,a.return=e,e=a):(a=ia(t.child,n),a.ref=e.ref,e.child=a,a.return=e,e=a),e;case 22:return hf(t,e,a);case 24:return ln(e),n=ge(re),t===null?(l=cu(),l===null&&(l=Ft,i=su(),l.pooledCache=i,i.refCount++,i!==null&&(l.pooledCacheLanes|=a),l=i),e.memoizedState={parent:n,cache:l},du(e),wa(e,re,l)):((t.lanes&a)!==0&&(hu(t,e),El(e,null,null,a),xl()),l=t.memoizedState,i=e.memoizedState,l.parent!==n?(l={parent:n,cache:n},e.memoizedState=l,e.lanes===0&&(e.memoizedState=e.updateQueue.baseState=l),wa(e,re,n)):(n=i.cache,wa(e,re,n),n!==l.cache&&uu(e,[re],a,!0))),de(t,e,e.pendingProps.children,a),e.child;case 29:throw e.pendingProps}throw Error(d(156,e.tag))}function ha(t){t.flags|=4}function xf(t,e){if(e.type!=="stylesheet"||(e.state.loading&4)!==0)t.flags&=-16777217;else if(t.flags|=16777216,!kd(e)){if(e=Ye.current,e!==null&&((jt&4194048)===jt?We!==null:(jt&62914560)!==jt&&(jt&536870912)===0||e!==We))throw _l=fu,lc;t.flags|=8192}}function Gi(t,e){e!==null&&(t.flags|=4),t.flags&16384&&(e=t.tag!==22?li():536870912,t.lanes|=e,Xn|=e)}function kl(t,e){if(!Ht)switch(t.tailMode){case"hidden":e=t.tail;for(var a=null;e!==null;)e.alternate!==null&&(a=e),e=e.sibling;a===null?t.tail=null:a.sibling=null;break;case"collapsed":a=t.tail;for(var n=null;a!==null;)a.alternate!==null&&(n=a),a=a.sibling;n===null?e||t.tail===null?t.tail=null:t.tail.sibling=null:n.sibling=null}}function $t(t){var e=t.alternate!==null&&t.alternate.child===t.child,a=0,n=0;if(e)for(var l=t.child;l!==null;)a|=l.lanes|l.childLanes,n|=l.subtreeFlags&65011712,n|=l.flags&65011712,l.return=t,l=l.sibling;else for(l=t.child;l!==null;)a|=l.lanes|l.childLanes,n|=l.subtreeFlags,n|=l.flags,l.return=t,l=l.sibling;return t.subtreeFlags|=n,t.childLanes=a,e}function jm(t,e,a){var n=e.pendingProps;switch(nu(e),e.tag){case 31:case 16:case 15:case 0:case 11:case 7:case 8:case 12:case 9:case 14:return $t(e),null;case 1:return $t(e),null;case 3:return a=e.stateNode,n=null,t!==null&&(n=t.memoizedState.cache),e.memoizedState.cache!==n&&(e.flags|=2048),oa(re),Ct(),a.pendingContext&&(a.context=a.pendingContext,a.pendingContext=null),(t===null||t.child===null)&&(hl(e)?ha(e):t===null||t.memoizedState.isDehydrated&&(e.flags&256)===0||(e.flags|=1024,Po())),$t(e),null;case 26:return a=e.memoizedState,t===null?(ha(e),a!==null?($t(e),xf(e,a)):($t(e),e.flags&=-16777217)):a?a!==t.memoizedState?(ha(e),$t(e),xf(e,a)):($t(e),e.flags&=-16777217):(t.memoizedProps!==n&&ha(e),$t(e),e.flags&=-16777217),null;case 27:Vt(e),a=_t.current;var l=e.type;if(t!==null&&e.stateNode!=null)t.memoizedProps!==n&&ha(e);else{if(!n){if(e.stateNode===null)throw Error(d(166));return $t(e),null}t=it.current,hl(e)?$o(e):(t=xd(l,n,a),e.stateNode=t,ha(e))}return $t(e),null;case 5:if(Vt(e),a=e.type,t!==null&&e.stateNode!=null)t.memoizedProps!==n&&ha(e);else{if(!n){if(e.stateNode===null)throw Error(d(166));return $t(e),null}if(t=it.current,hl(e))$o(e);else{switch(l=ar(_t.current),t){case 1:t=l.createElementNS("http://www.w3.org/2000/svg",a);break;case 2:t=l.createElementNS("http://www.w3.org/1998/Math/MathML",a);break;default:switch(a){case"svg":t=l.createElementNS("http://www.w3.org/2000/svg",a);break;case"math":t=l.createElementNS("http://www.w3.org/1998/Math/MathML",a);break;case"script":t=l.createElement("div"),t.innerHTML="<script><\/script>",t=t.removeChild(t.firstChild);break;case"select":t=typeof n.is=="string"?l.createElement("select",{is:n.is}):l.createElement("select"),n.multiple?t.multiple=!0:n.size&&(t.size=n.size);break;default:t=typeof n.is=="string"?l.createElement(a,{is:n.is}):l.createElement(a)}}t[pe]=e,t[xe]=n;t:for(l=e.child;l!==null;){if(l.tag===5||l.tag===6)t.appendChild(l.stateNode);else if(l.tag!==4&&l.tag!==27&&l.child!==null){l.child.return=l,l=l.child;continue}if(l===e)break t;for(;l.sibling===null;){if(l.return===null||l.return===e)break t;l=l.return}l.sibling.return=l.return,l=l.sibling}e.stateNode=t;t:switch(me(t,a,n),a){case"button":case"input":case"select":case"textarea":t=!!n.autoFocus;break t;case"img":t=!0;break t;default:t=!1}t&&ha(e)}}return $t(e),e.flags&=-16777217,null;case 6:if(t&&e.stateNode!=null)t.memoizedProps!==n&&ha(e);else{if(typeof n!="string"&&e.stateNode===null)throw Error(d(166));if(t=_t.current,hl(e)){if(t=e.stateNode,a=e.memoizedProps,n=null,l=be,l!==null)switch(l.tag){case 27:case 5:n=l.memoizedProps}t[pe]=e,t=!!(t.nodeValue===a||n!==null&&n.suppressHydrationWarning===!0||pd(t.nodeValue,a)),t||an(e)}else t=ar(t).createTextNode(n),t[pe]=e,e.stateNode=t}return $t(e),null;case 13:if(n=e.memoizedState,t===null||t.memoizedState!==null&&t.memoizedState.dehydrated!==null){if(l=hl(e),n!==null&&n.dehydrated!==null){if(t===null){if(!l)throw Error(d(318));if(l=e.memoizedState,l=l!==null?l.dehydrated:null,!l)throw Error(d(317));l[pe]=e}else ml(),(e.flags&128)===0&&(e.memoizedState=null),e.flags|=4;$t(e),l=!1}else l=Po(),t!==null&&t.memoizedState!==null&&(t.memoizedState.hydrationErrors=l),l=!0;if(!l)return e.flags&256?(fa(e),e):(fa(e),null)}if(fa(e),(e.flags&128)!==0)return e.lanes=a,e;if(a=n!==null,t=t!==null&&t.memoizedState!==null,a){n=e.child,l=null,n.alternate!==null&&n.alternate.memoizedState!==null&&n.alternate.memoizedState.cachePool!==null&&(l=n.alternate.memoizedState.cachePool.pool);var i=null;n.memoizedState!==null&&n.memoizedState.cachePool!==null&&(i=n.memoizedState.cachePool.pool),i!==l&&(n.flags|=2048)}return a!==t&&a&&(e.child.flags|=8192),Gi(e,e.updateQueue),$t(e),null;case 4:return Ct(),t===null&&ps(e.stateNode.containerInfo),$t(e),null;case 10:return oa(e.type),$t(e),null;case 19:if(B(ue),l=e.memoizedState,l===null)return $t(e),null;if(n=(e.flags&128)!==0,i=l.rendering,i===null)if(n)kl(l,!1);else{if(ee!==0||t!==null&&(t.flags&128)!==0)for(t=e.child;t!==null;){if(i=Hi(t),i!==null){for(e.flags|=128,kl(l,!1),t=i.updateQueue,e.updateQueue=t,Gi(e,t),e.subtreeFlags=0,t=a,a=e.child;a!==null;)Fo(a,t),a=a.sibling;return nt(ue,ue.current&1|2),e.child}t=t.sibling}l.tail!==null&&k()>Vi&&(e.flags|=128,n=!0,kl(l,!1),e.lanes=4194304)}else{if(!n)if(t=Hi(i),t!==null){if(e.flags|=128,n=!0,t=t.updateQueue,e.updateQueue=t,Gi(e,t),kl(l,!0),l.tail===null&&l.tailMode==="hidden"&&!i.alternate&&!Ht)return $t(e),null}else 2*k()-l.renderingStartTime>Vi&&a!==536870912&&(e.flags|=128,n=!0,kl(l,!1),e.lanes=4194304);l.isBackwards?(i.sibling=e.child,e.child=i):(t=l.last,t!==null?t.sibling=i:e.child=i,l.last=i)}return l.tail!==null?(e=l.tail,l.rendering=e,l.tail=e.sibling,l.renderingStartTime=k(),e.sibling=null,t=ue.current,nt(ue,n?t&1|2:t&1),e):($t(e),null);case 22:case 23:return fa(e),yu(),n=e.memoizedState!==null,t!==null?t.memoizedState!==null!==n&&(e.flags|=8192):n&&(e.flags|=8192),n?(a&536870912)!==0&&(e.flags&128)===0&&($t(e),e.subtreeFlags&6&&(e.flags|=8192)):$t(e),a=e.updateQueue,a!==null&&Gi(e,a.retryQueue),a=null,t!==null&&t.memoizedState!==null&&t.memoizedState.cachePool!==null&&(a=t.memoizedState.cachePool.pool),n=null,e.memoizedState!==null&&e.memoizedState.cachePool!==null&&(n=e.memoizedState.cachePool.pool),n!==a&&(e.flags|=2048),t!==null&&B(rn),null;case 24:return a=null,t!==null&&(a=t.memoizedState.cache),e.memoizedState.cache!==a&&(e.flags|=2048),oa(re),$t(e),null;case 25:return null;case 30:return null}throw Error(d(156,e.tag))}function Bm(t,e){switch(nu(e),e.tag){case 1:return t=e.flags,t&65536?(e.flags=t&-65537|128,e):null;case 3:return oa(re),Ct(),t=e.flags,(t&65536)!==0&&(t&128)===0?(e.flags=t&-65537|128,e):null;case 26:case 27:case 5:return Vt(e),null;case 13:if(fa(e),t=e.memoizedState,t!==null&&t.dehydrated!==null){if(e.alternate===null)throw Error(d(340));ml()}return t=e.flags,t&65536?(e.flags=t&-65537|128,e):null;case 19:return B(ue),null;case 4:return Ct(),null;case 10:return oa(e.type),null;case 22:case 23:return fa(e),yu(),t!==null&&B(rn),t=e.flags,t&65536?(e.flags=t&-65537|128,e):null;case 24:return oa(re),null;case 25:return null;default:return null}}function Ef(t,e){switch(nu(e),e.tag){case 3:oa(re),Ct();break;case 26:case 27:case 5:Vt(e);break;case 4:Ct();break;case 13:fa(e);break;case 19:B(ue);break;case 10:oa(e.type);break;case 22:case 23:fa(e),yu(),t!==null&&B(rn);break;case 24:oa(re)}}function Cl(t,e){try{var a=e.updateQueue,n=a!==null?a.lastEffect:null;if(n!==null){var l=n.next;a=l;do{if((a.tag&t)===t){n=void 0;var i=a.create,u=a.inst;n=i(),u.destroy=n}a=a.next}while(a!==l)}}catch(x){Qt(e,e.return,x)}}function Ca(t,e,a){try{var n=e.updateQueue,l=n!==null?n.lastEffect:null;if(l!==null){var i=l.next;n=i;do{if((n.tag&t)===t){var u=n.inst,x=u.destroy;if(x!==void 0){u.destroy=void 0,l=e;var U=a,V=x;try{V()}catch(lt){Qt(l,U,lt)}}}n=n.next}while(n!==i)}}catch(lt){Qt(e,e.return,lt)}}function wf(t){var e=t.updateQueue;if(e!==null){var a=t.stateNode;try{cc(e,a)}catch(n){Qt(t,t.return,n)}}}function Af(t,e,a){a.props=sn(t.type,t.memoizedProps),a.state=t.memoizedState;try{a.componentWillUnmount()}catch(n){Qt(t,e,n)}}function Dl(t,e){try{var a=t.ref;if(a!==null){switch(t.tag){case 26:case 27:case 5:var n=t.stateNode;break;case 30:n=t.stateNode;break;default:n=t.stateNode}typeof a=="function"?t.refCleanup=a(n):a.current=n}}catch(l){Qt(t,e,l)}}function $e(t,e){var a=t.ref,n=t.refCleanup;if(a!==null)if(typeof n=="function")try{n()}catch(l){Qt(t,e,l)}finally{t.refCleanup=null,t=t.alternate,t!=null&&(t.refCleanup=null)}else if(typeof a=="function")try{a(null)}catch(l){Qt(t,e,l)}else a.current=null}function zf(t){var e=t.type,a=t.memoizedProps,n=t.stateNode;try{t:switch(e){case"button":case"input":case"select":case"textarea":a.autoFocus&&n.focus();break t;case"img":a.src?n.src=a.src:a.srcSet&&(n.srcset=a.srcSet)}}catch(l){Qt(t,t.return,l)}}function Qu(t,e,a){try{var n=t.stateNode;l0(n,t.type,a,e),n[xe]=e}catch(l){Qt(t,t.return,l)}}function Tf(t){return t.tag===5||t.tag===3||t.tag===26||t.tag===27&&La(t.type)||t.tag===4}function Vu(t){t:for(;;){for(;t.sibling===null;){if(t.return===null||Tf(t.return))return null;t=t.return}for(t.sibling.return=t.return,t=t.sibling;t.tag!==5&&t.tag!==6&&t.tag!==18;){if(t.tag===27&&La(t.type)||t.flags&2||t.child===null||t.tag===4)continue t;t.child.return=t,t=t.child}if(!(t.flags&2))return t.stateNode}}function Ku(t,e,a){var n=t.tag;if(n===5||n===6)t=t.stateNode,e?(a.nodeType===9?a.body:a.nodeName==="HTML"?a.ownerDocument.body:a).insertBefore(t,e):(e=a.nodeType===9?a.body:a.nodeName==="HTML"?a.ownerDocument.body:a,e.appendChild(t),a=a._reactRootContainer,a!=null||e.onclick!==null||(e.onclick=er));else if(n!==4&&(n===27&&La(t.type)&&(a=t.stateNode,e=null),t=t.child,t!==null))for(Ku(t,e,a),t=t.sibling;t!==null;)Ku(t,e,a),t=t.sibling}function Xi(t,e,a){var n=t.tag;if(n===5||n===6)t=t.stateNode,e?a.insertBefore(t,e):a.appendChild(t);else if(n!==4&&(n===27&&La(t.type)&&(a=t.stateNode),t=t.child,t!==null))for(Xi(t,e,a),t=t.sibling;t!==null;)Xi(t,e,a),t=t.sibling}function Rf(t){var e=t.stateNode,a=t.memoizedProps;try{for(var n=t.type,l=e.attributes;l.length;)e.removeAttributeNode(l[0]);me(e,n,a),e[pe]=t,e[xe]=a}catch(i){Qt(t,t.return,i)}}var ma=!1,ne=!1,Ju=!1,Of=typeof WeakSet=="function"?WeakSet:Set,ce=null;function Hm(t,e){if(t=t.containerInfo,vs=sr,t=Lo(t),Vr(t)){if("selectionStart"in t)var a={start:t.selectionStart,end:t.selectionEnd};else t:{a=(a=t.ownerDocument)&&a.defaultView||window;var n=a.getSelection&&a.getSelection();if(n&&n.rangeCount!==0){a=n.anchorNode;var l=n.anchorOffset,i=n.focusNode;n=n.focusOffset;try{a.nodeType,i.nodeType}catch{a=null;break t}var u=0,x=-1,U=-1,V=0,lt=0,ct=t,J=null;e:for(;;){for(var W;ct!==a||l!==0&&ct.nodeType!==3||(x=u+l),ct!==i||n!==0&&ct.nodeType!==3||(U=u+n),ct.nodeType===3&&(u+=ct.nodeValue.length),(W=ct.firstChild)!==null;)J=ct,ct=W;for(;;){if(ct===t)break e;if(J===a&&++V===l&&(x=u),J===i&&++lt===n&&(U=u),(W=ct.nextSibling)!==null)break;ct=J,J=ct.parentNode}ct=W}a=x===-1||U===-1?null:{start:x,end:U}}else a=null}a=a||{start:0,end:0}}else a=null;for(bs={focusedElem:t,selectionRange:a},sr=!1,ce=e;ce!==null;)if(e=ce,t=e.child,(e.subtreeFlags&1024)!==0&&t!==null)t.return=e,ce=t;else for(;ce!==null;){switch(e=ce,i=e.alternate,t=e.flags,e.tag){case 0:break;case 11:case 15:break;case 1:if((t&1024)!==0&&i!==null){t=void 0,a=e,l=i.memoizedProps,i=i.memoizedState,n=a.stateNode;try{var Tt=sn(a.type,l,a.elementType===a.type);t=n.getSnapshotBeforeUpdate(Tt,i),n.__reactInternalSnapshotBeforeUpdate=t}catch(At){Qt(a,a.return,At)}}break;case 3:if((t&1024)!==0){if(t=e.stateNode.containerInfo,a=t.nodeType,a===9)xs(t);else if(a===1)switch(t.nodeName){case"HEAD":case"HTML":case"BODY":xs(t);break;default:t.textContent=""}}break;case 5:case 26:case 27:case 6:case 4:case 17:break;default:if((t&1024)!==0)throw Error(d(163))}if(t=e.sibling,t!==null){t.return=e.return,ce=t;break}ce=e.return}}function kf(t,e,a){var n=a.flags;switch(a.tag){case 0:case 11:case 15:Da(t,a),n&4&&Cl(5,a);break;case 1:if(Da(t,a),n&4)if(t=a.stateNode,e===null)try{t.componentDidMount()}catch(u){Qt(a,a.return,u)}else{var l=sn(a.type,e.memoizedProps);e=e.memoizedState;try{t.componentDidUpdate(l,e,t.__reactInternalSnapshotBeforeUpdate)}catch(u){Qt(a,a.return,u)}}n&64&&wf(a),n&512&&Dl(a,a.return);break;case 3:if(Da(t,a),n&64&&(t=a.updateQueue,t!==null)){if(e=null,a.child!==null)switch(a.child.tag){case 27:case 5:e=a.child.stateNode;break;case 1:e=a.child.stateNode}try{cc(t,e)}catch(u){Qt(a,a.return,u)}}break;case 27:e===null&&n&4&&Rf(a);case 26:case 5:Da(t,a),e===null&&n&4&&zf(a),n&512&&Dl(a,a.return);break;case 12:Da(t,a);break;case 13:Da(t,a),n&4&&Nf(t,a),n&64&&(t=a.memoizedState,t!==null&&(t=t.dehydrated,t!==null&&(a=Km.bind(null,a),f0(t,a))));break;case 22:if(n=a.memoizedState!==null||ma,!n){e=e!==null&&e.memoizedState!==null||ne,l=ma;var i=ne;ma=n,(ne=e)&&!i?Na(t,a,(a.subtreeFlags&8772)!==0):Da(t,a),ma=l,ne=i}break;case 30:break;default:Da(t,a)}}function Cf(t){var e=t.alternate;e!==null&&(t.alternate=null,Cf(e)),t.child=null,t.deletions=null,t.sibling=null,t.tag===5&&(e=t.stateNode,e!==null&&Tr(e)),t.stateNode=null,t.return=null,t.dependencies=null,t.memoizedProps=null,t.memoizedState=null,t.pendingProps=null,t.stateNode=null,t.updateQueue=null}var Wt=null,Ae=!1;function pa(t,e,a){for(a=a.child;a!==null;)Df(t,e,a),a=a.sibling}function Df(t,e,a){if(kt&&typeof kt.onCommitFiberUnmount=="function")try{kt.onCommitFiberUnmount(Et,a)}catch{}switch(a.tag){case 26:ne||$e(a,e),pa(t,e,a),a.memoizedState?a.memoizedState.count--:a.stateNode&&(a=a.stateNode,a.parentNode.removeChild(a));break;case 27:ne||$e(a,e);var n=Wt,l=Ae;La(a.type)&&(Wt=a.stateNode,Ae=!1),pa(t,e,a),Yl(a.stateNode),Wt=n,Ae=l;break;case 5:ne||$e(a,e);case 6:if(n=Wt,l=Ae,Wt=null,pa(t,e,a),Wt=n,Ae=l,Wt!==null)if(Ae)try{(Wt.nodeType===9?Wt.body:Wt.nodeName==="HTML"?Wt.ownerDocument.body:Wt).removeChild(a.stateNode)}catch(i){Qt(a,e,i)}else try{Wt.removeChild(a.stateNode)}catch(i){Qt(a,e,i)}break;case 18:Wt!==null&&(Ae?(t=Wt,_d(t.nodeType===9?t.body:t.nodeName==="HTML"?t.ownerDocument.body:t,a.stateNode),Fl(t)):_d(Wt,a.stateNode));break;case 4:n=Wt,l=Ae,Wt=a.stateNode.containerInfo,Ae=!0,pa(t,e,a),Wt=n,Ae=l;break;case 0:case 11:case 14:case 15:ne||Ca(2,a,e),ne||Ca(4,a,e),pa(t,e,a);break;case 1:ne||($e(a,e),n=a.stateNode,typeof n.componentWillUnmount=="function"&&Af(a,e,n)),pa(t,e,a);break;case 21:pa(t,e,a);break;case 22:ne=(n=ne)||a.memoizedState!==null,pa(t,e,a),ne=n;break;default:pa(t,e,a)}}function Nf(t,e){if(e.memoizedState===null&&(t=e.alternate,t!==null&&(t=t.memoizedState,t!==null&&(t=t.dehydrated,t!==null))))try{Fl(t)}catch(a){Qt(e,e.return,a)}}function Lm(t){switch(t.tag){case 13:case 19:var e=t.stateNode;return e===null&&(e=t.stateNode=new Of),e;case 22:return t=t.stateNode,e=t._retryCache,e===null&&(e=t._retryCache=new Of),e;default:throw Error(d(435,t.tag))}}function Fu(t,e){var a=Lm(t);e.forEach(function(n){var l=Jm.bind(null,t,n);a.has(n)||(a.add(n),n.then(l,l))})}function Oe(t,e){var a=e.deletions;if(a!==null)for(var n=0;n<a.length;n++){var l=a[n],i=t,u=e,x=u;t:for(;x!==null;){switch(x.tag){case 27:if(La(x.type)){Wt=x.stateNode,Ae=!1;break t}break;case 5:Wt=x.stateNode,Ae=!1;break t;case 3:case 4:Wt=x.stateNode.containerInfo,Ae=!0;break t}x=x.return}if(Wt===null)throw Error(d(160));Df(i,u,l),Wt=null,Ae=!1,i=l.alternate,i!==null&&(i.return=null),l.return=null}if(e.subtreeFlags&13878)for(e=e.child;e!==null;)Uf(e,t),e=e.sibling}var Ke=null;function Uf(t,e){var a=t.alternate,n=t.flags;switch(t.tag){case 0:case 11:case 14:case 15:Oe(e,t),ke(t),n&4&&(Ca(3,t,t.return),Cl(3,t),Ca(5,t,t.return));break;case 1:Oe(e,t),ke(t),n&512&&(ne||a===null||$e(a,a.return)),n&64&&ma&&(t=t.updateQueue,t!==null&&(n=t.callbacks,n!==null&&(a=t.shared.hiddenCallbacks,t.shared.hiddenCallbacks=a===null?n:a.concat(n))));break;case 26:var l=Ke;if(Oe(e,t),ke(t),n&512&&(ne||a===null||$e(a,a.return)),n&4){var i=a!==null?a.memoizedState:null;if(n=t.memoizedState,a===null)if(n===null)if(t.stateNode===null){t:{n=t.type,a=t.memoizedProps,l=l.ownerDocument||l;e:switch(n){case"title":i=l.getElementsByTagName("title")[0],(!i||i[al]||i[pe]||i.namespaceURI==="http://www.w3.org/2000/svg"||i.hasAttribute("itemprop"))&&(i=l.createElement(n),l.head.insertBefore(i,l.querySelector("head > title"))),me(i,n,a),i[pe]=t,se(i),n=i;break t;case"link":var u=Rd("link","href",l).get(n+(a.href||""));if(u){for(var x=0;x<u.length;x++)if(i=u[x],i.getAttribute("href")===(a.href==null||a.href===""?null:a.href)&&i.getAttribute("rel")===(a.rel==null?null:a.rel)&&i.getAttribute("title")===(a.title==null?null:a.title)&&i.getAttribute("crossorigin")===(a.crossOrigin==null?null:a.crossOrigin)){u.splice(x,1);break e}}i=l.createElement(n),me(i,n,a),l.head.appendChild(i);break;case"meta":if(u=Rd("meta","content",l).get(n+(a.content||""))){for(x=0;x<u.length;x++)if(i=u[x],i.getAttribute("content")===(a.content==null?null:""+a.content)&&i.getAttribute("name")===(a.name==null?null:a.name)&&i.getAttribute("property")===(a.property==null?null:a.property)&&i.getAttribute("http-equiv")===(a.httpEquiv==null?null:a.httpEquiv)&&i.getAttribute("charset")===(a.charSet==null?null:a.charSet)){u.splice(x,1);break e}}i=l.createElement(n),me(i,n,a),l.head.appendChild(i);break;default:throw Error(d(468,n))}i[pe]=t,se(i),n=i}t.stateNode=n}else Od(l,t.type,t.stateNode);else t.stateNode=Td(l,n,t.memoizedProps);else i!==n?(i===null?a.stateNode!==null&&(a=a.stateNode,a.parentNode.removeChild(a)):i.count--,n===null?Od(l,t.type,t.stateNode):Td(l,n,t.memoizedProps)):n===null&&t.stateNode!==null&&Qu(t,t.memoizedProps,a.memoizedProps)}break;case 27:Oe(e,t),ke(t),n&512&&(ne||a===null||$e(a,a.return)),a!==null&&n&4&&Qu(t,t.memoizedProps,a.memoizedProps);break;case 5:if(Oe(e,t),ke(t),n&512&&(ne||a===null||$e(a,a.return)),t.flags&32){l=t.stateNode;try{xn(l,"")}catch(W){Qt(t,t.return,W)}}n&4&&t.stateNode!=null&&(l=t.memoizedProps,Qu(t,l,a!==null?a.memoizedProps:l)),n&1024&&(Ju=!0);break;case 6:if(Oe(e,t),ke(t),n&4){if(t.stateNode===null)throw Error(d(162));n=t.memoizedProps,a=t.stateNode;try{a.nodeValue=n}catch(W){Qt(t,t.return,W)}}break;case 3:if(ir=null,l=Ke,Ke=nr(e.containerInfo),Oe(e,t),Ke=l,ke(t),n&4&&a!==null&&a.memoizedState.isDehydrated)try{Fl(e.containerInfo)}catch(W){Qt(t,t.return,W)}Ju&&(Ju=!1,Mf(t));break;case 4:n=Ke,Ke=nr(t.stateNode.containerInfo),Oe(e,t),ke(t),Ke=n;break;case 12:Oe(e,t),ke(t);break;case 13:Oe(e,t),ke(t),t.child.flags&8192&&t.memoizedState!==null!=(a!==null&&a.memoizedState!==null)&&(es=k()),n&4&&(n=t.updateQueue,n!==null&&(t.updateQueue=null,Fu(t,n)));break;case 22:l=t.memoizedState!==null;var U=a!==null&&a.memoizedState!==null,V=ma,lt=ne;if(ma=V||l,ne=lt||U,Oe(e,t),ne=lt,ma=V,ke(t),n&8192)t:for(e=t.stateNode,e._visibility=l?e._visibility&-2:e._visibility|1,l&&(a===null||U||ma||ne||on(t)),a=null,e=t;;){if(e.tag===5||e.tag===26){if(a===null){U=a=e;try{if(i=U.stateNode,l)u=i.style,typeof u.setProperty=="function"?u.setProperty("display","none","important"):u.display="none";else{x=U.stateNode;var ct=U.memoizedProps.style,J=ct!=null&&ct.hasOwnProperty("display")?ct.display:null;x.style.display=J==null||typeof J=="boolean"?"":(""+J).trim()}}catch(W){Qt(U,U.return,W)}}}else if(e.tag===6){if(a===null){U=e;try{U.stateNode.nodeValue=l?"":U.memoizedProps}catch(W){Qt(U,U.return,W)}}}else if((e.tag!==22&&e.tag!==23||e.memoizedState===null||e===t)&&e.child!==null){e.child.return=e,e=e.child;continue}if(e===t)break t;for(;e.sibling===null;){if(e.return===null||e.return===t)break t;a===e&&(a=null),e=e.return}a===e&&(a=null),e.sibling.return=e.return,e=e.sibling}n&4&&(n=t.updateQueue,n!==null&&(a=n.retryQueue,a!==null&&(n.retryQueue=null,Fu(t,a))));break;case 19:Oe(e,t),ke(t),n&4&&(n=t.updateQueue,n!==null&&(t.updateQueue=null,Fu(t,n)));break;case 30:break;case 21:break;default:Oe(e,t),ke(t)}}function ke(t){var e=t.flags;if(e&2){try{for(var a,n=t.return;n!==null;){if(Tf(n)){a=n;break}n=n.return}if(a==null)throw Error(d(160));switch(a.tag){case 27:var l=a.stateNode,i=Vu(t);Xi(t,i,l);break;case 5:var u=a.stateNode;a.flags&32&&(xn(u,""),a.flags&=-33);var x=Vu(t);Xi(t,x,u);break;case 3:case 4:var U=a.stateNode.containerInfo,V=Vu(t);Ku(t,V,U);break;default:throw Error(d(161))}}catch(lt){Qt(t,t.return,lt)}t.flags&=-3}e&4096&&(t.flags&=-4097)}function Mf(t){if(t.subtreeFlags&1024)for(t=t.child;t!==null;){var e=t;Mf(e),e.tag===5&&e.flags&1024&&e.stateNode.reset(),t=t.sibling}}function Da(t,e){if(e.subtreeFlags&8772)for(e=e.child;e!==null;)kf(t,e.alternate,e),e=e.sibling}function on(t){for(t=t.child;t!==null;){var e=t;switch(e.tag){case 0:case 11:case 14:case 15:Ca(4,e,e.return),on(e);break;case 1:$e(e,e.return);var a=e.stateNode;typeof a.componentWillUnmount=="function"&&Af(e,e.return,a),on(e);break;case 27:Yl(e.stateNode);case 26:case 5:$e(e,e.return),on(e);break;case 22:e.memoizedState===null&&on(e);break;case 30:on(e);break;default:on(e)}t=t.sibling}}function Na(t,e,a){for(a=a&&(e.subtreeFlags&8772)!==0,e=e.child;e!==null;){var n=e.alternate,l=t,i=e,u=i.flags;switch(i.tag){case 0:case 11:case 15:Na(l,i,a),Cl(4,i);break;case 1:if(Na(l,i,a),n=i,l=n.stateNode,typeof l.componentDidMount=="function")try{l.componentDidMount()}catch(V){Qt(n,n.return,V)}if(n=i,l=n.updateQueue,l!==null){var x=n.stateNode;try{var U=l.shared.hiddenCallbacks;if(U!==null)for(l.shared.hiddenCallbacks=null,l=0;l<U.length;l++)oc(U[l],x)}catch(V){Qt(n,n.return,V)}}a&&u&64&&wf(i),Dl(i,i.return);break;case 27:Rf(i);case 26:case 5:Na(l,i,a),a&&n===null&&u&4&&zf(i),Dl(i,i.return);break;case 12:Na(l,i,a);break;case 13:Na(l,i,a),a&&u&4&&Nf(l,i);break;case 22:i.memoizedState===null&&Na(l,i,a),Dl(i,i.return);break;case 30:break;default:Na(l,i,a)}e=e.sibling}}function Wu(t,e){var a=null;t!==null&&t.memoizedState!==null&&t.memoizedState.cachePool!==null&&(a=t.memoizedState.cachePool.pool),t=null,e.memoizedState!==null&&e.memoizedState.cachePool!==null&&(t=e.memoizedState.cachePool.pool),t!==a&&(t!=null&&t.refCount++,a!=null&&yl(a))}function $u(t,e){t=null,e.alternate!==null&&(t=e.alternate.memoizedState.cache),e=e.memoizedState.cache,e!==t&&(e.refCount++,t!=null&&yl(t))}function Ie(t,e,a,n){if(e.subtreeFlags&10256)for(e=e.child;e!==null;)jf(t,e,a,n),e=e.sibling}function jf(t,e,a,n){var l=e.flags;switch(e.tag){case 0:case 11:case 15:Ie(t,e,a,n),l&2048&&Cl(9,e);break;case 1:Ie(t,e,a,n);break;case 3:Ie(t,e,a,n),l&2048&&(t=null,e.alternate!==null&&(t=e.alternate.memoizedState.cache),e=e.memoizedState.cache,e!==t&&(e.refCount++,t!=null&&yl(t)));break;case 12:if(l&2048){Ie(t,e,a,n),t=e.stateNode;try{var i=e.memoizedProps,u=i.id,x=i.onPostCommit;typeof x=="function"&&x(u,e.alternate===null?"mount":"update",t.passiveEffectDuration,-0)}catch(U){Qt(e,e.return,U)}}else Ie(t,e,a,n);break;case 13:Ie(t,e,a,n);break;case 23:break;case 22:i=e.stateNode,u=e.alternate,e.memoizedState!==null?i._visibility&2?Ie(t,e,a,n):Nl(t,e):i._visibility&2?Ie(t,e,a,n):(i._visibility|=2,Yn(t,e,a,n,(e.subtreeFlags&10256)!==0)),l&2048&&Wu(u,e);break;case 24:Ie(t,e,a,n),l&2048&&$u(e.alternate,e);break;default:Ie(t,e,a,n)}}function Yn(t,e,a,n,l){for(l=l&&(e.subtreeFlags&10256)!==0,e=e.child;e!==null;){var i=t,u=e,x=a,U=n,V=u.flags;switch(u.tag){case 0:case 11:case 15:Yn(i,u,x,U,l),Cl(8,u);break;case 23:break;case 22:var lt=u.stateNode;u.memoizedState!==null?lt._visibility&2?Yn(i,u,x,U,l):Nl(i,u):(lt._visibility|=2,Yn(i,u,x,U,l)),l&&V&2048&&Wu(u.alternate,u);break;case 24:Yn(i,u,x,U,l),l&&V&2048&&$u(u.alternate,u);break;default:Yn(i,u,x,U,l)}e=e.sibling}}function Nl(t,e){if(e.subtreeFlags&10256)for(e=e.child;e!==null;){var a=t,n=e,l=n.flags;switch(n.tag){case 22:Nl(a,n),l&2048&&Wu(n.alternate,n);break;case 24:Nl(a,n),l&2048&&$u(n.alternate,n);break;default:Nl(a,n)}e=e.sibling}}var Ul=8192;function Zn(t){if(t.subtreeFlags&Ul)for(t=t.child;t!==null;)Bf(t),t=t.sibling}function Bf(t){switch(t.tag){case 26:Zn(t),t.flags&Ul&&t.memoizedState!==null&&w0(Ke,t.memoizedState,t.memoizedProps);break;case 5:Zn(t);break;case 3:case 4:var e=Ke;Ke=nr(t.stateNode.containerInfo),Zn(t),Ke=e;break;case 22:t.memoizedState===null&&(e=t.alternate,e!==null&&e.memoizedState!==null?(e=Ul,Ul=16777216,Zn(t),Ul=e):Zn(t));break;default:Zn(t)}}function Hf(t){var e=t.alternate;if(e!==null&&(t=e.child,t!==null)){e.child=null;do e=t.sibling,t.sibling=null,t=e;while(t!==null)}}function Ml(t){var e=t.deletions;if((t.flags&16)!==0){if(e!==null)for(var a=0;a<e.length;a++){var n=e[a];ce=n,qf(n,t)}Hf(t)}if(t.subtreeFlags&10256)for(t=t.child;t!==null;)Lf(t),t=t.sibling}function Lf(t){switch(t.tag){case 0:case 11:case 15:Ml(t),t.flags&2048&&Ca(9,t,t.return);break;case 3:Ml(t);break;case 12:Ml(t);break;case 22:var e=t.stateNode;t.memoizedState!==null&&e._visibility&2&&(t.return===null||t.return.tag!==13)?(e._visibility&=-3,Qi(t)):Ml(t);break;default:Ml(t)}}function Qi(t){var e=t.deletions;if((t.flags&16)!==0){if(e!==null)for(var a=0;a<e.length;a++){var n=e[a];ce=n,qf(n,t)}Hf(t)}for(t=t.child;t!==null;){switch(e=t,e.tag){case 0:case 11:case 15:Ca(8,e,e.return),Qi(e);break;case 22:a=e.stateNode,a._visibility&2&&(a._visibility&=-3,Qi(e));break;default:Qi(e)}t=t.sibling}}function qf(t,e){for(;ce!==null;){var a=ce;switch(a.tag){case 0:case 11:case 15:Ca(8,a,e);break;case 23:case 22:if(a.memoizedState!==null&&a.memoizedState.cachePool!==null){var n=a.memoizedState.cachePool.pool;n!=null&&n.refCount++}break;case 24:yl(a.memoizedState.cache)}if(n=a.child,n!==null)n.return=a,ce=n;else t:for(a=t;ce!==null;){n=ce;var l=n.sibling,i=n.return;if(Cf(n),n===a){ce=null;break t}if(l!==null){l.return=i,ce=l;break t}ce=i}}}var qm={getCacheForType:function(t){var e=ge(re),a=e.data.get(t);return a===void 0&&(a=t(),e.data.set(t,a)),a}},Ym=typeof WeakMap=="function"?WeakMap:Map,qt=0,Ft=null,Ut=null,jt=0,Yt=0,Ce=null,Ua=!1,Gn=!1,Iu=!1,ga=0,ee=0,Ma=0,cn=0,Pu=0,Ze=0,Xn=0,jl=null,ze=null,ts=!1,es=0,Vi=1/0,Ki=null,ja=null,he=0,Ba=null,Qn=null,Vn=0,as=0,ns=null,Yf=null,Bl=0,ls=null;function De(){if((qt&2)!==0&&jt!==0)return jt&-jt;if(G.T!==null){var t=Nn;return t!==0?t:fs()}return eo()}function Zf(){Ze===0&&(Ze=(jt&536870912)===0||Ht?ni():536870912);var t=Ye.current;return t!==null&&(t.flags|=32),Ze}function Ne(t,e,a){(t===Ft&&(Yt===2||Yt===9)||t.cancelPendingCommit!==null)&&(Kn(t,0),Ha(t,jt,Ze,!1)),Qe(t,a),((qt&2)===0||t!==Ft)&&(t===Ft&&((qt&2)===0&&(cn|=a),ee===4&&Ha(t,jt,Ze,!1)),Pe(t))}function Gf(t,e,a){if((qt&6)!==0)throw Error(d(327));var n=!a&&(e&124)===0&&(e&t.expiredLanes)===0||Sa(t,e),l=n?Xm(t,e):us(t,e,!0),i=n;do{if(l===0){Gn&&!n&&Ha(t,e,0,!1);break}else{if(a=t.current.alternate,i&&!Zm(a)){l=us(t,e,!1),i=!1;continue}if(l===2){if(i=e,t.errorRecoveryDisabledLanes&i)var u=0;else u=t.pendingLanes&-536870913,u=u!==0?u:u&536870912?536870912:0;if(u!==0){e=u;t:{var x=t;l=jl;var U=x.current.memoizedState.isDehydrated;if(U&&(Kn(x,u).flags|=256),u=us(x,u,!1),u!==2){if(Iu&&!U){x.errorRecoveryDisabledLanes|=i,cn|=i,l=4;break t}i=ze,ze=l,i!==null&&(ze===null?ze=i:ze.push.apply(ze,i))}l=u}if(i=!1,l!==2)continue}}if(l===1){Kn(t,0),Ha(t,e,0,!0);break}t:{switch(n=t,i=l,i){case 0:case 1:throw Error(d(345));case 4:if((e&4194048)!==e)break;case 6:Ha(n,e,Ze,!Ua);break t;case 2:ze=null;break;case 3:case 5:break;default:throw Error(d(329))}if((e&62914560)===e&&(l=es+300-k(),10<l)){if(Ha(n,e,Ze,!Ua),Se(n,0,!0)!==0)break t;n.timeoutHandle=vd(Xf.bind(null,n,a,ze,Ki,ts,e,Ze,cn,Xn,Ua,i,2,-0,0),l);break t}Xf(n,a,ze,Ki,ts,e,Ze,cn,Xn,Ua,i,0,-0,0)}}break}while(!0);Pe(t)}function Xf(t,e,a,n,l,i,u,x,U,V,lt,ct,J,W){if(t.timeoutHandle=-1,ct=e.subtreeFlags,(ct&8192||(ct&16785408)===16785408)&&(Xl={stylesheets:null,count:0,unsuspend:E0},Bf(e),ct=A0(),ct!==null)){t.cancelPendingCommit=ct($f.bind(null,t,e,i,a,n,l,u,x,U,lt,1,J,W)),Ha(t,i,u,!V);return}$f(t,e,i,a,n,l,u,x,U)}function Zm(t){for(var e=t;;){var a=e.tag;if((a===0||a===11||a===15)&&e.flags&16384&&(a=e.updateQueue,a!==null&&(a=a.stores,a!==null)))for(var n=0;n<a.length;n++){var l=a[n],i=l.getSnapshot;l=l.value;try{if(!Te(i(),l))return!1}catch{return!1}}if(a=e.child,e.subtreeFlags&16384&&a!==null)a.return=e,e=a;else{if(e===t)break;for(;e.sibling===null;){if(e.return===null||e.return===t)return!0;e=e.return}e.sibling.return=e.return,e=e.sibling}}return!0}function Ha(t,e,a,n){e&=~Pu,e&=~cn,t.suspendedLanes|=e,t.pingedLanes&=~e,n&&(t.warmLanes|=e),n=t.expirationTimes;for(var l=e;0<l;){var i=31-Kt(l),u=1<<i;n[i]=-1,l&=~u}a!==0&&Ps(t,a,e)}function Ji(){return(qt&6)===0?(Hl(0),!1):!0}function is(){if(Ut!==null){if(Yt===0)var t=Ut.return;else t=Ut,sa=nn=null,xu(t),Ln=null,Rl=0,t=Ut;for(;t!==null;)Ef(t.alternate,t),t=t.return;Ut=null}}function Kn(t,e){var a=t.timeoutHandle;a!==-1&&(t.timeoutHandle=-1,r0(a)),a=t.cancelPendingCommit,a!==null&&(t.cancelPendingCommit=null,a()),is(),Ft=t,Ut=a=ia(t.current,null),jt=e,Yt=0,Ce=null,Ua=!1,Gn=Sa(t,e),Iu=!1,Xn=Ze=Pu=cn=Ma=ee=0,ze=jl=null,ts=!1,(e&8)!==0&&(e|=e&32);var n=t.entangledLanes;if(n!==0)for(t=t.entanglements,n&=e;0<n;){var l=31-Kt(n),i=1<<l;e|=t[l],n&=~i}return ga=e,gi(),a}function Qf(t,e){Dt=null,G.H=Mi,e===bl||e===Ai?(e=uc(),Yt=3):e===lc?(e=uc(),Yt=4):Yt=e===of?8:e!==null&&typeof e=="object"&&typeof e.then=="function"?6:1,Ce=e,Ut===null&&(ee=1,qi(t,Be(e,t.current)))}function Vf(){var t=G.H;return G.H=Mi,t===null?Mi:t}function Kf(){var t=G.A;return G.A=qm,t}function rs(){ee=4,Ua||(jt&4194048)!==jt&&Ye.current!==null||(Gn=!0),(Ma&134217727)===0&&(cn&134217727)===0||Ft===null||Ha(Ft,jt,Ze,!1)}function us(t,e,a){var n=qt;qt|=2;var l=Vf(),i=Kf();(Ft!==t||jt!==e)&&(Ki=null,Kn(t,e)),e=!1;var u=ee;t:do try{if(Yt!==0&&Ut!==null){var x=Ut,U=Ce;switch(Yt){case 8:is(),u=6;break t;case 3:case 2:case 9:case 6:Ye.current===null&&(e=!0);var V=Yt;if(Yt=0,Ce=null,Jn(t,x,U,V),a&&Gn){u=0;break t}break;default:V=Yt,Yt=0,Ce=null,Jn(t,x,U,V)}}Gm(),u=ee;break}catch(lt){Qf(t,lt)}while(!0);return e&&t.shellSuspendCounter++,sa=nn=null,qt=n,G.H=l,G.A=i,Ut===null&&(Ft=null,jt=0,gi()),u}function Gm(){for(;Ut!==null;)Jf(Ut)}function Xm(t,e){var a=qt;qt|=2;var n=Vf(),l=Kf();Ft!==t||jt!==e?(Ki=null,Vi=k()+500,Kn(t,e)):Gn=Sa(t,e);t:do try{if(Yt!==0&&Ut!==null){e=Ut;var i=Ce;e:switch(Yt){case 1:Yt=0,Ce=null,Jn(t,e,i,1);break;case 2:case 9:if(ic(i)){Yt=0,Ce=null,Ff(e);break}e=function(){Yt!==2&&Yt!==9||Ft!==t||(Yt=7),Pe(t)},i.then(e,e);break t;case 3:Yt=7;break t;case 4:Yt=5;break t;case 7:ic(i)?(Yt=0,Ce=null,Ff(e)):(Yt=0,Ce=null,Jn(t,e,i,7));break;case 5:var u=null;switch(Ut.tag){case 26:u=Ut.memoizedState;case 5:case 27:var x=Ut;if(!u||kd(u)){Yt=0,Ce=null;var U=x.sibling;if(U!==null)Ut=U;else{var V=x.return;V!==null?(Ut=V,Fi(V)):Ut=null}break e}}Yt=0,Ce=null,Jn(t,e,i,5);break;case 6:Yt=0,Ce=null,Jn(t,e,i,6);break;case 8:is(),ee=6;break t;default:throw Error(d(462))}}Qm();break}catch(lt){Qf(t,lt)}while(!0);return sa=nn=null,G.H=n,G.A=l,qt=a,Ut!==null?0:(Ft=null,jt=0,gi(),ee)}function Qm(){for(;Ut!==null&&!ot();)Jf(Ut)}function Jf(t){var e=Sf(t.alternate,t,ga);t.memoizedProps=t.pendingProps,e===null?Fi(t):Ut=e}function Ff(t){var e=t,a=e.alternate;switch(e.tag){case 15:case 0:e=pf(a,e,e.pendingProps,e.type,void 0,jt);break;case 11:e=pf(a,e,e.pendingProps,e.type.render,e.ref,jt);break;case 5:xu(e);default:Ef(a,e),e=Ut=Fo(e,ga),e=Sf(a,e,ga)}t.memoizedProps=t.pendingProps,e===null?Fi(t):Ut=e}function Jn(t,e,a,n){sa=nn=null,xu(e),Ln=null,Rl=0;var l=e.return;try{if(Um(t,l,e,a,jt)){ee=1,qi(t,Be(a,t.current)),Ut=null;return}}catch(i){if(l!==null)throw Ut=l,i;ee=1,qi(t,Be(a,t.current)),Ut=null;return}e.flags&32768?(Ht||n===1?t=!0:Gn||(jt&536870912)!==0?t=!1:(Ua=t=!0,(n===2||n===9||n===3||n===6)&&(n=Ye.current,n!==null&&n.tag===13&&(n.flags|=16384))),Wf(e,t)):Fi(e)}function Fi(t){var e=t;do{if((e.flags&32768)!==0){Wf(e,Ua);return}t=e.return;var a=jm(e.alternate,e,ga);if(a!==null){Ut=a;return}if(e=e.sibling,e!==null){Ut=e;return}Ut=e=t}while(e!==null);ee===0&&(ee=5)}function Wf(t,e){do{var a=Bm(t.alternate,t);if(a!==null){a.flags&=32767,Ut=a;return}if(a=t.return,a!==null&&(a.flags|=32768,a.subtreeFlags=0,a.deletions=null),!e&&(t=t.sibling,t!==null)){Ut=t;return}Ut=t=a}while(t!==null);ee=6,Ut=null}function $f(t,e,a,n,l,i,u,x,U){t.cancelPendingCommit=null;do Wi();while(he!==0);if((qt&6)!==0)throw Error(d(327));if(e!==null){if(e===t.current)throw Error(d(177));if(i=e.lanes|e.childLanes,i|=$r,mn(t,a,i,u,x,U),t===Ft&&(Ut=Ft=null,jt=0),Qn=e,Ba=t,Vn=a,as=i,ns=l,Yf=n,(e.subtreeFlags&10256)!==0||(e.flags&10256)!==0?(t.callbackNode=null,t.callbackPriority=0,Fm(mt,function(){return ad(),null})):(t.callbackNode=null,t.callbackPriority=0),n=(e.flags&13878)!==0,(e.subtreeFlags&13878)!==0||n){n=G.T,G.T=null,l=X.p,X.p=2,u=qt,qt|=4;try{Hm(t,e,a)}finally{qt=u,X.p=l,G.T=n}}he=1,If(),Pf(),td()}}function If(){if(he===1){he=0;var t=Ba,e=Qn,a=(e.flags&13878)!==0;if((e.subtreeFlags&13878)!==0||a){a=G.T,G.T=null;var n=X.p;X.p=2;var l=qt;qt|=4;try{Uf(e,t);var i=bs,u=Lo(t.containerInfo),x=i.focusedElem,U=i.selectionRange;if(u!==x&&x&&x.ownerDocument&&Ho(x.ownerDocument.documentElement,x)){if(U!==null&&Vr(x)){var V=U.start,lt=U.end;if(lt===void 0&&(lt=V),"selectionStart"in x)x.selectionStart=V,x.selectionEnd=Math.min(lt,x.value.length);else{var ct=x.ownerDocument||document,J=ct&&ct.defaultView||window;if(J.getSelection){var W=J.getSelection(),Tt=x.textContent.length,At=Math.min(U.start,Tt),Xt=U.end===void 0?At:Math.min(U.end,Tt);!W.extend&&At>Xt&&(u=Xt,Xt=At,At=u);var Y=Bo(x,At),H=Bo(x,Xt);if(Y&&H&&(W.rangeCount!==1||W.anchorNode!==Y.node||W.anchorOffset!==Y.offset||W.focusNode!==H.node||W.focusOffset!==H.offset)){var Q=ct.createRange();Q.setStart(Y.node,Y.offset),W.removeAllRanges(),At>Xt?(W.addRange(Q),W.extend(H.node,H.offset)):(Q.setEnd(H.node,H.offset),W.addRange(Q))}}}}for(ct=[],W=x;W=W.parentNode;)W.nodeType===1&&ct.push({element:W,left:W.scrollLeft,top:W.scrollTop});for(typeof x.focus=="function"&&x.focus(),x=0;x<ct.length;x++){var ut=ct[x];ut.element.scrollLeft=ut.left,ut.element.scrollTop=ut.top}}sr=!!vs,bs=vs=null}finally{qt=l,X.p=n,G.T=a}}t.current=e,he=2}}function Pf(){if(he===2){he=0;var t=Ba,e=Qn,a=(e.flags&8772)!==0;if((e.subtreeFlags&8772)!==0||a){a=G.T,G.T=null;var n=X.p;X.p=2;var l=qt;qt|=4;try{kf(t,e.alternate,e)}finally{qt=l,X.p=n,G.T=a}}he=3}}function td(){if(he===4||he===3){he=0,P();var t=Ba,e=Qn,a=Vn,n=Yf;(e.subtreeFlags&10256)!==0||(e.flags&10256)!==0?he=5:(he=0,Qn=Ba=null,ed(t,t.pendingLanes));var l=t.pendingLanes;if(l===0&&(ja=null),Ar(a),e=e.stateNode,kt&&typeof kt.onCommitFiberRoot=="function")try{kt.onCommitFiberRoot(Et,e,void 0,(e.current.flags&128)===128)}catch{}if(n!==null){e=G.T,l=X.p,X.p=2,G.T=null;try{for(var i=t.onRecoverableError,u=0;u<n.length;u++){var x=n[u];i(x.value,{componentStack:x.stack})}}finally{G.T=e,X.p=l}}(Vn&3)!==0&&Wi(),Pe(t),l=t.pendingLanes,(a&4194090)!==0&&(l&42)!==0?t===ls?Bl++:(Bl=0,ls=t):Bl=0,Hl(0)}}function ed(t,e){(t.pooledCacheLanes&=e)===0&&(e=t.pooledCache,e!=null&&(t.pooledCache=null,yl(e)))}function Wi(t){return If(),Pf(),td(),ad()}function ad(){if(he!==5)return!1;var t=Ba,e=as;as=0;var a=Ar(Vn),n=G.T,l=X.p;try{X.p=32>a?32:a,G.T=null,a=ns,ns=null;var i=Ba,u=Vn;if(he=0,Qn=Ba=null,Vn=0,(qt&6)!==0)throw Error(d(331));var x=qt;if(qt|=4,Lf(i.current),jf(i,i.current,u,a),qt=x,Hl(0,!1),kt&&typeof kt.onPostCommitFiberRoot=="function")try{kt.onPostCommitFiberRoot(Et,i)}catch{}return!0}finally{X.p=l,G.T=n,ed(t,e)}}function nd(t,e,a){e=Be(a,e),e=ju(t.stateNode,e,2),t=Ta(t,e,2),t!==null&&(Qe(t,2),Pe(t))}function Qt(t,e,a){if(t.tag===3)nd(t,t,a);else for(;e!==null;){if(e.tag===3){nd(e,t,a);break}else if(e.tag===1){var n=e.stateNode;if(typeof e.type.getDerivedStateFromError=="function"||typeof n.componentDidCatch=="function"&&(ja===null||!ja.has(n))){t=Be(a,t),a=uf(2),n=Ta(e,a,2),n!==null&&(sf(a,n,e,t),Qe(n,2),Pe(n));break}}e=e.return}}function ss(t,e,a){var n=t.pingCache;if(n===null){n=t.pingCache=new Ym;var l=new Set;n.set(e,l)}else l=n.get(e),l===void 0&&(l=new Set,n.set(e,l));l.has(a)||(Iu=!0,l.add(a),t=Vm.bind(null,t,e,a),e.then(t,t))}function Vm(t,e,a){var n=t.pingCache;n!==null&&n.delete(e),t.pingedLanes|=t.suspendedLanes&a,t.warmLanes&=~a,Ft===t&&(jt&a)===a&&(ee===4||ee===3&&(jt&62914560)===jt&&300>k()-es?(qt&2)===0&&Kn(t,0):Pu|=a,Xn===jt&&(Xn=0)),Pe(t)}function ld(t,e){e===0&&(e=li()),t=On(t,e),t!==null&&(Qe(t,e),Pe(t))}function Km(t){var e=t.memoizedState,a=0;e!==null&&(a=e.retryLane),ld(t,a)}function Jm(t,e){var a=0;switch(t.tag){case 13:var n=t.stateNode,l=t.memoizedState;l!==null&&(a=l.retryLane);break;case 19:n=t.stateNode;break;case 22:n=t.stateNode._retryCache;break;default:throw Error(d(314))}n!==null&&n.delete(e),ld(t,a)}function Fm(t,e){return ve(t,e)}var $i=null,Fn=null,os=!1,Ii=!1,cs=!1,fn=0;function Pe(t){t!==Fn&&t.next===null&&(Fn===null?$i=Fn=t:Fn=Fn.next=t),Ii=!0,os||(os=!0,$m())}function Hl(t,e){if(!cs&&Ii){cs=!0;do for(var a=!1,n=$i;n!==null;){if(t!==0){var l=n.pendingLanes;if(l===0)var i=0;else{var u=n.suspendedLanes,x=n.pingedLanes;i=(1<<31-Kt(42|t)+1)-1,i&=l&~(u&~x),i=i&201326741?i&201326741|1:i?i|2:0}i!==0&&(a=!0,sd(n,i))}else i=jt,i=Se(n,n===Ft?i:0,n.cancelPendingCommit!==null||n.timeoutHandle!==-1),(i&3)===0||Sa(n,i)||(a=!0,sd(n,i));n=n.next}while(a);cs=!1}}function Wm(){id()}function id(){Ii=os=!1;var t=0;fn!==0&&(i0()&&(t=fn),fn=0);for(var e=k(),a=null,n=$i;n!==null;){var l=n.next,i=rd(n,e);i===0?(n.next=null,a===null?$i=l:a.next=l,l===null&&(Fn=a)):(a=n,(t!==0||(i&3)!==0)&&(Ii=!0)),n=l}Hl(t)}function rd(t,e){for(var a=t.suspendedLanes,n=t.pingedLanes,l=t.expirationTimes,i=t.pendingLanes&-62914561;0<i;){var u=31-Kt(i),x=1<<u,U=l[u];U===-1?((x&a)===0||(x&n)!==0)&&(l[u]=Er(x,e)):U<=e&&(t.expiredLanes|=x),i&=~x}if(e=Ft,a=jt,a=Se(t,t===e?a:0,t.cancelPendingCommit!==null||t.timeoutHandle!==-1),n=t.callbackNode,a===0||t===e&&(Yt===2||Yt===9)||t.cancelPendingCommit!==null)return n!==null&&n!==null&&f(n),t.callbackNode=null,t.callbackPriority=0;if((a&3)===0||Sa(t,a)){if(e=a&-a,e===t.callbackPriority)return e;switch(n!==null&&f(n),Ar(a)){case 2:case 8:a=ht;break;case 32:a=mt;break;case 268435456:a=pt;break;default:a=mt}return n=ud.bind(null,t),a=ve(a,n),t.callbackPriority=e,t.callbackNode=a,e}return n!==null&&n!==null&&f(n),t.callbackPriority=2,t.callbackNode=null,2}function ud(t,e){if(he!==0&&he!==5)return t.callbackNode=null,t.callbackPriority=0,null;var a=t.callbackNode;if(Wi()&&t.callbackNode!==a)return null;var n=jt;return n=Se(t,t===Ft?n:0,t.cancelPendingCommit!==null||t.timeoutHandle!==-1),n===0?null:(Gf(t,n,e),rd(t,k()),t.callbackNode!=null&&t.callbackNode===a?ud.bind(null,t):null)}function sd(t,e){if(Wi())return null;Gf(t,e,!0)}function $m(){u0(function(){(qt&6)!==0?ve(q,Wm):id()})}function fs(){return fn===0&&(fn=ni()),fn}function od(t){return t==null||typeof t=="symbol"||typeof t=="boolean"?null:typeof t=="function"?t:oi(""+t)}function cd(t,e){var a=e.ownerDocument.createElement("input");return a.name=e.name,a.value=e.value,t.id&&a.setAttribute("form",t.id),e.parentNode.insertBefore(a,e),t=new FormData(t),a.parentNode.removeChild(a),t}function Im(t,e,a,n,l){if(e==="submit"&&a&&a.stateNode===l){var i=od((l[xe]||null).action),u=n.submitter;u&&(e=(e=u[xe]||null)?od(e.formAction):u.getAttribute("formAction"),e!==null&&(i=e,u=null));var x=new hi("action","action",null,n,l);t.push({event:x,listeners:[{instance:null,listener:function(){if(n.defaultPrevented){if(fn!==0){var U=u?cd(l,u):new FormData(l);Cu(a,{pending:!0,data:U,method:l.method,action:i},null,U)}}else typeof i=="function"&&(x.preventDefault(),U=u?cd(l,u):new FormData(l),Cu(a,{pending:!0,data:U,method:l.method,action:i},i,U))},currentTarget:l}]})}}for(var ds=0;ds<Wr.length;ds++){var hs=Wr[ds],Pm=hs.toLowerCase(),t0=hs[0].toUpperCase()+hs.slice(1);Ve(Pm,"on"+t0)}Ve(Zo,"onAnimationEnd"),Ve(Go,"onAnimationIteration"),Ve(Xo,"onAnimationStart"),Ve("dblclick","onDoubleClick"),Ve("focusin","onFocus"),Ve("focusout","onBlur"),Ve(ym,"onTransitionRun"),Ve(vm,"onTransitionStart"),Ve(bm,"onTransitionCancel"),Ve(Qo,"onTransitionEnd"),bn("onMouseEnter",["mouseout","mouseover"]),bn("onMouseLeave",["mouseout","mouseover"]),bn("onPointerEnter",["pointerout","pointerover"]),bn("onPointerLeave",["pointerout","pointerover"]),Ja("onChange","change click focusin focusout input keydown keyup selectionchange".split(" ")),Ja("onSelect","focusout contextmenu dragend focusin keydown keyup mousedown mouseup selectionchange".split(" ")),Ja("onBeforeInput",["compositionend","keypress","textInput","paste"]),Ja("onCompositionEnd","compositionend focusout keydown keypress keyup mousedown".split(" ")),Ja("onCompositionStart","compositionstart focusout keydown keypress keyup mousedown".split(" ")),Ja("onCompositionUpdate","compositionupdate focusout keydown keypress keyup mousedown".split(" "));var Ll="abort canplay canplaythrough durationchange emptied encrypted ended error loadeddata loadedmetadata loadstart pause play playing progress ratechange resize seeked seeking stalled suspend timeupdate volumechange waiting".split(" "),e0=new Set("beforetoggle cancel close invalid load scroll scrollend toggle".split(" ").concat(Ll));function fd(t,e){e=(e&4)!==0;for(var a=0;a<t.length;a++){var n=t[a],l=n.event;n=n.listeners;t:{var i=void 0;if(e)for(var u=n.length-1;0<=u;u--){var x=n[u],U=x.instance,V=x.currentTarget;if(x=x.listener,U!==i&&l.isPropagationStopped())break t;i=x,l.currentTarget=V;try{i(l)}catch(lt){Li(lt)}l.currentTarget=null,i=U}else for(u=0;u<n.length;u++){if(x=n[u],U=x.instance,V=x.currentTarget,x=x.listener,U!==i&&l.isPropagationStopped())break t;i=x,l.currentTarget=V;try{i(l)}catch(lt){Li(lt)}l.currentTarget=null,i=U}}}}function Mt(t,e){var a=e[zr];a===void 0&&(a=e[zr]=new Set);var n=t+"__bubble";a.has(n)||(dd(e,t,2,!1),a.add(n))}function ms(t,e,a){var n=0;e&&(n|=4),dd(a,t,n,e)}var Pi="_reactListening"+Math.random().toString(36).slice(2);function ps(t){if(!t[Pi]){t[Pi]=!0,no.forEach(function(a){a!=="selectionchange"&&(e0.has(a)||ms(a,!1,t),ms(a,!0,t))});var e=t.nodeType===9?t:t.ownerDocument;e===null||e[Pi]||(e[Pi]=!0,ms("selectionchange",!1,e))}}function dd(t,e,a,n){switch(jd(e)){case 2:var l=R0;break;case 8:l=O0;break;default:l=Os}a=l.bind(null,e,a,t),l=void 0,!Br||e!=="touchstart"&&e!=="touchmove"&&e!=="wheel"||(l=!0),n?l!==void 0?t.addEventListener(e,a,{capture:!0,passive:l}):t.addEventListener(e,a,!0):l!==void 0?t.addEventListener(e,a,{passive:l}):t.addEventListener(e,a,!1)}function gs(t,e,a,n,l){var i=n;if((e&1)===0&&(e&2)===0&&n!==null)t:for(;;){if(n===null)return;var u=n.tag;if(u===3||u===4){var x=n.stateNode.containerInfo;if(x===l)break;if(u===4)for(u=n.return;u!==null;){var U=u.tag;if((U===3||U===4)&&u.stateNode.containerInfo===l)return;u=u.return}for(;x!==null;){if(u=gn(x),u===null)return;if(U=u.tag,U===5||U===6||U===26||U===27){n=i=u;continue t}x=x.parentNode}}n=n.return}vo(function(){var V=i,lt=Mr(a),ct=[];t:{var J=Vo.get(t);if(J!==void 0){var W=hi,Tt=t;switch(t){case"keypress":if(fi(a)===0)break t;case"keydown":case"keyup":W=Fh;break;case"focusin":Tt="focus",W=Yr;break;case"focusout":Tt="blur",W=Yr;break;case"beforeblur":case"afterblur":W=Yr;break;case"click":if(a.button===2)break t;case"auxclick":case"dblclick":case"mousedown":case"mousemove":case"mouseup":case"mouseout":case"mouseover":case"contextmenu":W=So;break;case"drag":case"dragend":case"dragenter":case"dragexit":case"dragleave":case"dragover":case"dragstart":case"drop":W=Bh;break;case"touchcancel":case"touchend":case"touchmove":case"touchstart":W=Ih;break;case Zo:case Go:case Xo:W=qh;break;case Qo:W=tm;break;case"scroll":case"scrollend":W=Mh;break;case"wheel":W=am;break;case"copy":case"cut":case"paste":W=Zh;break;case"gotpointercapture":case"lostpointercapture":case"pointercancel":case"pointerdown":case"pointermove":case"pointerout":case"pointerover":case"pointerup":W=Eo;break;case"toggle":case"beforetoggle":W=lm}var At=(e&4)!==0,Xt=!At&&(t==="scroll"||t==="scrollend"),Y=At?J!==null?J+"Capture":null:J;At=[];for(var H=V,Q;H!==null;){var ut=H;if(Q=ut.stateNode,ut=ut.tag,ut!==5&&ut!==26&&ut!==27||Q===null||Y===null||(ut=ll(H,Y),ut!=null&&At.push(ql(H,ut,Q))),Xt)break;H=H.return}0<At.length&&(J=new W(J,Tt,null,a,lt),ct.push({event:J,listeners:At}))}}if((e&7)===0){t:{if(J=t==="mouseover"||t==="pointerover",W=t==="mouseout"||t==="pointerout",J&&a!==Ur&&(Tt=a.relatedTarget||a.fromElement)&&(gn(Tt)||Tt[pn]))break t;if((W||J)&&(J=lt.window===lt?lt:(J=lt.ownerDocument)?J.defaultView||J.parentWindow:window,W?(Tt=a.relatedTarget||a.toElement,W=V,Tt=Tt?gn(Tt):null,Tt!==null&&(Xt=o(Tt),At=Tt.tag,Tt!==Xt||At!==5&&At!==27&&At!==6)&&(Tt=null)):(W=null,Tt=V),W!==Tt)){if(At=So,ut="onMouseLeave",Y="onMouseEnter",H="mouse",(t==="pointerout"||t==="pointerover")&&(At=Eo,ut="onPointerLeave",Y="onPointerEnter",H="pointer"),Xt=W==null?J:nl(W),Q=Tt==null?J:nl(Tt),J=new At(ut,H+"leave",W,a,lt),J.target=Xt,J.relatedTarget=Q,ut=null,gn(lt)===V&&(At=new At(Y,H+"enter",Tt,a,lt),At.target=Q,At.relatedTarget=Xt,ut=At),Xt=ut,W&&Tt)e:{for(At=W,Y=Tt,H=0,Q=At;Q;Q=Wn(Q))H++;for(Q=0,ut=Y;ut;ut=Wn(ut))Q++;for(;0<H-Q;)At=Wn(At),H--;for(;0<Q-H;)Y=Wn(Y),Q--;for(;H--;){if(At===Y||Y!==null&&At===Y.alternate)break e;At=Wn(At),Y=Wn(Y)}At=null}else At=null;W!==null&&hd(ct,J,W,At,!1),Tt!==null&&Xt!==null&&hd(ct,Xt,Tt,At,!0)}}t:{if(J=V?nl(V):window,W=J.nodeName&&J.nodeName.toLowerCase(),W==="select"||W==="input"&&J.type==="file")var St=Co;else if(Oo(J))if(Do)St=mm;else{St=dm;var Nt=fm}else W=J.nodeName,!W||W.toLowerCase()!=="input"||J.type!=="checkbox"&&J.type!=="radio"?V&&Nr(V.elementType)&&(St=Co):St=hm;if(St&&(St=St(t,V))){ko(ct,St,a,lt);break t}Nt&&Nt(t,J,V),t==="focusout"&&V&&J.type==="number"&&V.memoizedProps.value!=null&&Dr(J,"number",J.value)}switch(Nt=V?nl(V):window,t){case"focusin":(Oo(Nt)||Nt.contentEditable==="true")&&(zn=Nt,Kr=V,dl=null);break;case"focusout":dl=Kr=zn=null;break;case"mousedown":Jr=!0;break;case"contextmenu":case"mouseup":case"dragend":Jr=!1,qo(ct,a,lt);break;case"selectionchange":if(gm)break;case"keydown":case"keyup":qo(ct,a,lt)}var wt;if(Gr)t:{switch(t){case"compositionstart":var zt="onCompositionStart";break t;case"compositionend":zt="onCompositionEnd";break t;case"compositionupdate":zt="onCompositionUpdate";break t}zt=void 0}else An?To(t,a)&&(zt="onCompositionEnd"):t==="keydown"&&a.keyCode===229&&(zt="onCompositionStart");zt&&(wo&&a.locale!=="ko"&&(An||zt!=="onCompositionStart"?zt==="onCompositionEnd"&&An&&(wt=bo()):(Ea=lt,Hr="value"in Ea?Ea.value:Ea.textContent,An=!0)),Nt=tr(V,zt),0<Nt.length&&(zt=new xo(zt,t,null,a,lt),ct.push({event:zt,listeners:Nt}),wt?zt.data=wt:(wt=Ro(a),wt!==null&&(zt.data=wt)))),(wt=rm?um(t,a):sm(t,a))&&(zt=tr(V,"onBeforeInput"),0<zt.length&&(Nt=new xo("onBeforeInput","beforeinput",null,a,lt),ct.push({event:Nt,listeners:zt}),Nt.data=wt)),Im(ct,t,V,a,lt)}fd(ct,e)})}function ql(t,e,a){return{instance:t,listener:e,currentTarget:a}}function tr(t,e){for(var a=e+"Capture",n=[];t!==null;){var l=t,i=l.stateNode;if(l=l.tag,l!==5&&l!==26&&l!==27||i===null||(l=ll(t,a),l!=null&&n.unshift(ql(t,l,i)),l=ll(t,e),l!=null&&n.push(ql(t,l,i))),t.tag===3)return n;t=t.return}return[]}function Wn(t){if(t===null)return null;do t=t.return;while(t&&t.tag!==5&&t.tag!==27);return t||null}function hd(t,e,a,n,l){for(var i=e._reactName,u=[];a!==null&&a!==n;){var x=a,U=x.alternate,V=x.stateNode;if(x=x.tag,U!==null&&U===n)break;x!==5&&x!==26&&x!==27||V===null||(U=V,l?(V=ll(a,i),V!=null&&u.unshift(ql(a,V,U))):l||(V=ll(a,i),V!=null&&u.push(ql(a,V,U)))),a=a.return}u.length!==0&&t.push({event:e,listeners:u})}var a0=/\r\n?/g,n0=/\u0000|\uFFFD/g;function md(t){return(typeof t=="string"?t:""+t).replace(a0,`
`).replace(n0,"")}function pd(t,e){return e=md(e),md(t)===e}function er(){}function Gt(t,e,a,n,l,i){switch(a){case"children":typeof n=="string"?e==="body"||e==="textarea"&&n===""||xn(t,n):(typeof n=="number"||typeof n=="bigint")&&e!=="body"&&xn(t,""+n);break;case"className":ri(t,"class",n);break;case"tabIndex":ri(t,"tabindex",n);break;case"dir":case"role":case"viewBox":case"width":case"height":ri(t,a,n);break;case"style":go(t,n,i);break;case"data":if(e!=="object"){ri(t,"data",n);break}case"src":case"href":if(n===""&&(e!=="a"||a!=="href")){t.removeAttribute(a);break}if(n==null||typeof n=="function"||typeof n=="symbol"||typeof n=="boolean"){t.removeAttribute(a);break}n=oi(""+n),t.setAttribute(a,n);break;case"action":case"formAction":if(typeof n=="function"){t.setAttribute(a,"javascript:throw new Error('A React form was unexpectedly submitted. If you called form.submit() manually, consider using form.requestSubmit() instead. If you\\'re trying to use event.stopPropagation() in a submit event handler, consider also calling event.preventDefault().')");break}else typeof i=="function"&&(a==="formAction"?(e!=="input"&&Gt(t,e,"name",l.name,l,null),Gt(t,e,"formEncType",l.formEncType,l,null),Gt(t,e,"formMethod",l.formMethod,l,null),Gt(t,e,"formTarget",l.formTarget,l,null)):(Gt(t,e,"encType",l.encType,l,null),Gt(t,e,"method",l.method,l,null),Gt(t,e,"target",l.target,l,null)));if(n==null||typeof n=="symbol"||typeof n=="boolean"){t.removeAttribute(a);break}n=oi(""+n),t.setAttribute(a,n);break;case"onClick":n!=null&&(t.onclick=er);break;case"onScroll":n!=null&&Mt("scroll",t);break;case"onScrollEnd":n!=null&&Mt("scrollend",t);break;case"dangerouslySetInnerHTML":if(n!=null){if(typeof n!="object"||!("__html"in n))throw Error(d(61));if(a=n.__html,a!=null){if(l.children!=null)throw Error(d(60));t.innerHTML=a}}break;case"multiple":t.multiple=n&&typeof n!="function"&&typeof n!="symbol";break;case"muted":t.muted=n&&typeof n!="function"&&typeof n!="symbol";break;case"suppressContentEditableWarning":case"suppressHydrationWarning":case"defaultValue":case"defaultChecked":case"innerHTML":case"ref":break;case"autoFocus":break;case"xlinkHref":if(n==null||typeof n=="function"||typeof n=="boolean"||typeof n=="symbol"){t.removeAttribute("xlink:href");break}a=oi(""+n),t.setAttributeNS("http://www.w3.org/1999/xlink","xlink:href",a);break;case"contentEditable":case"spellCheck":case"draggable":case"value":case"autoReverse":case"externalResourcesRequired":case"focusable":case"preserveAlpha":n!=null&&typeof n!="function"&&typeof n!="symbol"?t.setAttribute(a,""+n):t.removeAttribute(a);break;case"inert":case"allowFullScreen":case"async":case"autoPlay":case"controls":case"default":case"defer":case"disabled":case"disablePictureInPicture":case"disableRemotePlayback":case"formNoValidate":case"hidden":case"loop":case"noModule":case"noValidate":case"open":case"playsInline":case"readOnly":case"required":case"reversed":case"scoped":case"seamless":case"itemScope":n&&typeof n!="function"&&typeof n!="symbol"?t.setAttribute(a,""):t.removeAttribute(a);break;case"capture":case"download":n===!0?t.setAttribute(a,""):n!==!1&&n!=null&&typeof n!="function"&&typeof n!="symbol"?t.setAttribute(a,n):t.removeAttribute(a);break;case"cols":case"rows":case"size":case"span":n!=null&&typeof n!="function"&&typeof n!="symbol"&&!isNaN(n)&&1<=n?t.setAttribute(a,n):t.removeAttribute(a);break;case"rowSpan":case"start":n==null||typeof n=="function"||typeof n=="symbol"||isNaN(n)?t.removeAttribute(a):t.setAttribute(a,n);break;case"popover":Mt("beforetoggle",t),Mt("toggle",t),ii(t,"popover",n);break;case"xlinkActuate":na(t,"http://www.w3.org/1999/xlink","xlink:actuate",n);break;case"xlinkArcrole":na(t,"http://www.w3.org/1999/xlink","xlink:arcrole",n);break;case"xlinkRole":na(t,"http://www.w3.org/1999/xlink","xlink:role",n);break;case"xlinkShow":na(t,"http://www.w3.org/1999/xlink","xlink:show",n);break;case"xlinkTitle":na(t,"http://www.w3.org/1999/xlink","xlink:title",n);break;case"xlinkType":na(t,"http://www.w3.org/1999/xlink","xlink:type",n);break;case"xmlBase":na(t,"http://www.w3.org/XML/1998/namespace","xml:base",n);break;case"xmlLang":na(t,"http://www.w3.org/XML/1998/namespace","xml:lang",n);break;case"xmlSpace":na(t,"http://www.w3.org/XML/1998/namespace","xml:space",n);break;case"is":ii(t,"is",n);break;case"innerText":case"textContent":break;default:(!(2<a.length)||a[0]!=="o"&&a[0]!=="O"||a[1]!=="n"&&a[1]!=="N")&&(a=Nh.get(a)||a,ii(t,a,n))}}function ys(t,e,a,n,l,i){switch(a){case"style":go(t,n,i);break;case"dangerouslySetInnerHTML":if(n!=null){if(typeof n!="object"||!("__html"in n))throw Error(d(61));if(a=n.__html,a!=null){if(l.children!=null)throw Error(d(60));t.innerHTML=a}}break;case"children":typeof n=="string"?xn(t,n):(typeof n=="number"||typeof n=="bigint")&&xn(t,""+n);break;case"onScroll":n!=null&&Mt("scroll",t);break;case"onScrollEnd":n!=null&&Mt("scrollend",t);break;case"onClick":n!=null&&(t.onclick=er);break;case"suppressContentEditableWarning":case"suppressHydrationWarning":case"innerHTML":case"ref":break;case"innerText":case"textContent":break;default:if(!lo.hasOwnProperty(a))t:{if(a[0]==="o"&&a[1]==="n"&&(l=a.endsWith("Capture"),e=a.slice(2,l?a.length-7:void 0),i=t[xe]||null,i=i!=null?i[a]:null,typeof i=="function"&&t.removeEventListener(e,i,l),typeof n=="function")){typeof i!="function"&&i!==null&&(a in t?t[a]=null:t.hasAttribute(a)&&t.removeAttribute(a)),t.addEventListener(e,n,l);break t}a in t?t[a]=n:n===!0?t.setAttribute(a,""):ii(t,a,n)}}}function me(t,e,a){switch(e){case"div":case"span":case"svg":case"path":case"a":case"g":case"p":case"li":break;case"img":Mt("error",t),Mt("load",t);var n=!1,l=!1,i;for(i in a)if(a.hasOwnProperty(i)){var u=a[i];if(u!=null)switch(i){case"src":n=!0;break;case"srcSet":l=!0;break;case"children":case"dangerouslySetInnerHTML":throw Error(d(137,e));default:Gt(t,e,i,u,a,null)}}l&&Gt(t,e,"srcSet",a.srcSet,a,null),n&&Gt(t,e,"src",a.src,a,null);return;case"input":Mt("invalid",t);var x=i=u=l=null,U=null,V=null;for(n in a)if(a.hasOwnProperty(n)){var lt=a[n];if(lt!=null)switch(n){case"name":l=lt;break;case"type":u=lt;break;case"checked":U=lt;break;case"defaultChecked":V=lt;break;case"value":i=lt;break;case"defaultValue":x=lt;break;case"children":case"dangerouslySetInnerHTML":if(lt!=null)throw Error(d(137,e));break;default:Gt(t,e,n,lt,a,null)}}fo(t,i,x,U,V,u,l,!1),ui(t);return;case"select":Mt("invalid",t),n=u=i=null;for(l in a)if(a.hasOwnProperty(l)&&(x=a[l],x!=null))switch(l){case"value":i=x;break;case"defaultValue":u=x;break;case"multiple":n=x;default:Gt(t,e,l,x,a,null)}e=i,a=u,t.multiple=!!n,e!=null?Sn(t,!!n,e,!1):a!=null&&Sn(t,!!n,a,!0);return;case"textarea":Mt("invalid",t),i=l=n=null;for(u in a)if(a.hasOwnProperty(u)&&(x=a[u],x!=null))switch(u){case"value":n=x;break;case"defaultValue":l=x;break;case"children":i=x;break;case"dangerouslySetInnerHTML":if(x!=null)throw Error(d(91));break;default:Gt(t,e,u,x,a,null)}mo(t,n,l,i),ui(t);return;case"option":for(U in a)if(a.hasOwnProperty(U)&&(n=a[U],n!=null))switch(U){case"selected":t.selected=n&&typeof n!="function"&&typeof n!="symbol";break;default:Gt(t,e,U,n,a,null)}return;case"dialog":Mt("beforetoggle",t),Mt("toggle",t),Mt("cancel",t),Mt("close",t);break;case"iframe":case"object":Mt("load",t);break;case"video":case"audio":for(n=0;n<Ll.length;n++)Mt(Ll[n],t);break;case"image":Mt("error",t),Mt("load",t);break;case"details":Mt("toggle",t);break;case"embed":case"source":case"link":Mt("error",t),Mt("load",t);case"area":case"base":case"br":case"col":case"hr":case"keygen":case"meta":case"param":case"track":case"wbr":case"menuitem":for(V in a)if(a.hasOwnProperty(V)&&(n=a[V],n!=null))switch(V){case"children":case"dangerouslySetInnerHTML":throw Error(d(137,e));default:Gt(t,e,V,n,a,null)}return;default:if(Nr(e)){for(lt in a)a.hasOwnProperty(lt)&&(n=a[lt],n!==void 0&&ys(t,e,lt,n,a,void 0));return}}for(x in a)a.hasOwnProperty(x)&&(n=a[x],n!=null&&Gt(t,e,x,n,a,null))}function l0(t,e,a,n){switch(e){case"div":case"span":case"svg":case"path":case"a":case"g":case"p":case"li":break;case"input":var l=null,i=null,u=null,x=null,U=null,V=null,lt=null;for(W in a){var ct=a[W];if(a.hasOwnProperty(W)&&ct!=null)switch(W){case"checked":break;case"value":break;case"defaultValue":U=ct;default:n.hasOwnProperty(W)||Gt(t,e,W,null,n,ct)}}for(var J in n){var W=n[J];if(ct=a[J],n.hasOwnProperty(J)&&(W!=null||ct!=null))switch(J){case"type":i=W;break;case"name":l=W;break;case"checked":V=W;break;case"defaultChecked":lt=W;break;case"value":u=W;break;case"defaultValue":x=W;break;case"children":case"dangerouslySetInnerHTML":if(W!=null)throw Error(d(137,e));break;default:W!==ct&&Gt(t,e,J,W,n,ct)}}Cr(t,u,x,U,V,lt,i,l);return;case"select":W=u=x=J=null;for(i in a)if(U=a[i],a.hasOwnProperty(i)&&U!=null)switch(i){case"value":break;case"multiple":W=U;default:n.hasOwnProperty(i)||Gt(t,e,i,null,n,U)}for(l in n)if(i=n[l],U=a[l],n.hasOwnProperty(l)&&(i!=null||U!=null))switch(l){case"value":J=i;break;case"defaultValue":x=i;break;case"multiple":u=i;default:i!==U&&Gt(t,e,l,i,n,U)}e=x,a=u,n=W,J!=null?Sn(t,!!a,J,!1):!!n!=!!a&&(e!=null?Sn(t,!!a,e,!0):Sn(t,!!a,a?[]:"",!1));return;case"textarea":W=J=null;for(x in a)if(l=a[x],a.hasOwnProperty(x)&&l!=null&&!n.hasOwnProperty(x))switch(x){case"value":break;case"children":break;default:Gt(t,e,x,null,n,l)}for(u in n)if(l=n[u],i=a[u],n.hasOwnProperty(u)&&(l!=null||i!=null))switch(u){case"value":J=l;break;case"defaultValue":W=l;break;case"children":break;case"dangerouslySetInnerHTML":if(l!=null)throw Error(d(91));break;default:l!==i&&Gt(t,e,u,l,n,i)}ho(t,J,W);return;case"option":for(var Tt in a)if(J=a[Tt],a.hasOwnProperty(Tt)&&J!=null&&!n.hasOwnProperty(Tt))switch(Tt){case"selected":t.selected=!1;break;default:Gt(t,e,Tt,null,n,J)}for(U in n)if(J=n[U],W=a[U],n.hasOwnProperty(U)&&J!==W&&(J!=null||W!=null))switch(U){case"selected":t.selected=J&&typeof J!="function"&&typeof J!="symbol";break;default:Gt(t,e,U,J,n,W)}return;case"img":case"link":case"area":case"base":case"br":case"col":case"embed":case"hr":case"keygen":case"meta":case"param":case"source":case"track":case"wbr":case"menuitem":for(var At in a)J=a[At],a.hasOwnProperty(At)&&J!=null&&!n.hasOwnProperty(At)&&Gt(t,e,At,null,n,J);for(V in n)if(J=n[V],W=a[V],n.hasOwnProperty(V)&&J!==W&&(J!=null||W!=null))switch(V){case"children":case"dangerouslySetInnerHTML":if(J!=null)throw Error(d(137,e));break;default:Gt(t,e,V,J,n,W)}return;default:if(Nr(e)){for(var Xt in a)J=a[Xt],a.hasOwnProperty(Xt)&&J!==void 0&&!n.hasOwnProperty(Xt)&&ys(t,e,Xt,void 0,n,J);for(lt in n)J=n[lt],W=a[lt],!n.hasOwnProperty(lt)||J===W||J===void 0&&W===void 0||ys(t,e,lt,J,n,W);return}}for(var Y in a)J=a[Y],a.hasOwnProperty(Y)&&J!=null&&!n.hasOwnProperty(Y)&&Gt(t,e,Y,null,n,J);for(ct in n)J=n[ct],W=a[ct],!n.hasOwnProperty(ct)||J===W||J==null&&W==null||Gt(t,e,ct,J,n,W)}var vs=null,bs=null;function ar(t){return t.nodeType===9?t:t.ownerDocument}function gd(t){switch(t){case"http://www.w3.org/2000/svg":return 1;case"http://www.w3.org/1998/Math/MathML":return 2;default:return 0}}function yd(t,e){if(t===0)switch(e){case"svg":return 1;case"math":return 2;default:return 0}return t===1&&e==="foreignObject"?0:t}function _s(t,e){return t==="textarea"||t==="noscript"||typeof e.children=="string"||typeof e.children=="number"||typeof e.children=="bigint"||typeof e.dangerouslySetInnerHTML=="object"&&e.dangerouslySetInnerHTML!==null&&e.dangerouslySetInnerHTML.__html!=null}var Ss=null;function i0(){var t=window.event;return t&&t.type==="popstate"?t===Ss?!1:(Ss=t,!0):(Ss=null,!1)}var vd=typeof setTimeout=="function"?setTimeout:void 0,r0=typeof clearTimeout=="function"?clearTimeout:void 0,bd=typeof Promise=="function"?Promise:void 0,u0=typeof queueMicrotask=="function"?queueMicrotask:typeof bd<"u"?function(t){return bd.resolve(null).then(t).catch(s0)}:vd;function s0(t){setTimeout(function(){throw t})}function La(t){return t==="head"}function _d(t,e){var a=e,n=0,l=0;do{var i=a.nextSibling;if(t.removeChild(a),i&&i.nodeType===8)if(a=i.data,a==="/$"){if(0<n&&8>n){a=n;var u=t.ownerDocument;if(a&1&&Yl(u.documentElement),a&2&&Yl(u.body),a&4)for(a=u.head,Yl(a),u=a.firstChild;u;){var x=u.nextSibling,U=u.nodeName;u[al]||U==="SCRIPT"||U==="STYLE"||U==="LINK"&&u.rel.toLowerCase()==="stylesheet"||a.removeChild(u),u=x}}if(l===0){t.removeChild(i),Fl(e);return}l--}else a==="$"||a==="$?"||a==="$!"?l++:n=a.charCodeAt(0)-48;else n=0;a=i}while(a);Fl(e)}function xs(t){var e=t.firstChild;for(e&&e.nodeType===10&&(e=e.nextSibling);e;){var a=e;switch(e=e.nextSibling,a.nodeName){case"HTML":case"HEAD":case"BODY":xs(a),Tr(a);continue;case"SCRIPT":case"STYLE":continue;case"LINK":if(a.rel.toLowerCase()==="stylesheet")continue}t.removeChild(a)}}function o0(t,e,a,n){for(;t.nodeType===1;){var l=a;if(t.nodeName.toLowerCase()!==e.toLowerCase()){if(!n&&(t.nodeName!=="INPUT"||t.type!=="hidden"))break}else if(n){if(!t[al])switch(e){case"meta":if(!t.hasAttribute("itemprop"))break;return t;case"link":if(i=t.getAttribute("rel"),i==="stylesheet"&&t.hasAttribute("data-precedence"))break;if(i!==l.rel||t.getAttribute("href")!==(l.href==null||l.href===""?null:l.href)||t.getAttribute("crossorigin")!==(l.crossOrigin==null?null:l.crossOrigin)||t.getAttribute("title")!==(l.title==null?null:l.title))break;return t;case"style":if(t.hasAttribute("data-precedence"))break;return t;case"script":if(i=t.getAttribute("src"),(i!==(l.src==null?null:l.src)||t.getAttribute("type")!==(l.type==null?null:l.type)||t.getAttribute("crossorigin")!==(l.crossOrigin==null?null:l.crossOrigin))&&i&&t.hasAttribute("async")&&!t.hasAttribute("itemprop"))break;return t;default:return t}}else if(e==="input"&&t.type==="hidden"){var i=l.name==null?null:""+l.name;if(l.type==="hidden"&&t.getAttribute("name")===i)return t}else return t;if(t=Je(t.nextSibling),t===null)break}return null}function c0(t,e,a){if(e==="")return null;for(;t.nodeType!==3;)if((t.nodeType!==1||t.nodeName!=="INPUT"||t.type!=="hidden")&&!a||(t=Je(t.nextSibling),t===null))return null;return t}function Es(t){return t.data==="$!"||t.data==="$?"&&t.ownerDocument.readyState==="complete"}function f0(t,e){var a=t.ownerDocument;if(t.data!=="$?"||a.readyState==="complete")e();else{var n=function(){e(),a.removeEventListener("DOMContentLoaded",n)};a.addEventListener("DOMContentLoaded",n),t._reactRetry=n}}function Je(t){for(;t!=null;t=t.nextSibling){var e=t.nodeType;if(e===1||e===3)break;if(e===8){if(e=t.data,e==="$"||e==="$!"||e==="$?"||e==="F!"||e==="F")break;if(e==="/$")return null}}return t}var ws=null;function Sd(t){t=t.previousSibling;for(var e=0;t;){if(t.nodeType===8){var a=t.data;if(a==="$"||a==="$!"||a==="$?"){if(e===0)return t;e--}else a==="/$"&&e++}t=t.previousSibling}return null}function xd(t,e,a){switch(e=ar(a),t){case"html":if(t=e.documentElement,!t)throw Error(d(452));return t;case"head":if(t=e.head,!t)throw Error(d(453));return t;case"body":if(t=e.body,!t)throw Error(d(454));return t;default:throw Error(d(451))}}function Yl(t){for(var e=t.attributes;e.length;)t.removeAttributeNode(e[0]);Tr(t)}var Ge=new Map,Ed=new Set;function nr(t){return typeof t.getRootNode=="function"?t.getRootNode():t.nodeType===9?t:t.ownerDocument}var ya=X.d;X.d={f:d0,r:h0,D:m0,C:p0,L:g0,m:y0,X:b0,S:v0,M:_0};function d0(){var t=ya.f(),e=Ji();return t||e}function h0(t){var e=yn(t);e!==null&&e.tag===5&&e.type==="form"?Zc(e):ya.r(t)}var $n=typeof document>"u"?null:document;function wd(t,e,a){var n=$n;if(n&&typeof e=="string"&&e){var l=je(e);l='link[rel="'+t+'"][href="'+l+'"]',typeof a=="string"&&(l+='[crossorigin="'+a+'"]'),Ed.has(l)||(Ed.add(l),t={rel:t,crossOrigin:a,href:e},n.querySelector(l)===null&&(e=n.createElement("link"),me(e,"link",t),se(e),n.head.appendChild(e)))}}function m0(t){ya.D(t),wd("dns-prefetch",t,null)}function p0(t,e){ya.C(t,e),wd("preconnect",t,e)}function g0(t,e,a){ya.L(t,e,a);var n=$n;if(n&&t&&e){var l='link[rel="preload"][as="'+je(e)+'"]';e==="image"&&a&&a.imageSrcSet?(l+='[imagesrcset="'+je(a.imageSrcSet)+'"]',typeof a.imageSizes=="string"&&(l+='[imagesizes="'+je(a.imageSizes)+'"]')):l+='[href="'+je(t)+'"]';var i=l;switch(e){case"style":i=In(t);break;case"script":i=Pn(t)}Ge.has(i)||(t=E({rel:"preload",href:e==="image"&&a&&a.imageSrcSet?void 0:t,as:e},a),Ge.set(i,t),n.querySelector(l)!==null||e==="style"&&n.querySelector(Zl(i))||e==="script"&&n.querySelector(Gl(i))||(e=n.createElement("link"),me(e,"link",t),se(e),n.head.appendChild(e)))}}function y0(t,e){ya.m(t,e);var a=$n;if(a&&t){var n=e&&typeof e.as=="string"?e.as:"script",l='link[rel="modulepreload"][as="'+je(n)+'"][href="'+je(t)+'"]',i=l;switch(n){case"audioworklet":case"paintworklet":case"serviceworker":case"sharedworker":case"worker":case"script":i=Pn(t)}if(!Ge.has(i)&&(t=E({rel:"modulepreload",href:t},e),Ge.set(i,t),a.querySelector(l)===null)){switch(n){case"audioworklet":case"paintworklet":case"serviceworker":case"sharedworker":case"worker":case"script":if(a.querySelector(Gl(i)))return}n=a.createElement("link"),me(n,"link",t),se(n),a.head.appendChild(n)}}}function v0(t,e,a){ya.S(t,e,a);var n=$n;if(n&&t){var l=vn(n).hoistableStyles,i=In(t);e=e||"default";var u=l.get(i);if(!u){var x={loading:0,preload:null};if(u=n.querySelector(Zl(i)))x.loading=5;else{t=E({rel:"stylesheet",href:t,"data-precedence":e},a),(a=Ge.get(i))&&As(t,a);var U=u=n.createElement("link");se(U),me(U,"link",t),U._p=new Promise(function(V,lt){U.onload=V,U.onerror=lt}),U.addEventListener("load",function(){x.loading|=1}),U.addEventListener("error",function(){x.loading|=2}),x.loading|=4,lr(u,e,n)}u={type:"stylesheet",instance:u,count:1,state:x},l.set(i,u)}}}function b0(t,e){ya.X(t,e);var a=$n;if(a&&t){var n=vn(a).hoistableScripts,l=Pn(t),i=n.get(l);i||(i=a.querySelector(Gl(l)),i||(t=E({src:t,async:!0},e),(e=Ge.get(l))&&zs(t,e),i=a.createElement("script"),se(i),me(i,"link",t),a.head.appendChild(i)),i={type:"script",instance:i,count:1,state:null},n.set(l,i))}}function _0(t,e){ya.M(t,e);var a=$n;if(a&&t){var n=vn(a).hoistableScripts,l=Pn(t),i=n.get(l);i||(i=a.querySelector(Gl(l)),i||(t=E({src:t,async:!0,type:"module"},e),(e=Ge.get(l))&&zs(t,e),i=a.createElement("script"),se(i),me(i,"link",t),a.head.appendChild(i)),i={type:"script",instance:i,count:1,state:null},n.set(l,i))}}function Ad(t,e,a,n){var l=(l=_t.current)?nr(l):null;if(!l)throw Error(d(446));switch(t){case"meta":case"title":return null;case"style":return typeof a.precedence=="string"&&typeof a.href=="string"?(e=In(a.href),a=vn(l).hoistableStyles,n=a.get(e),n||(n={type:"style",instance:null,count:0,state:null},a.set(e,n)),n):{type:"void",instance:null,count:0,state:null};case"link":if(a.rel==="stylesheet"&&typeof a.href=="string"&&typeof a.precedence=="string"){t=In(a.href);var i=vn(l).hoistableStyles,u=i.get(t);if(u||(l=l.ownerDocument||l,u={type:"stylesheet",instance:null,count:0,state:{loading:0,preload:null}},i.set(t,u),(i=l.querySelector(Zl(t)))&&!i._p&&(u.instance=i,u.state.loading=5),Ge.has(t)||(a={rel:"preload",as:"style",href:a.href,crossOrigin:a.crossOrigin,integrity:a.integrity,media:a.media,hrefLang:a.hrefLang,referrerPolicy:a.referrerPolicy},Ge.set(t,a),i||S0(l,t,a,u.state))),e&&n===null)throw Error(d(528,""));return u}if(e&&n!==null)throw Error(d(529,""));return null;case"script":return e=a.async,a=a.src,typeof a=="string"&&e&&typeof e!="function"&&typeof e!="symbol"?(e=Pn(a),a=vn(l).hoistableScripts,n=a.get(e),n||(n={type:"script",instance:null,count:0,state:null},a.set(e,n)),n):{type:"void",instance:null,count:0,state:null};default:throw Error(d(444,t))}}function In(t){return'href="'+je(t)+'"'}function Zl(t){return'link[rel="stylesheet"]['+t+"]"}function zd(t){return E({},t,{"data-precedence":t.precedence,precedence:null})}function S0(t,e,a,n){t.querySelector('link[rel="preload"][as="style"]['+e+"]")?n.loading=1:(e=t.createElement("link"),n.preload=e,e.addEventListener("load",function(){return n.loading|=1}),e.addEventListener("error",function(){return n.loading|=2}),me(e,"link",a),se(e),t.head.appendChild(e))}function Pn(t){return'[src="'+je(t)+'"]'}function Gl(t){return"script[async]"+t}function Td(t,e,a){if(e.count++,e.instance===null)switch(e.type){case"style":var n=t.querySelector('style[data-href~="'+je(a.href)+'"]');if(n)return e.instance=n,se(n),n;var l=E({},a,{"data-href":a.href,"data-precedence":a.precedence,href:null,precedence:null});return n=(t.ownerDocument||t).createElement("style"),se(n),me(n,"style",l),lr(n,a.precedence,t),e.instance=n;case"stylesheet":l=In(a.href);var i=t.querySelector(Zl(l));if(i)return e.state.loading|=4,e.instance=i,se(i),i;n=zd(a),(l=Ge.get(l))&&As(n,l),i=(t.ownerDocument||t).createElement("link"),se(i);var u=i;return u._p=new Promise(function(x,U){u.onload=x,u.onerror=U}),me(i,"link",n),e.state.loading|=4,lr(i,a.precedence,t),e.instance=i;case"script":return i=Pn(a.src),(l=t.querySelector(Gl(i)))?(e.instance=l,se(l),l):(n=a,(l=Ge.get(i))&&(n=E({},a),zs(n,l)),t=t.ownerDocument||t,l=t.createElement("script"),se(l),me(l,"link",n),t.head.appendChild(l),e.instance=l);case"void":return null;default:throw Error(d(443,e.type))}else e.type==="stylesheet"&&(e.state.loading&4)===0&&(n=e.instance,e.state.loading|=4,lr(n,a.precedence,t));return e.instance}function lr(t,e,a){for(var n=a.querySelectorAll('link[rel="stylesheet"][data-precedence],style[data-precedence]'),l=n.length?n[n.length-1]:null,i=l,u=0;u<n.length;u++){var x=n[u];if(x.dataset.precedence===e)i=x;else if(i!==l)break}i?i.parentNode.insertBefore(t,i.nextSibling):(e=a.nodeType===9?a.head:a,e.insertBefore(t,e.firstChild))}function As(t,e){t.crossOrigin==null&&(t.crossOrigin=e.crossOrigin),t.referrerPolicy==null&&(t.referrerPolicy=e.referrerPolicy),t.title==null&&(t.title=e.title)}function zs(t,e){t.crossOrigin==null&&(t.crossOrigin=e.crossOrigin),t.referrerPolicy==null&&(t.referrerPolicy=e.referrerPolicy),t.integrity==null&&(t.integrity=e.integrity)}var ir=null;function Rd(t,e,a){if(ir===null){var n=new Map,l=ir=new Map;l.set(a,n)}else l=ir,n=l.get(a),n||(n=new Map,l.set(a,n));if(n.has(t))return n;for(n.set(t,null),a=a.getElementsByTagName(t),l=0;l<a.length;l++){var i=a[l];if(!(i[al]||i[pe]||t==="link"&&i.getAttribute("rel")==="stylesheet")&&i.namespaceURI!=="http://www.w3.org/2000/svg"){var u=i.getAttribute(e)||"";u=t+u;var x=n.get(u);x?x.push(i):n.set(u,[i])}}return n}function Od(t,e,a){t=t.ownerDocument||t,t.head.insertBefore(a,e==="title"?t.querySelector("head > title"):null)}function x0(t,e,a){if(a===1||e.itemProp!=null)return!1;switch(t){case"meta":case"title":return!0;case"style":if(typeof e.precedence!="string"||typeof e.href!="string"||e.href==="")break;return!0;case"link":if(typeof e.rel!="string"||typeof e.href!="string"||e.href===""||e.onLoad||e.onError)break;switch(e.rel){case"stylesheet":return t=e.disabled,typeof e.precedence=="string"&&t==null;default:return!0}case"script":if(e.async&&typeof e.async!="function"&&typeof e.async!="symbol"&&!e.onLoad&&!e.onError&&e.src&&typeof e.src=="string")return!0}return!1}function kd(t){return!(t.type==="stylesheet"&&(t.state.loading&3)===0)}var Xl=null;function E0(){}function w0(t,e,a){if(Xl===null)throw Error(d(475));var n=Xl;if(e.type==="stylesheet"&&(typeof a.media!="string"||matchMedia(a.media).matches!==!1)&&(e.state.loading&4)===0){if(e.instance===null){var l=In(a.href),i=t.querySelector(Zl(l));if(i){t=i._p,t!==null&&typeof t=="object"&&typeof t.then=="function"&&(n.count++,n=rr.bind(n),t.then(n,n)),e.state.loading|=4,e.instance=i,se(i);return}i=t.ownerDocument||t,a=zd(a),(l=Ge.get(l))&&As(a,l),i=i.createElement("link"),se(i);var u=i;u._p=new Promise(function(x,U){u.onload=x,u.onerror=U}),me(i,"link",a),e.instance=i}n.stylesheets===null&&(n.stylesheets=new Map),n.stylesheets.set(e,t),(t=e.state.preload)&&(e.state.loading&3)===0&&(n.count++,e=rr.bind(n),t.addEventListener("load",e),t.addEventListener("error",e))}}function A0(){if(Xl===null)throw Error(d(475));var t=Xl;return t.stylesheets&&t.count===0&&Ts(t,t.stylesheets),0<t.count?function(e){var a=setTimeout(function(){if(t.stylesheets&&Ts(t,t.stylesheets),t.unsuspend){var n=t.unsuspend;t.unsuspend=null,n()}},6e4);return t.unsuspend=e,function(){t.unsuspend=null,clearTimeout(a)}}:null}function rr(){if(this.count--,this.count===0){if(this.stylesheets)Ts(this,this.stylesheets);else if(this.unsuspend){var t=this.unsuspend;this.unsuspend=null,t()}}}var ur=null;function Ts(t,e){t.stylesheets=null,t.unsuspend!==null&&(t.count++,ur=new Map,e.forEach(z0,t),ur=null,rr.call(t))}function z0(t,e){if(!(e.state.loading&4)){var a=ur.get(t);if(a)var n=a.get(null);else{a=new Map,ur.set(t,a);for(var l=t.querySelectorAll("link[data-precedence],style[data-precedence]"),i=0;i<l.length;i++){var u=l[i];(u.nodeName==="LINK"||u.getAttribute("media")!=="not all")&&(a.set(u.dataset.precedence,u),n=u)}n&&a.set(null,n)}l=e.instance,u=l.getAttribute("data-precedence"),i=a.get(u)||n,i===n&&a.set(null,l),a.set(u,l),this.count++,n=rr.bind(this),l.addEventListener("load",n),l.addEventListener("error",n),i?i.parentNode.insertBefore(l,i.nextSibling):(t=t.nodeType===9?t.head:t,t.insertBefore(l,t.firstChild)),e.state.loading|=4}}var Ql={$$typeof:D,Provider:null,Consumer:null,_currentValue:gt,_currentValue2:gt,_threadCount:0};function T0(t,e,a,n,l,i,u,x){this.tag=1,this.containerInfo=t,this.pingCache=this.current=this.pendingChildren=null,this.timeoutHandle=-1,this.callbackNode=this.next=this.pendingContext=this.context=this.cancelPendingCommit=null,this.callbackPriority=0,this.expirationTimes=hn(-1),this.entangledLanes=this.shellSuspendCounter=this.errorRecoveryDisabledLanes=this.expiredLanes=this.warmLanes=this.pingedLanes=this.suspendedLanes=this.pendingLanes=0,this.entanglements=hn(0),this.hiddenUpdates=hn(null),this.identifierPrefix=n,this.onUncaughtError=l,this.onCaughtError=i,this.onRecoverableError=u,this.pooledCache=null,this.pooledCacheLanes=0,this.formState=x,this.incompleteTransitions=new Map}function Cd(t,e,a,n,l,i,u,x,U,V,lt,ct){return t=new T0(t,e,a,u,x,U,V,ct),e=1,i===!0&&(e|=24),i=Re(3,null,null,e),t.current=i,i.stateNode=t,e=su(),e.refCount++,t.pooledCache=e,e.refCount++,i.memoizedState={element:n,isDehydrated:a,cache:e},du(i),t}function Dd(t){return t?(t=kn,t):kn}function Nd(t,e,a,n,l,i){l=Dd(l),n.context===null?n.context=l:n.pendingContext=l,n=za(e),n.payload={element:a},i=i===void 0?null:i,i!==null&&(n.callback=i),a=Ta(t,n,e),a!==null&&(Ne(a,t,e),Sl(a,t,e))}function Ud(t,e){if(t=t.memoizedState,t!==null&&t.dehydrated!==null){var a=t.retryLane;t.retryLane=a!==0&&a<e?a:e}}function Rs(t,e){Ud(t,e),(t=t.alternate)&&Ud(t,e)}function Md(t){if(t.tag===13){var e=On(t,67108864);e!==null&&Ne(e,t,67108864),Rs(t,67108864)}}var sr=!0;function R0(t,e,a,n){var l=G.T;G.T=null;var i=X.p;try{X.p=2,Os(t,e,a,n)}finally{X.p=i,G.T=l}}function O0(t,e,a,n){var l=G.T;G.T=null;var i=X.p;try{X.p=8,Os(t,e,a,n)}finally{X.p=i,G.T=l}}function Os(t,e,a,n){if(sr){var l=ks(n);if(l===null)gs(t,e,n,or,a),Bd(t,n);else if(C0(l,t,e,a,n))n.stopPropagation();else if(Bd(t,n),e&4&&-1<k0.indexOf(t)){for(;l!==null;){var i=yn(l);if(i!==null)switch(i.tag){case 3:if(i=i.stateNode,i.current.memoizedState.isDehydrated){var u=Xe(i.pendingLanes);if(u!==0){var x=i;for(x.pendingLanes|=2,x.entangledLanes|=2;u;){var U=1<<31-Kt(u);x.entanglements[1]|=U,u&=~U}Pe(i),(qt&6)===0&&(Vi=k()+500,Hl(0))}}break;case 13:x=On(i,2),x!==null&&Ne(x,i,2),Ji(),Rs(i,2)}if(i=ks(n),i===null&&gs(t,e,n,or,a),i===l)break;l=i}l!==null&&n.stopPropagation()}else gs(t,e,n,null,a)}}function ks(t){return t=Mr(t),Cs(t)}var or=null;function Cs(t){if(or=null,t=gn(t),t!==null){var e=o(t);if(e===null)t=null;else{var a=e.tag;if(a===13){if(t=y(e),t!==null)return t;t=null}else if(a===3){if(e.stateNode.current.memoizedState.isDehydrated)return e.tag===3?e.stateNode.containerInfo:null;t=null}else e!==t&&(t=null)}}return or=t,null}function jd(t){switch(t){case"beforetoggle":case"cancel":case"click":case"close":case"contextmenu":case"copy":case"cut":case"auxclick":case"dblclick":case"dragend":case"dragstart":case"drop":case"focusin":case"focusout":case"input":case"invalid":case"keydown":case"keypress":case"keyup":case"mousedown":case"mouseup":case"paste":case"pause":case"play":case"pointercancel":case"pointerdown":case"pointerup":case"ratechange":case"reset":case"resize":case"seeked":case"submit":case"toggle":case"touchcancel":case"touchend":case"touchstart":case"volumechange":case"change":case"selectionchange":case"textInput":case"compositionstart":case"compositionend":case"compositionupdate":case"beforeblur":case"afterblur":case"beforeinput":case"blur":case"fullscreenchange":case"focus":case"hashchange":case"popstate":case"select":case"selectstart":return 2;case"drag":case"dragenter":case"dragexit":case"dragleave":case"dragover":case"mousemove":case"mouseout":case"mouseover":case"pointermove":case"pointerout":case"pointerover":case"scroll":case"touchmove":case"wheel":case"mouseenter":case"mouseleave":case"pointerenter":case"pointerleave":return 8;case"message":switch(O()){case q:return 2;case ht:return 8;case mt:case K:return 32;case pt:return 268435456;default:return 32}default:return 32}}var Ds=!1,qa=null,Ya=null,Za=null,Vl=new Map,Kl=new Map,Ga=[],k0="mousedown mouseup touchcancel touchend touchstart auxclick dblclick pointercancel pointerdown pointerup dragend dragstart drop compositionend compositionstart keydown keypress keyup input textInput copy cut paste click change contextmenu reset".split(" ");function Bd(t,e){switch(t){case"focusin":case"focusout":qa=null;break;case"dragenter":case"dragleave":Ya=null;break;case"mouseover":case"mouseout":Za=null;break;case"pointerover":case"pointerout":Vl.delete(e.pointerId);break;case"gotpointercapture":case"lostpointercapture":Kl.delete(e.pointerId)}}function Jl(t,e,a,n,l,i){return t===null||t.nativeEvent!==i?(t={blockedOn:e,domEventName:a,eventSystemFlags:n,nativeEvent:i,targetContainers:[l]},e!==null&&(e=yn(e),e!==null&&Md(e)),t):(t.eventSystemFlags|=n,e=t.targetContainers,l!==null&&e.indexOf(l)===-1&&e.push(l),t)}function C0(t,e,a,n,l){switch(e){case"focusin":return qa=Jl(qa,t,e,a,n,l),!0;case"dragenter":return Ya=Jl(Ya,t,e,a,n,l),!0;case"mouseover":return Za=Jl(Za,t,e,a,n,l),!0;case"pointerover":var i=l.pointerId;return Vl.set(i,Jl(Vl.get(i)||null,t,e,a,n,l)),!0;case"gotpointercapture":return i=l.pointerId,Kl.set(i,Jl(Kl.get(i)||null,t,e,a,n,l)),!0}return!1}function Hd(t){var e=gn(t.target);if(e!==null){var a=o(e);if(a!==null){if(e=a.tag,e===13){if(e=y(a),e!==null){t.blockedOn=e,wh(t.priority,function(){if(a.tag===13){var n=De();n=wr(n);var l=On(a,n);l!==null&&Ne(l,a,n),Rs(a,n)}});return}}else if(e===3&&a.stateNode.current.memoizedState.isDehydrated){t.blockedOn=a.tag===3?a.stateNode.containerInfo:null;return}}}t.blockedOn=null}function cr(t){if(t.blockedOn!==null)return!1;for(var e=t.targetContainers;0<e.length;){var a=ks(t.nativeEvent);if(a===null){a=t.nativeEvent;var n=new a.constructor(a.type,a);Ur=n,a.target.dispatchEvent(n),Ur=null}else return e=yn(a),e!==null&&Md(e),t.blockedOn=a,!1;e.shift()}return!0}function Ld(t,e,a){cr(t)&&a.delete(e)}function D0(){Ds=!1,qa!==null&&cr(qa)&&(qa=null),Ya!==null&&cr(Ya)&&(Ya=null),Za!==null&&cr(Za)&&(Za=null),Vl.forEach(Ld),Kl.forEach(Ld)}function fr(t,e){t.blockedOn===e&&(t.blockedOn=null,Ds||(Ds=!0,s.unstable_scheduleCallback(s.unstable_NormalPriority,D0)))}var dr=null;function qd(t){dr!==t&&(dr=t,s.unstable_scheduleCallback(s.unstable_NormalPriority,function(){dr===t&&(dr=null);for(var e=0;e<t.length;e+=3){var a=t[e],n=t[e+1],l=t[e+2];if(typeof n!="function"){if(Cs(n||a)===null)continue;break}var i=yn(a);i!==null&&(t.splice(e,3),e-=3,Cu(i,{pending:!0,data:l,method:a.method,action:n},n,l))}}))}function Fl(t){function e(U){return fr(U,t)}qa!==null&&fr(qa,t),Ya!==null&&fr(Ya,t),Za!==null&&fr(Za,t),Vl.forEach(e),Kl.forEach(e);for(var a=0;a<Ga.length;a++){var n=Ga[a];n.blockedOn===t&&(n.blockedOn=null)}for(;0<Ga.length&&(a=Ga[0],a.blockedOn===null);)Hd(a),a.blockedOn===null&&Ga.shift();if(a=(t.ownerDocument||t).$$reactFormReplay,a!=null)for(n=0;n<a.length;n+=3){var l=a[n],i=a[n+1],u=l[xe]||null;if(typeof i=="function")u||qd(a);else if(u){var x=null;if(i&&i.hasAttribute("formAction")){if(l=i,u=i[xe]||null)x=u.formAction;else if(Cs(l)!==null)continue}else x=u.action;typeof x=="function"?a[n+1]=x:(a.splice(n,3),n-=3),qd(a)}}}function Ns(t){this._internalRoot=t}hr.prototype.render=Ns.prototype.render=function(t){var e=this._internalRoot;if(e===null)throw Error(d(409));var a=e.current,n=De();Nd(a,n,t,e,null,null)},hr.prototype.unmount=Ns.prototype.unmount=function(){var t=this._internalRoot;if(t!==null){this._internalRoot=null;var e=t.containerInfo;Nd(t.current,2,null,t,null,null),Ji(),e[pn]=null}};function hr(t){this._internalRoot=t}hr.prototype.unstable_scheduleHydration=function(t){if(t){var e=eo();t={blockedOn:null,target:t,priority:e};for(var a=0;a<Ga.length&&e!==0&&e<Ga[a].priority;a++);Ga.splice(a,0,t),a===0&&Hd(t)}};var Yd=R.version;if(Yd!=="19.1.1")throw Error(d(527,Yd,"19.1.1"));X.findDOMNode=function(t){var e=t._reactInternals;if(e===void 0)throw typeof t.render=="function"?Error(d(188)):(t=Object.keys(t).join(","),Error(d(268,t)));return t=h(e),t=t!==null?r(t):null,t=t===null?null:t.stateNode,t};var N0={bundleType:0,version:"19.1.1",rendererPackageName:"react-dom",currentDispatcherRef:G,reconcilerVersion:"19.1.1"};if(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__<"u"){var mr=__REACT_DEVTOOLS_GLOBAL_HOOK__;if(!mr.isDisabled&&mr.supportsFiber)try{Et=mr.inject(N0),kt=mr}catch{}}return $l.createRoot=function(t,e){if(!p(t))throw Error(d(299));var a=!1,n="",l=af,i=nf,u=lf,x=null;return e!=null&&(e.unstable_strictMode===!0&&(a=!0),e.identifierPrefix!==void 0&&(n=e.identifierPrefix),e.onUncaughtError!==void 0&&(l=e.onUncaughtError),e.onCaughtError!==void 0&&(i=e.onCaughtError),e.onRecoverableError!==void 0&&(u=e.onRecoverableError),e.unstable_transitionCallbacks!==void 0&&(x=e.unstable_transitionCallbacks)),e=Cd(t,1,!1,null,null,a,n,l,i,u,x,null),t[pn]=e.current,ps(t),new Ns(e)},$l.hydrateRoot=function(t,e,a){if(!p(t))throw Error(d(299));var n=!1,l="",i=af,u=nf,x=lf,U=null,V=null;return a!=null&&(a.unstable_strictMode===!0&&(n=!0),a.identifierPrefix!==void 0&&(l=a.identifierPrefix),a.onUncaughtError!==void 0&&(i=a.onUncaughtError),a.onCaughtError!==void 0&&(u=a.onCaughtError),a.onRecoverableError!==void 0&&(x=a.onRecoverableError),a.unstable_transitionCallbacks!==void 0&&(U=a.unstable_transitionCallbacks),a.formState!==void 0&&(V=a.formState)),e=Cd(t,1,!0,e,a??null,n,l,i,u,x,U,V),e.context=Dd(null),a=e.current,n=De(),n=wr(n),l=za(n),l.callback=null,Ta(a,l,n),a=n,e.current.lanes=a,Qe(e,a),Pe(e),t[pn]=e.current,ps(t),new hr(e)},$l.version="19.1.1",$l}var $d;function X0(){if($d)return js.exports;$d=1;function s(){if(!(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__>"u"||typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE!="function"))try{__REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE(s)}catch(R){console.error(R)}}return s(),js.exports=G0(),js.exports}var Q0=X0();/**
 * react-router v7.8.0
 *
 * Copyright (c) Remix Software Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE.md file in the root directory of this source tree.
 *
 * @license MIT
 */var Id="popstate";function V0(s={}){function R(d,p){let{pathname:o,search:y,hash:m}=d.location;return Gs("",{pathname:o,search:y,hash:m},p.state&&p.state.usr||null,p.state&&p.state.key||"default")}function c(d,p){return typeof p=="string"?p:Pl(p)}return J0(R,c,null,s)}function It(s,R){if(s===!1||s===null||typeof s>"u")throw new Error(R)}function ta(s,R){if(!s){typeof console<"u"&&console.warn(R);try{throw new Error(R)}catch{}}}function K0(){return Math.random().toString(36).substring(2,10)}function Pd(s,R){return{usr:s.state,key:s.key,idx:R}}function Gs(s,R,c=null,d){return{pathname:typeof s=="string"?s:s.pathname,search:"",hash:"",...typeof R=="string"?tl(R):R,state:c,key:R&&R.key||d||K0()}}function Pl({pathname:s="/",search:R="",hash:c=""}){return R&&R!=="?"&&(s+=R.charAt(0)==="?"?R:"?"+R),c&&c!=="#"&&(s+=c.charAt(0)==="#"?c:"#"+c),s}function tl(s){let R={};if(s){let c=s.indexOf("#");c>=0&&(R.hash=s.substring(c),s=s.substring(0,c));let d=s.indexOf("?");d>=0&&(R.search=s.substring(d),s=s.substring(0,d)),s&&(R.pathname=s)}return R}function J0(s,R,c,d={}){let{window:p=document.defaultView,v5Compat:o=!1}=d,y=p.history,m="POP",h=null,r=E();r==null&&(r=0,y.replaceState({...y.state,idx:r},""));function E(){return(y.state||{idx:null}).idx}function A(){m="POP";let S=E(),v=S==null?null:S-r;r=S,h&&h({action:m,location:g.location,delta:v})}function w(S,v){m="PUSH";let T=Gs(g.location,S,v);r=E()+1;let D=Pd(T,r),L=g.createHref(T);try{y.pushState(D,"",L)}catch(j){if(j instanceof DOMException&&j.name==="DataCloneError")throw j;p.location.assign(L)}o&&h&&h({action:m,location:g.location,delta:1})}function _(S,v){m="REPLACE";let T=Gs(g.location,S,v);r=E();let D=Pd(T,r),L=g.createHref(T);y.replaceState(D,"",L),o&&h&&h({action:m,location:g.location,delta:0})}function z(S){return F0(S)}let g={get action(){return m},get location(){return s(p,y)},listen(S){if(h)throw new Error("A history only accepts one active listener");return p.addEventListener(Id,A),h=S,()=>{p.removeEventListener(Id,A),h=null}},createHref(S){return R(p,S)},createURL:z,encodeLocation(S){let v=z(S);return{pathname:v.pathname,search:v.search,hash:v.hash}},push:w,replace:_,go(S){return y.go(S)}};return g}function F0(s,R=!1){let c="http://localhost";typeof window<"u"&&(c=window.location.origin!=="null"?window.location.origin:window.location.href),It(c,"No window.location.(origin|href) available to create URL");let d=typeof s=="string"?s:Pl(s);return d=d.replace(/ $/,"%20"),!R&&d.startsWith("//")&&(d=c+d),new URL(d,c)}function ih(s,R,c="/"){return W0(s,R,c,!1)}function W0(s,R,c,d){let p=typeof R=="string"?tl(R):R,o=ba(p.pathname||"/",c);if(o==null)return null;let y=rh(s);$0(y);let m=null;for(let h=0;m==null&&h<y.length;++h){let r=sp(o);m=rp(y[h],r,d)}return m}function rh(s,R=[],c=[],d=""){let p=(o,y,m)=>{let h={relativePath:m===void 0?o.path||"":m,caseSensitive:o.caseSensitive===!0,childrenIndex:y,route:o};h.relativePath.startsWith("/")&&(It(h.relativePath.startsWith(d),`Absolute route path "${h.relativePath}" nested under path "${d}" is not valid. An absolute child route path must start with the combined path of all its parent routes.`),h.relativePath=h.relativePath.slice(d.length));let r=va([d,h.relativePath]),E=c.concat(h);o.children&&o.children.length>0&&(It(o.index!==!0,`Index routes must not have child routes. Please remove all child routes from route path "${r}".`),rh(o.children,R,E,r)),!(o.path==null&&!o.index)&&R.push({path:r,score:lp(r,o.index),routesMeta:E})};return s.forEach((o,y)=>{if(o.path===""||!o.path?.includes("?"))p(o,y);else for(let m of uh(o.path))p(o,y,m)}),R}function uh(s){let R=s.split("/");if(R.length===0)return[];let[c,...d]=R,p=c.endsWith("?"),o=c.replace(/\?$/,"");if(d.length===0)return p?[o,""]:[o];let y=uh(d.join("/")),m=[];return m.push(...y.map(h=>h===""?o:[o,h].join("/"))),p&&m.push(...y),m.map(h=>s.startsWith("/")&&h===""?"/":h)}function $0(s){s.sort((R,c)=>R.score!==c.score?c.score-R.score:ip(R.routesMeta.map(d=>d.childrenIndex),c.routesMeta.map(d=>d.childrenIndex)))}var I0=/^:[\w-]+$/,P0=3,tp=2,ep=1,ap=10,np=-2,th=s=>s==="*";function lp(s,R){let c=s.split("/"),d=c.length;return c.some(th)&&(d+=np),R&&(d+=tp),c.filter(p=>!th(p)).reduce((p,o)=>p+(I0.test(o)?P0:o===""?ep:ap),d)}function ip(s,R){return s.length===R.length&&s.slice(0,-1).every((d,p)=>d===R[p])?s[s.length-1]-R[R.length-1]:0}function rp(s,R,c=!1){let{routesMeta:d}=s,p={},o="/",y=[];for(let m=0;m<d.length;++m){let h=d[m],r=m===d.length-1,E=o==="/"?R:R.slice(o.length)||"/",A=_r({path:h.relativePath,caseSensitive:h.caseSensitive,end:r},E),w=h.route;if(!A&&r&&c&&!d[d.length-1].route.index&&(A=_r({path:h.relativePath,caseSensitive:h.caseSensitive,end:!1},E)),!A)return null;Object.assign(p,A.params),y.push({params:p,pathname:va([o,A.pathname]),pathnameBase:dp(va([o,A.pathnameBase])),route:w}),A.pathnameBase!=="/"&&(o=va([o,A.pathnameBase]))}return y}function _r(s,R){typeof s=="string"&&(s={path:s,caseSensitive:!1,end:!0});let[c,d]=up(s.path,s.caseSensitive,s.end),p=R.match(c);if(!p)return null;let o=p[0],y=o.replace(/(.)\/+$/,"$1"),m=p.slice(1);return{params:d.reduce((r,{paramName:E,isOptional:A},w)=>{if(E==="*"){let z=m[w]||"";y=o.slice(0,o.length-z.length).replace(/(.)\/+$/,"$1")}const _=m[w];return A&&!_?r[E]=void 0:r[E]=(_||"").replace(/%2F/g,"/"),r},{}),pathname:o,pathnameBase:y,pattern:s}}function up(s,R=!1,c=!0){ta(s==="*"||!s.endsWith("*")||s.endsWith("/*"),`Route path "${s}" will be treated as if it were "${s.replace(/\*$/,"/*")}" because the \`*\` character must always follow a \`/\` in the pattern. To get rid of this warning, please change the route path to "${s.replace(/\*$/,"/*")}".`);let d=[],p="^"+s.replace(/\/*\*?$/,"").replace(/^\/*/,"/").replace(/[\\.*+^${}|()[\]]/g,"\\$&").replace(/\/:([\w-]+)(\?)?/g,(y,m,h)=>(d.push({paramName:m,isOptional:h!=null}),h?"/?([^\\/]+)?":"/([^\\/]+)"));return s.endsWith("*")?(d.push({paramName:"*"}),p+=s==="*"||s==="/*"?"(.*)$":"(?:\\/(.+)|\\/*)$"):c?p+="\\/*$":s!==""&&s!=="/"&&(p+="(?:(?=\\/|$))"),[new RegExp(p,R?void 0:"i"),d]}function sp(s){try{return s.split("/").map(R=>decodeURIComponent(R).replace(/\//g,"%2F")).join("/")}catch(R){return ta(!1,`The URL path "${s}" could not be decoded because it is a malformed URL segment. This is probably due to a bad percent encoding (${R}).`),s}}function ba(s,R){if(R==="/")return s;if(!s.toLowerCase().startsWith(R.toLowerCase()))return null;let c=R.endsWith("/")?R.length-1:R.length,d=s.charAt(c);return d&&d!=="/"?null:s.slice(c)||"/"}function op(s,R="/"){let{pathname:c,search:d="",hash:p=""}=typeof s=="string"?tl(s):s;return{pathname:c?c.startsWith("/")?c:cp(c,R):R,search:hp(d),hash:mp(p)}}function cp(s,R){let c=R.replace(/\/+$/,"").split("/");return s.split("/").forEach(p=>{p===".."?c.length>1&&c.pop():p!=="."&&c.push(p)}),c.length>1?c.join("/"):"/"}function qs(s,R,c,d){return`Cannot include a '${s}' character in a manually specified \`to.${R}\` field [${JSON.stringify(d)}].  Please separate it out to the \`to.${c}\` field. Alternatively you may provide the full path as a string in <Link to="..."> and the router will parse it for you.`}function fp(s){return s.filter((R,c)=>c===0||R.route.path&&R.route.path.length>0)}function sh(s){let R=fp(s);return R.map((c,d)=>d===R.length-1?c.pathname:c.pathnameBase)}function oh(s,R,c,d=!1){let p;typeof s=="string"?p=tl(s):(p={...s},It(!p.pathname||!p.pathname.includes("?"),qs("?","pathname","search",p)),It(!p.pathname||!p.pathname.includes("#"),qs("#","pathname","hash",p)),It(!p.search||!p.search.includes("#"),qs("#","search","hash",p)));let o=s===""||p.pathname==="",y=o?"/":p.pathname,m;if(y==null)m=c;else{let A=R.length-1;if(!d&&y.startsWith("..")){let w=y.split("/");for(;w[0]==="..";)w.shift(),A-=1;p.pathname=w.join("/")}m=A>=0?R[A]:"/"}let h=op(p,m),r=y&&y!=="/"&&y.endsWith("/"),E=(o||y===".")&&c.endsWith("/");return!h.pathname.endsWith("/")&&(r||E)&&(h.pathname+="/"),h}var va=s=>s.join("/").replace(/\/\/+/g,"/"),dp=s=>s.replace(/\/+$/,"").replace(/^\/*/,"/"),hp=s=>!s||s==="?"?"":s.startsWith("?")?s:"?"+s,mp=s=>!s||s==="#"?"":s.startsWith("#")?s:"#"+s;function pp(s){return s!=null&&typeof s.status=="number"&&typeof s.statusText=="string"&&typeof s.internal=="boolean"&&"data"in s}var ch=["POST","PUT","PATCH","DELETE"];new Set(ch);var gp=["GET",...ch];new Set(gp);var el=I.createContext(null);el.displayName="DataRouter";var Sr=I.createContext(null);Sr.displayName="DataRouterState";I.createContext(!1);var fh=I.createContext({isTransitioning:!1});fh.displayName="ViewTransition";var yp=I.createContext(new Map);yp.displayName="Fetchers";var vp=I.createContext(null);vp.displayName="Await";var ea=I.createContext(null);ea.displayName="Navigation";var ti=I.createContext(null);ti.displayName="Location";var _a=I.createContext({outlet:null,matches:[],isDataRoute:!1});_a.displayName="Route";var Ks=I.createContext(null);Ks.displayName="RouteError";function bp(s,{relative:R}={}){It(ei(),"useHref() may be used only in the context of a <Router> component.");let{basename:c,navigator:d}=I.useContext(ea),{hash:p,pathname:o,search:y}=ai(s,{relative:R}),m=o;return c!=="/"&&(m=o==="/"?c:va([c,o])),d.createHref({pathname:m,search:y,hash:p})}function ei(){return I.useContext(ti)!=null}function dn(){return It(ei(),"useLocation() may be used only in the context of a <Router> component."),I.useContext(ti).location}var dh="You should call navigate() in a React.useEffect(), not when your component is first rendered.";function hh(s){I.useContext(ea).static||I.useLayoutEffect(s)}function Js(){let{isDataRoute:s}=I.useContext(_a);return s?Dp():_p()}function _p(){It(ei(),"useNavigate() may be used only in the context of a <Router> component.");let s=I.useContext(el),{basename:R,navigator:c}=I.useContext(ea),{matches:d}=I.useContext(_a),{pathname:p}=dn(),o=JSON.stringify(sh(d)),y=I.useRef(!1);return hh(()=>{y.current=!0}),I.useCallback((h,r={})=>{if(ta(y.current,dh),!y.current)return;if(typeof h=="number"){c.go(h);return}let E=oh(h,JSON.parse(o),p,r.relative==="path");s==null&&R!=="/"&&(E.pathname=E.pathname==="/"?R:va([R,E.pathname])),(r.replace?c.replace:c.push)(E,r.state,r)},[R,c,o,p,s])}I.createContext(null);function ai(s,{relative:R}={}){let{matches:c}=I.useContext(_a),{pathname:d}=dn(),p=JSON.stringify(sh(c));return I.useMemo(()=>oh(s,JSON.parse(p),d,R==="path"),[s,p,d,R])}function Sp(s,R){return mh(s,R)}function mh(s,R,c,d){It(ei(),"useRoutes() may be used only in the context of a <Router> component.");let{navigator:p}=I.useContext(ea),{matches:o}=I.useContext(_a),y=o[o.length-1],m=y?y.params:{},h=y?y.pathname:"/",r=y?y.pathnameBase:"/",E=y&&y.route;{let v=E&&E.path||"";ph(h,!E||v.endsWith("*")||v.endsWith("*?"),`You rendered descendant <Routes> (or called \`useRoutes()\`) at "${h}" (under <Route path="${v}">) but the parent route path has no trailing "*". This means if you navigate deeper, the parent won't match anymore and therefore the child routes will never render.

Please change the parent <Route path="${v}"> to <Route path="${v==="/"?"*":`${v}/*`}">.`)}let A=dn(),w;if(R){let v=typeof R=="string"?tl(R):R;It(r==="/"||v.pathname?.startsWith(r),`When overriding the location using \`<Routes location>\` or \`useRoutes(routes, location)\`, the location pathname must begin with the portion of the URL pathname that was matched by all parent routes. The current pathname base is "${r}" but pathname "${v.pathname}" was given in the \`location\` prop.`),w=v}else w=A;let _=w.pathname||"/",z=_;if(r!=="/"){let v=r.replace(/^\//,"").split("/");z="/"+_.replace(/^\//,"").split("/").slice(v.length).join("/")}let g=ih(s,{pathname:z});ta(E||g!=null,`No routes matched location "${w.pathname}${w.search}${w.hash}" `),ta(g==null||g[g.length-1].route.element!==void 0||g[g.length-1].route.Component!==void 0||g[g.length-1].route.lazy!==void 0,`Matched leaf route at location "${w.pathname}${w.search}${w.hash}" does not have an element or Component. This means it will render an <Outlet /> with a null value by default resulting in an "empty" page.`);let S=zp(g&&g.map(v=>Object.assign({},v,{params:Object.assign({},m,v.params),pathname:va([r,p.encodeLocation?p.encodeLocation(v.pathname).pathname:v.pathname]),pathnameBase:v.pathnameBase==="/"?r:va([r,p.encodeLocation?p.encodeLocation(v.pathnameBase).pathname:v.pathnameBase])})),o,c,d);return R&&S?I.createElement(ti.Provider,{value:{location:{pathname:"/",search:"",hash:"",state:null,key:"default",...w},navigationType:"POP"}},S):S}function xp(){let s=Cp(),R=pp(s)?`${s.status} ${s.statusText}`:s instanceof Error?s.message:JSON.stringify(s),c=s instanceof Error?s.stack:null,d="rgba(200,200,200, 0.5)",p={padding:"0.5rem",backgroundColor:d},o={padding:"2px 4px",backgroundColor:d},y=null;return console.error("Error handled by React Router default ErrorBoundary:",s),y=I.createElement(I.Fragment,null,I.createElement("p",null,"💿 Hey developer 👋"),I.createElement("p",null,"You can provide a way better UX than this when your app throws errors by providing your own ",I.createElement("code",{style:o},"ErrorBoundary")," or"," ",I.createElement("code",{style:o},"errorElement")," prop on your route.")),I.createElement(I.Fragment,null,I.createElement("h2",null,"Unexpected Application Error!"),I.createElement("h3",{style:{fontStyle:"italic"}},R),c?I.createElement("pre",{style:p},c):null,y)}var Ep=I.createElement(xp,null),wp=class extends I.Component{constructor(s){super(s),this.state={location:s.location,revalidation:s.revalidation,error:s.error}}static getDerivedStateFromError(s){return{error:s}}static getDerivedStateFromProps(s,R){return R.location!==s.location||R.revalidation!=="idle"&&s.revalidation==="idle"?{error:s.error,location:s.location,revalidation:s.revalidation}:{error:s.error!==void 0?s.error:R.error,location:R.location,revalidation:s.revalidation||R.revalidation}}componentDidCatch(s,R){console.error("React Router caught the following error during render",s,R)}render(){return this.state.error!==void 0?I.createElement(_a.Provider,{value:this.props.routeContext},I.createElement(Ks.Provider,{value:this.state.error,children:this.props.component})):this.props.children}};function Ap({routeContext:s,match:R,children:c}){let d=I.useContext(el);return d&&d.static&&d.staticContext&&(R.route.errorElement||R.route.ErrorBoundary)&&(d.staticContext._deepestRenderedBoundaryId=R.route.id),I.createElement(_a.Provider,{value:s},c)}function zp(s,R=[],c=null,d=null){if(s==null){if(!c)return null;if(c.errors)s=c.matches;else if(R.length===0&&!c.initialized&&c.matches.length>0)s=c.matches;else return null}let p=s,o=c?.errors;if(o!=null){let h=p.findIndex(r=>r.route.id&&o?.[r.route.id]!==void 0);It(h>=0,`Could not find a matching route for errors on route IDs: ${Object.keys(o).join(",")}`),p=p.slice(0,Math.min(p.length,h+1))}let y=!1,m=-1;if(c)for(let h=0;h<p.length;h++){let r=p[h];if((r.route.HydrateFallback||r.route.hydrateFallbackElement)&&(m=h),r.route.id){let{loaderData:E,errors:A}=c,w=r.route.loader&&!E.hasOwnProperty(r.route.id)&&(!A||A[r.route.id]===void 0);if(r.route.lazy||w){y=!0,m>=0?p=p.slice(0,m+1):p=[p[0]];break}}}return p.reduceRight((h,r,E)=>{let A,w=!1,_=null,z=null;c&&(A=o&&r.route.id?o[r.route.id]:void 0,_=r.route.errorElement||Ep,y&&(m<0&&E===0?(ph("route-fallback",!1,"No `HydrateFallback` element provided to render during initial hydration"),w=!0,z=null):m===E&&(w=!0,z=r.route.hydrateFallbackElement||null)));let g=R.concat(p.slice(0,E+1)),S=()=>{let v;return A?v=_:w?v=z:r.route.Component?v=I.createElement(r.route.Component,null):r.route.element?v=r.route.element:v=h,I.createElement(Ap,{match:r,routeContext:{outlet:h,matches:g,isDataRoute:c!=null},children:v})};return c&&(r.route.ErrorBoundary||r.route.errorElement||E===0)?I.createElement(wp,{location:c.location,revalidation:c.revalidation,component:_,error:A,children:S(),routeContext:{outlet:null,matches:g,isDataRoute:!0}}):S()},null)}function Fs(s){return`${s} must be used within a data router.  See https://reactrouter.com/en/main/routers/picking-a-router.`}function Tp(s){let R=I.useContext(el);return It(R,Fs(s)),R}function Rp(s){let R=I.useContext(Sr);return It(R,Fs(s)),R}function Op(s){let R=I.useContext(_a);return It(R,Fs(s)),R}function Ws(s){let R=Op(s),c=R.matches[R.matches.length-1];return It(c.route.id,`${s} can only be used on routes that contain a unique "id"`),c.route.id}function kp(){return Ws("useRouteId")}function Cp(){let s=I.useContext(Ks),R=Rp("useRouteError"),c=Ws("useRouteError");return s!==void 0?s:R.errors?.[c]}function Dp(){let{router:s}=Tp("useNavigate"),R=Ws("useNavigate"),c=I.useRef(!1);return hh(()=>{c.current=!0}),I.useCallback(async(p,o={})=>{ta(c.current,dh),c.current&&(typeof p=="number"?s.navigate(p):await s.navigate(p,{fromRouteId:R,...o}))},[s,R])}var eh={};function ph(s,R,c){!R&&!eh[s]&&(eh[s]=!0,ta(!1,c))}I.memo(Np);function Np({routes:s,future:R,state:c}){return mh(s,void 0,c,R)}function Xs(s){It(!1,"A <Route> is only ever to be used as the child of <Routes> element, never rendered directly. Please wrap your <Route> in a <Routes>.")}function Up({basename:s="/",children:R=null,location:c,navigationType:d="POP",navigator:p,static:o=!1}){It(!ei(),"You cannot render a <Router> inside another <Router>. You should never have more than one in your app.");let y=s.replace(/^\/*/,"/"),m=I.useMemo(()=>({basename:y,navigator:p,static:o,future:{}}),[y,p,o]);typeof c=="string"&&(c=tl(c));let{pathname:h="/",search:r="",hash:E="",state:A=null,key:w="default"}=c,_=I.useMemo(()=>{let z=ba(h,y);return z==null?null:{location:{pathname:z,search:r,hash:E,state:A,key:w},navigationType:d}},[y,h,r,E,A,w,d]);return ta(_!=null,`<Router basename="${y}"> is not able to match the URL "${h}${r}${E}" because it does not start with the basename, so the <Router> won't render anything.`),_==null?null:I.createElement(ea.Provider,{value:m},I.createElement(ti.Provider,{children:R,value:_}))}function Mp({children:s,location:R}){return Sp(Qs(s),R)}function Qs(s,R=[]){let c=[];return I.Children.forEach(s,(d,p)=>{if(!I.isValidElement(d))return;let o=[...R,p];if(d.type===I.Fragment){c.push.apply(c,Qs(d.props.children,o));return}It(d.type===Xs,`[${typeof d.type=="string"?d.type:d.type.name}] is not a <Route> component. All component children of <Routes> must be a <Route> or <React.Fragment>`),It(!d.props.index||!d.props.children,"An index route cannot have child routes.");let y={id:d.props.id||o.join("-"),caseSensitive:d.props.caseSensitive,element:d.props.element,Component:d.props.Component,index:d.props.index,path:d.props.path,loader:d.props.loader,action:d.props.action,hydrateFallbackElement:d.props.hydrateFallbackElement,HydrateFallback:d.props.HydrateFallback,errorElement:d.props.errorElement,ErrorBoundary:d.props.ErrorBoundary,hasErrorBoundary:d.props.hasErrorBoundary===!0||d.props.ErrorBoundary!=null||d.props.errorElement!=null,shouldRevalidate:d.props.shouldRevalidate,handle:d.props.handle,lazy:d.props.lazy};d.props.children&&(y.children=Qs(d.props.children,o)),c.push(y)}),c}var yr="get",vr="application/x-www-form-urlencoded";function xr(s){return s!=null&&typeof s.tagName=="string"}function jp(s){return xr(s)&&s.tagName.toLowerCase()==="button"}function Bp(s){return xr(s)&&s.tagName.toLowerCase()==="form"}function Hp(s){return xr(s)&&s.tagName.toLowerCase()==="input"}function Lp(s){return!!(s.metaKey||s.altKey||s.ctrlKey||s.shiftKey)}function qp(s,R){return s.button===0&&(!R||R==="_self")&&!Lp(s)}var pr=null;function Yp(){if(pr===null)try{new FormData(document.createElement("form"),0),pr=!1}catch{pr=!0}return pr}var Zp=new Set(["application/x-www-form-urlencoded","multipart/form-data","text/plain"]);function Ys(s){return s!=null&&!Zp.has(s)?(ta(!1,`"${s}" is not a valid \`encType\` for \`<Form>\`/\`<fetcher.Form>\` and will default to "${vr}"`),null):s}function Gp(s,R){let c,d,p,o,y;if(Bp(s)){let m=s.getAttribute("action");d=m?ba(m,R):null,c=s.getAttribute("method")||yr,p=Ys(s.getAttribute("enctype"))||vr,o=new FormData(s)}else if(jp(s)||Hp(s)&&(s.type==="submit"||s.type==="image")){let m=s.form;if(m==null)throw new Error('Cannot submit a <button> or <input type="submit"> without a <form>');let h=s.getAttribute("formaction")||m.getAttribute("action");if(d=h?ba(h,R):null,c=s.getAttribute("formmethod")||m.getAttribute("method")||yr,p=Ys(s.getAttribute("formenctype"))||Ys(m.getAttribute("enctype"))||vr,o=new FormData(m,s),!Yp()){let{name:r,type:E,value:A}=s;if(E==="image"){let w=r?`${r}.`:"";o.append(`${w}x`,"0"),o.append(`${w}y`,"0")}else r&&o.append(r,A)}}else{if(xr(s))throw new Error('Cannot submit element that is not <form>, <button>, or <input type="submit|image">');c=yr,d=null,p=vr,y=s}return o&&p==="text/plain"&&(y=o,o=void 0),{action:d,method:c.toLowerCase(),encType:p,formData:o,body:y}}Object.getOwnPropertyNames(Object.prototype).sort().join("\0");function $s(s,R){if(s===!1||s===null||typeof s>"u")throw new Error(R)}function Xp(s,R,c){let d=typeof s=="string"?new URL(s,typeof window>"u"?"server://singlefetch/":window.location.origin):s;return d.pathname==="/"?d.pathname=`_root.${c}`:R&&ba(d.pathname,R)==="/"?d.pathname=`${R.replace(/\/$/,"")}/_root.${c}`:d.pathname=`${d.pathname.replace(/\/$/,"")}.${c}`,d}async function Qp(s,R){if(s.id in R)return R[s.id];try{let c=await import(s.module);return R[s.id]=c,c}catch(c){return console.error(`Error loading route module \`${s.module}\`, reloading page...`),console.error(c),window.__reactRouterContext&&window.__reactRouterContext.isSpaMode,window.location.reload(),new Promise(()=>{})}}function Vp(s){return s==null?!1:s.href==null?s.rel==="preload"&&typeof s.imageSrcSet=="string"&&typeof s.imageSizes=="string":typeof s.rel=="string"&&typeof s.href=="string"}async function Kp(s,R,c){let d=await Promise.all(s.map(async p=>{let o=R.routes[p.route.id];if(o){let y=await Qp(o,c);return y.links?y.links():[]}return[]}));return $p(d.flat(1).filter(Vp).filter(p=>p.rel==="stylesheet"||p.rel==="preload").map(p=>p.rel==="stylesheet"?{...p,rel:"prefetch",as:"style"}:{...p,rel:"prefetch"}))}function ah(s,R,c,d,p,o){let y=(h,r)=>c[r]?h.route.id!==c[r].route.id:!0,m=(h,r)=>c[r].pathname!==h.pathname||c[r].route.path?.endsWith("*")&&c[r].params["*"]!==h.params["*"];return o==="assets"?R.filter((h,r)=>y(h,r)||m(h,r)):o==="data"?R.filter((h,r)=>{let E=d.routes[h.route.id];if(!E||!E.hasLoader)return!1;if(y(h,r)||m(h,r))return!0;if(h.route.shouldRevalidate){let A=h.route.shouldRevalidate({currentUrl:new URL(p.pathname+p.search+p.hash,window.origin),currentParams:c[0]?.params||{},nextUrl:new URL(s,window.origin),nextParams:h.params,defaultShouldRevalidate:!0});if(typeof A=="boolean")return A}return!0}):[]}function Jp(s,R,{includeHydrateFallback:c}={}){return Fp(s.map(d=>{let p=R.routes[d.route.id];if(!p)return[];let o=[p.module];return p.clientActionModule&&(o=o.concat(p.clientActionModule)),p.clientLoaderModule&&(o=o.concat(p.clientLoaderModule)),c&&p.hydrateFallbackModule&&(o=o.concat(p.hydrateFallbackModule)),p.imports&&(o=o.concat(p.imports)),o}).flat(1))}function Fp(s){return[...new Set(s)]}function Wp(s){let R={},c=Object.keys(s).sort();for(let d of c)R[d]=s[d];return R}function $p(s,R){let c=new Set;return new Set(R),s.reduce((d,p)=>{let o=JSON.stringify(Wp(p));return c.has(o)||(c.add(o),d.push({key:o,link:p})),d},[])}function gh(){let s=I.useContext(el);return $s(s,"You must render this element inside a <DataRouterContext.Provider> element"),s}function Ip(){let s=I.useContext(Sr);return $s(s,"You must render this element inside a <DataRouterStateContext.Provider> element"),s}var Is=I.createContext(void 0);Is.displayName="FrameworkContext";function yh(){let s=I.useContext(Is);return $s(s,"You must render this element inside a <HydratedRouter> element"),s}function Pp(s,R){let c=I.useContext(Is),[d,p]=I.useState(!1),[o,y]=I.useState(!1),{onFocus:m,onBlur:h,onMouseEnter:r,onMouseLeave:E,onTouchStart:A}=R,w=I.useRef(null);I.useEffect(()=>{if(s==="render"&&y(!0),s==="viewport"){let g=v=>{v.forEach(T=>{y(T.isIntersecting)})},S=new IntersectionObserver(g,{threshold:.5});return w.current&&S.observe(w.current),()=>{S.disconnect()}}},[s]),I.useEffect(()=>{if(d){let g=setTimeout(()=>{y(!0)},100);return()=>{clearTimeout(g)}}},[d]);let _=()=>{p(!0)},z=()=>{p(!1),y(!1)};return c?s!=="intent"?[o,w,{}]:[o,w,{onFocus:Il(m,_),onBlur:Il(h,z),onMouseEnter:Il(r,_),onMouseLeave:Il(E,z),onTouchStart:Il(A,_)}]:[!1,w,{}]}function Il(s,R){return c=>{s&&s(c),c.defaultPrevented||R(c)}}function tg({page:s,...R}){let{router:c}=gh(),d=I.useMemo(()=>ih(c.routes,s,c.basename),[c.routes,s,c.basename]);return d?I.createElement(ag,{page:s,matches:d,...R}):null}function eg(s){let{manifest:R,routeModules:c}=yh(),[d,p]=I.useState([]);return I.useEffect(()=>{let o=!1;return Kp(s,R,c).then(y=>{o||p(y)}),()=>{o=!0}},[s,R,c]),d}function ag({page:s,matches:R,...c}){let d=dn(),{manifest:p,routeModules:o}=yh(),{basename:y}=gh(),{loaderData:m,matches:h}=Ip(),r=I.useMemo(()=>ah(s,R,h,p,d,"data"),[s,R,h,p,d]),E=I.useMemo(()=>ah(s,R,h,p,d,"assets"),[s,R,h,p,d]),A=I.useMemo(()=>{if(s===d.pathname+d.search+d.hash)return[];let z=new Set,g=!1;if(R.forEach(v=>{let T=p.routes[v.route.id];!T||!T.hasLoader||(!r.some(D=>D.route.id===v.route.id)&&v.route.id in m&&o[v.route.id]?.shouldRevalidate||T.hasClientLoader?g=!0:z.add(v.route.id))}),z.size===0)return[];let S=Xp(s,y,"data");return g&&z.size>0&&S.searchParams.set("_routes",R.filter(v=>z.has(v.route.id)).map(v=>v.route.id).join(",")),[S.pathname+S.search]},[y,m,d,p,r,R,s,o]),w=I.useMemo(()=>Jp(E,p),[E,p]),_=eg(E);return I.createElement(I.Fragment,null,A.map(z=>I.createElement("link",{key:z,rel:"prefetch",as:"fetch",href:z,...c})),w.map(z=>I.createElement("link",{key:z,rel:"modulepreload",href:z,...c})),_.map(({key:z,link:g})=>I.createElement("link",{key:z,nonce:c.nonce,...g})))}function ng(...s){return R=>{s.forEach(c=>{typeof c=="function"?c(R):c!=null&&(c.current=R)})}}var vh=typeof window<"u"&&typeof window.document<"u"&&typeof window.document.createElement<"u";try{vh&&(window.__reactRouterVersion="7.8.0")}catch{}function lg({basename:s,children:R,window:c}){let d=I.useRef();d.current==null&&(d.current=V0({window:c,v5Compat:!0}));let p=d.current,[o,y]=I.useState({action:p.action,location:p.location}),m=I.useCallback(h=>{I.startTransition(()=>y(h))},[y]);return I.useLayoutEffect(()=>p.listen(m),[p,m]),I.createElement(Up,{basename:s,children:R,location:o.location,navigationType:o.action,navigator:p})}var bh=/^(?:[a-z][a-z0-9+.-]*:|\/\/)/i,_h=I.forwardRef(function({onClick:R,discover:c="render",prefetch:d="none",relative:p,reloadDocument:o,replace:y,state:m,target:h,to:r,preventScrollReset:E,viewTransition:A,...w},_){let{basename:z}=I.useContext(ea),g=typeof r=="string"&&bh.test(r),S,v=!1;if(typeof r=="string"&&g&&(S=r,vh))try{let $=new URL(window.location.href),rt=r.startsWith("//")?new URL($.protocol+r):new URL(r),vt=ba(rt.pathname,z);rt.origin===$.origin&&vt!=null?r=vt+rt.search+rt.hash:v=!0}catch{ta(!1,`<Link to="${r}"> contains an invalid URL which will probably break when clicked - please update to a valid URL path.`)}let T=bp(r,{relative:p}),[D,L,j]=Pp(d,w),tt=sg(r,{replace:y,state:m,target:h,preventScrollReset:E,relative:p,viewTransition:A});function F($){R&&R($),$.defaultPrevented||tt($)}let ft=I.createElement("a",{...w,...j,href:S||T,onClick:v||o?R:F,ref:ng(_,L),target:h,"data-discover":!g&&c==="render"?"true":void 0});return D&&!g?I.createElement(I.Fragment,null,ft,I.createElement(tg,{page:T})):ft});_h.displayName="Link";var ig=I.forwardRef(function({"aria-current":R="page",caseSensitive:c=!1,className:d="",end:p=!1,style:o,to:y,viewTransition:m,children:h,...r},E){let A=ai(y,{relative:r.relative}),w=dn(),_=I.useContext(Sr),{navigator:z,basename:g}=I.useContext(ea),S=_!=null&&hg(A)&&m===!0,v=z.encodeLocation?z.encodeLocation(A).pathname:A.pathname,T=w.pathname,D=_&&_.navigation&&_.navigation.location?_.navigation.location.pathname:null;c||(T=T.toLowerCase(),D=D?D.toLowerCase():null,v=v.toLowerCase()),D&&g&&(D=ba(D,g)||D);const L=v!=="/"&&v.endsWith("/")?v.length-1:v.length;let j=T===v||!p&&T.startsWith(v)&&T.charAt(L)==="/",tt=D!=null&&(D===v||!p&&D.startsWith(v)&&D.charAt(v.length)==="/"),F={isActive:j,isPending:tt,isTransitioning:S},ft=j?R:void 0,$;typeof d=="function"?$=d(F):$=[d,j?"active":null,tt?"pending":null,S?"transitioning":null].filter(Boolean).join(" ");let rt=typeof o=="function"?o(F):o;return I.createElement(_h,{...r,"aria-current":ft,className:$,ref:E,style:rt,to:y,viewTransition:m},typeof h=="function"?h(F):h)});ig.displayName="NavLink";var rg=I.forwardRef(({discover:s="render",fetcherKey:R,navigate:c,reloadDocument:d,replace:p,state:o,method:y=yr,action:m,onSubmit:h,relative:r,preventScrollReset:E,viewTransition:A,...w},_)=>{let z=fg(),g=dg(m,{relative:r}),S=y.toLowerCase()==="get"?"get":"post",v=typeof m=="string"&&bh.test(m),T=D=>{if(h&&h(D),D.defaultPrevented)return;D.preventDefault();let L=D.nativeEvent.submitter,j=L?.getAttribute("formmethod")||y;z(L||D.currentTarget,{fetcherKey:R,method:j,navigate:c,replace:p,state:o,relative:r,preventScrollReset:E,viewTransition:A})};return I.createElement("form",{ref:_,method:S,action:g,onSubmit:d?h:T,...w,"data-discover":!v&&s==="render"?"true":void 0})});rg.displayName="Form";function ug(s){return`${s} must be used within a data router.  See https://reactrouter.com/en/main/routers/picking-a-router.`}function Sh(s){let R=I.useContext(el);return It(R,ug(s)),R}function sg(s,{target:R,replace:c,state:d,preventScrollReset:p,relative:o,viewTransition:y}={}){let m=Js(),h=dn(),r=ai(s,{relative:o});return I.useCallback(E=>{if(qp(E,R)){E.preventDefault();let A=c!==void 0?c:Pl(h)===Pl(r);m(s,{replace:A,state:d,preventScrollReset:p,relative:o,viewTransition:y})}},[h,m,r,c,d,R,s,p,o,y])}var og=0,cg=()=>`__${String(++og)}__`;function fg(){let{router:s}=Sh("useSubmit"),{basename:R}=I.useContext(ea),c=kp();return I.useCallback(async(d,p={})=>{let{action:o,method:y,encType:m,formData:h,body:r}=Gp(d,R);if(p.navigate===!1){let E=p.fetcherKey||cg();await s.fetch(E,c,p.action||o,{preventScrollReset:p.preventScrollReset,formData:h,body:r,formMethod:p.method||y,formEncType:p.encType||m,flushSync:p.flushSync})}else await s.navigate(p.action||o,{preventScrollReset:p.preventScrollReset,formData:h,body:r,formMethod:p.method||y,formEncType:p.encType||m,replace:p.replace,state:p.state,fromRouteId:c,flushSync:p.flushSync,viewTransition:p.viewTransition})},[s,R,c])}function dg(s,{relative:R}={}){let{basename:c}=I.useContext(ea),d=I.useContext(_a);It(d,"useFormAction must be used inside a RouteContext");let[p]=d.matches.slice(-1),o={...ai(s||".",{relative:R})},y=dn();if(s==null){o.search=y.search;let m=new URLSearchParams(o.search),h=m.getAll("index");if(h.some(E=>E==="")){m.delete("index"),h.filter(A=>A).forEach(A=>m.append("index",A));let E=m.toString();o.search=E?`?${E}`:""}}return(!s||s===".")&&p.route.index&&(o.search=o.search?o.search.replace(/^\?/,"?index&"):"?index"),c!=="/"&&(o.pathname=o.pathname==="/"?c:va([c,o.pathname])),Pl(o)}function hg(s,{relative:R}={}){let c=I.useContext(fh);It(c!=null,"`useViewTransitionState` must be used within `react-router-dom`'s `RouterProvider`.  Did you accidentally import `RouterProvider` from `react-router`?");let{basename:d}=Sh("useViewTransitionState"),p=ai(s,{relative:R});if(!c.isTransitioning)return!1;let o=ba(c.currentLocation.pathname,d)||c.currentLocation.pathname,y=ba(c.nextLocation.pathname,d)||c.nextLocation.pathname;return _r(p.pathname,y)!=null||_r(p.pathname,o)!=null}const xh=I.createContext(void 0);function mg({children:s}){const[R,c]=I.useState("dark");I.useEffect(()=>{const p=localStorage.getItem("moonlabel_theme");p&&c(p)},[]),I.useEffect(()=>{document.documentElement.classList.remove("light","dark"),document.documentElement.classList.add(R),localStorage.setItem("moonlabel_theme",R)},[R]);const d=()=>{c(p=>p==="dark"?"light":"dark")};return Z.jsx(xh.Provider,{value:{theme:R,toggleTheme:d},children:s})}function Eh(){const s=I.useContext(xh);if(s===void 0)throw new Error("useTheme must be used within a ThemeProvider");return s}function pg({onDetection:s,isProcessing:R}){const[c,d]=I.useState(!1),[p,o]=I.useState([]),[y,m]=I.useState("person"),[h,r]=I.useState(""),[E,A]=I.useState("http://localhost:2020/v1"),[w,_]=I.useState("cloud"),[z,g]=I.useState(!1),[S,v]=I.useState({processed:0,total:0}),T=I.useRef(null);I.useEffect(()=>{const rt=localStorage.getItem("moondream_api_key");rt&&r(rt);const vt=localStorage.getItem("moondream_station_endpoint");vt&&A(vt);const M=localStorage.getItem("moondream_mode");(M==="local"||M==="cloud"||M==="station")&&_(M)},[]);const D=I.useCallback(rt=>{rt.preventDefault(),d(!0)},[]),L=I.useCallback(rt=>{rt.preventDefault(),d(!1)},[]),j=I.useCallback(rt=>{rt.preventDefault(),d(!1);const M=Array.from(rt.dataTransfer.files).filter(et=>et.type.startsWith("image/"));M.length>0&&o(M)},[]),tt=I.useCallback(rt=>{const vt=rt.target.files;if(vt){const M=Array.from(vt);o(M)}},[]),F=()=>{T.current?.click()},ft=async()=>{if(p.length===0||w==="cloud"&&!h.trim()||w==="station"&&!E.trim())return;const rt=y.split(",").map(vt=>vt.trim()).filter(vt=>vt);g(!0),v({processed:0,total:p.length});for(const[vt,M]of p.entries())await s(M,rt,w==="cloud"?h:"",w==="station"?E:void 0),v({processed:vt+1,total:p.length});g(!1)},$=()=>{o([]),T.current&&(T.current.value="")};return Z.jsxs("div",{className:"space-y-6",children:[Z.jsxs("div",{className:"bg-white dark:bg-gray-800 rounded-lg p-6 border border-gray-200 dark:border-gray-700",children:[Z.jsx("h2",{className:"text-xl font-semibold mb-4 flex items-center gap-2",children:"📤 Upload Images"}),Z.jsx("p",{className:"text-gray-600 dark:text-gray-400 mb-6",children:"Drag and drop images or click to select files"}),Z.jsx("div",{className:`border-2 border-dashed rounded-lg p-12 text-center transition-all ${c?"border-blue-400 bg-blue-50 dark:bg-blue-900/20":p.length>0?"border-green-400 bg-green-50 dark:bg-green-900/20":"border-gray-300 dark:border-gray-600 bg-gray-50 dark:bg-gray-700/30"}`,onDragOver:D,onDragLeave:L,onDrop:j,children:p.length>0?Z.jsxs("div",{className:"space-y-4",children:[Z.jsx("div",{className:"text-green-500 dark:text-green-400 text-4xl",children:"✓"}),Z.jsxs("div",{children:[Z.jsx("p",{className:"text-lg font-medium text-green-600 dark:text-green-400",children:"Files selected"}),Z.jsx("p",{className:"text-gray-800 dark:text-gray-300",children:p.map(rt=>rt.name).join(", ")}),Z.jsxs("p",{className:"text-sm text-gray-600 dark:text-gray-400",children:[p.reduce((rt,vt)=>rt+vt.size/1024/1024,0).toFixed(2)," MB"]})]}),Z.jsx("button",{onClick:$,className:"text-blue-600 dark:text-blue-400 hover:text-blue-700 dark:hover:text-blue-300 underline",children:"Choose different files"})]}):Z.jsxs("div",{className:"space-y-4",children:[Z.jsx("div",{className:"text-gray-500 dark:text-gray-400 text-4xl",children:"📤"}),Z.jsxs("div",{children:[Z.jsx("p",{className:"text-lg font-medium text-gray-800 dark:text-white",children:"Drag and drop images here"}),Z.jsx("p",{className:"text-gray-600 dark:text-gray-400",children:"or click to browse files"}),Z.jsx("p",{className:"text-sm text-gray-500 dark:text-gray-500 mt-2",children:"Supports: JPG, PNG, GIF, WebP"})]}),Z.jsx("button",{onClick:F,className:"px-6 py-2 bg-blue-600 hover:bg-blue-700 text-white rounded-lg transition-colors",children:"Browse Files"})]})}),Z.jsx("input",{ref:T,type:"file",accept:"image/*",onChange:tt,className:"hidden",multiple:!0})]}),Z.jsxs("div",{className:"bg-white dark:bg-gray-800 rounded-lg p-6 space-y-4 border border-gray-200 dark:border-gray-700",children:[Z.jsx("h3",{className:"text-lg font-semibold",children:"Detection Configuration"}),Z.jsxs("div",{children:[Z.jsx("label",{className:"block text-sm font-medium mb-2",children:"Objects to detect (comma-separated)"}),Z.jsx("input",{type:"text",value:y,onChange:rt=>m(rt.target.value),placeholder:"person, car, dog",className:"w-full px-3 py-2 bg-gray-100 dark:bg-gray-700 border border-gray-300 dark:border-gray-600 rounded-lg focus:outline-none focus:border-blue-400 text-gray-900 dark:text-white"})]}),w==="cloud"&&!h&&Z.jsx("div",{className:"bg-yellow-50 dark:bg-yellow-900/50 border border-yellow-200 dark:border-yellow-700 rounded-lg p-4",children:Z.jsxs("p",{className:"text-yellow-800 dark:text-yellow-200 text-sm",children:["⚠️ No API key configured. Please go to ",Z.jsx("strong",{children:"Settings"})," to add your API key."]})}),w==="station"&&!E&&Z.jsx("div",{className:"bg-yellow-50 dark:bg-yellow-900/50 border border-yellow-200 dark:border-yellow-700 rounded-lg p-4",children:Z.jsxs("p",{className:"text-yellow-800 dark:text-yellow-200 text-sm",children:["⚠️ No station endpoint configured. Please go to ",Z.jsx("strong",{children:"Settings"})," to configure your Moondream Station endpoint."]})}),w==="station"&&E&&Z.jsx("div",{className:"bg-blue-50 dark:bg-blue-900/20 border border-blue-200 dark:border-blue-700 rounded-lg p-4",children:Z.jsxs("p",{className:"text-blue-800 dark:text-blue-200 text-sm",children:["🚀 Using Moondream Station at ",E,". Make sure your Station is running."]})}),w==="local"&&Z.jsx("div",{className:"bg-green-50 dark:bg-green-900/20 border border-green-200 dark:border-green-700 rounded-lg p-4",children:Z.jsx("p",{className:"text-green-800 dark:text-green-200 text-sm",children:"🖥️ Using local Hugging Face model. Ensure the backend has downloaded it first use."})}),Z.jsxs("button",{onClick:ft,disabled:p.length===0||w==="cloud"&&!h.trim()||w==="station"&&!E.trim()||z||R,className:`w-full py-3 rounded-lg font-medium transition-colors flex items-center justify-center gap-2 ${p.length>0&&(w==="local"||w==="cloud"&&h.trim()||w==="station"&&E.trim())&&!z&&!R?"bg-blue-600 hover:bg-blue-700 text-white":"bg-gray-300 dark:bg-gray-600 text-gray-500 dark:text-gray-400 cursor-not-allowed"}`,children:[(z||R)&&Z.jsx("div",{className:"animate-spin rounded-full h-4 w-4 border-2 border-white border-t-transparent"}),z||R?`Processing ${S.processed}/${S.total}`:"Detect Objects"]}),z&&Z.jsx("div",{className:"w-full bg-gray-200 dark:bg-gray-700 rounded-full h-3 mt-4 overflow-hidden",children:Z.jsx("div",{className:"bg-blue-600 h-full transition-all",style:{width:`${S.processed/S.total*100}%`}})})]})]})}function gr(s){throw new Error('Could not dynamically require "'+s+'". Please configure the dynamicRequireTargets or/and ignoreDynamicRequires option of @rollup/plugin-commonjs appropriately for this require call to work.')}var Zs={exports:{}};/*!

JSZip v3.10.1 - A JavaScript class for generating and reading zip files
<http://stuartk.com/jszip>

(c) 2009-2016 Stuart Knightley <stuart [at] stuartk.com>
Dual licenced under the MIT license or GPLv3. See https://raw.github.com/Stuk/jszip/main/LICENSE.markdown.

JSZip uses the library pako released under the MIT license :
https://github.com/nodeca/pako/blob/main/LICENSE
*/var nh;function gg(){return nh||(nh=1,function(s,R){(function(c){s.exports=c()})(function(){return function c(d,p,o){function y(r,E){if(!p[r]){if(!d[r]){var A=typeof gr=="function"&&gr;if(!E&&A)return A(r,!0);if(m)return m(r,!0);var w=new Error("Cannot find module '"+r+"'");throw w.code="MODULE_NOT_FOUND",w}var _=p[r]={exports:{}};d[r][0].call(_.exports,function(z){var g=d[r][1][z];return y(g||z)},_,_.exports,c,d,p,o)}return p[r].exports}for(var m=typeof gr=="function"&&gr,h=0;h<o.length;h++)y(o[h]);return y}({1:[function(c,d,p){var o=c("./utils"),y=c("./support"),m="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";p.encode=function(h){for(var r,E,A,w,_,z,g,S=[],v=0,T=h.length,D=T,L=o.getTypeOf(h)!=="string";v<h.length;)D=T-v,A=L?(r=h[v++],E=v<T?h[v++]:0,v<T?h[v++]:0):(r=h.charCodeAt(v++),E=v<T?h.charCodeAt(v++):0,v<T?h.charCodeAt(v++):0),w=r>>2,_=(3&r)<<4|E>>4,z=1<D?(15&E)<<2|A>>6:64,g=2<D?63&A:64,S.push(m.charAt(w)+m.charAt(_)+m.charAt(z)+m.charAt(g));return S.join("")},p.decode=function(h){var r,E,A,w,_,z,g=0,S=0,v="data:";if(h.substr(0,v.length)===v)throw new Error("Invalid base64 input, it looks like a data url.");var T,D=3*(h=h.replace(/[^A-Za-z0-9+/=]/g,"")).length/4;if(h.charAt(h.length-1)===m.charAt(64)&&D--,h.charAt(h.length-2)===m.charAt(64)&&D--,D%1!=0)throw new Error("Invalid base64 input, bad content length.");for(T=y.uint8array?new Uint8Array(0|D):new Array(0|D);g<h.length;)r=m.indexOf(h.charAt(g++))<<2|(w=m.indexOf(h.charAt(g++)))>>4,E=(15&w)<<4|(_=m.indexOf(h.charAt(g++)))>>2,A=(3&_)<<6|(z=m.indexOf(h.charAt(g++))),T[S++]=r,_!==64&&(T[S++]=E),z!==64&&(T[S++]=A);return T}},{"./support":30,"./utils":32}],2:[function(c,d,p){var o=c("./external"),y=c("./stream/DataWorker"),m=c("./stream/Crc32Probe"),h=c("./stream/DataLengthProbe");function r(E,A,w,_,z){this.compressedSize=E,this.uncompressedSize=A,this.crc32=w,this.compression=_,this.compressedContent=z}r.prototype={getContentWorker:function(){var E=new y(o.Promise.resolve(this.compressedContent)).pipe(this.compression.uncompressWorker()).pipe(new h("data_length")),A=this;return E.on("end",function(){if(this.streamInfo.data_length!==A.uncompressedSize)throw new Error("Bug : uncompressed data size mismatch")}),E},getCompressedWorker:function(){return new y(o.Promise.resolve(this.compressedContent)).withStreamInfo("compressedSize",this.compressedSize).withStreamInfo("uncompressedSize",this.uncompressedSize).withStreamInfo("crc32",this.crc32).withStreamInfo("compression",this.compression)}},r.createWorkerFrom=function(E,A,w){return E.pipe(new m).pipe(new h("uncompressedSize")).pipe(A.compressWorker(w)).pipe(new h("compressedSize")).withStreamInfo("compression",A)},d.exports=r},{"./external":6,"./stream/Crc32Probe":25,"./stream/DataLengthProbe":26,"./stream/DataWorker":27}],3:[function(c,d,p){var o=c("./stream/GenericWorker");p.STORE={magic:"\0\0",compressWorker:function(){return new o("STORE compression")},uncompressWorker:function(){return new o("STORE decompression")}},p.DEFLATE=c("./flate")},{"./flate":7,"./stream/GenericWorker":28}],4:[function(c,d,p){var o=c("./utils"),y=function(){for(var m,h=[],r=0;r<256;r++){m=r;for(var E=0;E<8;E++)m=1&m?3988292384^m>>>1:m>>>1;h[r]=m}return h}();d.exports=function(m,h){return m!==void 0&&m.length?o.getTypeOf(m)!=="string"?function(r,E,A,w){var _=y,z=w+A;r^=-1;for(var g=w;g<z;g++)r=r>>>8^_[255&(r^E[g])];return-1^r}(0|h,m,m.length,0):function(r,E,A,w){var _=y,z=w+A;r^=-1;for(var g=w;g<z;g++)r=r>>>8^_[255&(r^E.charCodeAt(g))];return-1^r}(0|h,m,m.length,0):0}},{"./utils":32}],5:[function(c,d,p){p.base64=!1,p.binary=!1,p.dir=!1,p.createFolders=!0,p.date=null,p.compression=null,p.compressionOptions=null,p.comment=null,p.unixPermissions=null,p.dosPermissions=null},{}],6:[function(c,d,p){var o=null;o=typeof Promise<"u"?Promise:c("lie"),d.exports={Promise:o}},{lie:37}],7:[function(c,d,p){var o=typeof Uint8Array<"u"&&typeof Uint16Array<"u"&&typeof Uint32Array<"u",y=c("pako"),m=c("./utils"),h=c("./stream/GenericWorker"),r=o?"uint8array":"array";function E(A,w){h.call(this,"FlateWorker/"+A),this._pako=null,this._pakoAction=A,this._pakoOptions=w,this.meta={}}p.magic="\b\0",m.inherits(E,h),E.prototype.processChunk=function(A){this.meta=A.meta,this._pako===null&&this._createPako(),this._pako.push(m.transformTo(r,A.data),!1)},E.prototype.flush=function(){h.prototype.flush.call(this),this._pako===null&&this._createPako(),this._pako.push([],!0)},E.prototype.cleanUp=function(){h.prototype.cleanUp.call(this),this._pako=null},E.prototype._createPako=function(){this._pako=new y[this._pakoAction]({raw:!0,level:this._pakoOptions.level||-1});var A=this;this._pako.onData=function(w){A.push({data:w,meta:A.meta})}},p.compressWorker=function(A){return new E("Deflate",A)},p.uncompressWorker=function(){return new E("Inflate",{})}},{"./stream/GenericWorker":28,"./utils":32,pako:38}],8:[function(c,d,p){function o(_,z){var g,S="";for(g=0;g<z;g++)S+=String.fromCharCode(255&_),_>>>=8;return S}function y(_,z,g,S,v,T){var D,L,j=_.file,tt=_.compression,F=T!==r.utf8encode,ft=m.transformTo("string",T(j.name)),$=m.transformTo("string",r.utf8encode(j.name)),rt=j.comment,vt=m.transformTo("string",T(rt)),M=m.transformTo("string",r.utf8encode(rt)),et=$.length!==j.name.length,b=M.length!==rt.length,at="",G="",X="",gt=j.dir,st=j.date,C={crc32:0,compressedSize:0,uncompressedSize:0};z&&!g||(C.crc32=_.crc32,C.compressedSize=_.compressedSize,C.uncompressedSize=_.uncompressedSize);var N=0;z&&(N|=8),F||!et&&!b||(N|=2048);var B=0,nt=0;gt&&(B|=16),v==="UNIX"?(nt=798,B|=function(dt,_t){var Lt=dt;return dt||(Lt=_t?16893:33204),(65535&Lt)<<16}(j.unixPermissions,gt)):(nt=20,B|=function(dt){return 63&(dt||0)}(j.dosPermissions)),D=st.getUTCHours(),D<<=6,D|=st.getUTCMinutes(),D<<=5,D|=st.getUTCSeconds()/2,L=st.getUTCFullYear()-1980,L<<=4,L|=st.getUTCMonth()+1,L<<=5,L|=st.getUTCDate(),et&&(G=o(1,1)+o(E(ft),4)+$,at+="up"+o(G.length,2)+G),b&&(X=o(1,1)+o(E(vt),4)+M,at+="uc"+o(X.length,2)+X);var it="";return it+=`
\0`,it+=o(N,2),it+=tt.magic,it+=o(D,2),it+=o(L,2),it+=o(C.crc32,4),it+=o(C.compressedSize,4),it+=o(C.uncompressedSize,4),it+=o(ft.length,2),it+=o(at.length,2),{fileRecord:A.LOCAL_FILE_HEADER+it+ft+at,dirRecord:A.CENTRAL_FILE_HEADER+o(nt,2)+it+o(vt.length,2)+"\0\0\0\0"+o(B,4)+o(S,4)+ft+at+vt}}var m=c("../utils"),h=c("../stream/GenericWorker"),r=c("../utf8"),E=c("../crc32"),A=c("../signature");function w(_,z,g,S){h.call(this,"ZipFileWorker"),this.bytesWritten=0,this.zipComment=z,this.zipPlatform=g,this.encodeFileName=S,this.streamFiles=_,this.accumulate=!1,this.contentBuffer=[],this.dirRecords=[],this.currentSourceOffset=0,this.entriesCount=0,this.currentFile=null,this._sources=[]}m.inherits(w,h),w.prototype.push=function(_){var z=_.meta.percent||0,g=this.entriesCount,S=this._sources.length;this.accumulate?this.contentBuffer.push(_):(this.bytesWritten+=_.data.length,h.prototype.push.call(this,{data:_.data,meta:{currentFile:this.currentFile,percent:g?(z+100*(g-S-1))/g:100}}))},w.prototype.openedSource=function(_){this.currentSourceOffset=this.bytesWritten,this.currentFile=_.file.name;var z=this.streamFiles&&!_.file.dir;if(z){var g=y(_,z,!1,this.currentSourceOffset,this.zipPlatform,this.encodeFileName);this.push({data:g.fileRecord,meta:{percent:0}})}else this.accumulate=!0},w.prototype.closedSource=function(_){this.accumulate=!1;var z=this.streamFiles&&!_.file.dir,g=y(_,z,!0,this.currentSourceOffset,this.zipPlatform,this.encodeFileName);if(this.dirRecords.push(g.dirRecord),z)this.push({data:function(S){return A.DATA_DESCRIPTOR+o(S.crc32,4)+o(S.compressedSize,4)+o(S.uncompressedSize,4)}(_),meta:{percent:100}});else for(this.push({data:g.fileRecord,meta:{percent:0}});this.contentBuffer.length;)this.push(this.contentBuffer.shift());this.currentFile=null},w.prototype.flush=function(){for(var _=this.bytesWritten,z=0;z<this.dirRecords.length;z++)this.push({data:this.dirRecords[z],meta:{percent:100}});var g=this.bytesWritten-_,S=function(v,T,D,L,j){var tt=m.transformTo("string",j(L));return A.CENTRAL_DIRECTORY_END+"\0\0\0\0"+o(v,2)+o(v,2)+o(T,4)+o(D,4)+o(tt.length,2)+tt}(this.dirRecords.length,g,_,this.zipComment,this.encodeFileName);this.push({data:S,meta:{percent:100}})},w.prototype.prepareNextSource=function(){this.previous=this._sources.shift(),this.openedSource(this.previous.streamInfo),this.isPaused?this.previous.pause():this.previous.resume()},w.prototype.registerPrevious=function(_){this._sources.push(_);var z=this;return _.on("data",function(g){z.processChunk(g)}),_.on("end",function(){z.closedSource(z.previous.streamInfo),z._sources.length?z.prepareNextSource():z.end()}),_.on("error",function(g){z.error(g)}),this},w.prototype.resume=function(){return!!h.prototype.resume.call(this)&&(!this.previous&&this._sources.length?(this.prepareNextSource(),!0):this.previous||this._sources.length||this.generatedError?void 0:(this.end(),!0))},w.prototype.error=function(_){var z=this._sources;if(!h.prototype.error.call(this,_))return!1;for(var g=0;g<z.length;g++)try{z[g].error(_)}catch{}return!0},w.prototype.lock=function(){h.prototype.lock.call(this);for(var _=this._sources,z=0;z<_.length;z++)_[z].lock()},d.exports=w},{"../crc32":4,"../signature":23,"../stream/GenericWorker":28,"../utf8":31,"../utils":32}],9:[function(c,d,p){var o=c("../compressions"),y=c("./ZipFileWorker");p.generateWorker=function(m,h,r){var E=new y(h.streamFiles,r,h.platform,h.encodeFileName),A=0;try{m.forEach(function(w,_){A++;var z=function(T,D){var L=T||D,j=o[L];if(!j)throw new Error(L+" is not a valid compression method !");return j}(_.options.compression,h.compression),g=_.options.compressionOptions||h.compressionOptions||{},S=_.dir,v=_.date;_._compressWorker(z,g).withStreamInfo("file",{name:w,dir:S,date:v,comment:_.comment||"",unixPermissions:_.unixPermissions,dosPermissions:_.dosPermissions}).pipe(E)}),E.entriesCount=A}catch(w){E.error(w)}return E}},{"../compressions":3,"./ZipFileWorker":8}],10:[function(c,d,p){function o(){if(!(this instanceof o))return new o;if(arguments.length)throw new Error("The constructor with parameters has been removed in JSZip 3.0, please check the upgrade guide.");this.files=Object.create(null),this.comment=null,this.root="",this.clone=function(){var y=new o;for(var m in this)typeof this[m]!="function"&&(y[m]=this[m]);return y}}(o.prototype=c("./object")).loadAsync=c("./load"),o.support=c("./support"),o.defaults=c("./defaults"),o.version="3.10.1",o.loadAsync=function(y,m){return new o().loadAsync(y,m)},o.external=c("./external"),d.exports=o},{"./defaults":5,"./external":6,"./load":11,"./object":15,"./support":30}],11:[function(c,d,p){var o=c("./utils"),y=c("./external"),m=c("./utf8"),h=c("./zipEntries"),r=c("./stream/Crc32Probe"),E=c("./nodejsUtils");function A(w){return new y.Promise(function(_,z){var g=w.decompressed.getContentWorker().pipe(new r);g.on("error",function(S){z(S)}).on("end",function(){g.streamInfo.crc32!==w.decompressed.crc32?z(new Error("Corrupted zip : CRC32 mismatch")):_()}).resume()})}d.exports=function(w,_){var z=this;return _=o.extend(_||{},{base64:!1,checkCRC32:!1,optimizedBinaryString:!1,createFolders:!1,decodeFileName:m.utf8decode}),E.isNode&&E.isStream(w)?y.Promise.reject(new Error("JSZip can't accept a stream when loading a zip file.")):o.prepareContent("the loaded zip file",w,!0,_.optimizedBinaryString,_.base64).then(function(g){var S=new h(_);return S.load(g),S}).then(function(g){var S=[y.Promise.resolve(g)],v=g.files;if(_.checkCRC32)for(var T=0;T<v.length;T++)S.push(A(v[T]));return y.Promise.all(S)}).then(function(g){for(var S=g.shift(),v=S.files,T=0;T<v.length;T++){var D=v[T],L=D.fileNameStr,j=o.resolve(D.fileNameStr);z.file(j,D.decompressed,{binary:!0,optimizedBinaryString:!0,date:D.date,dir:D.dir,comment:D.fileCommentStr.length?D.fileCommentStr:null,unixPermissions:D.unixPermissions,dosPermissions:D.dosPermissions,createFolders:_.createFolders}),D.dir||(z.file(j).unsafeOriginalName=L)}return S.zipComment.length&&(z.comment=S.zipComment),z})}},{"./external":6,"./nodejsUtils":14,"./stream/Crc32Probe":25,"./utf8":31,"./utils":32,"./zipEntries":33}],12:[function(c,d,p){var o=c("../utils"),y=c("../stream/GenericWorker");function m(h,r){y.call(this,"Nodejs stream input adapter for "+h),this._upstreamEnded=!1,this._bindStream(r)}o.inherits(m,y),m.prototype._bindStream=function(h){var r=this;(this._stream=h).pause(),h.on("data",function(E){r.push({data:E,meta:{percent:0}})}).on("error",function(E){r.isPaused?this.generatedError=E:r.error(E)}).on("end",function(){r.isPaused?r._upstreamEnded=!0:r.end()})},m.prototype.pause=function(){return!!y.prototype.pause.call(this)&&(this._stream.pause(),!0)},m.prototype.resume=function(){return!!y.prototype.resume.call(this)&&(this._upstreamEnded?this.end():this._stream.resume(),!0)},d.exports=m},{"../stream/GenericWorker":28,"../utils":32}],13:[function(c,d,p){var o=c("readable-stream").Readable;function y(m,h,r){o.call(this,h),this._helper=m;var E=this;m.on("data",function(A,w){E.push(A)||E._helper.pause(),r&&r(w)}).on("error",function(A){E.emit("error",A)}).on("end",function(){E.push(null)})}c("../utils").inherits(y,o),y.prototype._read=function(){this._helper.resume()},d.exports=y},{"../utils":32,"readable-stream":16}],14:[function(c,d,p){d.exports={isNode:typeof Buffer<"u",newBufferFrom:function(o,y){if(Buffer.from&&Buffer.from!==Uint8Array.from)return Buffer.from(o,y);if(typeof o=="number")throw new Error('The "data" argument must not be a number');return new Buffer(o,y)},allocBuffer:function(o){if(Buffer.alloc)return Buffer.alloc(o);var y=new Buffer(o);return y.fill(0),y},isBuffer:function(o){return Buffer.isBuffer(o)},isStream:function(o){return o&&typeof o.on=="function"&&typeof o.pause=="function"&&typeof o.resume=="function"}}},{}],15:[function(c,d,p){function o(j,tt,F){var ft,$=m.getTypeOf(tt),rt=m.extend(F||{},E);rt.date=rt.date||new Date,rt.compression!==null&&(rt.compression=rt.compression.toUpperCase()),typeof rt.unixPermissions=="string"&&(rt.unixPermissions=parseInt(rt.unixPermissions,8)),rt.unixPermissions&&16384&rt.unixPermissions&&(rt.dir=!0),rt.dosPermissions&&16&rt.dosPermissions&&(rt.dir=!0),rt.dir&&(j=v(j)),rt.createFolders&&(ft=S(j))&&T.call(this,ft,!0);var vt=$==="string"&&rt.binary===!1&&rt.base64===!1;F&&F.binary!==void 0||(rt.binary=!vt),(tt instanceof A&&tt.uncompressedSize===0||rt.dir||!tt||tt.length===0)&&(rt.base64=!1,rt.binary=!0,tt="",rt.compression="STORE",$="string");var M=null;M=tt instanceof A||tt instanceof h?tt:z.isNode&&z.isStream(tt)?new g(j,tt):m.prepareContent(j,tt,rt.binary,rt.optimizedBinaryString,rt.base64);var et=new w(j,M,rt);this.files[j]=et}var y=c("./utf8"),m=c("./utils"),h=c("./stream/GenericWorker"),r=c("./stream/StreamHelper"),E=c("./defaults"),A=c("./compressedObject"),w=c("./zipObject"),_=c("./generate"),z=c("./nodejsUtils"),g=c("./nodejs/NodejsStreamInputAdapter"),S=function(j){j.slice(-1)==="/"&&(j=j.substring(0,j.length-1));var tt=j.lastIndexOf("/");return 0<tt?j.substring(0,tt):""},v=function(j){return j.slice(-1)!=="/"&&(j+="/"),j},T=function(j,tt){return tt=tt!==void 0?tt:E.createFolders,j=v(j),this.files[j]||o.call(this,j,null,{dir:!0,createFolders:tt}),this.files[j]};function D(j){return Object.prototype.toString.call(j)==="[object RegExp]"}var L={load:function(){throw new Error("This method has been removed in JSZip 3.0, please check the upgrade guide.")},forEach:function(j){var tt,F,ft;for(tt in this.files)ft=this.files[tt],(F=tt.slice(this.root.length,tt.length))&&tt.slice(0,this.root.length)===this.root&&j(F,ft)},filter:function(j){var tt=[];return this.forEach(function(F,ft){j(F,ft)&&tt.push(ft)}),tt},file:function(j,tt,F){if(arguments.length!==1)return j=this.root+j,o.call(this,j,tt,F),this;if(D(j)){var ft=j;return this.filter(function(rt,vt){return!vt.dir&&ft.test(rt)})}var $=this.files[this.root+j];return $&&!$.dir?$:null},folder:function(j){if(!j)return this;if(D(j))return this.filter(function($,rt){return rt.dir&&j.test($)});var tt=this.root+j,F=T.call(this,tt),ft=this.clone();return ft.root=F.name,ft},remove:function(j){j=this.root+j;var tt=this.files[j];if(tt||(j.slice(-1)!=="/"&&(j+="/"),tt=this.files[j]),tt&&!tt.dir)delete this.files[j];else for(var F=this.filter(function($,rt){return rt.name.slice(0,j.length)===j}),ft=0;ft<F.length;ft++)delete this.files[F[ft].name];return this},generate:function(){throw new Error("This method has been removed in JSZip 3.0, please check the upgrade guide.")},generateInternalStream:function(j){var tt,F={};try{if((F=m.extend(j||{},{streamFiles:!1,compression:"STORE",compressionOptions:null,type:"",platform:"DOS",comment:null,mimeType:"application/zip",encodeFileName:y.utf8encode})).type=F.type.toLowerCase(),F.compression=F.compression.toUpperCase(),F.type==="binarystring"&&(F.type="string"),!F.type)throw new Error("No output type specified.");m.checkSupport(F.type),F.platform!=="darwin"&&F.platform!=="freebsd"&&F.platform!=="linux"&&F.platform!=="sunos"||(F.platform="UNIX"),F.platform==="win32"&&(F.platform="DOS");var ft=F.comment||this.comment||"";tt=_.generateWorker(this,F,ft)}catch($){(tt=new h("error")).error($)}return new r(tt,F.type||"string",F.mimeType)},generateAsync:function(j,tt){return this.generateInternalStream(j).accumulate(tt)},generateNodeStream:function(j,tt){return(j=j||{}).type||(j.type="nodebuffer"),this.generateInternalStream(j).toNodejsStream(tt)}};d.exports=L},{"./compressedObject":2,"./defaults":5,"./generate":9,"./nodejs/NodejsStreamInputAdapter":12,"./nodejsUtils":14,"./stream/GenericWorker":28,"./stream/StreamHelper":29,"./utf8":31,"./utils":32,"./zipObject":35}],16:[function(c,d,p){d.exports=c("stream")},{stream:void 0}],17:[function(c,d,p){var o=c("./DataReader");function y(m){o.call(this,m);for(var h=0;h<this.data.length;h++)m[h]=255&m[h]}c("../utils").inherits(y,o),y.prototype.byteAt=function(m){return this.data[this.zero+m]},y.prototype.lastIndexOfSignature=function(m){for(var h=m.charCodeAt(0),r=m.charCodeAt(1),E=m.charCodeAt(2),A=m.charCodeAt(3),w=this.length-4;0<=w;--w)if(this.data[w]===h&&this.data[w+1]===r&&this.data[w+2]===E&&this.data[w+3]===A)return w-this.zero;return-1},y.prototype.readAndCheckSignature=function(m){var h=m.charCodeAt(0),r=m.charCodeAt(1),E=m.charCodeAt(2),A=m.charCodeAt(3),w=this.readData(4);return h===w[0]&&r===w[1]&&E===w[2]&&A===w[3]},y.prototype.readData=function(m){if(this.checkOffset(m),m===0)return[];var h=this.data.slice(this.zero+this.index,this.zero+this.index+m);return this.index+=m,h},d.exports=y},{"../utils":32,"./DataReader":18}],18:[function(c,d,p){var o=c("../utils");function y(m){this.data=m,this.length=m.length,this.index=0,this.zero=0}y.prototype={checkOffset:function(m){this.checkIndex(this.index+m)},checkIndex:function(m){if(this.length<this.zero+m||m<0)throw new Error("End of data reached (data length = "+this.length+", asked index = "+m+"). Corrupted zip ?")},setIndex:function(m){this.checkIndex(m),this.index=m},skip:function(m){this.setIndex(this.index+m)},byteAt:function(){},readInt:function(m){var h,r=0;for(this.checkOffset(m),h=this.index+m-1;h>=this.index;h--)r=(r<<8)+this.byteAt(h);return this.index+=m,r},readString:function(m){return o.transformTo("string",this.readData(m))},readData:function(){},lastIndexOfSignature:function(){},readAndCheckSignature:function(){},readDate:function(){var m=this.readInt(4);return new Date(Date.UTC(1980+(m>>25&127),(m>>21&15)-1,m>>16&31,m>>11&31,m>>5&63,(31&m)<<1))}},d.exports=y},{"../utils":32}],19:[function(c,d,p){var o=c("./Uint8ArrayReader");function y(m){o.call(this,m)}c("../utils").inherits(y,o),y.prototype.readData=function(m){this.checkOffset(m);var h=this.data.slice(this.zero+this.index,this.zero+this.index+m);return this.index+=m,h},d.exports=y},{"../utils":32,"./Uint8ArrayReader":21}],20:[function(c,d,p){var o=c("./DataReader");function y(m){o.call(this,m)}c("../utils").inherits(y,o),y.prototype.byteAt=function(m){return this.data.charCodeAt(this.zero+m)},y.prototype.lastIndexOfSignature=function(m){return this.data.lastIndexOf(m)-this.zero},y.prototype.readAndCheckSignature=function(m){return m===this.readData(4)},y.prototype.readData=function(m){this.checkOffset(m);var h=this.data.slice(this.zero+this.index,this.zero+this.index+m);return this.index+=m,h},d.exports=y},{"../utils":32,"./DataReader":18}],21:[function(c,d,p){var o=c("./ArrayReader");function y(m){o.call(this,m)}c("../utils").inherits(y,o),y.prototype.readData=function(m){if(this.checkOffset(m),m===0)return new Uint8Array(0);var h=this.data.subarray(this.zero+this.index,this.zero+this.index+m);return this.index+=m,h},d.exports=y},{"../utils":32,"./ArrayReader":17}],22:[function(c,d,p){var o=c("../utils"),y=c("../support"),m=c("./ArrayReader"),h=c("./StringReader"),r=c("./NodeBufferReader"),E=c("./Uint8ArrayReader");d.exports=function(A){var w=o.getTypeOf(A);return o.checkSupport(w),w!=="string"||y.uint8array?w==="nodebuffer"?new r(A):y.uint8array?new E(o.transformTo("uint8array",A)):new m(o.transformTo("array",A)):new h(A)}},{"../support":30,"../utils":32,"./ArrayReader":17,"./NodeBufferReader":19,"./StringReader":20,"./Uint8ArrayReader":21}],23:[function(c,d,p){p.LOCAL_FILE_HEADER="PK",p.CENTRAL_FILE_HEADER="PK",p.CENTRAL_DIRECTORY_END="PK",p.ZIP64_CENTRAL_DIRECTORY_LOCATOR="PK\x07",p.ZIP64_CENTRAL_DIRECTORY_END="PK",p.DATA_DESCRIPTOR="PK\x07\b"},{}],24:[function(c,d,p){var o=c("./GenericWorker"),y=c("../utils");function m(h){o.call(this,"ConvertWorker to "+h),this.destType=h}y.inherits(m,o),m.prototype.processChunk=function(h){this.push({data:y.transformTo(this.destType,h.data),meta:h.meta})},d.exports=m},{"../utils":32,"./GenericWorker":28}],25:[function(c,d,p){var o=c("./GenericWorker"),y=c("../crc32");function m(){o.call(this,"Crc32Probe"),this.withStreamInfo("crc32",0)}c("../utils").inherits(m,o),m.prototype.processChunk=function(h){this.streamInfo.crc32=y(h.data,this.streamInfo.crc32||0),this.push(h)},d.exports=m},{"../crc32":4,"../utils":32,"./GenericWorker":28}],26:[function(c,d,p){var o=c("../utils"),y=c("./GenericWorker");function m(h){y.call(this,"DataLengthProbe for "+h),this.propName=h,this.withStreamInfo(h,0)}o.inherits(m,y),m.prototype.processChunk=function(h){if(h){var r=this.streamInfo[this.propName]||0;this.streamInfo[this.propName]=r+h.data.length}y.prototype.processChunk.call(this,h)},d.exports=m},{"../utils":32,"./GenericWorker":28}],27:[function(c,d,p){var o=c("../utils"),y=c("./GenericWorker");function m(h){y.call(this,"DataWorker");var r=this;this.dataIsReady=!1,this.index=0,this.max=0,this.data=null,this.type="",this._tickScheduled=!1,h.then(function(E){r.dataIsReady=!0,r.data=E,r.max=E&&E.length||0,r.type=o.getTypeOf(E),r.isPaused||r._tickAndRepeat()},function(E){r.error(E)})}o.inherits(m,y),m.prototype.cleanUp=function(){y.prototype.cleanUp.call(this),this.data=null},m.prototype.resume=function(){return!!y.prototype.resume.call(this)&&(!this._tickScheduled&&this.dataIsReady&&(this._tickScheduled=!0,o.delay(this._tickAndRepeat,[],this)),!0)},m.prototype._tickAndRepeat=function(){this._tickScheduled=!1,this.isPaused||this.isFinished||(this._tick(),this.isFinished||(o.delay(this._tickAndRepeat,[],this),this._tickScheduled=!0))},m.prototype._tick=function(){if(this.isPaused||this.isFinished)return!1;var h=null,r=Math.min(this.max,this.index+16384);if(this.index>=this.max)return this.end();switch(this.type){case"string":h=this.data.substring(this.index,r);break;case"uint8array":h=this.data.subarray(this.index,r);break;case"array":case"nodebuffer":h=this.data.slice(this.index,r)}return this.index=r,this.push({data:h,meta:{percent:this.max?this.index/this.max*100:0}})},d.exports=m},{"../utils":32,"./GenericWorker":28}],28:[function(c,d,p){function o(y){this.name=y||"default",this.streamInfo={},this.generatedError=null,this.extraStreamInfo={},this.isPaused=!0,this.isFinished=!1,this.isLocked=!1,this._listeners={data:[],end:[],error:[]},this.previous=null}o.prototype={push:function(y){this.emit("data",y)},end:function(){if(this.isFinished)return!1;this.flush();try{this.emit("end"),this.cleanUp(),this.isFinished=!0}catch(y){this.emit("error",y)}return!0},error:function(y){return!this.isFinished&&(this.isPaused?this.generatedError=y:(this.isFinished=!0,this.emit("error",y),this.previous&&this.previous.error(y),this.cleanUp()),!0)},on:function(y,m){return this._listeners[y].push(m),this},cleanUp:function(){this.streamInfo=this.generatedError=this.extraStreamInfo=null,this._listeners=[]},emit:function(y,m){if(this._listeners[y])for(var h=0;h<this._listeners[y].length;h++)this._listeners[y][h].call(this,m)},pipe:function(y){return y.registerPrevious(this)},registerPrevious:function(y){if(this.isLocked)throw new Error("The stream '"+this+"' has already been used.");this.streamInfo=y.streamInfo,this.mergeStreamInfo(),this.previous=y;var m=this;return y.on("data",function(h){m.processChunk(h)}),y.on("end",function(){m.end()}),y.on("error",function(h){m.error(h)}),this},pause:function(){return!this.isPaused&&!this.isFinished&&(this.isPaused=!0,this.previous&&this.previous.pause(),!0)},resume:function(){if(!this.isPaused||this.isFinished)return!1;var y=this.isPaused=!1;return this.generatedError&&(this.error(this.generatedError),y=!0),this.previous&&this.previous.resume(),!y},flush:function(){},processChunk:function(y){this.push(y)},withStreamInfo:function(y,m){return this.extraStreamInfo[y]=m,this.mergeStreamInfo(),this},mergeStreamInfo:function(){for(var y in this.extraStreamInfo)Object.prototype.hasOwnProperty.call(this.extraStreamInfo,y)&&(this.streamInfo[y]=this.extraStreamInfo[y])},lock:function(){if(this.isLocked)throw new Error("The stream '"+this+"' has already been used.");this.isLocked=!0,this.previous&&this.previous.lock()},toString:function(){var y="Worker "+this.name;return this.previous?this.previous+" -> "+y:y}},d.exports=o},{}],29:[function(c,d,p){var o=c("../utils"),y=c("./ConvertWorker"),m=c("./GenericWorker"),h=c("../base64"),r=c("../support"),E=c("../external"),A=null;if(r.nodestream)try{A=c("../nodejs/NodejsStreamOutputAdapter")}catch{}function w(z,g){return new E.Promise(function(S,v){var T=[],D=z._internalType,L=z._outputType,j=z._mimeType;z.on("data",function(tt,F){T.push(tt),g&&g(F)}).on("error",function(tt){T=[],v(tt)}).on("end",function(){try{var tt=function(F,ft,$){switch(F){case"blob":return o.newBlob(o.transformTo("arraybuffer",ft),$);case"base64":return h.encode(ft);default:return o.transformTo(F,ft)}}(L,function(F,ft){var $,rt=0,vt=null,M=0;for($=0;$<ft.length;$++)M+=ft[$].length;switch(F){case"string":return ft.join("");case"array":return Array.prototype.concat.apply([],ft);case"uint8array":for(vt=new Uint8Array(M),$=0;$<ft.length;$++)vt.set(ft[$],rt),rt+=ft[$].length;return vt;case"nodebuffer":return Buffer.concat(ft);default:throw new Error("concat : unsupported type '"+F+"'")}}(D,T),j);S(tt)}catch(F){v(F)}T=[]}).resume()})}function _(z,g,S){var v=g;switch(g){case"blob":case"arraybuffer":v="uint8array";break;case"base64":v="string"}try{this._internalType=v,this._outputType=g,this._mimeType=S,o.checkSupport(v),this._worker=z.pipe(new y(v)),z.lock()}catch(T){this._worker=new m("error"),this._worker.error(T)}}_.prototype={accumulate:function(z){return w(this,z)},on:function(z,g){var S=this;return z==="data"?this._worker.on(z,function(v){g.call(S,v.data,v.meta)}):this._worker.on(z,function(){o.delay(g,arguments,S)}),this},resume:function(){return o.delay(this._worker.resume,[],this._worker),this},pause:function(){return this._worker.pause(),this},toNodejsStream:function(z){if(o.checkSupport("nodestream"),this._outputType!=="nodebuffer")throw new Error(this._outputType+" is not supported by this method");return new A(this,{objectMode:this._outputType!=="nodebuffer"},z)}},d.exports=_},{"../base64":1,"../external":6,"../nodejs/NodejsStreamOutputAdapter":13,"../support":30,"../utils":32,"./ConvertWorker":24,"./GenericWorker":28}],30:[function(c,d,p){if(p.base64=!0,p.array=!0,p.string=!0,p.arraybuffer=typeof ArrayBuffer<"u"&&typeof Uint8Array<"u",p.nodebuffer=typeof Buffer<"u",p.uint8array=typeof Uint8Array<"u",typeof ArrayBuffer>"u")p.blob=!1;else{var o=new ArrayBuffer(0);try{p.blob=new Blob([o],{type:"application/zip"}).size===0}catch{try{var y=new(self.BlobBuilder||self.WebKitBlobBuilder||self.MozBlobBuilder||self.MSBlobBuilder);y.append(o),p.blob=y.getBlob("application/zip").size===0}catch{p.blob=!1}}}try{p.nodestream=!!c("readable-stream").Readable}catch{p.nodestream=!1}},{"readable-stream":16}],31:[function(c,d,p){for(var o=c("./utils"),y=c("./support"),m=c("./nodejsUtils"),h=c("./stream/GenericWorker"),r=new Array(256),E=0;E<256;E++)r[E]=252<=E?6:248<=E?5:240<=E?4:224<=E?3:192<=E?2:1;r[254]=r[254]=1;function A(){h.call(this,"utf-8 decode"),this.leftOver=null}function w(){h.call(this,"utf-8 encode")}p.utf8encode=function(_){return y.nodebuffer?m.newBufferFrom(_,"utf-8"):function(z){var g,S,v,T,D,L=z.length,j=0;for(T=0;T<L;T++)(64512&(S=z.charCodeAt(T)))==55296&&T+1<L&&(64512&(v=z.charCodeAt(T+1)))==56320&&(S=65536+(S-55296<<10)+(v-56320),T++),j+=S<128?1:S<2048?2:S<65536?3:4;for(g=y.uint8array?new Uint8Array(j):new Array(j),T=D=0;D<j;T++)(64512&(S=z.charCodeAt(T)))==55296&&T+1<L&&(64512&(v=z.charCodeAt(T+1)))==56320&&(S=65536+(S-55296<<10)+(v-56320),T++),S<128?g[D++]=S:(S<2048?g[D++]=192|S>>>6:(S<65536?g[D++]=224|S>>>12:(g[D++]=240|S>>>18,g[D++]=128|S>>>12&63),g[D++]=128|S>>>6&63),g[D++]=128|63&S);return g}(_)},p.utf8decode=function(_){return y.nodebuffer?o.transformTo("nodebuffer",_).toString("utf-8"):function(z){var g,S,v,T,D=z.length,L=new Array(2*D);for(g=S=0;g<D;)if((v=z[g++])<128)L[S++]=v;else if(4<(T=r[v]))L[S++]=65533,g+=T-1;else{for(v&=T===2?31:T===3?15:7;1<T&&g<D;)v=v<<6|63&z[g++],T--;1<T?L[S++]=65533:v<65536?L[S++]=v:(v-=65536,L[S++]=55296|v>>10&1023,L[S++]=56320|1023&v)}return L.length!==S&&(L.subarray?L=L.subarray(0,S):L.length=S),o.applyFromCharCode(L)}(_=o.transformTo(y.uint8array?"uint8array":"array",_))},o.inherits(A,h),A.prototype.processChunk=function(_){var z=o.transformTo(y.uint8array?"uint8array":"array",_.data);if(this.leftOver&&this.leftOver.length){if(y.uint8array){var g=z;(z=new Uint8Array(g.length+this.leftOver.length)).set(this.leftOver,0),z.set(g,this.leftOver.length)}else z=this.leftOver.concat(z);this.leftOver=null}var S=function(T,D){var L;for((D=D||T.length)>T.length&&(D=T.length),L=D-1;0<=L&&(192&T[L])==128;)L--;return L<0||L===0?D:L+r[T[L]]>D?L:D}(z),v=z;S!==z.length&&(y.uint8array?(v=z.subarray(0,S),this.leftOver=z.subarray(S,z.length)):(v=z.slice(0,S),this.leftOver=z.slice(S,z.length))),this.push({data:p.utf8decode(v),meta:_.meta})},A.prototype.flush=function(){this.leftOver&&this.leftOver.length&&(this.push({data:p.utf8decode(this.leftOver),meta:{}}),this.leftOver=null)},p.Utf8DecodeWorker=A,o.inherits(w,h),w.prototype.processChunk=function(_){this.push({data:p.utf8encode(_.data),meta:_.meta})},p.Utf8EncodeWorker=w},{"./nodejsUtils":14,"./stream/GenericWorker":28,"./support":30,"./utils":32}],32:[function(c,d,p){var o=c("./support"),y=c("./base64"),m=c("./nodejsUtils"),h=c("./external");function r(g){return g}function E(g,S){for(var v=0;v<g.length;++v)S[v]=255&g.charCodeAt(v);return S}c("setimmediate"),p.newBlob=function(g,S){p.checkSupport("blob");try{return new Blob([g],{type:S})}catch{try{var v=new(self.BlobBuilder||self.WebKitBlobBuilder||self.MozBlobBuilder||self.MSBlobBuilder);return v.append(g),v.getBlob(S)}catch{throw new Error("Bug : can't construct the Blob.")}}};var A={stringifyByChunk:function(g,S,v){var T=[],D=0,L=g.length;if(L<=v)return String.fromCharCode.apply(null,g);for(;D<L;)S==="array"||S==="nodebuffer"?T.push(String.fromCharCode.apply(null,g.slice(D,Math.min(D+v,L)))):T.push(String.fromCharCode.apply(null,g.subarray(D,Math.min(D+v,L)))),D+=v;return T.join("")},stringifyByChar:function(g){for(var S="",v=0;v<g.length;v++)S+=String.fromCharCode(g[v]);return S},applyCanBeUsed:{uint8array:function(){try{return o.uint8array&&String.fromCharCode.apply(null,new Uint8Array(1)).length===1}catch{return!1}}(),nodebuffer:function(){try{return o.nodebuffer&&String.fromCharCode.apply(null,m.allocBuffer(1)).length===1}catch{return!1}}()}};function w(g){var S=65536,v=p.getTypeOf(g),T=!0;if(v==="uint8array"?T=A.applyCanBeUsed.uint8array:v==="nodebuffer"&&(T=A.applyCanBeUsed.nodebuffer),T)for(;1<S;)try{return A.stringifyByChunk(g,v,S)}catch{S=Math.floor(S/2)}return A.stringifyByChar(g)}function _(g,S){for(var v=0;v<g.length;v++)S[v]=g[v];return S}p.applyFromCharCode=w;var z={};z.string={string:r,array:function(g){return E(g,new Array(g.length))},arraybuffer:function(g){return z.string.uint8array(g).buffer},uint8array:function(g){return E(g,new Uint8Array(g.length))},nodebuffer:function(g){return E(g,m.allocBuffer(g.length))}},z.array={string:w,array:r,arraybuffer:function(g){return new Uint8Array(g).buffer},uint8array:function(g){return new Uint8Array(g)},nodebuffer:function(g){return m.newBufferFrom(g)}},z.arraybuffer={string:function(g){return w(new Uint8Array(g))},array:function(g){return _(new Uint8Array(g),new Array(g.byteLength))},arraybuffer:r,uint8array:function(g){return new Uint8Array(g)},nodebuffer:function(g){return m.newBufferFrom(new Uint8Array(g))}},z.uint8array={string:w,array:function(g){return _(g,new Array(g.length))},arraybuffer:function(g){return g.buffer},uint8array:r,nodebuffer:function(g){return m.newBufferFrom(g)}},z.nodebuffer={string:w,array:function(g){return _(g,new Array(g.length))},arraybuffer:function(g){return z.nodebuffer.uint8array(g).buffer},uint8array:function(g){return _(g,new Uint8Array(g.length))},nodebuffer:r},p.transformTo=function(g,S){if(S=S||"",!g)return S;p.checkSupport(g);var v=p.getTypeOf(S);return z[v][g](S)},p.resolve=function(g){for(var S=g.split("/"),v=[],T=0;T<S.length;T++){var D=S[T];D==="."||D===""&&T!==0&&T!==S.length-1||(D===".."?v.pop():v.push(D))}return v.join("/")},p.getTypeOf=function(g){return typeof g=="string"?"string":Object.prototype.toString.call(g)==="[object Array]"?"array":o.nodebuffer&&m.isBuffer(g)?"nodebuffer":o.uint8array&&g instanceof Uint8Array?"uint8array":o.arraybuffer&&g instanceof ArrayBuffer?"arraybuffer":void 0},p.checkSupport=function(g){if(!o[g.toLowerCase()])throw new Error(g+" is not supported by this platform")},p.MAX_VALUE_16BITS=65535,p.MAX_VALUE_32BITS=-1,p.pretty=function(g){var S,v,T="";for(v=0;v<(g||"").length;v++)T+="\\x"+((S=g.charCodeAt(v))<16?"0":"")+S.toString(16).toUpperCase();return T},p.delay=function(g,S,v){setImmediate(function(){g.apply(v||null,S||[])})},p.inherits=function(g,S){function v(){}v.prototype=S.prototype,g.prototype=new v},p.extend=function(){var g,S,v={};for(g=0;g<arguments.length;g++)for(S in arguments[g])Object.prototype.hasOwnProperty.call(arguments[g],S)&&v[S]===void 0&&(v[S]=arguments[g][S]);return v},p.prepareContent=function(g,S,v,T,D){return h.Promise.resolve(S).then(function(L){return o.blob&&(L instanceof Blob||["[object File]","[object Blob]"].indexOf(Object.prototype.toString.call(L))!==-1)&&typeof FileReader<"u"?new h.Promise(function(j,tt){var F=new FileReader;F.onload=function(ft){j(ft.target.result)},F.onerror=function(ft){tt(ft.target.error)},F.readAsArrayBuffer(L)}):L}).then(function(L){var j=p.getTypeOf(L);return j?(j==="arraybuffer"?L=p.transformTo("uint8array",L):j==="string"&&(D?L=y.decode(L):v&&T!==!0&&(L=function(tt){return E(tt,o.uint8array?new Uint8Array(tt.length):new Array(tt.length))}(L))),L):h.Promise.reject(new Error("Can't read the data of '"+g+"'. Is it in a supported JavaScript type (String, Blob, ArrayBuffer, etc) ?"))})}},{"./base64":1,"./external":6,"./nodejsUtils":14,"./support":30,setimmediate:54}],33:[function(c,d,p){var o=c("./reader/readerFor"),y=c("./utils"),m=c("./signature"),h=c("./zipEntry"),r=c("./support");function E(A){this.files=[],this.loadOptions=A}E.prototype={checkSignature:function(A){if(!this.reader.readAndCheckSignature(A)){this.reader.index-=4;var w=this.reader.readString(4);throw new Error("Corrupted zip or bug: unexpected signature ("+y.pretty(w)+", expected "+y.pretty(A)+")")}},isSignature:function(A,w){var _=this.reader.index;this.reader.setIndex(A);var z=this.reader.readString(4)===w;return this.reader.setIndex(_),z},readBlockEndOfCentral:function(){this.diskNumber=this.reader.readInt(2),this.diskWithCentralDirStart=this.reader.readInt(2),this.centralDirRecordsOnThisDisk=this.reader.readInt(2),this.centralDirRecords=this.reader.readInt(2),this.centralDirSize=this.reader.readInt(4),this.centralDirOffset=this.reader.readInt(4),this.zipCommentLength=this.reader.readInt(2);var A=this.reader.readData(this.zipCommentLength),w=r.uint8array?"uint8array":"array",_=y.transformTo(w,A);this.zipComment=this.loadOptions.decodeFileName(_)},readBlockZip64EndOfCentral:function(){this.zip64EndOfCentralSize=this.reader.readInt(8),this.reader.skip(4),this.diskNumber=this.reader.readInt(4),this.diskWithCentralDirStart=this.reader.readInt(4),this.centralDirRecordsOnThisDisk=this.reader.readInt(8),this.centralDirRecords=this.reader.readInt(8),this.centralDirSize=this.reader.readInt(8),this.centralDirOffset=this.reader.readInt(8),this.zip64ExtensibleData={};for(var A,w,_,z=this.zip64EndOfCentralSize-44;0<z;)A=this.reader.readInt(2),w=this.reader.readInt(4),_=this.reader.readData(w),this.zip64ExtensibleData[A]={id:A,length:w,value:_}},readBlockZip64EndOfCentralLocator:function(){if(this.diskWithZip64CentralDirStart=this.reader.readInt(4),this.relativeOffsetEndOfZip64CentralDir=this.reader.readInt(8),this.disksCount=this.reader.readInt(4),1<this.disksCount)throw new Error("Multi-volumes zip are not supported")},readLocalFiles:function(){var A,w;for(A=0;A<this.files.length;A++)w=this.files[A],this.reader.setIndex(w.localHeaderOffset),this.checkSignature(m.LOCAL_FILE_HEADER),w.readLocalPart(this.reader),w.handleUTF8(),w.processAttributes()},readCentralDir:function(){var A;for(this.reader.setIndex(this.centralDirOffset);this.reader.readAndCheckSignature(m.CENTRAL_FILE_HEADER);)(A=new h({zip64:this.zip64},this.loadOptions)).readCentralPart(this.reader),this.files.push(A);if(this.centralDirRecords!==this.files.length&&this.centralDirRecords!==0&&this.files.length===0)throw new Error("Corrupted zip or bug: expected "+this.centralDirRecords+" records in central dir, got "+this.files.length)},readEndOfCentral:function(){var A=this.reader.lastIndexOfSignature(m.CENTRAL_DIRECTORY_END);if(A<0)throw this.isSignature(0,m.LOCAL_FILE_HEADER)?new Error("Corrupted zip: can't find end of central directory"):new Error("Can't find end of central directory : is this a zip file ? If it is, see https://stuk.github.io/jszip/documentation/howto/read_zip.html");this.reader.setIndex(A);var w=A;if(this.checkSignature(m.CENTRAL_DIRECTORY_END),this.readBlockEndOfCentral(),this.diskNumber===y.MAX_VALUE_16BITS||this.diskWithCentralDirStart===y.MAX_VALUE_16BITS||this.centralDirRecordsOnThisDisk===y.MAX_VALUE_16BITS||this.centralDirRecords===y.MAX_VALUE_16BITS||this.centralDirSize===y.MAX_VALUE_32BITS||this.centralDirOffset===y.MAX_VALUE_32BITS){if(this.zip64=!0,(A=this.reader.lastIndexOfSignature(m.ZIP64_CENTRAL_DIRECTORY_LOCATOR))<0)throw new Error("Corrupted zip: can't find the ZIP64 end of central directory locator");if(this.reader.setIndex(A),this.checkSignature(m.ZIP64_CENTRAL_DIRECTORY_LOCATOR),this.readBlockZip64EndOfCentralLocator(),!this.isSignature(this.relativeOffsetEndOfZip64CentralDir,m.ZIP64_CENTRAL_DIRECTORY_END)&&(this.relativeOffsetEndOfZip64CentralDir=this.reader.lastIndexOfSignature(m.ZIP64_CENTRAL_DIRECTORY_END),this.relativeOffsetEndOfZip64CentralDir<0))throw new Error("Corrupted zip: can't find the ZIP64 end of central directory");this.reader.setIndex(this.relativeOffsetEndOfZip64CentralDir),this.checkSignature(m.ZIP64_CENTRAL_DIRECTORY_END),this.readBlockZip64EndOfCentral()}var _=this.centralDirOffset+this.centralDirSize;this.zip64&&(_+=20,_+=12+this.zip64EndOfCentralSize);var z=w-_;if(0<z)this.isSignature(w,m.CENTRAL_FILE_HEADER)||(this.reader.zero=z);else if(z<0)throw new Error("Corrupted zip: missing "+Math.abs(z)+" bytes.")},prepareReader:function(A){this.reader=o(A)},load:function(A){this.prepareReader(A),this.readEndOfCentral(),this.readCentralDir(),this.readLocalFiles()}},d.exports=E},{"./reader/readerFor":22,"./signature":23,"./support":30,"./utils":32,"./zipEntry":34}],34:[function(c,d,p){var o=c("./reader/readerFor"),y=c("./utils"),m=c("./compressedObject"),h=c("./crc32"),r=c("./utf8"),E=c("./compressions"),A=c("./support");function w(_,z){this.options=_,this.loadOptions=z}w.prototype={isEncrypted:function(){return(1&this.bitFlag)==1},useUTF8:function(){return(2048&this.bitFlag)==2048},readLocalPart:function(_){var z,g;if(_.skip(22),this.fileNameLength=_.readInt(2),g=_.readInt(2),this.fileName=_.readData(this.fileNameLength),_.skip(g),this.compressedSize===-1||this.uncompressedSize===-1)throw new Error("Bug or corrupted zip : didn't get enough information from the central directory (compressedSize === -1 || uncompressedSize === -1)");if((z=function(S){for(var v in E)if(Object.prototype.hasOwnProperty.call(E,v)&&E[v].magic===S)return E[v];return null}(this.compressionMethod))===null)throw new Error("Corrupted zip : compression "+y.pretty(this.compressionMethod)+" unknown (inner file : "+y.transformTo("string",this.fileName)+")");this.decompressed=new m(this.compressedSize,this.uncompressedSize,this.crc32,z,_.readData(this.compressedSize))},readCentralPart:function(_){this.versionMadeBy=_.readInt(2),_.skip(2),this.bitFlag=_.readInt(2),this.compressionMethod=_.readString(2),this.date=_.readDate(),this.crc32=_.readInt(4),this.compressedSize=_.readInt(4),this.uncompressedSize=_.readInt(4);var z=_.readInt(2);if(this.extraFieldsLength=_.readInt(2),this.fileCommentLength=_.readInt(2),this.diskNumberStart=_.readInt(2),this.internalFileAttributes=_.readInt(2),this.externalFileAttributes=_.readInt(4),this.localHeaderOffset=_.readInt(4),this.isEncrypted())throw new Error("Encrypted zip are not supported");_.skip(z),this.readExtraFields(_),this.parseZIP64ExtraField(_),this.fileComment=_.readData(this.fileCommentLength)},processAttributes:function(){this.unixPermissions=null,this.dosPermissions=null;var _=this.versionMadeBy>>8;this.dir=!!(16&this.externalFileAttributes),_==0&&(this.dosPermissions=63&this.externalFileAttributes),_==3&&(this.unixPermissions=this.externalFileAttributes>>16&65535),this.dir||this.fileNameStr.slice(-1)!=="/"||(this.dir=!0)},parseZIP64ExtraField:function(){if(this.extraFields[1]){var _=o(this.extraFields[1].value);this.uncompressedSize===y.MAX_VALUE_32BITS&&(this.uncompressedSize=_.readInt(8)),this.compressedSize===y.MAX_VALUE_32BITS&&(this.compressedSize=_.readInt(8)),this.localHeaderOffset===y.MAX_VALUE_32BITS&&(this.localHeaderOffset=_.readInt(8)),this.diskNumberStart===y.MAX_VALUE_32BITS&&(this.diskNumberStart=_.readInt(4))}},readExtraFields:function(_){var z,g,S,v=_.index+this.extraFieldsLength;for(this.extraFields||(this.extraFields={});_.index+4<v;)z=_.readInt(2),g=_.readInt(2),S=_.readData(g),this.extraFields[z]={id:z,length:g,value:S};_.setIndex(v)},handleUTF8:function(){var _=A.uint8array?"uint8array":"array";if(this.useUTF8())this.fileNameStr=r.utf8decode(this.fileName),this.fileCommentStr=r.utf8decode(this.fileComment);else{var z=this.findExtraFieldUnicodePath();if(z!==null)this.fileNameStr=z;else{var g=y.transformTo(_,this.fileName);this.fileNameStr=this.loadOptions.decodeFileName(g)}var S=this.findExtraFieldUnicodeComment();if(S!==null)this.fileCommentStr=S;else{var v=y.transformTo(_,this.fileComment);this.fileCommentStr=this.loadOptions.decodeFileName(v)}}},findExtraFieldUnicodePath:function(){var _=this.extraFields[28789];if(_){var z=o(_.value);return z.readInt(1)!==1||h(this.fileName)!==z.readInt(4)?null:r.utf8decode(z.readData(_.length-5))}return null},findExtraFieldUnicodeComment:function(){var _=this.extraFields[25461];if(_){var z=o(_.value);return z.readInt(1)!==1||h(this.fileComment)!==z.readInt(4)?null:r.utf8decode(z.readData(_.length-5))}return null}},d.exports=w},{"./compressedObject":2,"./compressions":3,"./crc32":4,"./reader/readerFor":22,"./support":30,"./utf8":31,"./utils":32}],35:[function(c,d,p){function o(z,g,S){this.name=z,this.dir=S.dir,this.date=S.date,this.comment=S.comment,this.unixPermissions=S.unixPermissions,this.dosPermissions=S.dosPermissions,this._data=g,this._dataBinary=S.binary,this.options={compression:S.compression,compressionOptions:S.compressionOptions}}var y=c("./stream/StreamHelper"),m=c("./stream/DataWorker"),h=c("./utf8"),r=c("./compressedObject"),E=c("./stream/GenericWorker");o.prototype={internalStream:function(z){var g=null,S="string";try{if(!z)throw new Error("No output type specified.");var v=(S=z.toLowerCase())==="string"||S==="text";S!=="binarystring"&&S!=="text"||(S="string"),g=this._decompressWorker();var T=!this._dataBinary;T&&!v&&(g=g.pipe(new h.Utf8EncodeWorker)),!T&&v&&(g=g.pipe(new h.Utf8DecodeWorker))}catch(D){(g=new E("error")).error(D)}return new y(g,S,"")},async:function(z,g){return this.internalStream(z).accumulate(g)},nodeStream:function(z,g){return this.internalStream(z||"nodebuffer").toNodejsStream(g)},_compressWorker:function(z,g){if(this._data instanceof r&&this._data.compression.magic===z.magic)return this._data.getCompressedWorker();var S=this._decompressWorker();return this._dataBinary||(S=S.pipe(new h.Utf8EncodeWorker)),r.createWorkerFrom(S,z,g)},_decompressWorker:function(){return this._data instanceof r?this._data.getContentWorker():this._data instanceof E?this._data:new m(this._data)}};for(var A=["asText","asBinary","asNodeBuffer","asUint8Array","asArrayBuffer"],w=function(){throw new Error("This method has been removed in JSZip 3.0, please check the upgrade guide.")},_=0;_<A.length;_++)o.prototype[A[_]]=w;d.exports=o},{"./compressedObject":2,"./stream/DataWorker":27,"./stream/GenericWorker":28,"./stream/StreamHelper":29,"./utf8":31}],36:[function(c,d,p){(function(o){var y,m,h=o.MutationObserver||o.WebKitMutationObserver;if(h){var r=0,E=new h(z),A=o.document.createTextNode("");E.observe(A,{characterData:!0}),y=function(){A.data=r=++r%2}}else if(o.setImmediate||o.MessageChannel===void 0)y="document"in o&&"onreadystatechange"in o.document.createElement("script")?function(){var g=o.document.createElement("script");g.onreadystatechange=function(){z(),g.onreadystatechange=null,g.parentNode.removeChild(g),g=null},o.document.documentElement.appendChild(g)}:function(){setTimeout(z,0)};else{var w=new o.MessageChannel;w.port1.onmessage=z,y=function(){w.port2.postMessage(0)}}var _=[];function z(){var g,S;m=!0;for(var v=_.length;v;){for(S=_,_=[],g=-1;++g<v;)S[g]();v=_.length}m=!1}d.exports=function(g){_.push(g)!==1||m||y()}}).call(this,typeof Qa<"u"?Qa:typeof self<"u"?self:typeof window<"u"?window:{})},{}],37:[function(c,d,p){var o=c("immediate");function y(){}var m={},h=["REJECTED"],r=["FULFILLED"],E=["PENDING"];function A(v){if(typeof v!="function")throw new TypeError("resolver must be a function");this.state=E,this.queue=[],this.outcome=void 0,v!==y&&g(this,v)}function w(v,T,D){this.promise=v,typeof T=="function"&&(this.onFulfilled=T,this.callFulfilled=this.otherCallFulfilled),typeof D=="function"&&(this.onRejected=D,this.callRejected=this.otherCallRejected)}function _(v,T,D){o(function(){var L;try{L=T(D)}catch(j){return m.reject(v,j)}L===v?m.reject(v,new TypeError("Cannot resolve promise with itself")):m.resolve(v,L)})}function z(v){var T=v&&v.then;if(v&&(typeof v=="object"||typeof v=="function")&&typeof T=="function")return function(){T.apply(v,arguments)}}function g(v,T){var D=!1;function L(F){D||(D=!0,m.reject(v,F))}function j(F){D||(D=!0,m.resolve(v,F))}var tt=S(function(){T(j,L)});tt.status==="error"&&L(tt.value)}function S(v,T){var D={};try{D.value=v(T),D.status="success"}catch(L){D.status="error",D.value=L}return D}(d.exports=A).prototype.finally=function(v){if(typeof v!="function")return this;var T=this.constructor;return this.then(function(D){return T.resolve(v()).then(function(){return D})},function(D){return T.resolve(v()).then(function(){throw D})})},A.prototype.catch=function(v){return this.then(null,v)},A.prototype.then=function(v,T){if(typeof v!="function"&&this.state===r||typeof T!="function"&&this.state===h)return this;var D=new this.constructor(y);return this.state!==E?_(D,this.state===r?v:T,this.outcome):this.queue.push(new w(D,v,T)),D},w.prototype.callFulfilled=function(v){m.resolve(this.promise,v)},w.prototype.otherCallFulfilled=function(v){_(this.promise,this.onFulfilled,v)},w.prototype.callRejected=function(v){m.reject(this.promise,v)},w.prototype.otherCallRejected=function(v){_(this.promise,this.onRejected,v)},m.resolve=function(v,T){var D=S(z,T);if(D.status==="error")return m.reject(v,D.value);var L=D.value;if(L)g(v,L);else{v.state=r,v.outcome=T;for(var j=-1,tt=v.queue.length;++j<tt;)v.queue[j].callFulfilled(T)}return v},m.reject=function(v,T){v.state=h,v.outcome=T;for(var D=-1,L=v.queue.length;++D<L;)v.queue[D].callRejected(T);return v},A.resolve=function(v){return v instanceof this?v:m.resolve(new this(y),v)},A.reject=function(v){var T=new this(y);return m.reject(T,v)},A.all=function(v){var T=this;if(Object.prototype.toString.call(v)!=="[object Array]")return this.reject(new TypeError("must be an array"));var D=v.length,L=!1;if(!D)return this.resolve([]);for(var j=new Array(D),tt=0,F=-1,ft=new this(y);++F<D;)$(v[F],F);return ft;function $(rt,vt){T.resolve(rt).then(function(M){j[vt]=M,++tt!==D||L||(L=!0,m.resolve(ft,j))},function(M){L||(L=!0,m.reject(ft,M))})}},A.race=function(v){var T=this;if(Object.prototype.toString.call(v)!=="[object Array]")return this.reject(new TypeError("must be an array"));var D=v.length,L=!1;if(!D)return this.resolve([]);for(var j=-1,tt=new this(y);++j<D;)F=v[j],T.resolve(F).then(function(ft){L||(L=!0,m.resolve(tt,ft))},function(ft){L||(L=!0,m.reject(tt,ft))});var F;return tt}},{immediate:36}],38:[function(c,d,p){var o={};(0,c("./lib/utils/common").assign)(o,c("./lib/deflate"),c("./lib/inflate"),c("./lib/zlib/constants")),d.exports=o},{"./lib/deflate":39,"./lib/inflate":40,"./lib/utils/common":41,"./lib/zlib/constants":44}],39:[function(c,d,p){var o=c("./zlib/deflate"),y=c("./utils/common"),m=c("./utils/strings"),h=c("./zlib/messages"),r=c("./zlib/zstream"),E=Object.prototype.toString,A=0,w=-1,_=0,z=8;function g(v){if(!(this instanceof g))return new g(v);this.options=y.assign({level:w,method:z,chunkSize:16384,windowBits:15,memLevel:8,strategy:_,to:""},v||{});var T=this.options;T.raw&&0<T.windowBits?T.windowBits=-T.windowBits:T.gzip&&0<T.windowBits&&T.windowBits<16&&(T.windowBits+=16),this.err=0,this.msg="",this.ended=!1,this.chunks=[],this.strm=new r,this.strm.avail_out=0;var D=o.deflateInit2(this.strm,T.level,T.method,T.windowBits,T.memLevel,T.strategy);if(D!==A)throw new Error(h[D]);if(T.header&&o.deflateSetHeader(this.strm,T.header),T.dictionary){var L;if(L=typeof T.dictionary=="string"?m.string2buf(T.dictionary):E.call(T.dictionary)==="[object ArrayBuffer]"?new Uint8Array(T.dictionary):T.dictionary,(D=o.deflateSetDictionary(this.strm,L))!==A)throw new Error(h[D]);this._dict_set=!0}}function S(v,T){var D=new g(T);if(D.push(v,!0),D.err)throw D.msg||h[D.err];return D.result}g.prototype.push=function(v,T){var D,L,j=this.strm,tt=this.options.chunkSize;if(this.ended)return!1;L=T===~~T?T:T===!0?4:0,typeof v=="string"?j.input=m.string2buf(v):E.call(v)==="[object ArrayBuffer]"?j.input=new Uint8Array(v):j.input=v,j.next_in=0,j.avail_in=j.input.length;do{if(j.avail_out===0&&(j.output=new y.Buf8(tt),j.next_out=0,j.avail_out=tt),(D=o.deflate(j,L))!==1&&D!==A)return this.onEnd(D),!(this.ended=!0);j.avail_out!==0&&(j.avail_in!==0||L!==4&&L!==2)||(this.options.to==="string"?this.onData(m.buf2binstring(y.shrinkBuf(j.output,j.next_out))):this.onData(y.shrinkBuf(j.output,j.next_out)))}while((0<j.avail_in||j.avail_out===0)&&D!==1);return L===4?(D=o.deflateEnd(this.strm),this.onEnd(D),this.ended=!0,D===A):L!==2||(this.onEnd(A),!(j.avail_out=0))},g.prototype.onData=function(v){this.chunks.push(v)},g.prototype.onEnd=function(v){v===A&&(this.options.to==="string"?this.result=this.chunks.join(""):this.result=y.flattenChunks(this.chunks)),this.chunks=[],this.err=v,this.msg=this.strm.msg},p.Deflate=g,p.deflate=S,p.deflateRaw=function(v,T){return(T=T||{}).raw=!0,S(v,T)},p.gzip=function(v,T){return(T=T||{}).gzip=!0,S(v,T)}},{"./utils/common":41,"./utils/strings":42,"./zlib/deflate":46,"./zlib/messages":51,"./zlib/zstream":53}],40:[function(c,d,p){var o=c("./zlib/inflate"),y=c("./utils/common"),m=c("./utils/strings"),h=c("./zlib/constants"),r=c("./zlib/messages"),E=c("./zlib/zstream"),A=c("./zlib/gzheader"),w=Object.prototype.toString;function _(g){if(!(this instanceof _))return new _(g);this.options=y.assign({chunkSize:16384,windowBits:0,to:""},g||{});var S=this.options;S.raw&&0<=S.windowBits&&S.windowBits<16&&(S.windowBits=-S.windowBits,S.windowBits===0&&(S.windowBits=-15)),!(0<=S.windowBits&&S.windowBits<16)||g&&g.windowBits||(S.windowBits+=32),15<S.windowBits&&S.windowBits<48&&(15&S.windowBits)==0&&(S.windowBits|=15),this.err=0,this.msg="",this.ended=!1,this.chunks=[],this.strm=new E,this.strm.avail_out=0;var v=o.inflateInit2(this.strm,S.windowBits);if(v!==h.Z_OK)throw new Error(r[v]);this.header=new A,o.inflateGetHeader(this.strm,this.header)}function z(g,S){var v=new _(S);if(v.push(g,!0),v.err)throw v.msg||r[v.err];return v.result}_.prototype.push=function(g,S){var v,T,D,L,j,tt,F=this.strm,ft=this.options.chunkSize,$=this.options.dictionary,rt=!1;if(this.ended)return!1;T=S===~~S?S:S===!0?h.Z_FINISH:h.Z_NO_FLUSH,typeof g=="string"?F.input=m.binstring2buf(g):w.call(g)==="[object ArrayBuffer]"?F.input=new Uint8Array(g):F.input=g,F.next_in=0,F.avail_in=F.input.length;do{if(F.avail_out===0&&(F.output=new y.Buf8(ft),F.next_out=0,F.avail_out=ft),(v=o.inflate(F,h.Z_NO_FLUSH))===h.Z_NEED_DICT&&$&&(tt=typeof $=="string"?m.string2buf($):w.call($)==="[object ArrayBuffer]"?new Uint8Array($):$,v=o.inflateSetDictionary(this.strm,tt)),v===h.Z_BUF_ERROR&&rt===!0&&(v=h.Z_OK,rt=!1),v!==h.Z_STREAM_END&&v!==h.Z_OK)return this.onEnd(v),!(this.ended=!0);F.next_out&&(F.avail_out!==0&&v!==h.Z_STREAM_END&&(F.avail_in!==0||T!==h.Z_FINISH&&T!==h.Z_SYNC_FLUSH)||(this.options.to==="string"?(D=m.utf8border(F.output,F.next_out),L=F.next_out-D,j=m.buf2string(F.output,D),F.next_out=L,F.avail_out=ft-L,L&&y.arraySet(F.output,F.output,D,L,0),this.onData(j)):this.onData(y.shrinkBuf(F.output,F.next_out)))),F.avail_in===0&&F.avail_out===0&&(rt=!0)}while((0<F.avail_in||F.avail_out===0)&&v!==h.Z_STREAM_END);return v===h.Z_STREAM_END&&(T=h.Z_FINISH),T===h.Z_FINISH?(v=o.inflateEnd(this.strm),this.onEnd(v),this.ended=!0,v===h.Z_OK):T!==h.Z_SYNC_FLUSH||(this.onEnd(h.Z_OK),!(F.avail_out=0))},_.prototype.onData=function(g){this.chunks.push(g)},_.prototype.onEnd=function(g){g===h.Z_OK&&(this.options.to==="string"?this.result=this.chunks.join(""):this.result=y.flattenChunks(this.chunks)),this.chunks=[],this.err=g,this.msg=this.strm.msg},p.Inflate=_,p.inflate=z,p.inflateRaw=function(g,S){return(S=S||{}).raw=!0,z(g,S)},p.ungzip=z},{"./utils/common":41,"./utils/strings":42,"./zlib/constants":44,"./zlib/gzheader":47,"./zlib/inflate":49,"./zlib/messages":51,"./zlib/zstream":53}],41:[function(c,d,p){var o=typeof Uint8Array<"u"&&typeof Uint16Array<"u"&&typeof Int32Array<"u";p.assign=function(h){for(var r=Array.prototype.slice.call(arguments,1);r.length;){var E=r.shift();if(E){if(typeof E!="object")throw new TypeError(E+"must be non-object");for(var A in E)E.hasOwnProperty(A)&&(h[A]=E[A])}}return h},p.shrinkBuf=function(h,r){return h.length===r?h:h.subarray?h.subarray(0,r):(h.length=r,h)};var y={arraySet:function(h,r,E,A,w){if(r.subarray&&h.subarray)h.set(r.subarray(E,E+A),w);else for(var _=0;_<A;_++)h[w+_]=r[E+_]},flattenChunks:function(h){var r,E,A,w,_,z;for(r=A=0,E=h.length;r<E;r++)A+=h[r].length;for(z=new Uint8Array(A),r=w=0,E=h.length;r<E;r++)_=h[r],z.set(_,w),w+=_.length;return z}},m={arraySet:function(h,r,E,A,w){for(var _=0;_<A;_++)h[w+_]=r[E+_]},flattenChunks:function(h){return[].concat.apply([],h)}};p.setTyped=function(h){h?(p.Buf8=Uint8Array,p.Buf16=Uint16Array,p.Buf32=Int32Array,p.assign(p,y)):(p.Buf8=Array,p.Buf16=Array,p.Buf32=Array,p.assign(p,m))},p.setTyped(o)},{}],42:[function(c,d,p){var o=c("./common"),y=!0,m=!0;try{String.fromCharCode.apply(null,[0])}catch{y=!1}try{String.fromCharCode.apply(null,new Uint8Array(1))}catch{m=!1}for(var h=new o.Buf8(256),r=0;r<256;r++)h[r]=252<=r?6:248<=r?5:240<=r?4:224<=r?3:192<=r?2:1;function E(A,w){if(w<65537&&(A.subarray&&m||!A.subarray&&y))return String.fromCharCode.apply(null,o.shrinkBuf(A,w));for(var _="",z=0;z<w;z++)_+=String.fromCharCode(A[z]);return _}h[254]=h[254]=1,p.string2buf=function(A){var w,_,z,g,S,v=A.length,T=0;for(g=0;g<v;g++)(64512&(_=A.charCodeAt(g)))==55296&&g+1<v&&(64512&(z=A.charCodeAt(g+1)))==56320&&(_=65536+(_-55296<<10)+(z-56320),g++),T+=_<128?1:_<2048?2:_<65536?3:4;for(w=new o.Buf8(T),g=S=0;S<T;g++)(64512&(_=A.charCodeAt(g)))==55296&&g+1<v&&(64512&(z=A.charCodeAt(g+1)))==56320&&(_=65536+(_-55296<<10)+(z-56320),g++),_<128?w[S++]=_:(_<2048?w[S++]=192|_>>>6:(_<65536?w[S++]=224|_>>>12:(w[S++]=240|_>>>18,w[S++]=128|_>>>12&63),w[S++]=128|_>>>6&63),w[S++]=128|63&_);return w},p.buf2binstring=function(A){return E(A,A.length)},p.binstring2buf=function(A){for(var w=new o.Buf8(A.length),_=0,z=w.length;_<z;_++)w[_]=A.charCodeAt(_);return w},p.buf2string=function(A,w){var _,z,g,S,v=w||A.length,T=new Array(2*v);for(_=z=0;_<v;)if((g=A[_++])<128)T[z++]=g;else if(4<(S=h[g]))T[z++]=65533,_+=S-1;else{for(g&=S===2?31:S===3?15:7;1<S&&_<v;)g=g<<6|63&A[_++],S--;1<S?T[z++]=65533:g<65536?T[z++]=g:(g-=65536,T[z++]=55296|g>>10&1023,T[z++]=56320|1023&g)}return E(T,z)},p.utf8border=function(A,w){var _;for((w=w||A.length)>A.length&&(w=A.length),_=w-1;0<=_&&(192&A[_])==128;)_--;return _<0||_===0?w:_+h[A[_]]>w?_:w}},{"./common":41}],43:[function(c,d,p){d.exports=function(o,y,m,h){for(var r=65535&o|0,E=o>>>16&65535|0,A=0;m!==0;){for(m-=A=2e3<m?2e3:m;E=E+(r=r+y[h++]|0)|0,--A;);r%=65521,E%=65521}return r|E<<16|0}},{}],44:[function(c,d,p){d.exports={Z_NO_FLUSH:0,Z_PARTIAL_FLUSH:1,Z_SYNC_FLUSH:2,Z_FULL_FLUSH:3,Z_FINISH:4,Z_BLOCK:5,Z_TREES:6,Z_OK:0,Z_STREAM_END:1,Z_NEED_DICT:2,Z_ERRNO:-1,Z_STREAM_ERROR:-2,Z_DATA_ERROR:-3,Z_BUF_ERROR:-5,Z_NO_COMPRESSION:0,Z_BEST_SPEED:1,Z_BEST_COMPRESSION:9,Z_DEFAULT_COMPRESSION:-1,Z_FILTERED:1,Z_HUFFMAN_ONLY:2,Z_RLE:3,Z_FIXED:4,Z_DEFAULT_STRATEGY:0,Z_BINARY:0,Z_TEXT:1,Z_UNKNOWN:2,Z_DEFLATED:8}},{}],45:[function(c,d,p){var o=function(){for(var y,m=[],h=0;h<256;h++){y=h;for(var r=0;r<8;r++)y=1&y?3988292384^y>>>1:y>>>1;m[h]=y}return m}();d.exports=function(y,m,h,r){var E=o,A=r+h;y^=-1;for(var w=r;w<A;w++)y=y>>>8^E[255&(y^m[w])];return-1^y}},{}],46:[function(c,d,p){var o,y=c("../utils/common"),m=c("./trees"),h=c("./adler32"),r=c("./crc32"),E=c("./messages"),A=0,w=4,_=0,z=-2,g=-1,S=4,v=2,T=8,D=9,L=286,j=30,tt=19,F=2*L+1,ft=15,$=3,rt=258,vt=rt+$+1,M=42,et=113,b=1,at=2,G=3,X=4;function gt(f,ot){return f.msg=E[ot],ot}function st(f){return(f<<1)-(4<f?9:0)}function C(f){for(var ot=f.length;0<=--ot;)f[ot]=0}function N(f){var ot=f.state,P=ot.pending;P>f.avail_out&&(P=f.avail_out),P!==0&&(y.arraySet(f.output,ot.pending_buf,ot.pending_out,P,f.next_out),f.next_out+=P,ot.pending_out+=P,f.total_out+=P,f.avail_out-=P,ot.pending-=P,ot.pending===0&&(ot.pending_out=0))}function B(f,ot){m._tr_flush_block(f,0<=f.block_start?f.block_start:-1,f.strstart-f.block_start,ot),f.block_start=f.strstart,N(f.strm)}function nt(f,ot){f.pending_buf[f.pending++]=ot}function it(f,ot){f.pending_buf[f.pending++]=ot>>>8&255,f.pending_buf[f.pending++]=255&ot}function dt(f,ot){var P,k,O=f.max_chain_length,q=f.strstart,ht=f.prev_length,mt=f.nice_match,K=f.strstart>f.w_size-vt?f.strstart-(f.w_size-vt):0,pt=f.window,bt=f.w_mask,yt=f.prev,Et=f.strstart+rt,kt=pt[q+ht-1],Rt=pt[q+ht];f.prev_length>=f.good_match&&(O>>=2),mt>f.lookahead&&(mt=f.lookahead);do if(pt[(P=ot)+ht]===Rt&&pt[P+ht-1]===kt&&pt[P]===pt[q]&&pt[++P]===pt[q+1]){q+=2,P++;do;while(pt[++q]===pt[++P]&&pt[++q]===pt[++P]&&pt[++q]===pt[++P]&&pt[++q]===pt[++P]&&pt[++q]===pt[++P]&&pt[++q]===pt[++P]&&pt[++q]===pt[++P]&&pt[++q]===pt[++P]&&q<Et);if(k=rt-(Et-q),q=Et-rt,ht<k){if(f.match_start=ot,mt<=(ht=k))break;kt=pt[q+ht-1],Rt=pt[q+ht]}}while((ot=yt[ot&bt])>K&&--O!=0);return ht<=f.lookahead?ht:f.lookahead}function _t(f){var ot,P,k,O,q,ht,mt,K,pt,bt,yt=f.w_size;do{if(O=f.window_size-f.lookahead-f.strstart,f.strstart>=yt+(yt-vt)){for(y.arraySet(f.window,f.window,yt,yt,0),f.match_start-=yt,f.strstart-=yt,f.block_start-=yt,ot=P=f.hash_size;k=f.head[--ot],f.head[ot]=yt<=k?k-yt:0,--P;);for(ot=P=yt;k=f.prev[--ot],f.prev[ot]=yt<=k?k-yt:0,--P;);O+=yt}if(f.strm.avail_in===0)break;if(ht=f.strm,mt=f.window,K=f.strstart+f.lookahead,pt=O,bt=void 0,bt=ht.avail_in,pt<bt&&(bt=pt),P=bt===0?0:(ht.avail_in-=bt,y.arraySet(mt,ht.input,ht.next_in,bt,K),ht.state.wrap===1?ht.adler=h(ht.adler,mt,bt,K):ht.state.wrap===2&&(ht.adler=r(ht.adler,mt,bt,K)),ht.next_in+=bt,ht.total_in+=bt,bt),f.lookahead+=P,f.lookahead+f.insert>=$)for(q=f.strstart-f.insert,f.ins_h=f.window[q],f.ins_h=(f.ins_h<<f.hash_shift^f.window[q+1])&f.hash_mask;f.insert&&(f.ins_h=(f.ins_h<<f.hash_shift^f.window[q+$-1])&f.hash_mask,f.prev[q&f.w_mask]=f.head[f.ins_h],f.head[f.ins_h]=q,q++,f.insert--,!(f.lookahead+f.insert<$)););}while(f.lookahead<vt&&f.strm.avail_in!==0)}function Lt(f,ot){for(var P,k;;){if(f.lookahead<vt){if(_t(f),f.lookahead<vt&&ot===A)return b;if(f.lookahead===0)break}if(P=0,f.lookahead>=$&&(f.ins_h=(f.ins_h<<f.hash_shift^f.window[f.strstart+$-1])&f.hash_mask,P=f.prev[f.strstart&f.w_mask]=f.head[f.ins_h],f.head[f.ins_h]=f.strstart),P!==0&&f.strstart-P<=f.w_size-vt&&(f.match_length=dt(f,P)),f.match_length>=$)if(k=m._tr_tally(f,f.strstart-f.match_start,f.match_length-$),f.lookahead-=f.match_length,f.match_length<=f.max_lazy_match&&f.lookahead>=$){for(f.match_length--;f.strstart++,f.ins_h=(f.ins_h<<f.hash_shift^f.window[f.strstart+$-1])&f.hash_mask,P=f.prev[f.strstart&f.w_mask]=f.head[f.ins_h],f.head[f.ins_h]=f.strstart,--f.match_length!=0;);f.strstart++}else f.strstart+=f.match_length,f.match_length=0,f.ins_h=f.window[f.strstart],f.ins_h=(f.ins_h<<f.hash_shift^f.window[f.strstart+1])&f.hash_mask;else k=m._tr_tally(f,0,f.window[f.strstart]),f.lookahead--,f.strstart++;if(k&&(B(f,!1),f.strm.avail_out===0))return b}return f.insert=f.strstart<$-1?f.strstart:$-1,ot===w?(B(f,!0),f.strm.avail_out===0?G:X):f.last_lit&&(B(f,!1),f.strm.avail_out===0)?b:at}function xt(f,ot){for(var P,k,O;;){if(f.lookahead<vt){if(_t(f),f.lookahead<vt&&ot===A)return b;if(f.lookahead===0)break}if(P=0,f.lookahead>=$&&(f.ins_h=(f.ins_h<<f.hash_shift^f.window[f.strstart+$-1])&f.hash_mask,P=f.prev[f.strstart&f.w_mask]=f.head[f.ins_h],f.head[f.ins_h]=f.strstart),f.prev_length=f.match_length,f.prev_match=f.match_start,f.match_length=$-1,P!==0&&f.prev_length<f.max_lazy_match&&f.strstart-P<=f.w_size-vt&&(f.match_length=dt(f,P),f.match_length<=5&&(f.strategy===1||f.match_length===$&&4096<f.strstart-f.match_start)&&(f.match_length=$-1)),f.prev_length>=$&&f.match_length<=f.prev_length){for(O=f.strstart+f.lookahead-$,k=m._tr_tally(f,f.strstart-1-f.prev_match,f.prev_length-$),f.lookahead-=f.prev_length-1,f.prev_length-=2;++f.strstart<=O&&(f.ins_h=(f.ins_h<<f.hash_shift^f.window[f.strstart+$-1])&f.hash_mask,P=f.prev[f.strstart&f.w_mask]=f.head[f.ins_h],f.head[f.ins_h]=f.strstart),--f.prev_length!=0;);if(f.match_available=0,f.match_length=$-1,f.strstart++,k&&(B(f,!1),f.strm.avail_out===0))return b}else if(f.match_available){if((k=m._tr_tally(f,0,f.window[f.strstart-1]))&&B(f,!1),f.strstart++,f.lookahead--,f.strm.avail_out===0)return b}else f.match_available=1,f.strstart++,f.lookahead--}return f.match_available&&(k=m._tr_tally(f,0,f.window[f.strstart-1]),f.match_available=0),f.insert=f.strstart<$-1?f.strstart:$-1,ot===w?(B(f,!0),f.strm.avail_out===0?G:X):f.last_lit&&(B(f,!1),f.strm.avail_out===0)?b:at}function Ct(f,ot,P,k,O){this.good_length=f,this.max_lazy=ot,this.nice_length=P,this.max_chain=k,this.func=O}function Pt(){this.strm=null,this.status=0,this.pending_buf=null,this.pending_buf_size=0,this.pending_out=0,this.pending=0,this.wrap=0,this.gzhead=null,this.gzindex=0,this.method=T,this.last_flush=-1,this.w_size=0,this.w_bits=0,this.w_mask=0,this.window=null,this.window_size=0,this.prev=null,this.head=null,this.ins_h=0,this.hash_size=0,this.hash_bits=0,this.hash_mask=0,this.hash_shift=0,this.block_start=0,this.match_length=0,this.prev_match=0,this.match_available=0,this.strstart=0,this.match_start=0,this.lookahead=0,this.prev_length=0,this.max_chain_length=0,this.max_lazy_match=0,this.level=0,this.strategy=0,this.good_match=0,this.nice_match=0,this.dyn_ltree=new y.Buf16(2*F),this.dyn_dtree=new y.Buf16(2*(2*j+1)),this.bl_tree=new y.Buf16(2*(2*tt+1)),C(this.dyn_ltree),C(this.dyn_dtree),C(this.bl_tree),this.l_desc=null,this.d_desc=null,this.bl_desc=null,this.bl_count=new y.Buf16(ft+1),this.heap=new y.Buf16(2*L+1),C(this.heap),this.heap_len=0,this.heap_max=0,this.depth=new y.Buf16(2*L+1),C(this.depth),this.l_buf=0,this.lit_bufsize=0,this.last_lit=0,this.d_buf=0,this.opt_len=0,this.static_len=0,this.matches=0,this.insert=0,this.bi_buf=0,this.bi_valid=0}function Vt(f){var ot;return f&&f.state?(f.total_in=f.total_out=0,f.data_type=v,(ot=f.state).pending=0,ot.pending_out=0,ot.wrap<0&&(ot.wrap=-ot.wrap),ot.status=ot.wrap?M:et,f.adler=ot.wrap===2?0:1,ot.last_flush=A,m._tr_init(ot),_):gt(f,z)}function _e(f){var ot=Vt(f);return ot===_&&function(P){P.window_size=2*P.w_size,C(P.head),P.max_lazy_match=o[P.level].max_lazy,P.good_match=o[P.level].good_length,P.nice_match=o[P.level].nice_length,P.max_chain_length=o[P.level].max_chain,P.strstart=0,P.block_start=0,P.lookahead=0,P.insert=0,P.match_length=P.prev_length=$-1,P.match_available=0,P.ins_h=0}(f.state),ot}function ve(f,ot,P,k,O,q){if(!f)return z;var ht=1;if(ot===g&&(ot=6),k<0?(ht=0,k=-k):15<k&&(ht=2,k-=16),O<1||D<O||P!==T||k<8||15<k||ot<0||9<ot||q<0||S<q)return gt(f,z);k===8&&(k=9);var mt=new Pt;return(f.state=mt).strm=f,mt.wrap=ht,mt.gzhead=null,mt.w_bits=k,mt.w_size=1<<mt.w_bits,mt.w_mask=mt.w_size-1,mt.hash_bits=O+7,mt.hash_size=1<<mt.hash_bits,mt.hash_mask=mt.hash_size-1,mt.hash_shift=~~((mt.hash_bits+$-1)/$),mt.window=new y.Buf8(2*mt.w_size),mt.head=new y.Buf16(mt.hash_size),mt.prev=new y.Buf16(mt.w_size),mt.lit_bufsize=1<<O+6,mt.pending_buf_size=4*mt.lit_bufsize,mt.pending_buf=new y.Buf8(mt.pending_buf_size),mt.d_buf=1*mt.lit_bufsize,mt.l_buf=3*mt.lit_bufsize,mt.level=ot,mt.strategy=q,mt.method=P,_e(f)}o=[new Ct(0,0,0,0,function(f,ot){var P=65535;for(P>f.pending_buf_size-5&&(P=f.pending_buf_size-5);;){if(f.lookahead<=1){if(_t(f),f.lookahead===0&&ot===A)return b;if(f.lookahead===0)break}f.strstart+=f.lookahead,f.lookahead=0;var k=f.block_start+P;if((f.strstart===0||f.strstart>=k)&&(f.lookahead=f.strstart-k,f.strstart=k,B(f,!1),f.strm.avail_out===0)||f.strstart-f.block_start>=f.w_size-vt&&(B(f,!1),f.strm.avail_out===0))return b}return f.insert=0,ot===w?(B(f,!0),f.strm.avail_out===0?G:X):(f.strstart>f.block_start&&(B(f,!1),f.strm.avail_out),b)}),new Ct(4,4,8,4,Lt),new Ct(4,5,16,8,Lt),new Ct(4,6,32,32,Lt),new Ct(4,4,16,16,xt),new Ct(8,16,32,32,xt),new Ct(8,16,128,128,xt),new Ct(8,32,128,256,xt),new Ct(32,128,258,1024,xt),new Ct(32,258,258,4096,xt)],p.deflateInit=function(f,ot){return ve(f,ot,T,15,8,0)},p.deflateInit2=ve,p.deflateReset=_e,p.deflateResetKeep=Vt,p.deflateSetHeader=function(f,ot){return f&&f.state?f.state.wrap!==2?z:(f.state.gzhead=ot,_):z},p.deflate=function(f,ot){var P,k,O,q;if(!f||!f.state||5<ot||ot<0)return f?gt(f,z):z;if(k=f.state,!f.output||!f.input&&f.avail_in!==0||k.status===666&&ot!==w)return gt(f,f.avail_out===0?-5:z);if(k.strm=f,P=k.last_flush,k.last_flush=ot,k.status===M)if(k.wrap===2)f.adler=0,nt(k,31),nt(k,139),nt(k,8),k.gzhead?(nt(k,(k.gzhead.text?1:0)+(k.gzhead.hcrc?2:0)+(k.gzhead.extra?4:0)+(k.gzhead.name?8:0)+(k.gzhead.comment?16:0)),nt(k,255&k.gzhead.time),nt(k,k.gzhead.time>>8&255),nt(k,k.gzhead.time>>16&255),nt(k,k.gzhead.time>>24&255),nt(k,k.level===9?2:2<=k.strategy||k.level<2?4:0),nt(k,255&k.gzhead.os),k.gzhead.extra&&k.gzhead.extra.length&&(nt(k,255&k.gzhead.extra.length),nt(k,k.gzhead.extra.length>>8&255)),k.gzhead.hcrc&&(f.adler=r(f.adler,k.pending_buf,k.pending,0)),k.gzindex=0,k.status=69):(nt(k,0),nt(k,0),nt(k,0),nt(k,0),nt(k,0),nt(k,k.level===9?2:2<=k.strategy||k.level<2?4:0),nt(k,3),k.status=et);else{var ht=T+(k.w_bits-8<<4)<<8;ht|=(2<=k.strategy||k.level<2?0:k.level<6?1:k.level===6?2:3)<<6,k.strstart!==0&&(ht|=32),ht+=31-ht%31,k.status=et,it(k,ht),k.strstart!==0&&(it(k,f.adler>>>16),it(k,65535&f.adler)),f.adler=1}if(k.status===69)if(k.gzhead.extra){for(O=k.pending;k.gzindex<(65535&k.gzhead.extra.length)&&(k.pending!==k.pending_buf_size||(k.gzhead.hcrc&&k.pending>O&&(f.adler=r(f.adler,k.pending_buf,k.pending-O,O)),N(f),O=k.pending,k.pending!==k.pending_buf_size));)nt(k,255&k.gzhead.extra[k.gzindex]),k.gzindex++;k.gzhead.hcrc&&k.pending>O&&(f.adler=r(f.adler,k.pending_buf,k.pending-O,O)),k.gzindex===k.gzhead.extra.length&&(k.gzindex=0,k.status=73)}else k.status=73;if(k.status===73)if(k.gzhead.name){O=k.pending;do{if(k.pending===k.pending_buf_size&&(k.gzhead.hcrc&&k.pending>O&&(f.adler=r(f.adler,k.pending_buf,k.pending-O,O)),N(f),O=k.pending,k.pending===k.pending_buf_size)){q=1;break}q=k.gzindex<k.gzhead.name.length?255&k.gzhead.name.charCodeAt(k.gzindex++):0,nt(k,q)}while(q!==0);k.gzhead.hcrc&&k.pending>O&&(f.adler=r(f.adler,k.pending_buf,k.pending-O,O)),q===0&&(k.gzindex=0,k.status=91)}else k.status=91;if(k.status===91)if(k.gzhead.comment){O=k.pending;do{if(k.pending===k.pending_buf_size&&(k.gzhead.hcrc&&k.pending>O&&(f.adler=r(f.adler,k.pending_buf,k.pending-O,O)),N(f),O=k.pending,k.pending===k.pending_buf_size)){q=1;break}q=k.gzindex<k.gzhead.comment.length?255&k.gzhead.comment.charCodeAt(k.gzindex++):0,nt(k,q)}while(q!==0);k.gzhead.hcrc&&k.pending>O&&(f.adler=r(f.adler,k.pending_buf,k.pending-O,O)),q===0&&(k.status=103)}else k.status=103;if(k.status===103&&(k.gzhead.hcrc?(k.pending+2>k.pending_buf_size&&N(f),k.pending+2<=k.pending_buf_size&&(nt(k,255&f.adler),nt(k,f.adler>>8&255),f.adler=0,k.status=et)):k.status=et),k.pending!==0){if(N(f),f.avail_out===0)return k.last_flush=-1,_}else if(f.avail_in===0&&st(ot)<=st(P)&&ot!==w)return gt(f,-5);if(k.status===666&&f.avail_in!==0)return gt(f,-5);if(f.avail_in!==0||k.lookahead!==0||ot!==A&&k.status!==666){var mt=k.strategy===2?function(K,pt){for(var bt;;){if(K.lookahead===0&&(_t(K),K.lookahead===0)){if(pt===A)return b;break}if(K.match_length=0,bt=m._tr_tally(K,0,K.window[K.strstart]),K.lookahead--,K.strstart++,bt&&(B(K,!1),K.strm.avail_out===0))return b}return K.insert=0,pt===w?(B(K,!0),K.strm.avail_out===0?G:X):K.last_lit&&(B(K,!1),K.strm.avail_out===0)?b:at}(k,ot):k.strategy===3?function(K,pt){for(var bt,yt,Et,kt,Rt=K.window;;){if(K.lookahead<=rt){if(_t(K),K.lookahead<=rt&&pt===A)return b;if(K.lookahead===0)break}if(K.match_length=0,K.lookahead>=$&&0<K.strstart&&(yt=Rt[Et=K.strstart-1])===Rt[++Et]&&yt===Rt[++Et]&&yt===Rt[++Et]){kt=K.strstart+rt;do;while(yt===Rt[++Et]&&yt===Rt[++Et]&&yt===Rt[++Et]&&yt===Rt[++Et]&&yt===Rt[++Et]&&yt===Rt[++Et]&&yt===Rt[++Et]&&yt===Rt[++Et]&&Et<kt);K.match_length=rt-(kt-Et),K.match_length>K.lookahead&&(K.match_length=K.lookahead)}if(K.match_length>=$?(bt=m._tr_tally(K,1,K.match_length-$),K.lookahead-=K.match_length,K.strstart+=K.match_length,K.match_length=0):(bt=m._tr_tally(K,0,K.window[K.strstart]),K.lookahead--,K.strstart++),bt&&(B(K,!1),K.strm.avail_out===0))return b}return K.insert=0,pt===w?(B(K,!0),K.strm.avail_out===0?G:X):K.last_lit&&(B(K,!1),K.strm.avail_out===0)?b:at}(k,ot):o[k.level].func(k,ot);if(mt!==G&&mt!==X||(k.status=666),mt===b||mt===G)return f.avail_out===0&&(k.last_flush=-1),_;if(mt===at&&(ot===1?m._tr_align(k):ot!==5&&(m._tr_stored_block(k,0,0,!1),ot===3&&(C(k.head),k.lookahead===0&&(k.strstart=0,k.block_start=0,k.insert=0))),N(f),f.avail_out===0))return k.last_flush=-1,_}return ot!==w?_:k.wrap<=0?1:(k.wrap===2?(nt(k,255&f.adler),nt(k,f.adler>>8&255),nt(k,f.adler>>16&255),nt(k,f.adler>>24&255),nt(k,255&f.total_in),nt(k,f.total_in>>8&255),nt(k,f.total_in>>16&255),nt(k,f.total_in>>24&255)):(it(k,f.adler>>>16),it(k,65535&f.adler)),N(f),0<k.wrap&&(k.wrap=-k.wrap),k.pending!==0?_:1)},p.deflateEnd=function(f){var ot;return f&&f.state?(ot=f.state.status)!==M&&ot!==69&&ot!==73&&ot!==91&&ot!==103&&ot!==et&&ot!==666?gt(f,z):(f.state=null,ot===et?gt(f,-3):_):z},p.deflateSetDictionary=function(f,ot){var P,k,O,q,ht,mt,K,pt,bt=ot.length;if(!f||!f.state||(q=(P=f.state).wrap)===2||q===1&&P.status!==M||P.lookahead)return z;for(q===1&&(f.adler=h(f.adler,ot,bt,0)),P.wrap=0,bt>=P.w_size&&(q===0&&(C(P.head),P.strstart=0,P.block_start=0,P.insert=0),pt=new y.Buf8(P.w_size),y.arraySet(pt,ot,bt-P.w_size,P.w_size,0),ot=pt,bt=P.w_size),ht=f.avail_in,mt=f.next_in,K=f.input,f.avail_in=bt,f.next_in=0,f.input=ot,_t(P);P.lookahead>=$;){for(k=P.strstart,O=P.lookahead-($-1);P.ins_h=(P.ins_h<<P.hash_shift^P.window[k+$-1])&P.hash_mask,P.prev[k&P.w_mask]=P.head[P.ins_h],P.head[P.ins_h]=k,k++,--O;);P.strstart=k,P.lookahead=$-1,_t(P)}return P.strstart+=P.lookahead,P.block_start=P.strstart,P.insert=P.lookahead,P.lookahead=0,P.match_length=P.prev_length=$-1,P.match_available=0,f.next_in=mt,f.input=K,f.avail_in=ht,P.wrap=q,_},p.deflateInfo="pako deflate (from Nodeca project)"},{"../utils/common":41,"./adler32":43,"./crc32":45,"./messages":51,"./trees":52}],47:[function(c,d,p){d.exports=function(){this.text=0,this.time=0,this.xflags=0,this.os=0,this.extra=null,this.extra_len=0,this.name="",this.comment="",this.hcrc=0,this.done=!1}},{}],48:[function(c,d,p){d.exports=function(o,y){var m,h,r,E,A,w,_,z,g,S,v,T,D,L,j,tt,F,ft,$,rt,vt,M,et,b,at;m=o.state,h=o.next_in,b=o.input,r=h+(o.avail_in-5),E=o.next_out,at=o.output,A=E-(y-o.avail_out),w=E+(o.avail_out-257),_=m.dmax,z=m.wsize,g=m.whave,S=m.wnext,v=m.window,T=m.hold,D=m.bits,L=m.lencode,j=m.distcode,tt=(1<<m.lenbits)-1,F=(1<<m.distbits)-1;t:do{D<15&&(T+=b[h++]<<D,D+=8,T+=b[h++]<<D,D+=8),ft=L[T&tt];e:for(;;){if(T>>>=$=ft>>>24,D-=$,($=ft>>>16&255)===0)at[E++]=65535&ft;else{if(!(16&$)){if((64&$)==0){ft=L[(65535&ft)+(T&(1<<$)-1)];continue e}if(32&$){m.mode=12;break t}o.msg="invalid literal/length code",m.mode=30;break t}rt=65535&ft,($&=15)&&(D<$&&(T+=b[h++]<<D,D+=8),rt+=T&(1<<$)-1,T>>>=$,D-=$),D<15&&(T+=b[h++]<<D,D+=8,T+=b[h++]<<D,D+=8),ft=j[T&F];a:for(;;){if(T>>>=$=ft>>>24,D-=$,!(16&($=ft>>>16&255))){if((64&$)==0){ft=j[(65535&ft)+(T&(1<<$)-1)];continue a}o.msg="invalid distance code",m.mode=30;break t}if(vt=65535&ft,D<($&=15)&&(T+=b[h++]<<D,(D+=8)<$&&(T+=b[h++]<<D,D+=8)),_<(vt+=T&(1<<$)-1)){o.msg="invalid distance too far back",m.mode=30;break t}if(T>>>=$,D-=$,($=E-A)<vt){if(g<($=vt-$)&&m.sane){o.msg="invalid distance too far back",m.mode=30;break t}if(et=v,(M=0)===S){if(M+=z-$,$<rt){for(rt-=$;at[E++]=v[M++],--$;);M=E-vt,et=at}}else if(S<$){if(M+=z+S-$,($-=S)<rt){for(rt-=$;at[E++]=v[M++],--$;);if(M=0,S<rt){for(rt-=$=S;at[E++]=v[M++],--$;);M=E-vt,et=at}}}else if(M+=S-$,$<rt){for(rt-=$;at[E++]=v[M++],--$;);M=E-vt,et=at}for(;2<rt;)at[E++]=et[M++],at[E++]=et[M++],at[E++]=et[M++],rt-=3;rt&&(at[E++]=et[M++],1<rt&&(at[E++]=et[M++]))}else{for(M=E-vt;at[E++]=at[M++],at[E++]=at[M++],at[E++]=at[M++],2<(rt-=3););rt&&(at[E++]=at[M++],1<rt&&(at[E++]=at[M++]))}break}}break}}while(h<r&&E<w);h-=rt=D>>3,T&=(1<<(D-=rt<<3))-1,o.next_in=h,o.next_out=E,o.avail_in=h<r?r-h+5:5-(h-r),o.avail_out=E<w?w-E+257:257-(E-w),m.hold=T,m.bits=D}},{}],49:[function(c,d,p){var o=c("../utils/common"),y=c("./adler32"),m=c("./crc32"),h=c("./inffast"),r=c("./inftrees"),E=1,A=2,w=0,_=-2,z=1,g=852,S=592;function v(M){return(M>>>24&255)+(M>>>8&65280)+((65280&M)<<8)+((255&M)<<24)}function T(){this.mode=0,this.last=!1,this.wrap=0,this.havedict=!1,this.flags=0,this.dmax=0,this.check=0,this.total=0,this.head=null,this.wbits=0,this.wsize=0,this.whave=0,this.wnext=0,this.window=null,this.hold=0,this.bits=0,this.length=0,this.offset=0,this.extra=0,this.lencode=null,this.distcode=null,this.lenbits=0,this.distbits=0,this.ncode=0,this.nlen=0,this.ndist=0,this.have=0,this.next=null,this.lens=new o.Buf16(320),this.work=new o.Buf16(288),this.lendyn=null,this.distdyn=null,this.sane=0,this.back=0,this.was=0}function D(M){var et;return M&&M.state?(et=M.state,M.total_in=M.total_out=et.total=0,M.msg="",et.wrap&&(M.adler=1&et.wrap),et.mode=z,et.last=0,et.havedict=0,et.dmax=32768,et.head=null,et.hold=0,et.bits=0,et.lencode=et.lendyn=new o.Buf32(g),et.distcode=et.distdyn=new o.Buf32(S),et.sane=1,et.back=-1,w):_}function L(M){var et;return M&&M.state?((et=M.state).wsize=0,et.whave=0,et.wnext=0,D(M)):_}function j(M,et){var b,at;return M&&M.state?(at=M.state,et<0?(b=0,et=-et):(b=1+(et>>4),et<48&&(et&=15)),et&&(et<8||15<et)?_:(at.window!==null&&at.wbits!==et&&(at.window=null),at.wrap=b,at.wbits=et,L(M))):_}function tt(M,et){var b,at;return M?(at=new T,(M.state=at).window=null,(b=j(M,et))!==w&&(M.state=null),b):_}var F,ft,$=!0;function rt(M){if($){var et;for(F=new o.Buf32(512),ft=new o.Buf32(32),et=0;et<144;)M.lens[et++]=8;for(;et<256;)M.lens[et++]=9;for(;et<280;)M.lens[et++]=7;for(;et<288;)M.lens[et++]=8;for(r(E,M.lens,0,288,F,0,M.work,{bits:9}),et=0;et<32;)M.lens[et++]=5;r(A,M.lens,0,32,ft,0,M.work,{bits:5}),$=!1}M.lencode=F,M.lenbits=9,M.distcode=ft,M.distbits=5}function vt(M,et,b,at){var G,X=M.state;return X.window===null&&(X.wsize=1<<X.wbits,X.wnext=0,X.whave=0,X.window=new o.Buf8(X.wsize)),at>=X.wsize?(o.arraySet(X.window,et,b-X.wsize,X.wsize,0),X.wnext=0,X.whave=X.wsize):(at<(G=X.wsize-X.wnext)&&(G=at),o.arraySet(X.window,et,b-at,G,X.wnext),(at-=G)?(o.arraySet(X.window,et,b-at,at,0),X.wnext=at,X.whave=X.wsize):(X.wnext+=G,X.wnext===X.wsize&&(X.wnext=0),X.whave<X.wsize&&(X.whave+=G))),0}p.inflateReset=L,p.inflateReset2=j,p.inflateResetKeep=D,p.inflateInit=function(M){return tt(M,15)},p.inflateInit2=tt,p.inflate=function(M,et){var b,at,G,X,gt,st,C,N,B,nt,it,dt,_t,Lt,xt,Ct,Pt,Vt,_e,ve,f,ot,P,k,O=0,q=new o.Buf8(4),ht=[16,17,18,0,8,7,9,6,10,5,11,4,12,3,13,2,14,1,15];if(!M||!M.state||!M.output||!M.input&&M.avail_in!==0)return _;(b=M.state).mode===12&&(b.mode=13),gt=M.next_out,G=M.output,C=M.avail_out,X=M.next_in,at=M.input,st=M.avail_in,N=b.hold,B=b.bits,nt=st,it=C,ot=w;t:for(;;)switch(b.mode){case z:if(b.wrap===0){b.mode=13;break}for(;B<16;){if(st===0)break t;st--,N+=at[X++]<<B,B+=8}if(2&b.wrap&&N===35615){q[b.check=0]=255&N,q[1]=N>>>8&255,b.check=m(b.check,q,2,0),B=N=0,b.mode=2;break}if(b.flags=0,b.head&&(b.head.done=!1),!(1&b.wrap)||(((255&N)<<8)+(N>>8))%31){M.msg="incorrect header check",b.mode=30;break}if((15&N)!=8){M.msg="unknown compression method",b.mode=30;break}if(B-=4,f=8+(15&(N>>>=4)),b.wbits===0)b.wbits=f;else if(f>b.wbits){M.msg="invalid window size",b.mode=30;break}b.dmax=1<<f,M.adler=b.check=1,b.mode=512&N?10:12,B=N=0;break;case 2:for(;B<16;){if(st===0)break t;st--,N+=at[X++]<<B,B+=8}if(b.flags=N,(255&b.flags)!=8){M.msg="unknown compression method",b.mode=30;break}if(57344&b.flags){M.msg="unknown header flags set",b.mode=30;break}b.head&&(b.head.text=N>>8&1),512&b.flags&&(q[0]=255&N,q[1]=N>>>8&255,b.check=m(b.check,q,2,0)),B=N=0,b.mode=3;case 3:for(;B<32;){if(st===0)break t;st--,N+=at[X++]<<B,B+=8}b.head&&(b.head.time=N),512&b.flags&&(q[0]=255&N,q[1]=N>>>8&255,q[2]=N>>>16&255,q[3]=N>>>24&255,b.check=m(b.check,q,4,0)),B=N=0,b.mode=4;case 4:for(;B<16;){if(st===0)break t;st--,N+=at[X++]<<B,B+=8}b.head&&(b.head.xflags=255&N,b.head.os=N>>8),512&b.flags&&(q[0]=255&N,q[1]=N>>>8&255,b.check=m(b.check,q,2,0)),B=N=0,b.mode=5;case 5:if(1024&b.flags){for(;B<16;){if(st===0)break t;st--,N+=at[X++]<<B,B+=8}b.length=N,b.head&&(b.head.extra_len=N),512&b.flags&&(q[0]=255&N,q[1]=N>>>8&255,b.check=m(b.check,q,2,0)),B=N=0}else b.head&&(b.head.extra=null);b.mode=6;case 6:if(1024&b.flags&&(st<(dt=b.length)&&(dt=st),dt&&(b.head&&(f=b.head.extra_len-b.length,b.head.extra||(b.head.extra=new Array(b.head.extra_len)),o.arraySet(b.head.extra,at,X,dt,f)),512&b.flags&&(b.check=m(b.check,at,dt,X)),st-=dt,X+=dt,b.length-=dt),b.length))break t;b.length=0,b.mode=7;case 7:if(2048&b.flags){if(st===0)break t;for(dt=0;f=at[X+dt++],b.head&&f&&b.length<65536&&(b.head.name+=String.fromCharCode(f)),f&&dt<st;);if(512&b.flags&&(b.check=m(b.check,at,dt,X)),st-=dt,X+=dt,f)break t}else b.head&&(b.head.name=null);b.length=0,b.mode=8;case 8:if(4096&b.flags){if(st===0)break t;for(dt=0;f=at[X+dt++],b.head&&f&&b.length<65536&&(b.head.comment+=String.fromCharCode(f)),f&&dt<st;);if(512&b.flags&&(b.check=m(b.check,at,dt,X)),st-=dt,X+=dt,f)break t}else b.head&&(b.head.comment=null);b.mode=9;case 9:if(512&b.flags){for(;B<16;){if(st===0)break t;st--,N+=at[X++]<<B,B+=8}if(N!==(65535&b.check)){M.msg="header crc mismatch",b.mode=30;break}B=N=0}b.head&&(b.head.hcrc=b.flags>>9&1,b.head.done=!0),M.adler=b.check=0,b.mode=12;break;case 10:for(;B<32;){if(st===0)break t;st--,N+=at[X++]<<B,B+=8}M.adler=b.check=v(N),B=N=0,b.mode=11;case 11:if(b.havedict===0)return M.next_out=gt,M.avail_out=C,M.next_in=X,M.avail_in=st,b.hold=N,b.bits=B,2;M.adler=b.check=1,b.mode=12;case 12:if(et===5||et===6)break t;case 13:if(b.last){N>>>=7&B,B-=7&B,b.mode=27;break}for(;B<3;){if(st===0)break t;st--,N+=at[X++]<<B,B+=8}switch(b.last=1&N,B-=1,3&(N>>>=1)){case 0:b.mode=14;break;case 1:if(rt(b),b.mode=20,et!==6)break;N>>>=2,B-=2;break t;case 2:b.mode=17;break;case 3:M.msg="invalid block type",b.mode=30}N>>>=2,B-=2;break;case 14:for(N>>>=7&B,B-=7&B;B<32;){if(st===0)break t;st--,N+=at[X++]<<B,B+=8}if((65535&N)!=(N>>>16^65535)){M.msg="invalid stored block lengths",b.mode=30;break}if(b.length=65535&N,B=N=0,b.mode=15,et===6)break t;case 15:b.mode=16;case 16:if(dt=b.length){if(st<dt&&(dt=st),C<dt&&(dt=C),dt===0)break t;o.arraySet(G,at,X,dt,gt),st-=dt,X+=dt,C-=dt,gt+=dt,b.length-=dt;break}b.mode=12;break;case 17:for(;B<14;){if(st===0)break t;st--,N+=at[X++]<<B,B+=8}if(b.nlen=257+(31&N),N>>>=5,B-=5,b.ndist=1+(31&N),N>>>=5,B-=5,b.ncode=4+(15&N),N>>>=4,B-=4,286<b.nlen||30<b.ndist){M.msg="too many length or distance symbols",b.mode=30;break}b.have=0,b.mode=18;case 18:for(;b.have<b.ncode;){for(;B<3;){if(st===0)break t;st--,N+=at[X++]<<B,B+=8}b.lens[ht[b.have++]]=7&N,N>>>=3,B-=3}for(;b.have<19;)b.lens[ht[b.have++]]=0;if(b.lencode=b.lendyn,b.lenbits=7,P={bits:b.lenbits},ot=r(0,b.lens,0,19,b.lencode,0,b.work,P),b.lenbits=P.bits,ot){M.msg="invalid code lengths set",b.mode=30;break}b.have=0,b.mode=19;case 19:for(;b.have<b.nlen+b.ndist;){for(;Ct=(O=b.lencode[N&(1<<b.lenbits)-1])>>>16&255,Pt=65535&O,!((xt=O>>>24)<=B);){if(st===0)break t;st--,N+=at[X++]<<B,B+=8}if(Pt<16)N>>>=xt,B-=xt,b.lens[b.have++]=Pt;else{if(Pt===16){for(k=xt+2;B<k;){if(st===0)break t;st--,N+=at[X++]<<B,B+=8}if(N>>>=xt,B-=xt,b.have===0){M.msg="invalid bit length repeat",b.mode=30;break}f=b.lens[b.have-1],dt=3+(3&N),N>>>=2,B-=2}else if(Pt===17){for(k=xt+3;B<k;){if(st===0)break t;st--,N+=at[X++]<<B,B+=8}B-=xt,f=0,dt=3+(7&(N>>>=xt)),N>>>=3,B-=3}else{for(k=xt+7;B<k;){if(st===0)break t;st--,N+=at[X++]<<B,B+=8}B-=xt,f=0,dt=11+(127&(N>>>=xt)),N>>>=7,B-=7}if(b.have+dt>b.nlen+b.ndist){M.msg="invalid bit length repeat",b.mode=30;break}for(;dt--;)b.lens[b.have++]=f}}if(b.mode===30)break;if(b.lens[256]===0){M.msg="invalid code -- missing end-of-block",b.mode=30;break}if(b.lenbits=9,P={bits:b.lenbits},ot=r(E,b.lens,0,b.nlen,b.lencode,0,b.work,P),b.lenbits=P.bits,ot){M.msg="invalid literal/lengths set",b.mode=30;break}if(b.distbits=6,b.distcode=b.distdyn,P={bits:b.distbits},ot=r(A,b.lens,b.nlen,b.ndist,b.distcode,0,b.work,P),b.distbits=P.bits,ot){M.msg="invalid distances set",b.mode=30;break}if(b.mode=20,et===6)break t;case 20:b.mode=21;case 21:if(6<=st&&258<=C){M.next_out=gt,M.avail_out=C,M.next_in=X,M.avail_in=st,b.hold=N,b.bits=B,h(M,it),gt=M.next_out,G=M.output,C=M.avail_out,X=M.next_in,at=M.input,st=M.avail_in,N=b.hold,B=b.bits,b.mode===12&&(b.back=-1);break}for(b.back=0;Ct=(O=b.lencode[N&(1<<b.lenbits)-1])>>>16&255,Pt=65535&O,!((xt=O>>>24)<=B);){if(st===0)break t;st--,N+=at[X++]<<B,B+=8}if(Ct&&(240&Ct)==0){for(Vt=xt,_e=Ct,ve=Pt;Ct=(O=b.lencode[ve+((N&(1<<Vt+_e)-1)>>Vt)])>>>16&255,Pt=65535&O,!(Vt+(xt=O>>>24)<=B);){if(st===0)break t;st--,N+=at[X++]<<B,B+=8}N>>>=Vt,B-=Vt,b.back+=Vt}if(N>>>=xt,B-=xt,b.back+=xt,b.length=Pt,Ct===0){b.mode=26;break}if(32&Ct){b.back=-1,b.mode=12;break}if(64&Ct){M.msg="invalid literal/length code",b.mode=30;break}b.extra=15&Ct,b.mode=22;case 22:if(b.extra){for(k=b.extra;B<k;){if(st===0)break t;st--,N+=at[X++]<<B,B+=8}b.length+=N&(1<<b.extra)-1,N>>>=b.extra,B-=b.extra,b.back+=b.extra}b.was=b.length,b.mode=23;case 23:for(;Ct=(O=b.distcode[N&(1<<b.distbits)-1])>>>16&255,Pt=65535&O,!((xt=O>>>24)<=B);){if(st===0)break t;st--,N+=at[X++]<<B,B+=8}if((240&Ct)==0){for(Vt=xt,_e=Ct,ve=Pt;Ct=(O=b.distcode[ve+((N&(1<<Vt+_e)-1)>>Vt)])>>>16&255,Pt=65535&O,!(Vt+(xt=O>>>24)<=B);){if(st===0)break t;st--,N+=at[X++]<<B,B+=8}N>>>=Vt,B-=Vt,b.back+=Vt}if(N>>>=xt,B-=xt,b.back+=xt,64&Ct){M.msg="invalid distance code",b.mode=30;break}b.offset=Pt,b.extra=15&Ct,b.mode=24;case 24:if(b.extra){for(k=b.extra;B<k;){if(st===0)break t;st--,N+=at[X++]<<B,B+=8}b.offset+=N&(1<<b.extra)-1,N>>>=b.extra,B-=b.extra,b.back+=b.extra}if(b.offset>b.dmax){M.msg="invalid distance too far back",b.mode=30;break}b.mode=25;case 25:if(C===0)break t;if(dt=it-C,b.offset>dt){if((dt=b.offset-dt)>b.whave&&b.sane){M.msg="invalid distance too far back",b.mode=30;break}_t=dt>b.wnext?(dt-=b.wnext,b.wsize-dt):b.wnext-dt,dt>b.length&&(dt=b.length),Lt=b.window}else Lt=G,_t=gt-b.offset,dt=b.length;for(C<dt&&(dt=C),C-=dt,b.length-=dt;G[gt++]=Lt[_t++],--dt;);b.length===0&&(b.mode=21);break;case 26:if(C===0)break t;G[gt++]=b.length,C--,b.mode=21;break;case 27:if(b.wrap){for(;B<32;){if(st===0)break t;st--,N|=at[X++]<<B,B+=8}if(it-=C,M.total_out+=it,b.total+=it,it&&(M.adler=b.check=b.flags?m(b.check,G,it,gt-it):y(b.check,G,it,gt-it)),it=C,(b.flags?N:v(N))!==b.check){M.msg="incorrect data check",b.mode=30;break}B=N=0}b.mode=28;case 28:if(b.wrap&&b.flags){for(;B<32;){if(st===0)break t;st--,N+=at[X++]<<B,B+=8}if(N!==(4294967295&b.total)){M.msg="incorrect length check",b.mode=30;break}B=N=0}b.mode=29;case 29:ot=1;break t;case 30:ot=-3;break t;case 31:return-4;case 32:default:return _}return M.next_out=gt,M.avail_out=C,M.next_in=X,M.avail_in=st,b.hold=N,b.bits=B,(b.wsize||it!==M.avail_out&&b.mode<30&&(b.mode<27||et!==4))&&vt(M,M.output,M.next_out,it-M.avail_out)?(b.mode=31,-4):(nt-=M.avail_in,it-=M.avail_out,M.total_in+=nt,M.total_out+=it,b.total+=it,b.wrap&&it&&(M.adler=b.check=b.flags?m(b.check,G,it,M.next_out-it):y(b.check,G,it,M.next_out-it)),M.data_type=b.bits+(b.last?64:0)+(b.mode===12?128:0)+(b.mode===20||b.mode===15?256:0),(nt==0&&it===0||et===4)&&ot===w&&(ot=-5),ot)},p.inflateEnd=function(M){if(!M||!M.state)return _;var et=M.state;return et.window&&(et.window=null),M.state=null,w},p.inflateGetHeader=function(M,et){var b;return M&&M.state?(2&(b=M.state).wrap)==0?_:((b.head=et).done=!1,w):_},p.inflateSetDictionary=function(M,et){var b,at=et.length;return M&&M.state?(b=M.state).wrap!==0&&b.mode!==11?_:b.mode===11&&y(1,et,at,0)!==b.check?-3:vt(M,et,at,at)?(b.mode=31,-4):(b.havedict=1,w):_},p.inflateInfo="pako inflate (from Nodeca project)"},{"../utils/common":41,"./adler32":43,"./crc32":45,"./inffast":48,"./inftrees":50}],50:[function(c,d,p){var o=c("../utils/common"),y=[3,4,5,6,7,8,9,10,11,13,15,17,19,23,27,31,35,43,51,59,67,83,99,115,131,163,195,227,258,0,0],m=[16,16,16,16,16,16,16,16,17,17,17,17,18,18,18,18,19,19,19,19,20,20,20,20,21,21,21,21,16,72,78],h=[1,2,3,4,5,7,9,13,17,25,33,49,65,97,129,193,257,385,513,769,1025,1537,2049,3073,4097,6145,8193,12289,16385,24577,0,0],r=[16,16,16,16,17,17,18,18,19,19,20,20,21,21,22,22,23,23,24,24,25,25,26,26,27,27,28,28,29,29,64,64];d.exports=function(E,A,w,_,z,g,S,v){var T,D,L,j,tt,F,ft,$,rt,vt=v.bits,M=0,et=0,b=0,at=0,G=0,X=0,gt=0,st=0,C=0,N=0,B=null,nt=0,it=new o.Buf16(16),dt=new o.Buf16(16),_t=null,Lt=0;for(M=0;M<=15;M++)it[M]=0;for(et=0;et<_;et++)it[A[w+et]]++;for(G=vt,at=15;1<=at&&it[at]===0;at--);if(at<G&&(G=at),at===0)return z[g++]=20971520,z[g++]=20971520,v.bits=1,0;for(b=1;b<at&&it[b]===0;b++);for(G<b&&(G=b),M=st=1;M<=15;M++)if(st<<=1,(st-=it[M])<0)return-1;if(0<st&&(E===0||at!==1))return-1;for(dt[1]=0,M=1;M<15;M++)dt[M+1]=dt[M]+it[M];for(et=0;et<_;et++)A[w+et]!==0&&(S[dt[A[w+et]]++]=et);if(F=E===0?(B=_t=S,19):E===1?(B=y,nt-=257,_t=m,Lt-=257,256):(B=h,_t=r,-1),M=b,tt=g,gt=et=N=0,L=-1,j=(C=1<<(X=G))-1,E===1&&852<C||E===2&&592<C)return 1;for(;;){for(ft=M-gt,rt=S[et]<F?($=0,S[et]):S[et]>F?($=_t[Lt+S[et]],B[nt+S[et]]):($=96,0),T=1<<M-gt,b=D=1<<X;z[tt+(N>>gt)+(D-=T)]=ft<<24|$<<16|rt|0,D!==0;);for(T=1<<M-1;N&T;)T>>=1;if(T!==0?(N&=T-1,N+=T):N=0,et++,--it[M]==0){if(M===at)break;M=A[w+S[et]]}if(G<M&&(N&j)!==L){for(gt===0&&(gt=G),tt+=b,st=1<<(X=M-gt);X+gt<at&&!((st-=it[X+gt])<=0);)X++,st<<=1;if(C+=1<<X,E===1&&852<C||E===2&&592<C)return 1;z[L=N&j]=G<<24|X<<16|tt-g|0}}return N!==0&&(z[tt+N]=M-gt<<24|64<<16|0),v.bits=G,0}},{"../utils/common":41}],51:[function(c,d,p){d.exports={2:"need dictionary",1:"stream end",0:"","-1":"file error","-2":"stream error","-3":"data error","-4":"insufficient memory","-5":"buffer error","-6":"incompatible version"}},{}],52:[function(c,d,p){var o=c("../utils/common"),y=0,m=1;function h(O){for(var q=O.length;0<=--q;)O[q]=0}var r=0,E=29,A=256,w=A+1+E,_=30,z=19,g=2*w+1,S=15,v=16,T=7,D=256,L=16,j=17,tt=18,F=[0,0,0,0,0,0,0,0,1,1,1,1,2,2,2,2,3,3,3,3,4,4,4,4,5,5,5,5,0],ft=[0,0,0,0,1,1,2,2,3,3,4,4,5,5,6,6,7,7,8,8,9,9,10,10,11,11,12,12,13,13],$=[0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,3,7],rt=[16,17,18,0,8,7,9,6,10,5,11,4,12,3,13,2,14,1,15],vt=new Array(2*(w+2));h(vt);var M=new Array(2*_);h(M);var et=new Array(512);h(et);var b=new Array(256);h(b);var at=new Array(E);h(at);var G,X,gt,st=new Array(_);function C(O,q,ht,mt,K){this.static_tree=O,this.extra_bits=q,this.extra_base=ht,this.elems=mt,this.max_length=K,this.has_stree=O&&O.length}function N(O,q){this.dyn_tree=O,this.max_code=0,this.stat_desc=q}function B(O){return O<256?et[O]:et[256+(O>>>7)]}function nt(O,q){O.pending_buf[O.pending++]=255&q,O.pending_buf[O.pending++]=q>>>8&255}function it(O,q,ht){O.bi_valid>v-ht?(O.bi_buf|=q<<O.bi_valid&65535,nt(O,O.bi_buf),O.bi_buf=q>>v-O.bi_valid,O.bi_valid+=ht-v):(O.bi_buf|=q<<O.bi_valid&65535,O.bi_valid+=ht)}function dt(O,q,ht){it(O,ht[2*q],ht[2*q+1])}function _t(O,q){for(var ht=0;ht|=1&O,O>>>=1,ht<<=1,0<--q;);return ht>>>1}function Lt(O,q,ht){var mt,K,pt=new Array(S+1),bt=0;for(mt=1;mt<=S;mt++)pt[mt]=bt=bt+ht[mt-1]<<1;for(K=0;K<=q;K++){var yt=O[2*K+1];yt!==0&&(O[2*K]=_t(pt[yt]++,yt))}}function xt(O){var q;for(q=0;q<w;q++)O.dyn_ltree[2*q]=0;for(q=0;q<_;q++)O.dyn_dtree[2*q]=0;for(q=0;q<z;q++)O.bl_tree[2*q]=0;O.dyn_ltree[2*D]=1,O.opt_len=O.static_len=0,O.last_lit=O.matches=0}function Ct(O){8<O.bi_valid?nt(O,O.bi_buf):0<O.bi_valid&&(O.pending_buf[O.pending++]=O.bi_buf),O.bi_buf=0,O.bi_valid=0}function Pt(O,q,ht,mt){var K=2*q,pt=2*ht;return O[K]<O[pt]||O[K]===O[pt]&&mt[q]<=mt[ht]}function Vt(O,q,ht){for(var mt=O.heap[ht],K=ht<<1;K<=O.heap_len&&(K<O.heap_len&&Pt(q,O.heap[K+1],O.heap[K],O.depth)&&K++,!Pt(q,mt,O.heap[K],O.depth));)O.heap[ht]=O.heap[K],ht=K,K<<=1;O.heap[ht]=mt}function _e(O,q,ht){var mt,K,pt,bt,yt=0;if(O.last_lit!==0)for(;mt=O.pending_buf[O.d_buf+2*yt]<<8|O.pending_buf[O.d_buf+2*yt+1],K=O.pending_buf[O.l_buf+yt],yt++,mt===0?dt(O,K,q):(dt(O,(pt=b[K])+A+1,q),(bt=F[pt])!==0&&it(O,K-=at[pt],bt),dt(O,pt=B(--mt),ht),(bt=ft[pt])!==0&&it(O,mt-=st[pt],bt)),yt<O.last_lit;);dt(O,D,q)}function ve(O,q){var ht,mt,K,pt=q.dyn_tree,bt=q.stat_desc.static_tree,yt=q.stat_desc.has_stree,Et=q.stat_desc.elems,kt=-1;for(O.heap_len=0,O.heap_max=g,ht=0;ht<Et;ht++)pt[2*ht]!==0?(O.heap[++O.heap_len]=kt=ht,O.depth[ht]=0):pt[2*ht+1]=0;for(;O.heap_len<2;)pt[2*(K=O.heap[++O.heap_len]=kt<2?++kt:0)]=1,O.depth[K]=0,O.opt_len--,yt&&(O.static_len-=bt[2*K+1]);for(q.max_code=kt,ht=O.heap_len>>1;1<=ht;ht--)Vt(O,pt,ht);for(K=Et;ht=O.heap[1],O.heap[1]=O.heap[O.heap_len--],Vt(O,pt,1),mt=O.heap[1],O.heap[--O.heap_max]=ht,O.heap[--O.heap_max]=mt,pt[2*K]=pt[2*ht]+pt[2*mt],O.depth[K]=(O.depth[ht]>=O.depth[mt]?O.depth[ht]:O.depth[mt])+1,pt[2*ht+1]=pt[2*mt+1]=K,O.heap[1]=K++,Vt(O,pt,1),2<=O.heap_len;);O.heap[--O.heap_max]=O.heap[1],function(Rt,Kt){var Va,Ue,Ka,Jt,aa,Xe,Se=Kt.dyn_tree,Sa=Kt.max_code,Er=Kt.stat_desc.static_tree,ni=Kt.stat_desc.has_stree,li=Kt.stat_desc.extra_bits,hn=Kt.stat_desc.extra_base,Qe=Kt.stat_desc.max_length,mn=0;for(Jt=0;Jt<=S;Jt++)Rt.bl_count[Jt]=0;for(Se[2*Rt.heap[Rt.heap_max]+1]=0,Va=Rt.heap_max+1;Va<g;Va++)Qe<(Jt=Se[2*Se[2*(Ue=Rt.heap[Va])+1]+1]+1)&&(Jt=Qe,mn++),Se[2*Ue+1]=Jt,Sa<Ue||(Rt.bl_count[Jt]++,aa=0,hn<=Ue&&(aa=li[Ue-hn]),Xe=Se[2*Ue],Rt.opt_len+=Xe*(Jt+aa),ni&&(Rt.static_len+=Xe*(Er[2*Ue+1]+aa)));if(mn!==0){do{for(Jt=Qe-1;Rt.bl_count[Jt]===0;)Jt--;Rt.bl_count[Jt]--,Rt.bl_count[Jt+1]+=2,Rt.bl_count[Qe]--,mn-=2}while(0<mn);for(Jt=Qe;Jt!==0;Jt--)for(Ue=Rt.bl_count[Jt];Ue!==0;)Sa<(Ka=Rt.heap[--Va])||(Se[2*Ka+1]!==Jt&&(Rt.opt_len+=(Jt-Se[2*Ka+1])*Se[2*Ka],Se[2*Ka+1]=Jt),Ue--)}}(O,q),Lt(pt,kt,O.bl_count)}function f(O,q,ht){var mt,K,pt=-1,bt=q[1],yt=0,Et=7,kt=4;for(bt===0&&(Et=138,kt=3),q[2*(ht+1)+1]=65535,mt=0;mt<=ht;mt++)K=bt,bt=q[2*(mt+1)+1],++yt<Et&&K===bt||(yt<kt?O.bl_tree[2*K]+=yt:K!==0?(K!==pt&&O.bl_tree[2*K]++,O.bl_tree[2*L]++):yt<=10?O.bl_tree[2*j]++:O.bl_tree[2*tt]++,pt=K,kt=(yt=0)===bt?(Et=138,3):K===bt?(Et=6,3):(Et=7,4))}function ot(O,q,ht){var mt,K,pt=-1,bt=q[1],yt=0,Et=7,kt=4;for(bt===0&&(Et=138,kt=3),mt=0;mt<=ht;mt++)if(K=bt,bt=q[2*(mt+1)+1],!(++yt<Et&&K===bt)){if(yt<kt)for(;dt(O,K,O.bl_tree),--yt!=0;);else K!==0?(K!==pt&&(dt(O,K,O.bl_tree),yt--),dt(O,L,O.bl_tree),it(O,yt-3,2)):yt<=10?(dt(O,j,O.bl_tree),it(O,yt-3,3)):(dt(O,tt,O.bl_tree),it(O,yt-11,7));pt=K,kt=(yt=0)===bt?(Et=138,3):K===bt?(Et=6,3):(Et=7,4)}}h(st);var P=!1;function k(O,q,ht,mt){it(O,(r<<1)+(mt?1:0),3),function(K,pt,bt,yt){Ct(K),nt(K,bt),nt(K,~bt),o.arraySet(K.pending_buf,K.window,pt,bt,K.pending),K.pending+=bt}(O,q,ht)}p._tr_init=function(O){P||(function(){var q,ht,mt,K,pt,bt=new Array(S+1);for(K=mt=0;K<E-1;K++)for(at[K]=mt,q=0;q<1<<F[K];q++)b[mt++]=K;for(b[mt-1]=K,K=pt=0;K<16;K++)for(st[K]=pt,q=0;q<1<<ft[K];q++)et[pt++]=K;for(pt>>=7;K<_;K++)for(st[K]=pt<<7,q=0;q<1<<ft[K]-7;q++)et[256+pt++]=K;for(ht=0;ht<=S;ht++)bt[ht]=0;for(q=0;q<=143;)vt[2*q+1]=8,q++,bt[8]++;for(;q<=255;)vt[2*q+1]=9,q++,bt[9]++;for(;q<=279;)vt[2*q+1]=7,q++,bt[7]++;for(;q<=287;)vt[2*q+1]=8,q++,bt[8]++;for(Lt(vt,w+1,bt),q=0;q<_;q++)M[2*q+1]=5,M[2*q]=_t(q,5);G=new C(vt,F,A+1,w,S),X=new C(M,ft,0,_,S),gt=new C(new Array(0),$,0,z,T)}(),P=!0),O.l_desc=new N(O.dyn_ltree,G),O.d_desc=new N(O.dyn_dtree,X),O.bl_desc=new N(O.bl_tree,gt),O.bi_buf=0,O.bi_valid=0,xt(O)},p._tr_stored_block=k,p._tr_flush_block=function(O,q,ht,mt){var K,pt,bt=0;0<O.level?(O.strm.data_type===2&&(O.strm.data_type=function(yt){var Et,kt=4093624447;for(Et=0;Et<=31;Et++,kt>>>=1)if(1&kt&&yt.dyn_ltree[2*Et]!==0)return y;if(yt.dyn_ltree[18]!==0||yt.dyn_ltree[20]!==0||yt.dyn_ltree[26]!==0)return m;for(Et=32;Et<A;Et++)if(yt.dyn_ltree[2*Et]!==0)return m;return y}(O)),ve(O,O.l_desc),ve(O,O.d_desc),bt=function(yt){var Et;for(f(yt,yt.dyn_ltree,yt.l_desc.max_code),f(yt,yt.dyn_dtree,yt.d_desc.max_code),ve(yt,yt.bl_desc),Et=z-1;3<=Et&&yt.bl_tree[2*rt[Et]+1]===0;Et--);return yt.opt_len+=3*(Et+1)+5+5+4,Et}(O),K=O.opt_len+3+7>>>3,(pt=O.static_len+3+7>>>3)<=K&&(K=pt)):K=pt=ht+5,ht+4<=K&&q!==-1?k(O,q,ht,mt):O.strategy===4||pt===K?(it(O,2+(mt?1:0),3),_e(O,vt,M)):(it(O,4+(mt?1:0),3),function(yt,Et,kt,Rt){var Kt;for(it(yt,Et-257,5),it(yt,kt-1,5),it(yt,Rt-4,4),Kt=0;Kt<Rt;Kt++)it(yt,yt.bl_tree[2*rt[Kt]+1],3);ot(yt,yt.dyn_ltree,Et-1),ot(yt,yt.dyn_dtree,kt-1)}(O,O.l_desc.max_code+1,O.d_desc.max_code+1,bt+1),_e(O,O.dyn_ltree,O.dyn_dtree)),xt(O),mt&&Ct(O)},p._tr_tally=function(O,q,ht){return O.pending_buf[O.d_buf+2*O.last_lit]=q>>>8&255,O.pending_buf[O.d_buf+2*O.last_lit+1]=255&q,O.pending_buf[O.l_buf+O.last_lit]=255&ht,O.last_lit++,q===0?O.dyn_ltree[2*ht]++:(O.matches++,q--,O.dyn_ltree[2*(b[ht]+A+1)]++,O.dyn_dtree[2*B(q)]++),O.last_lit===O.lit_bufsize-1},p._tr_align=function(O){it(O,2,3),dt(O,D,vt),function(q){q.bi_valid===16?(nt(q,q.bi_buf),q.bi_buf=0,q.bi_valid=0):8<=q.bi_valid&&(q.pending_buf[q.pending++]=255&q.bi_buf,q.bi_buf>>=8,q.bi_valid-=8)}(O)}},{"../utils/common":41}],53:[function(c,d,p){d.exports=function(){this.input=null,this.next_in=0,this.avail_in=0,this.total_in=0,this.output=null,this.next_out=0,this.avail_out=0,this.total_out=0,this.msg="",this.state=null,this.data_type=2,this.adler=0}},{}],54:[function(c,d,p){(function(o){(function(y,m){if(!y.setImmediate){var h,r,E,A,w=1,_={},z=!1,g=y.document,S=Object.getPrototypeOf&&Object.getPrototypeOf(y);S=S&&S.setTimeout?S:y,h={}.toString.call(y.process)==="[object process]"?function(L){process.nextTick(function(){T(L)})}:function(){if(y.postMessage&&!y.importScripts){var L=!0,j=y.onmessage;return y.onmessage=function(){L=!1},y.postMessage("","*"),y.onmessage=j,L}}()?(A="setImmediate$"+Math.random()+"$",y.addEventListener?y.addEventListener("message",D,!1):y.attachEvent("onmessage",D),function(L){y.postMessage(A+L,"*")}):y.MessageChannel?((E=new MessageChannel).port1.onmessage=function(L){T(L.data)},function(L){E.port2.postMessage(L)}):g&&"onreadystatechange"in g.createElement("script")?(r=g.documentElement,function(L){var j=g.createElement("script");j.onreadystatechange=function(){T(L),j.onreadystatechange=null,r.removeChild(j),j=null},r.appendChild(j)}):function(L){setTimeout(T,0,L)},S.setImmediate=function(L){typeof L!="function"&&(L=new Function(""+L));for(var j=new Array(arguments.length-1),tt=0;tt<j.length;tt++)j[tt]=arguments[tt+1];var F={callback:L,args:j};return _[w]=F,h(w),w++},S.clearImmediate=v}function v(L){delete _[L]}function T(L){if(z)setTimeout(T,0,L);else{var j=_[L];if(j){z=!0;try{(function(tt){var F=tt.callback,ft=tt.args;switch(ft.length){case 0:F();break;case 1:F(ft[0]);break;case 2:F(ft[0],ft[1]);break;case 3:F(ft[0],ft[1],ft[2]);break;default:F.apply(m,ft)}})(j)}finally{v(L),z=!1}}}}function D(L){L.source===y&&typeof L.data=="string"&&L.data.indexOf(A)===0&&T(+L.data.slice(A.length))}})(typeof self>"u"?o===void 0?this:o:self)}).call(this,typeof Qa<"u"?Qa:typeof self<"u"?self:typeof window<"u"?window:{})},{}]},{},[10])(10)})}(Zs)),Zs.exports}var yg=gg();const vg=M0(yg);var br={exports:{}},bg=br.exports,lh;function _g(){return lh||(lh=1,function(s,R){(function(c,d){d()})(bg,function(){function c(r,E){return typeof E>"u"?E={autoBom:!1}:typeof E!="object"&&(console.warn("Deprecated: Expected third argument to be a object"),E={autoBom:!E}),E.autoBom&&/^\s*(?:text\/\S*|application\/xml|\S*\/\S*\+xml)\s*;.*charset\s*=\s*utf-8/i.test(r.type)?new Blob(["\uFEFF",r],{type:r.type}):r}function d(r,E,A){var w=new XMLHttpRequest;w.open("GET",r),w.responseType="blob",w.onload=function(){h(w.response,E,A)},w.onerror=function(){console.error("could not download file")},w.send()}function p(r){var E=new XMLHttpRequest;E.open("HEAD",r,!1);try{E.send()}catch{}return 200<=E.status&&299>=E.status}function o(r){try{r.dispatchEvent(new MouseEvent("click"))}catch{var E=document.createEvent("MouseEvents");E.initMouseEvent("click",!0,!0,window,0,0,0,80,20,!1,!1,!1,!1,0,null),r.dispatchEvent(E)}}var y=typeof window=="object"&&window.window===window?window:typeof self=="object"&&self.self===self?self:typeof Qa=="object"&&Qa.global===Qa?Qa:void 0,m=y.navigator&&/Macintosh/.test(navigator.userAgent)&&/AppleWebKit/.test(navigator.userAgent)&&!/Safari/.test(navigator.userAgent),h=y.saveAs||(typeof window!="object"||window!==y?function(){}:"download"in HTMLAnchorElement.prototype&&!m?function(r,E,A){var w=y.URL||y.webkitURL,_=document.createElement("a");E=E||r.name||"download",_.download=E,_.rel="noopener",typeof r=="string"?(_.href=r,_.origin===location.origin?o(_):p(_.href)?d(r,E,A):o(_,_.target="_blank")):(_.href=w.createObjectURL(r),setTimeout(function(){w.revokeObjectURL(_.href)},4e4),setTimeout(function(){o(_)},0))}:"msSaveOrOpenBlob"in navigator?function(r,E,A){if(E=E||r.name||"download",typeof r!="string")navigator.msSaveOrOpenBlob(c(r,A),E);else if(p(r))d(r,E,A);else{var w=document.createElement("a");w.href=r,w.target="_blank",setTimeout(function(){o(w)})}}:function(r,E,A,w){if(w=w||open("","_blank"),w&&(w.document.title=w.document.body.innerText="downloading..."),typeof r=="string")return d(r,E,A);var _=r.type==="application/octet-stream",z=/constructor/i.test(y.HTMLElement)||y.safari,g=/CriOS\/[\d]+/.test(navigator.userAgent);if((g||_&&z||m)&&typeof FileReader<"u"){var S=new FileReader;S.onloadend=function(){var D=S.result;D=g?D:D.replace(/^data:[^;]*;/,"data:attachment/file;"),w?w.location.href=D:location=D,w=null},S.readAsDataURL(r)}else{var v=y.URL||y.webkitURL,T=v.createObjectURL(r);w?w.location=T:location.href=T,w=null,setTimeout(function(){v.revokeObjectURL(T)},4e4)}});y.saveAs=h.saveAs=h,s.exports=h})}(br)),br.exports}var Sg=_g();function xg({results:s}){const[R,c]=I.useState(null),d=s.flatMap(g=>g.detections),p=Array.from(new Set(d.map(g=>g.label))),o=Object.fromEntries(p.map((g,S)=>[g,S])),y=["#ef4444","#3b82f6","#10b981","#f59e0b","#8b5cf6","#ec4899","#6366f1","#f97316","#14b8a6","#06b6d4","#84cc16","#059669","#7c3aed","#d946ef","#f43f5e","#f59e0b"],m=g=>{const S=o[g];return y[S%y.length]},h=I.useCallback(g=>{if(R!==null)switch(g.key){case"Escape":c(null);break;case"ArrowLeft":g.preventDefault(),c(S=>S===null?null:S>0?S-1:s.length-1);break;case"ArrowRight":g.preventDefault(),c(S=>S===null?null:S<s.length-1?S+1:0);break}},[R,s.length]);I.useEffect(()=>{if(R!==null)return document.addEventListener("keydown",h),document.body.style.overflow="hidden",()=>{document.removeEventListener("keydown",h),document.body.style.overflow="unset"}},[h,R]);const r=g=>{c(g)},E=()=>{c(null)},A=()=>{c(g=>g===null?null:g>0?g-1:s.length-1)},w=()=>{c(g=>g===null?null:g<s.length-1?g+1:0)},_=async()=>{const g=d.map(S=>`${o[S.label]} ${S.x_center} ${S.y_center} ${S.width} ${S.height}`).join(`
`);try{await navigator.clipboard.writeText(g)}catch{}},z=async()=>{const g=new vg,S=g.folder("images"),v=g.folder("labels");for(const L of s){const j=L.file.name.replace(/\.[^.]+$/,"");S?.file(L.file.name,L.file);const tt=L.detections.map(F=>`${o[F.label]} ${F.x_center} ${F.y_center} ${F.width} ${F.height}`).join(`
`);v?.file(`${j}.txt`,tt)}const T=["path: .","train: images","val: images","test: images","","names:",...p.map((L,j)=>`  ${j}: ${L}`)].join(`
`);g.file("data.yaml",T);const D=await g.generateAsync({type:"blob"});Sg.saveAs(D,"dataset.zip")};return Z.jsxs("div",{className:"mt-6 space-y-6",children:[Z.jsxs("div",{className:"bg-white dark:bg-gray-800 rounded-lg p-6 border border-gray-200 dark:border-gray-700",children:[Z.jsxs("div",{className:"flex justify-between items-center mb-4",children:[Z.jsxs("h3",{className:"text-lg font-semibold",children:["Detection Results (",d.length," objects in ",s.length," image",s.length!==1?"s":"",")"]}),Z.jsxs("div",{className:"flex gap-2",children:[Z.jsx("button",{onClick:_,className:"px-4 py-2 bg-gray-200 dark:bg-gray-700 hover:bg-gray-300 dark:hover:bg-gray-600 rounded-lg text-sm transition-colors",children:"📋 Copy"}),Z.jsx("button",{onClick:z,className:"px-4 py-2 bg-green-600 hover:bg-green-700 text-white rounded-lg text-sm transition-colors",children:"📦 Download Dataset"})]})]}),Z.jsxs("div",{className:"grid grid-cols-1 md:grid-cols-3 gap-4",children:[Z.jsxs("div",{className:"bg-gray-100 dark:bg-gray-700 rounded-lg p-4 text-center border border-gray-200 dark:border-gray-600",children:[Z.jsx("div",{className:"text-2xl font-bold text-blue-600 dark:text-blue-400",children:s.length}),Z.jsx("div",{className:"text-sm text-gray-600 dark:text-gray-400",children:"Images"})]}),Z.jsxs("div",{className:"bg-gray-100 dark:bg-gray-700 rounded-lg p-4 text-center border border-gray-200 dark:border-gray-600",children:[Z.jsx("div",{className:"text-2xl font-bold text-green-600 dark:text-green-400",children:d.length}),Z.jsx("div",{className:"text-sm text-gray-600 dark:text-gray-400",children:"Objects Detected"})]}),Z.jsxs("div",{className:"bg-gray-100 dark:bg-gray-700 rounded-lg p-4 text-center border border-gray-200 dark:border-gray-600",children:[Z.jsx("div",{className:"text-2xl font-bold text-purple-600 dark:text-purple-400",children:new Set(d.map(g=>g.label)).size}),Z.jsx("div",{className:"text-sm text-gray-600 dark:text-gray-400",children:"Unique Labels"})]})]})]}),Z.jsx("div",{className:"flex flex-wrap gap-6 items-start",children:s.map((g,S)=>Z.jsx("div",{className:"bg-white dark:bg-gray-800 border border-gray-200 dark:border-gray-700 rounded-lg overflow-hidden w-full md:w-[calc(50%-0.75rem)] lg:w-[calc(33.333%-0.75rem)]",children:Z.jsxs("div",{className:"relative cursor-pointer w-full",onClick:()=>r(S),children:[Z.jsx("img",{src:g.previewUrl,alt:`preview-${S}`,className:"w-full h-auto object-contain z-0"}),Z.jsx("div",{className:"absolute inset-0 w-full h-full pointer-events-none z-10 bg-transparent",children:g.detections.map((v,T)=>{const D=(v.x_center-v.width/2)*100,L=(v.y_center-v.height/2)*100,j=v.width*100,tt=v.height*100,F=m(v.label);return Z.jsx("div",{className:"absolute border-2 pointer-events-none bg-transparent",style:{left:`${D}%`,top:`${L}%`,width:`${j}%`,height:`${tt}%`,borderColor:F},children:Z.jsx("span",{className:"absolute top-0 left-0 text-xs font-semibold px-1",style:{color:F,textShadow:"1px 1px 2px rgba(0,0,0,0.8)"},children:v.label})},T)})})]})},S))}),R!==null&&Z.jsxs("div",{className:"fixed inset-0 bg-black bg-opacity-90 flex items-center justify-center z-50",children:[Z.jsxs("div",{className:"relative max-w-7xl max-h-full w-full h-full flex items-center justify-center p-4",children:[Z.jsx("button",{onClick:E,className:"absolute top-4 right-4 text-white hover:text-gray-300 text-2xl z-10",children:"✕"}),s.length>1&&Z.jsxs(Z.Fragment,{children:[Z.jsx("button",{onClick:A,className:"absolute left-4 top-1/2 -translate-y-1/2 text-white hover:text-gray-300 text-3xl z-10",children:"‹"}),Z.jsx("button",{onClick:w,className:"absolute right-4 top-1/2 -translate-y-1/2 text-white hover:text-gray-300 text-3xl z-10",children:"›"})]}),Z.jsxs("div",{className:"relative max-w-full max-h-full",children:[Z.jsx("img",{src:s[R].previewUrl,alt:`full-${R}`,className:"max-w-full max-h-full object-contain"}),s[R].detections.map((g,S)=>{const v=(g.x_center-g.width/2)*100,T=(g.y_center-g.height/2)*100,D=g.width*100,L=g.height*100,j=m(g.label);return Z.jsx("div",{className:"absolute border-2",style:{left:`${v}%`,top:`${T}%`,width:`${D}%`,height:`${L}%`,borderColor:j},children:Z.jsx("span",{className:"absolute top-0 left-0 text-sm font-semibold px-1",style:{color:j,textShadow:"1px 1px 2px rgba(0,0,0,0.8)"},children:g.label})},S)})]}),Z.jsxs("div",{className:"absolute bottom-4 left-4 text-white bg-black bg-opacity-50 px-3 py-2 rounded",children:[Z.jsx("div",{className:"text-sm",children:s[R].file.name}),Z.jsxs("div",{className:"text-xs opacity-75",children:[R+1," of ",s.length," • ",s[R].detections.length," detection",s[R].detections.length!==1?"s":""]})]})]}),Z.jsx("div",{className:"absolute inset-0 -z-10",onClick:E})]})]})}const Eg="/assets/moonlabellogo-BPx9MdO3.svg";function wg(){const s=Js(),{theme:R,toggleTheme:c}=Eh(),[d,p]=I.useState(!1),[o,y]=I.useState([]),[m,h]=I.useState(null),r=async(E,A,w,_)=>{p(!0),h(null);try{const z=[];for(const S of A){const v=new FormData;v.append("image",E),v.append("objects",S),v.append("api_key",w),_&&v.append("station_endpoint",_);const T=await fetch("/api/detect",{method:"POST",body:v});if(!T.ok)throw new Error(`Detection failed: ${T.statusText}`);const D=await T.json();z.push(...D.detections)}const g=URL.createObjectURL(E);y(S=>[...S,{file:E,previewUrl:g,detections:z}])}catch(z){h(z instanceof Error?z.message:"Error")}finally{p(!1)}};return Z.jsxs("div",{className:"min-h-screen bg-gray-50 dark:bg-gray-900 text-gray-900 dark:text-white transition-colors",children:[Z.jsxs("header",{className:"flex justify-between items-center p-6 border-b border-gray-200 dark:border-gray-700",children:[Z.jsx("div",{children:Z.jsxs("h1",{className:"text-3xl font-bold flex items-center gap-3",children:[Z.jsx("img",{src:Eg,alt:"MoonLabel",className:"w-15 h-15"}),"MoonLabel"]})}),Z.jsxs("div",{className:"flex items-center gap-4",children:[Z.jsx("button",{onClick:c,className:"p-2 rounded-lg bg-gray-200 dark:bg-gray-800 hover:bg-gray-300 dark:hover:bg-gray-700 transition-colors",title:`Switch to ${R==="dark"?"light":"dark"} mode`,children:R==="dark"?"☀️":"🌙"}),Z.jsx("button",{onClick:()=>s("/settings"),className:"flex items-center gap-2 px-4 py-2 rounded-lg bg-gray-200 dark:bg-gray-800 hover:bg-gray-300 dark:hover:bg-gray-700 transition-colors",children:"⚙️ Settings"})]})]}),Z.jsx("main",{className:"px-6 pb-6",children:Z.jsxs("div",{className:"max-w-4xl mx-auto",children:[Z.jsx(pg,{onDetection:r,isProcessing:d}),m&&Z.jsx("div",{className:"mt-6 p-4 bg-red-100 dark:bg-red-900/50 border border-red-300 dark:border-red-700 rounded-lg",children:Z.jsxs("p",{className:"text-red-800 dark:text-red-200",children:["Error: ",m]})}),o.length>0&&Z.jsx(xg,{results:o})]})})]})}function Ag(){const s=Js(),{theme:R,toggleTheme:c}=Eh(),[d,p]=I.useState(""),[o,y]=I.useState("http://localhost:2020/v1"),[m,h]=I.useState("cloud"),[r,E]=I.useState(!1),[A,w]=I.useState(!1);I.useEffect(()=>{const S=localStorage.getItem("moondream_api_key");S&&p(S);const v=localStorage.getItem("moondream_station_endpoint");v&&y(v);const T=localStorage.getItem("moondream_mode");(T==="local"||T==="cloud"||T==="station")&&h(T)},[]);const _=()=>{m==="cloud"&&!d.trim()||m==="station"&&!o.trim()||(m==="cloud"?localStorage.setItem("moondream_api_key",d.trim()):m==="station"&&localStorage.setItem("moondream_station_endpoint",o.trim()),localStorage.setItem("moondream_mode",m),w(!0),setTimeout(()=>w(!1),2e3))},z=()=>{p(""),localStorage.removeItem("moondream_api_key")},g=()=>{y("http://localhost:2020/v1"),localStorage.removeItem("moondream_station_endpoint")};return Z.jsxs("div",{className:"min-h-screen bg-gray-50 dark:bg-gray-900 text-gray-900 dark:text-white transition-colors",children:[Z.jsxs("header",{className:"flex justify-between items-center p-6 border-b border-gray-200 dark:border-gray-700",children:[Z.jsxs("div",{className:"flex items-center gap-4",children:[Z.jsx("button",{onClick:()=>s("/"),className:"flex items-center gap-2 p-2 rounded-lg bg-gray-200 dark:bg-gray-800 hover:bg-gray-300 dark:hover:bg-gray-700 transition-colors",children:"← Back to Home"}),Z.jsxs("div",{children:[Z.jsx("h1",{className:"text-2xl font-bold",children:"Settings"}),Z.jsx("p",{className:"text-gray-600 dark:text-gray-400 text-sm",children:"Configure your Moondream API settings"})]})]}),Z.jsx("button",{onClick:c,className:"p-2 rounded-lg bg-gray-200 dark:bg-gray-800 hover:bg-gray-300 dark:hover:bg-gray-700 transition-colors",title:`Switch to ${R==="dark"?"light":"dark"} mode`,children:R==="dark"?"☀️":"🌙"})]}),Z.jsx("main",{className:"p-6",children:Z.jsxs("div",{className:"max-w-4xl mx-auto space-y-6",children:[Z.jsxs("div",{className:"bg-white dark:bg-gray-800 rounded-lg p-6 border border-gray-200 dark:border-gray-700",children:[Z.jsx("h2",{className:"text-xl font-semibold mb-2 flex items-center gap-2",children:"🔗 AI Backend Configuration"}),Z.jsx("p",{className:"text-gray-600 dark:text-gray-400 mb-6",children:"Select which backend to use for object detection and provide credentials if required."}),Z.jsxs("div",{className:"mb-4",children:[Z.jsx("label",{className:"block text-sm font-medium mb-2",children:"Backend Mode"}),Z.jsxs("div",{className:"flex gap-4",children:[Z.jsxs("label",{className:"inline-flex items-center gap-2 cursor-pointer",children:[Z.jsx("input",{type:"radio",name:"backend-mode",value:"cloud",checked:m==="cloud",onChange:()=>h("cloud")}),Z.jsx("span",{children:"Moondream Cloud"})]}),Z.jsxs("label",{className:"inline-flex items-center gap-2 cursor-pointer",children:[Z.jsx("input",{type:"radio",name:"backend-mode",value:"station",checked:m==="station",onChange:()=>h("station")}),Z.jsx("span",{children:"Moondream Station"})]}),Z.jsxs("label",{className:"inline-flex items-center gap-2 cursor-pointer",children:[Z.jsx("input",{type:"radio",name:"backend-mode",value:"local",checked:m==="local",onChange:()=>h("local")}),Z.jsx("span",{children:"Local (HuggingFace)"})]})]})]}),Z.jsxs("div",{className:"space-y-4",children:[m==="cloud"&&Z.jsxs("div",{children:[Z.jsx("label",{className:"block text-sm font-medium mb-2",children:"API Key"}),Z.jsxs("div",{className:"relative",children:[Z.jsx("input",{type:r?"text":"password",value:d,onChange:S=>p(S.target.value),placeholder:"Enter your AI API key",className:"w-full px-3 py-2 pr-12 bg-gray-100 dark:bg-gray-700 border border-gray-300 dark:border-gray-600 rounded-lg focus:outline-none focus:border-blue-400 text-gray-900 dark:text-white"}),Z.jsx("button",{type:"button",onClick:()=>E(!r),className:"absolute right-3 top-1/2 transform -translate-y-1/2 text-gray-500 dark:text-gray-400 hover:text-gray-700 dark:hover:text-white",children:r?"🙈":"👁️"})]})]}),m==="station"&&Z.jsxs("div",{children:[Z.jsx("label",{className:"block text-sm font-medium mb-2",children:"Station Endpoint"}),Z.jsx("input",{type:"url",value:o,onChange:S=>y(S.target.value),placeholder:"http://localhost:2020/v1",className:"w-full px-3 py-2 bg-gray-100 dark:bg-gray-700 border border-gray-300 dark:border-gray-600 rounded-lg focus:outline-none focus:border-blue-400 text-gray-900 dark:text-white"}),Z.jsx("p",{className:"text-sm text-gray-500 dark:text-gray-500 mt-2",children:"Make sure your Moondream Station is running locally"})]}),Z.jsxs("div",{className:"flex gap-3",children:[Z.jsx("button",{onClick:_,disabled:m==="cloud"&&!d.trim()||m==="station"&&!o.trim(),className:`px-4 py-2 rounded-lg font-medium transition-colors ${m==="cloud"&&d.trim()||m==="station"&&o.trim()||m==="local"?"bg-blue-600 hover:bg-blue-700 text-white":"bg-gray-300 dark:bg-gray-600 text-gray-500 dark:text-gray-400 cursor-not-allowed"}`,children:"💾 Save Settings"}),m==="cloud"&&d&&Z.jsx("button",{onClick:z,className:"px-4 py-2 rounded-lg bg-red-600 hover:bg-red-700 text-white transition-colors",children:"Clear API Key"}),m==="station"&&o&&Z.jsx("button",{onClick:g,className:"px-4 py-2 rounded-lg bg-red-600 hover:bg-red-700 text-white transition-colors",children:"Reset Endpoint"}),A&&Z.jsx("div",{className:"flex items-center text-green-600 dark:text-green-400 text-sm",children:"✅ Settings saved successfully!"})]})]})]}),Z.jsxs("div",{className:"bg-white dark:bg-gray-800 rounded-lg p-6 border border-gray-200 dark:border-gray-700",children:[Z.jsx("h2",{className:"text-xl font-semibold mb-2",children:"Supported AI Services"}),Z.jsx("p",{className:"text-gray-600 dark:text-gray-400 mb-6",children:"Compatible AI services for object detection"}),Z.jsx("div",{className:"space-y-4",children:Z.jsxs("div",{className:"bg-gray-100 dark:bg-gray-700 rounded-lg p-4 border border-gray-200 dark:border-gray-600",children:[Z.jsx("h3",{className:"font-semibold mb-2",children:"Moondream Vision API"}),Z.jsx("p",{className:"text-gray-700 dark:text-gray-300 mb-3",children:"Use Moondream for object detection and annotation with three different deployment options"}),Z.jsxs("div",{className:"space-y-4",children:[Z.jsxs("div",{children:[Z.jsx("h4",{className:"font-medium text-blue-600 dark:text-blue-400 mb-2",children:"Option A: Cloud"}),Z.jsx("p",{className:"text-sm text-gray-700 dark:text-gray-300 mb-2",children:"Fastest setup - no downloads or installations required:"}),Z.jsxs("ul",{className:"text-sm text-gray-700 dark:text-gray-300 space-y-1 ml-4",children:[Z.jsxs("li",{children:["• Create an API key at the ",Z.jsx("a",{href:"https://moondream.ai",target:"_blank",rel:"noopener noreferrer",className:"text-blue-600 dark:text-blue-400 hover:text-blue-700 dark:hover:text-blue-300 underline",children:"Moondream Cloud Console"})," (free tier available, no credit card required)"]}),Z.jsx("li",{children:"• Copy and save your API key"}),Z.jsx("li",{children:"• Ready to use immediately"})]})]}),Z.jsxs("div",{children:[Z.jsx("h4",{className:"font-medium text-purple-600 dark:text-purple-400 mb-2",children:"Option B: Station"}),Z.jsx("p",{className:"text-sm text-gray-700 dark:text-gray-300 mb-2",children:"Native on-device inference with Moondream Station (Mac/Linux):"}),Z.jsxs("ul",{className:"text-sm text-gray-700 dark:text-gray-300 space-y-1 ml-4",children:[Z.jsxs("li",{children:["• Download and install ",Z.jsx("a",{href:"https://moondream.ai/station",target:"_blank",rel:"noopener noreferrer",className:"text-purple-600 dark:text-purple-400 hover:text-purple-700 dark:hover:text-purple-300 underline",children:"Moondream Station"})]}),Z.jsxs("li",{children:["• Run Station locally (typically on ",Z.jsx("code",{children:"http://localhost:2020/v1"}),")"]}),Z.jsx("li",{children:"• Works completely offline once installed"})]})]}),Z.jsxs("div",{children:[Z.jsx("h4",{className:"font-medium text-green-600 dark:text-green-400 mb-2",children:"Option C: Local (Hugging Face)"}),Z.jsxs("p",{className:"text-sm text-gray-700 dark:text-gray-300 mb-2",children:["Runs the open-source ",Z.jsx("code",{children:"vikhyatk/moondream2"})," model directly in the backend:"]}),Z.jsxs("ul",{className:"text-sm text-gray-700 dark:text-gray-300 space-y-1 ml-4",children:[Z.jsx("li",{children:"• Works offline once the model is downloaded (first run)"}),Z.jsx("li",{children:"• Automatically uses GPU (CUDA) or Apple Silicon (MPS) if available"})]})]})]})]})})]})]})})]})}function zg(){return Z.jsx(mg,{children:Z.jsx(lg,{children:Z.jsx("div",{className:"min-h-screen bg-gray-50 dark:bg-gray-900 text-gray-900 dark:text-white transition-colors",children:Z.jsxs(Mp,{children:[Z.jsx(Xs,{path:"/",element:Z.jsx(wg,{})}),Z.jsx(Xs,{path:"/settings",element:Z.jsx(Ag,{})})]})})})})}Q0.createRoot(document.getElementById("root")).render(Z.jsx(I.StrictMode,{children:Z.jsx(zg,{})}));
